// Copyright (c) 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "chrome/browser/extensions/api/generated_api_registration.h"

#include "chrome/browser/extensions/api/webrtc_logging_private/webrtc_logging_private_api.h"
#include "chrome/browser/extensions/api/cookies/cookies_api.h"
#include "chrome/browser/extensions/api/top_sites/top_sites_api.h"
#include "chrome/browser/extensions/api/web_view/chrome_web_view_internal_api.h"
#include "chrome/browser/extensions/api/command_line_private/command_line_private_api.h"
#include "chrome/browser/extensions/api/braille_display_private/braille_display_private_api.h"
#include "chrome/browser/extensions/api/autofill_private/autofill_private_api.h"
#include "chrome/browser/extensions/api/activity_log_private/activity_log_private_api.h"
#include "chrome/browser/extensions/api/settings_private/settings_private_api.h"
#include "chrome/browser/extensions/api/resources_private/resources_private_api.h"
#include "chrome/browser/extensions/api/enterprise_reporting_private/enterprise_reporting_private_api.h"
#include "chrome/browser/extensions/api/bookmarks/bookmarks_api.h"
#include "chrome/browser/extensions/api/context_menus/context_menus_api.h"
#include "chrome/browser/extensions/api/debugger/debugger_api.h"
#include "chrome/browser/extensions/api/mdns/mdns_api.h"
#include "chrome/browser/extensions/api/dashboard_private/dashboard_private_api.h"
#include "chrome/browser/extensions/api/instance_id/instance_id_api.h"
#include "chrome/browser/extensions/api/sessions/sessions_api.h"
#include "chrome/browser/extensions/api/gcm/gcm_api.h"
#include "chrome/browser/extensions/api/desktop_capture/desktop_capture_api.h"
#include "chrome/browser/extensions/api/signed_in_devices/signed_in_devices_api.h"
#if defined(OS_CHROMEOS)
#include "chrome/browser/chromeos/extensions/autotest_private/autotest_private_api.h"
#endif  // defined(OS_CHROMEOS)
#include "chrome/browser/extensions/api/system_indicator/system_indicator_api.h"
#include "chrome/browser/extensions/api/web_navigation/web_navigation_api.h"
#include "chrome/browser/extensions/api/webrtc_desktop_capture_private/webrtc_desktop_capture_private_api.h"
#include "chrome/browser/extensions/api/system_private/system_private_api.h"
#include "chrome/browser/extensions/api/tab_capture/tab_capture_api.h"
#include "chrome/browser/extensions/api/safe_browsing_private/safe_browsing_private_api.h"
#include "chrome/browser/extensions/api/cryptotoken_private/cryptotoken_private_api.h"
#include "chrome/browser/extensions/api/data_reduction_proxy/data_reduction_proxy_api.h"
#include "chrome/browser/extensions/api/image_writer_private/image_writer_private_api.h"
#include "chrome/browser/extensions/api/downloads/downloads_api.h"
#include "chrome/browser/extensions/api/font_settings/font_settings_api.h"
#include "chrome/browser/extensions/api/enterprise_hardware_platform/enterprise_hardware_platform_api.h"
#include "chrome/browser/extensions/api/notifications/notifications_api.h"
#include "chrome/browser/extensions/api/tabs/tabs_api.h"
#include "chrome/browser/extensions/api/webrtc_audio_private/webrtc_audio_private_api.h"
#include "chrome/browser/extensions/api/i18n/i18n_api.h"
#include "chrome/browser/extensions/api/language_settings_private/language_settings_private_api.h"
#include "chrome/browser/extensions/api/cloud_print_private/cloud_print_private_api.h"
#include "chrome/browser/extensions/api/idltest/idltest_api.h"
#include "chrome/browser/extensions/api/identity/identity_api.h"
#include "chrome/browser/extensions/api/permissions/permissions_api.h"
#include "chrome/browser/extensions/api/processes/processes_api.h"
#include "chrome/browser/extensions/api/input_ime/input_ime_api.h"
#include "chrome/browser/extensions/api/downloads_internal/downloads_internal_api.h"
#include "chrome/browser/extensions/api/module/module.h"
#include "chrome/browser/extensions/api/bookmark_manager_private/bookmark_manager_private_api.h"
#include "chrome/browser/extensions/api/developer_private/developer_private_api.h"
#include "chrome/browser/extensions/api/tabs/tabs_api.h"
#include "chrome/browser/extensions/api/browsing_data/browsing_data_api.h"
#include "chrome/browser/extensions/api/omnibox/omnibox_api.h"
#include "chrome/browser/extensions/api/content_settings/content_settings_api.h"
#include "chrome/browser/extensions/api/webstore_private/webstore_private_api.h"
#include "chrome/browser/extensions/api/page_capture/page_capture_api.h"
#include "chrome/browser/extensions/api/passwords_private/passwords_private_api.h"
#include "chrome/browser/accessibility/accessibility_extension_api.h"
#include "chrome/browser/extensions/api/history/history_api.h"

#include "extensions/browser/extension_function_registry.h"

namespace extensions {
namespace api {

// static
void ChromeGeneratedFunctionRegistry::RegisterAll(ExtensionFunctionRegistry* registry) {
  constexpr ExtensionFunctionRegistry::FactoryEntry kEntries[] = {
    {
      &NewExtensionFunction<WebrtcLoggingPrivateSetMetaDataFunction>,
      WebrtcLoggingPrivateSetMetaDataFunction::function_name(),
      WebrtcLoggingPrivateSetMetaDataFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateStartFunction>,
      WebrtcLoggingPrivateStartFunction::function_name(),
      WebrtcLoggingPrivateStartFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateSetUploadOnRenderCloseFunction>,
      WebrtcLoggingPrivateSetUploadOnRenderCloseFunction::function_name(),
      WebrtcLoggingPrivateSetUploadOnRenderCloseFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateStopFunction>,
      WebrtcLoggingPrivateStopFunction::function_name(),
      WebrtcLoggingPrivateStopFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateStoreFunction>,
      WebrtcLoggingPrivateStoreFunction::function_name(),
      WebrtcLoggingPrivateStoreFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateUploadStoredFunction>,
      WebrtcLoggingPrivateUploadStoredFunction::function_name(),
      WebrtcLoggingPrivateUploadStoredFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateUploadFunction>,
      WebrtcLoggingPrivateUploadFunction::function_name(),
      WebrtcLoggingPrivateUploadFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateDiscardFunction>,
      WebrtcLoggingPrivateDiscardFunction::function_name(),
      WebrtcLoggingPrivateDiscardFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateStartRtpDumpFunction>,
      WebrtcLoggingPrivateStartRtpDumpFunction::function_name(),
      WebrtcLoggingPrivateStartRtpDumpFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateStopRtpDumpFunction>,
      WebrtcLoggingPrivateStopRtpDumpFunction::function_name(),
      WebrtcLoggingPrivateStopRtpDumpFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateStartAudioDebugRecordingsFunction>,
      WebrtcLoggingPrivateStartAudioDebugRecordingsFunction::function_name(),
      WebrtcLoggingPrivateStartAudioDebugRecordingsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateStopAudioDebugRecordingsFunction>,
      WebrtcLoggingPrivateStopAudioDebugRecordingsFunction::function_name(),
      WebrtcLoggingPrivateStopAudioDebugRecordingsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateStartEventLoggingFunction>,
      WebrtcLoggingPrivateStartEventLoggingFunction::function_name(),
      WebrtcLoggingPrivateStartEventLoggingFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcLoggingPrivateGetLogsDirectoryFunction>,
      WebrtcLoggingPrivateGetLogsDirectoryFunction::function_name(),
      WebrtcLoggingPrivateGetLogsDirectoryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CookiesGetFunction>,
      CookiesGetFunction::function_name(),
      CookiesGetFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CookiesGetAllFunction>,
      CookiesGetAllFunction::function_name(),
      CookiesGetAllFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CookiesSetFunction>,
      CookiesSetFunction::function_name(),
      CookiesSetFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CookiesRemoveFunction>,
      CookiesRemoveFunction::function_name(),
      CookiesRemoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CookiesGetAllCookieStoresFunction>,
      CookiesGetAllCookieStoresFunction::function_name(),
      CookiesGetAllCookieStoresFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TopSitesGetFunction>,
      TopSitesGetFunction::function_name(),
      TopSitesGetFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ChromeWebViewInternalContextMenusCreateFunction>,
      ChromeWebViewInternalContextMenusCreateFunction::function_name(),
      ChromeWebViewInternalContextMenusCreateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ChromeWebViewInternalContextMenusUpdateFunction>,
      ChromeWebViewInternalContextMenusUpdateFunction::function_name(),
      ChromeWebViewInternalContextMenusUpdateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ChromeWebViewInternalContextMenusRemoveFunction>,
      ChromeWebViewInternalContextMenusRemoveFunction::function_name(),
      ChromeWebViewInternalContextMenusRemoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ChromeWebViewInternalContextMenusRemoveAllFunction>,
      ChromeWebViewInternalContextMenusRemoveAllFunction::function_name(),
      ChromeWebViewInternalContextMenusRemoveAllFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ChromeWebViewInternalShowContextMenuFunction>,
      ChromeWebViewInternalShowContextMenuFunction::function_name(),
      ChromeWebViewInternalShowContextMenuFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CommandLinePrivateHasSwitchFunction>,
      CommandLinePrivateHasSwitchFunction::function_name(),
      CommandLinePrivateHasSwitchFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrailleDisplayPrivateGetDisplayStateFunction>,
      BrailleDisplayPrivateGetDisplayStateFunction::function_name(),
      BrailleDisplayPrivateGetDisplayStateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrailleDisplayPrivateWriteDotsFunction>,
      BrailleDisplayPrivateWriteDotsFunction::function_name(),
      BrailleDisplayPrivateWriteDotsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrailleDisplayPrivateUpdateBluetoothBrailleDisplayAddressFunction>,
      BrailleDisplayPrivateUpdateBluetoothBrailleDisplayAddressFunction::function_name(),
      BrailleDisplayPrivateUpdateBluetoothBrailleDisplayAddressFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateSaveAddressFunction>,
      AutofillPrivateSaveAddressFunction::function_name(),
      AutofillPrivateSaveAddressFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateGetCountryListFunction>,
      AutofillPrivateGetCountryListFunction::function_name(),
      AutofillPrivateGetCountryListFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateGetAddressComponentsFunction>,
      AutofillPrivateGetAddressComponentsFunction::function_name(),
      AutofillPrivateGetAddressComponentsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateGetAddressListFunction>,
      AutofillPrivateGetAddressListFunction::function_name(),
      AutofillPrivateGetAddressListFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateSaveCreditCardFunction>,
      AutofillPrivateSaveCreditCardFunction::function_name(),
      AutofillPrivateSaveCreditCardFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateRemoveEntryFunction>,
      AutofillPrivateRemoveEntryFunction::function_name(),
      AutofillPrivateRemoveEntryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateValidatePhoneNumbersFunction>,
      AutofillPrivateValidatePhoneNumbersFunction::function_name(),
      AutofillPrivateValidatePhoneNumbersFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateGetCreditCardListFunction>,
      AutofillPrivateGetCreditCardListFunction::function_name(),
      AutofillPrivateGetCreditCardListFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateMaskCreditCardFunction>,
      AutofillPrivateMaskCreditCardFunction::function_name(),
      AutofillPrivateMaskCreditCardFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateMigrateCreditCardsFunction>,
      AutofillPrivateMigrateCreditCardsFunction::function_name(),
      AutofillPrivateMigrateCreditCardsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutofillPrivateLogServerCardLinkClickedFunction>,
      AutofillPrivateLogServerCardLinkClickedFunction::function_name(),
      AutofillPrivateLogServerCardLinkClickedFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ActivityLogPrivateGetExtensionActivitiesFunction>,
      ActivityLogPrivateGetExtensionActivitiesFunction::function_name(),
      ActivityLogPrivateGetExtensionActivitiesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ActivityLogPrivateDeleteActivitiesFunction>,
      ActivityLogPrivateDeleteActivitiesFunction::function_name(),
      ActivityLogPrivateDeleteActivitiesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ActivityLogPrivateDeleteActivitiesByExtensionFunction>,
      ActivityLogPrivateDeleteActivitiesByExtensionFunction::function_name(),
      ActivityLogPrivateDeleteActivitiesByExtensionFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ActivityLogPrivateDeleteDatabaseFunction>,
      ActivityLogPrivateDeleteDatabaseFunction::function_name(),
      ActivityLogPrivateDeleteDatabaseFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ActivityLogPrivateDeleteUrlsFunction>,
      ActivityLogPrivateDeleteUrlsFunction::function_name(),
      ActivityLogPrivateDeleteUrlsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SettingsPrivateSetPrefFunction>,
      SettingsPrivateSetPrefFunction::function_name(),
      SettingsPrivateSetPrefFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SettingsPrivateGetAllPrefsFunction>,
      SettingsPrivateGetAllPrefsFunction::function_name(),
      SettingsPrivateGetAllPrefsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SettingsPrivateGetPrefFunction>,
      SettingsPrivateGetPrefFunction::function_name(),
      SettingsPrivateGetPrefFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SettingsPrivateGetDefaultZoomFunction>,
      SettingsPrivateGetDefaultZoomFunction::function_name(),
      SettingsPrivateGetDefaultZoomFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SettingsPrivateSetDefaultZoomFunction>,
      SettingsPrivateSetDefaultZoomFunction::function_name(),
      SettingsPrivateSetDefaultZoomFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ResourcesPrivateGetStringsFunction>,
      ResourcesPrivateGetStringsFunction::function_name(),
      ResourcesPrivateGetStringsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<EnterpriseReportingPrivateUploadChromeDesktopReportFunction>,
      EnterpriseReportingPrivateUploadChromeDesktopReportFunction::function_name(),
      EnterpriseReportingPrivateUploadChromeDesktopReportFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<EnterpriseReportingPrivateGetDeviceIdFunction>,
      EnterpriseReportingPrivateGetDeviceIdFunction::function_name(),
      EnterpriseReportingPrivateGetDeviceIdFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksGetFunction>,
      BookmarksGetFunction::function_name(),
      BookmarksGetFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksGetChildrenFunction>,
      BookmarksGetChildrenFunction::function_name(),
      BookmarksGetChildrenFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksGetRecentFunction>,
      BookmarksGetRecentFunction::function_name(),
      BookmarksGetRecentFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksGetTreeFunction>,
      BookmarksGetTreeFunction::function_name(),
      BookmarksGetTreeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksGetSubTreeFunction>,
      BookmarksGetSubTreeFunction::function_name(),
      BookmarksGetSubTreeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksSearchFunction>,
      BookmarksSearchFunction::function_name(),
      BookmarksSearchFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksCreateFunction>,
      BookmarksCreateFunction::function_name(),
      BookmarksCreateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksMoveFunction>,
      BookmarksMoveFunction::function_name(),
      BookmarksMoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksUpdateFunction>,
      BookmarksUpdateFunction::function_name(),
      BookmarksUpdateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksRemoveFunction>,
      BookmarksRemoveFunction::function_name(),
      BookmarksRemoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksRemoveTreeFunction>,
      BookmarksRemoveTreeFunction::function_name(),
      BookmarksRemoveTreeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksImportFunction>,
      BookmarksImportFunction::function_name(),
      BookmarksImportFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarksExportFunction>,
      BookmarksExportFunction::function_name(),
      BookmarksExportFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ContextMenusCreateFunction>,
      ContextMenusCreateFunction::function_name(),
      ContextMenusCreateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ContextMenusUpdateFunction>,
      ContextMenusUpdateFunction::function_name(),
      ContextMenusUpdateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ContextMenusRemoveFunction>,
      ContextMenusRemoveFunction::function_name(),
      ContextMenusRemoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ContextMenusRemoveAllFunction>,
      ContextMenusRemoveAllFunction::function_name(),
      ContextMenusRemoveAllFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DebuggerAttachFunction>,
      DebuggerAttachFunction::function_name(),
      DebuggerAttachFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DebuggerDetachFunction>,
      DebuggerDetachFunction::function_name(),
      DebuggerDetachFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DebuggerSendCommandFunction>,
      DebuggerSendCommandFunction::function_name(),
      DebuggerSendCommandFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DebuggerGetTargetsFunction>,
      DebuggerGetTargetsFunction::function_name(),
      DebuggerGetTargetsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<MdnsForceDiscoveryFunction>,
      MdnsForceDiscoveryFunction::function_name(),
      MdnsForceDiscoveryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DashboardPrivateShowPermissionPromptForDelegatedInstallFunction>,
      DashboardPrivateShowPermissionPromptForDelegatedInstallFunction::function_name(),
      DashboardPrivateShowPermissionPromptForDelegatedInstallFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<InstanceIDGetIDFunction>,
      InstanceIDGetIDFunction::function_name(),
      InstanceIDGetIDFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<InstanceIDGetCreationTimeFunction>,
      InstanceIDGetCreationTimeFunction::function_name(),
      InstanceIDGetCreationTimeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<InstanceIDGetTokenFunction>,
      InstanceIDGetTokenFunction::function_name(),
      InstanceIDGetTokenFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<InstanceIDDeleteTokenFunction>,
      InstanceIDDeleteTokenFunction::function_name(),
      InstanceIDDeleteTokenFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<InstanceIDDeleteIDFunction>,
      InstanceIDDeleteIDFunction::function_name(),
      InstanceIDDeleteIDFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SessionsGetRecentlyClosedFunction>,
      SessionsGetRecentlyClosedFunction::function_name(),
      SessionsGetRecentlyClosedFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SessionsGetDevicesFunction>,
      SessionsGetDevicesFunction::function_name(),
      SessionsGetDevicesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SessionsRestoreFunction>,
      SessionsRestoreFunction::function_name(),
      SessionsRestoreFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<GcmRegisterFunction>,
      GcmRegisterFunction::function_name(),
      GcmRegisterFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<GcmUnregisterFunction>,
      GcmUnregisterFunction::function_name(),
      GcmUnregisterFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<GcmSendFunction>,
      GcmSendFunction::function_name(),
      GcmSendFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DesktopCaptureChooseDesktopMediaFunction>,
      DesktopCaptureChooseDesktopMediaFunction::function_name(),
      DesktopCaptureChooseDesktopMediaFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DesktopCaptureCancelChooseDesktopMediaFunction>,
      DesktopCaptureCancelChooseDesktopMediaFunction::function_name(),
      DesktopCaptureCancelChooseDesktopMediaFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SignedInDevicesGetFunction>,
      SignedInDevicesGetFunction::function_name(),
      SignedInDevicesGetFunction::histogram_value(),
    },
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AutotestPrivateLogoutFunction>,
      AutotestPrivateLogoutFunction::function_name(),
      AutotestPrivateLogoutFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateRestartFunction>,
      AutotestPrivateRestartFunction::function_name(),
      AutotestPrivateRestartFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateShutdownFunction>,
      AutotestPrivateShutdownFunction::function_name(),
      AutotestPrivateShutdownFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateLoginStatusFunction>,
      AutotestPrivateLoginStatusFunction::function_name(),
      AutotestPrivateLoginStatusFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateLockScreenFunction>,
      AutotestPrivateLockScreenFunction::function_name(),
      AutotestPrivateLockScreenFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetExtensionsInfoFunction>,
      AutotestPrivateGetExtensionsInfoFunction::function_name(),
      AutotestPrivateGetExtensionsInfoFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSimulateAsanMemoryBugFunction>,
      AutotestPrivateSimulateAsanMemoryBugFunction::function_name(),
      AutotestPrivateSimulateAsanMemoryBugFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetTouchpadSensitivityFunction>,
      AutotestPrivateSetTouchpadSensitivityFunction::function_name(),
      AutotestPrivateSetTouchpadSensitivityFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetTapToClickFunction>,
      AutotestPrivateSetTapToClickFunction::function_name(),
      AutotestPrivateSetTapToClickFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetThreeFingerClickFunction>,
      AutotestPrivateSetThreeFingerClickFunction::function_name(),
      AutotestPrivateSetThreeFingerClickFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetTapDraggingFunction>,
      AutotestPrivateSetTapDraggingFunction::function_name(),
      AutotestPrivateSetTapDraggingFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetNaturalScrollFunction>,
      AutotestPrivateSetNaturalScrollFunction::function_name(),
      AutotestPrivateSetNaturalScrollFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetMouseSensitivityFunction>,
      AutotestPrivateSetMouseSensitivityFunction::function_name(),
      AutotestPrivateSetMouseSensitivityFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetPrimaryButtonRightFunction>,
      AutotestPrivateSetPrimaryButtonRightFunction::function_name(),
      AutotestPrivateSetPrimaryButtonRightFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetMouseReverseScrollFunction>,
      AutotestPrivateSetMouseReverseScrollFunction::function_name(),
      AutotestPrivateSetMouseReverseScrollFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetVisibleNotificationsFunction>,
      AutotestPrivateGetVisibleNotificationsFunction::function_name(),
      AutotestPrivateGetVisibleNotificationsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetArcStateFunction>,
      AutotestPrivateGetArcStateFunction::function_name(),
      AutotestPrivateGetArcStateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetPlayStoreStateFunction>,
      AutotestPrivateGetPlayStoreStateFunction::function_name(),
      AutotestPrivateGetPlayStoreStateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetPrinterListFunction>,
      AutotestPrivateGetPrinterListFunction::function_name(),
      AutotestPrivateGetPrinterListFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateIsAppShownFunction>,
      AutotestPrivateIsAppShownFunction::function_name(),
      AutotestPrivateIsAppShownFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateIsArcProvisionedFunction>,
      AutotestPrivateIsArcProvisionedFunction::function_name(),
      AutotestPrivateIsArcProvisionedFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetArcAppFunction>,
      AutotestPrivateGetArcAppFunction::function_name(),
      AutotestPrivateGetArcAppFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetArcPackageFunction>,
      AutotestPrivateGetArcPackageFunction::function_name(),
      AutotestPrivateGetArcPackageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateLaunchArcAppFunction>,
      AutotestPrivateLaunchArcAppFunction::function_name(),
      AutotestPrivateLaunchArcAppFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateLaunchAppFunction>,
      AutotestPrivateLaunchAppFunction::function_name(),
      AutotestPrivateLaunchAppFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateCloseAppFunction>,
      AutotestPrivateCloseAppFunction::function_name(),
      AutotestPrivateCloseAppFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateUpdatePrinterFunction>,
      AutotestPrivateUpdatePrinterFunction::function_name(),
      AutotestPrivateUpdatePrinterFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateRemovePrinterFunction>,
      AutotestPrivateRemovePrinterFunction::function_name(),
      AutotestPrivateRemovePrinterFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetPlayStoreEnabledFunction>,
      AutotestPrivateSetPlayStoreEnabledFunction::function_name(),
      AutotestPrivateSetPlayStoreEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetHistogramFunction>,
      AutotestPrivateGetHistogramFunction::function_name(),
      AutotestPrivateGetHistogramFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateRunCrostiniInstallerFunction>,
      AutotestPrivateRunCrostiniInstallerFunction::function_name(),
      AutotestPrivateRunCrostiniInstallerFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateRunCrostiniUninstallerFunction>,
      AutotestPrivateRunCrostiniUninstallerFunction::function_name(),
      AutotestPrivateRunCrostiniUninstallerFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetCrostiniEnabledFunction>,
      AutotestPrivateSetCrostiniEnabledFunction::function_name(),
      AutotestPrivateSetCrostiniEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateExportCrostiniFunction>,
      AutotestPrivateExportCrostiniFunction::function_name(),
      AutotestPrivateExportCrostiniFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateImportCrostiniFunction>,
      AutotestPrivateImportCrostiniFunction::function_name(),
      AutotestPrivateImportCrostiniFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateTakeScreenshotFunction>,
      AutotestPrivateTakeScreenshotFunction::function_name(),
      AutotestPrivateTakeScreenshotFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateBootstrapMachineLearningServiceFunction>,
      AutotestPrivateBootstrapMachineLearningServiceFunction::function_name(),
      AutotestPrivateBootstrapMachineLearningServiceFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetAssistantEnabledFunction>,
      AutotestPrivateSetAssistantEnabledFunction::function_name(),
      AutotestPrivateSetAssistantEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSendAssistantTextQueryFunction>,
      AutotestPrivateSendAssistantTextQueryFunction::function_name(),
      AutotestPrivateSendAssistantTextQueryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetWhitelistedPrefFunction>,
      AutotestPrivateSetWhitelistedPrefFunction::function_name(),
      AutotestPrivateSetWhitelistedPrefFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetCrostiniAppScaledFunction>,
      AutotestPrivateSetCrostiniAppScaledFunction::function_name(),
      AutotestPrivateSetCrostiniAppScaledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetPrimaryDisplayScaleFactorFunction>,
      AutotestPrivateGetPrimaryDisplayScaleFactorFunction::function_name(),
      AutotestPrivateGetPrimaryDisplayScaleFactorFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateIsTabletModeEnabledFunction>,
      AutotestPrivateIsTabletModeEnabledFunction::function_name(),
      AutotestPrivateIsTabletModeEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetTabletModeEnabledFunction>,
      AutotestPrivateSetTabletModeEnabledFunction::function_name(),
      AutotestPrivateSetTabletModeEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetShelfAutoHideBehaviorFunction>,
      AutotestPrivateGetShelfAutoHideBehaviorFunction::function_name(),
      AutotestPrivateGetShelfAutoHideBehaviorFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetShelfAutoHideBehaviorFunction>,
      AutotestPrivateSetShelfAutoHideBehaviorFunction::function_name(),
      AutotestPrivateSetShelfAutoHideBehaviorFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateGetShelfAlignmentFunction>,
      AutotestPrivateGetShelfAlignmentFunction::function_name(),
      AutotestPrivateGetShelfAlignmentFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateSetShelfAlignmentFunction>,
      AutotestPrivateSetShelfAlignmentFunction::function_name(),
      AutotestPrivateSetShelfAlignmentFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AutotestPrivateShowVirtualKeyboardIfEnabledFunction>,
      AutotestPrivateShowVirtualKeyboardIfEnabledFunction::function_name(),
      AutotestPrivateShowVirtualKeyboardIfEnabledFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<SystemIndicatorSetIconFunction>,
      SystemIndicatorSetIconFunction::function_name(),
      SystemIndicatorSetIconFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SystemIndicatorEnableFunction>,
      SystemIndicatorEnableFunction::function_name(),
      SystemIndicatorEnableFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SystemIndicatorDisableFunction>,
      SystemIndicatorDisableFunction::function_name(),
      SystemIndicatorDisableFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebNavigationGetFrameFunction>,
      WebNavigationGetFrameFunction::function_name(),
      WebNavigationGetFrameFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebNavigationGetAllFramesFunction>,
      WebNavigationGetAllFramesFunction::function_name(),
      WebNavigationGetAllFramesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcDesktopCapturePrivateChooseDesktopMediaFunction>,
      WebrtcDesktopCapturePrivateChooseDesktopMediaFunction::function_name(),
      WebrtcDesktopCapturePrivateChooseDesktopMediaFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcDesktopCapturePrivateCancelChooseDesktopMediaFunction>,
      WebrtcDesktopCapturePrivateCancelChooseDesktopMediaFunction::function_name(),
      WebrtcDesktopCapturePrivateCancelChooseDesktopMediaFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SystemPrivateGetIncognitoModeAvailabilityFunction>,
      SystemPrivateGetIncognitoModeAvailabilityFunction::function_name(),
      SystemPrivateGetIncognitoModeAvailabilityFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SystemPrivateGetUpdateStatusFunction>,
      SystemPrivateGetUpdateStatusFunction::function_name(),
      SystemPrivateGetUpdateStatusFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SystemPrivateGetApiKeyFunction>,
      SystemPrivateGetApiKeyFunction::function_name(),
      SystemPrivateGetApiKeyFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabCaptureCaptureFunction>,
      TabCaptureCaptureFunction::function_name(),
      TabCaptureCaptureFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabCaptureGetCapturedTabsFunction>,
      TabCaptureGetCapturedTabsFunction::function_name(),
      TabCaptureGetCapturedTabsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabCaptureCaptureOffscreenTabFunction>,
      TabCaptureCaptureOffscreenTabFunction::function_name(),
      TabCaptureCaptureOffscreenTabFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabCaptureGetMediaStreamIdFunction>,
      TabCaptureGetMediaStreamIdFunction::function_name(),
      TabCaptureGetMediaStreamIdFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<SafeBrowsingPrivateGetReferrerChainFunction>,
      SafeBrowsingPrivateGetReferrerChainFunction::function_name(),
      SafeBrowsingPrivateGetReferrerChainFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CryptotokenPrivateCanOriginAssertAppIdFunction>,
      CryptotokenPrivateCanOriginAssertAppIdFunction::function_name(),
      CryptotokenPrivateCanOriginAssertAppIdFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CryptotokenPrivateIsAppIdHashInEnterpriseContextFunction>,
      CryptotokenPrivateIsAppIdHashInEnterpriseContextFunction::function_name(),
      CryptotokenPrivateIsAppIdHashInEnterpriseContextFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CryptotokenPrivateCanAppIdGetAttestationFunction>,
      CryptotokenPrivateCanAppIdGetAttestationFunction::function_name(),
      CryptotokenPrivateCanAppIdGetAttestationFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DataReductionProxyClearDataSavingsFunction>,
      DataReductionProxyClearDataSavingsFunction::function_name(),
      DataReductionProxyClearDataSavingsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DataReductionProxyGetDataUsageFunction>,
      DataReductionProxyGetDataUsageFunction::function_name(),
      DataReductionProxyGetDataUsageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ImageWriterPrivateWriteFromUrlFunction>,
      ImageWriterPrivateWriteFromUrlFunction::function_name(),
      ImageWriterPrivateWriteFromUrlFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ImageWriterPrivateWriteFromFileFunction>,
      ImageWriterPrivateWriteFromFileFunction::function_name(),
      ImageWriterPrivateWriteFromFileFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ImageWriterPrivateCancelWriteFunction>,
      ImageWriterPrivateCancelWriteFunction::function_name(),
      ImageWriterPrivateCancelWriteFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ImageWriterPrivateDestroyPartitionsFunction>,
      ImageWriterPrivateDestroyPartitionsFunction::function_name(),
      ImageWriterPrivateDestroyPartitionsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ImageWriterPrivateListRemovableStorageDevicesFunction>,
      ImageWriterPrivateListRemovableStorageDevicesFunction::function_name(),
      ImageWriterPrivateListRemovableStorageDevicesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsDownloadFunction>,
      DownloadsDownloadFunction::function_name(),
      DownloadsDownloadFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsSearchFunction>,
      DownloadsSearchFunction::function_name(),
      DownloadsSearchFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsPauseFunction>,
      DownloadsPauseFunction::function_name(),
      DownloadsPauseFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsResumeFunction>,
      DownloadsResumeFunction::function_name(),
      DownloadsResumeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsCancelFunction>,
      DownloadsCancelFunction::function_name(),
      DownloadsCancelFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsGetFileIconFunction>,
      DownloadsGetFileIconFunction::function_name(),
      DownloadsGetFileIconFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsOpenFunction>,
      DownloadsOpenFunction::function_name(),
      DownloadsOpenFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsShowFunction>,
      DownloadsShowFunction::function_name(),
      DownloadsShowFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsShowDefaultFolderFunction>,
      DownloadsShowDefaultFolderFunction::function_name(),
      DownloadsShowDefaultFolderFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsEraseFunction>,
      DownloadsEraseFunction::function_name(),
      DownloadsEraseFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsRemoveFileFunction>,
      DownloadsRemoveFileFunction::function_name(),
      DownloadsRemoveFileFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsAcceptDangerFunction>,
      DownloadsAcceptDangerFunction::function_name(),
      DownloadsAcceptDangerFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsDragFunction>,
      DownloadsDragFunction::function_name(),
      DownloadsDragFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DownloadsSetShelfEnabledFunction>,
      DownloadsSetShelfEnabledFunction::function_name(),
      DownloadsSetShelfEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsClearFontFunction>,
      FontSettingsClearFontFunction::function_name(),
      FontSettingsClearFontFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsGetFontFunction>,
      FontSettingsGetFontFunction::function_name(),
      FontSettingsGetFontFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsSetFontFunction>,
      FontSettingsSetFontFunction::function_name(),
      FontSettingsSetFontFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsGetFontListFunction>,
      FontSettingsGetFontListFunction::function_name(),
      FontSettingsGetFontListFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsClearDefaultFontSizeFunction>,
      FontSettingsClearDefaultFontSizeFunction::function_name(),
      FontSettingsClearDefaultFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsGetDefaultFontSizeFunction>,
      FontSettingsGetDefaultFontSizeFunction::function_name(),
      FontSettingsGetDefaultFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsSetDefaultFontSizeFunction>,
      FontSettingsSetDefaultFontSizeFunction::function_name(),
      FontSettingsSetDefaultFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsClearDefaultFixedFontSizeFunction>,
      FontSettingsClearDefaultFixedFontSizeFunction::function_name(),
      FontSettingsClearDefaultFixedFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsGetDefaultFixedFontSizeFunction>,
      FontSettingsGetDefaultFixedFontSizeFunction::function_name(),
      FontSettingsGetDefaultFixedFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsSetDefaultFixedFontSizeFunction>,
      FontSettingsSetDefaultFixedFontSizeFunction::function_name(),
      FontSettingsSetDefaultFixedFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsClearMinimumFontSizeFunction>,
      FontSettingsClearMinimumFontSizeFunction::function_name(),
      FontSettingsClearMinimumFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsGetMinimumFontSizeFunction>,
      FontSettingsGetMinimumFontSizeFunction::function_name(),
      FontSettingsGetMinimumFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<FontSettingsSetMinimumFontSizeFunction>,
      FontSettingsSetMinimumFontSizeFunction::function_name(),
      FontSettingsSetMinimumFontSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<EnterpriseHardwarePlatformGetHardwarePlatformInfoFunction>,
      EnterpriseHardwarePlatformGetHardwarePlatformInfoFunction::function_name(),
      EnterpriseHardwarePlatformGetHardwarePlatformInfoFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<NotificationsCreateFunction>,
      NotificationsCreateFunction::function_name(),
      NotificationsCreateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<NotificationsUpdateFunction>,
      NotificationsUpdateFunction::function_name(),
      NotificationsUpdateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<NotificationsClearFunction>,
      NotificationsClearFunction::function_name(),
      NotificationsClearFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<NotificationsGetAllFunction>,
      NotificationsGetAllFunction::function_name(),
      NotificationsGetAllFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<NotificationsGetPermissionLevelFunction>,
      NotificationsGetPermissionLevelFunction::function_name(),
      NotificationsGetPermissionLevelFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsGetFunction>,
      TabsGetFunction::function_name(),
      TabsGetFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsGetCurrentFunction>,
      TabsGetCurrentFunction::function_name(),
      TabsGetCurrentFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsGetSelectedFunction>,
      TabsGetSelectedFunction::function_name(),
      TabsGetSelectedFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsGetAllInWindowFunction>,
      TabsGetAllInWindowFunction::function_name(),
      TabsGetAllInWindowFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsCreateFunction>,
      TabsCreateFunction::function_name(),
      TabsCreateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsDuplicateFunction>,
      TabsDuplicateFunction::function_name(),
      TabsDuplicateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsQueryFunction>,
      TabsQueryFunction::function_name(),
      TabsQueryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsHighlightFunction>,
      TabsHighlightFunction::function_name(),
      TabsHighlightFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsUpdateFunction>,
      TabsUpdateFunction::function_name(),
      TabsUpdateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsMoveFunction>,
      TabsMoveFunction::function_name(),
      TabsMoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsReloadFunction>,
      TabsReloadFunction::function_name(),
      TabsReloadFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsRemoveFunction>,
      TabsRemoveFunction::function_name(),
      TabsRemoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsDetectLanguageFunction>,
      TabsDetectLanguageFunction::function_name(),
      TabsDetectLanguageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsCaptureVisibleTabFunction>,
      TabsCaptureVisibleTabFunction::function_name(),
      TabsCaptureVisibleTabFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsExecuteScriptFunction>,
      TabsExecuteScriptFunction::function_name(),
      TabsExecuteScriptFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsInsertCSSFunction>,
      TabsInsertCSSFunction::function_name(),
      TabsInsertCSSFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsSetZoomFunction>,
      TabsSetZoomFunction::function_name(),
      TabsSetZoomFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsGetZoomFunction>,
      TabsGetZoomFunction::function_name(),
      TabsGetZoomFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsSetZoomSettingsFunction>,
      TabsSetZoomSettingsFunction::function_name(),
      TabsSetZoomSettingsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsGetZoomSettingsFunction>,
      TabsGetZoomSettingsFunction::function_name(),
      TabsGetZoomSettingsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsDiscardFunction>,
      TabsDiscardFunction::function_name(),
      TabsDiscardFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsGoForwardFunction>,
      TabsGoForwardFunction::function_name(),
      TabsGoForwardFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<TabsGoBackFunction>,
      TabsGoBackFunction::function_name(),
      TabsGoBackFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcAudioPrivateGetSinksFunction>,
      WebrtcAudioPrivateGetSinksFunction::function_name(),
      WebrtcAudioPrivateGetSinksFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebrtcAudioPrivateGetAssociatedSinkFunction>,
      WebrtcAudioPrivateGetAssociatedSinkFunction::function_name(),
      WebrtcAudioPrivateGetAssociatedSinkFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<I18nGetAcceptLanguagesFunction>,
      I18nGetAcceptLanguagesFunction::function_name(),
      I18nGetAcceptLanguagesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateGetLanguageListFunction>,
      LanguageSettingsPrivateGetLanguageListFunction::function_name(),
      LanguageSettingsPrivateGetLanguageListFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateEnableLanguageFunction>,
      LanguageSettingsPrivateEnableLanguageFunction::function_name(),
      LanguageSettingsPrivateEnableLanguageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateDisableLanguageFunction>,
      LanguageSettingsPrivateDisableLanguageFunction::function_name(),
      LanguageSettingsPrivateDisableLanguageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateSetEnableTranslationForLanguageFunction>,
      LanguageSettingsPrivateSetEnableTranslationForLanguageFunction::function_name(),
      LanguageSettingsPrivateSetEnableTranslationForLanguageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateMoveLanguageFunction>,
      LanguageSettingsPrivateMoveLanguageFunction::function_name(),
      LanguageSettingsPrivateMoveLanguageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateGetSpellcheckDictionaryStatusesFunction>,
      LanguageSettingsPrivateGetSpellcheckDictionaryStatusesFunction::function_name(),
      LanguageSettingsPrivateGetSpellcheckDictionaryStatusesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateGetSpellcheckWordsFunction>,
      LanguageSettingsPrivateGetSpellcheckWordsFunction::function_name(),
      LanguageSettingsPrivateGetSpellcheckWordsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateAddSpellcheckWordFunction>,
      LanguageSettingsPrivateAddSpellcheckWordFunction::function_name(),
      LanguageSettingsPrivateAddSpellcheckWordFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateRemoveSpellcheckWordFunction>,
      LanguageSettingsPrivateRemoveSpellcheckWordFunction::function_name(),
      LanguageSettingsPrivateRemoveSpellcheckWordFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateGetTranslateTargetLanguageFunction>,
      LanguageSettingsPrivateGetTranslateTargetLanguageFunction::function_name(),
      LanguageSettingsPrivateGetTranslateTargetLanguageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateGetInputMethodListsFunction>,
      LanguageSettingsPrivateGetInputMethodListsFunction::function_name(),
      LanguageSettingsPrivateGetInputMethodListsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateAddInputMethodFunction>,
      LanguageSettingsPrivateAddInputMethodFunction::function_name(),
      LanguageSettingsPrivateAddInputMethodFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateRemoveInputMethodFunction>,
      LanguageSettingsPrivateRemoveInputMethodFunction::function_name(),
      LanguageSettingsPrivateRemoveInputMethodFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<LanguageSettingsPrivateRetryDownloadDictionaryFunction>,
      LanguageSettingsPrivateRetryDownloadDictionaryFunction::function_name(),
      LanguageSettingsPrivateRetryDownloadDictionaryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CloudPrintPrivateSetupConnectorFunction>,
      CloudPrintPrivateSetupConnectorFunction::function_name(),
      CloudPrintPrivateSetupConnectorFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CloudPrintPrivateGetHostNameFunction>,
      CloudPrintPrivateGetHostNameFunction::function_name(),
      CloudPrintPrivateGetHostNameFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CloudPrintPrivateGetPrintersFunction>,
      CloudPrintPrivateGetPrintersFunction::function_name(),
      CloudPrintPrivateGetPrintersFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<CloudPrintPrivateGetClientIdFunction>,
      CloudPrintPrivateGetClientIdFunction::function_name(),
      CloudPrintPrivateGetClientIdFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<IdltestSendArrayBufferFunction>,
      IdltestSendArrayBufferFunction::function_name(),
      IdltestSendArrayBufferFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<IdltestSendArrayBufferViewFunction>,
      IdltestSendArrayBufferViewFunction::function_name(),
      IdltestSendArrayBufferViewFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<IdltestGetArrayBufferFunction>,
      IdltestGetArrayBufferFunction::function_name(),
      IdltestGetArrayBufferFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<IdentityGetAccountsFunction>,
      IdentityGetAccountsFunction::function_name(),
      IdentityGetAccountsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<IdentityGetAuthTokenFunction>,
      IdentityGetAuthTokenFunction::function_name(),
      IdentityGetAuthTokenFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<IdentityGetProfileUserInfoFunction>,
      IdentityGetProfileUserInfoFunction::function_name(),
      IdentityGetProfileUserInfoFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<IdentityRemoveCachedAuthTokenFunction>,
      IdentityRemoveCachedAuthTokenFunction::function_name(),
      IdentityRemoveCachedAuthTokenFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<IdentityLaunchWebAuthFlowFunction>,
      IdentityLaunchWebAuthFlowFunction::function_name(),
      IdentityLaunchWebAuthFlowFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PermissionsGetAllFunction>,
      PermissionsGetAllFunction::function_name(),
      PermissionsGetAllFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PermissionsContainsFunction>,
      PermissionsContainsFunction::function_name(),
      PermissionsContainsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PermissionsRequestFunction>,
      PermissionsRequestFunction::function_name(),
      PermissionsRequestFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PermissionsRemoveFunction>,
      PermissionsRemoveFunction::function_name(),
      PermissionsRemoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ProcessesGetProcessIdForTabFunction>,
      ProcessesGetProcessIdForTabFunction::function_name(),
      ProcessesGetProcessIdForTabFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ProcessesTerminateFunction>,
      ProcessesTerminateFunction::function_name(),
      ProcessesTerminateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ProcessesGetProcessInfoFunction>,
      ProcessesGetProcessInfoFunction::function_name(),
      ProcessesGetProcessInfoFunction::histogram_value(),
    },
    #if defined(OS_CHROMEOS) || defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeSetCompositionFunction>,
      InputImeSetCompositionFunction::function_name(),
      InputImeSetCompositionFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS) || defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<InputImeClearCompositionFunction>,
      InputImeClearCompositionFunction::function_name(),
      InputImeClearCompositionFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS) || defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeCommitTextFunction>,
      InputImeCommitTextFunction::function_name(),
      InputImeCommitTextFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS) || defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    #if defined(OS_CHROMEOS) || defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeSendKeyEventsFunction>,
      InputImeSendKeyEventsFunction::function_name(),
      InputImeSendKeyEventsFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS) || defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<InputImeHideInputViewFunction>,
      InputImeHideInputViewFunction::function_name(),
      InputImeHideInputViewFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<InputImeSetCandidateWindowPropertiesFunction>,
      InputImeSetCandidateWindowPropertiesFunction::function_name(),
      InputImeSetCandidateWindowPropertiesFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<InputImeSetCandidatesFunction>,
      InputImeSetCandidatesFunction::function_name(),
      InputImeSetCandidatesFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<InputImeSetCursorPositionFunction>,
      InputImeSetCursorPositionFunction::function_name(),
      InputImeSetCursorPositionFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<InputImeSetMenuItemsFunction>,
      InputImeSetMenuItemsFunction::function_name(),
      InputImeSetMenuItemsFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<InputImeUpdateMenuItemsFunction>,
      InputImeUpdateMenuItemsFunction::function_name(),
      InputImeUpdateMenuItemsFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<InputImeDeleteSurroundingTextFunction>,
      InputImeDeleteSurroundingTextFunction::function_name(),
      InputImeDeleteSurroundingTextFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS) || defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeKeyEventHandledFunction>,
      InputImeKeyEventHandledFunction::function_name(),
      InputImeKeyEventHandledFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS) || defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    #if defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeCreateWindowFunction>,
      InputImeCreateWindowFunction::function_name(),
      InputImeCreateWindowFunction::histogram_value(),
    },
    #endif  // defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    #if defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeShowWindowFunction>,
      InputImeShowWindowFunction::function_name(),
      InputImeShowWindowFunction::histogram_value(),
    },
    #endif  // defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    #if defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeHideWindowFunction>,
      InputImeHideWindowFunction::function_name(),
      InputImeHideWindowFunction::histogram_value(),
    },
    #endif  // defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    #if defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeActivateFunction>,
      InputImeActivateFunction::function_name(),
      InputImeActivateFunction::histogram_value(),
    },
    #endif  // defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    #if defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<InputImeDeactivateFunction>,
      InputImeDeactivateFunction::function_name(),
      InputImeDeactivateFunction::histogram_value(),
    },
    #endif  // defined(OS_WIN) || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
    {
      &NewExtensionFunction<DownloadsInternalDetermineFilenameFunction>,
      DownloadsInternalDetermineFilenameFunction::function_name(),
      DownloadsInternalDetermineFilenameFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ExtensionIsAllowedIncognitoAccessFunction>,
      ExtensionIsAllowedIncognitoAccessFunction::function_name(),
      ExtensionIsAllowedIncognitoAccessFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ExtensionIsAllowedFileSchemeAccessFunction>,
      ExtensionIsAllowedFileSchemeAccessFunction::function_name(),
      ExtensionIsAllowedFileSchemeAccessFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ExtensionSetUpdateUrlDataFunction>,
      ExtensionSetUpdateUrlDataFunction::function_name(),
      ExtensionSetUpdateUrlDataFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateCopyFunction>,
      BookmarkManagerPrivateCopyFunction::function_name(),
      BookmarkManagerPrivateCopyFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateCutFunction>,
      BookmarkManagerPrivateCutFunction::function_name(),
      BookmarkManagerPrivateCutFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivatePasteFunction>,
      BookmarkManagerPrivatePasteFunction::function_name(),
      BookmarkManagerPrivatePasteFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateSortChildrenFunction>,
      BookmarkManagerPrivateSortChildrenFunction::function_name(),
      BookmarkManagerPrivateSortChildrenFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateStartDragFunction>,
      BookmarkManagerPrivateStartDragFunction::function_name(),
      BookmarkManagerPrivateStartDragFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateDropFunction>,
      BookmarkManagerPrivateDropFunction::function_name(),
      BookmarkManagerPrivateDropFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateGetSubtreeFunction>,
      BookmarkManagerPrivateGetSubtreeFunction::function_name(),
      BookmarkManagerPrivateGetSubtreeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateRemoveTreesFunction>,
      BookmarkManagerPrivateRemoveTreesFunction::function_name(),
      BookmarkManagerPrivateRemoveTreesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateUndoFunction>,
      BookmarkManagerPrivateUndoFunction::function_name(),
      BookmarkManagerPrivateUndoFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BookmarkManagerPrivateRedoFunction>,
      BookmarkManagerPrivateRedoFunction::function_name(),
      BookmarkManagerPrivateRedoFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateAutoUpdateFunction>,
      DeveloperPrivateAutoUpdateFunction::function_name(),
      DeveloperPrivateAutoUpdateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateGetExtensionsInfoFunction>,
      DeveloperPrivateGetExtensionsInfoFunction::function_name(),
      DeveloperPrivateGetExtensionsInfoFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateGetExtensionInfoFunction>,
      DeveloperPrivateGetExtensionInfoFunction::function_name(),
      DeveloperPrivateGetExtensionInfoFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateGetExtensionSizeFunction>,
      DeveloperPrivateGetExtensionSizeFunction::function_name(),
      DeveloperPrivateGetExtensionSizeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateGetItemsInfoFunction>,
      DeveloperPrivateGetItemsInfoFunction::function_name(),
      DeveloperPrivateGetItemsInfoFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateGetProfileConfigurationFunction>,
      DeveloperPrivateGetProfileConfigurationFunction::function_name(),
      DeveloperPrivateGetProfileConfigurationFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateUpdateProfileConfigurationFunction>,
      DeveloperPrivateUpdateProfileConfigurationFunction::function_name(),
      DeveloperPrivateUpdateProfileConfigurationFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateShowPermissionsDialogFunction>,
      DeveloperPrivateShowPermissionsDialogFunction::function_name(),
      DeveloperPrivateShowPermissionsDialogFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateReloadFunction>,
      DeveloperPrivateReloadFunction::function_name(),
      DeveloperPrivateReloadFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateUpdateExtensionConfigurationFunction>,
      DeveloperPrivateUpdateExtensionConfigurationFunction::function_name(),
      DeveloperPrivateUpdateExtensionConfigurationFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateLoadUnpackedFunction>,
      DeveloperPrivateLoadUnpackedFunction::function_name(),
      DeveloperPrivateLoadUnpackedFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateInstallDroppedFileFunction>,
      DeveloperPrivateInstallDroppedFileFunction::function_name(),
      DeveloperPrivateInstallDroppedFileFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateNotifyDragInstallInProgressFunction>,
      DeveloperPrivateNotifyDragInstallInProgressFunction::function_name(),
      DeveloperPrivateNotifyDragInstallInProgressFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateLoadDirectoryFunction>,
      DeveloperPrivateLoadDirectoryFunction::function_name(),
      DeveloperPrivateLoadDirectoryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateChoosePathFunction>,
      DeveloperPrivateChoosePathFunction::function_name(),
      DeveloperPrivateChoosePathFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivatePackDirectoryFunction>,
      DeveloperPrivatePackDirectoryFunction::function_name(),
      DeveloperPrivatePackDirectoryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateIsProfileManagedFunction>,
      DeveloperPrivateIsProfileManagedFunction::function_name(),
      DeveloperPrivateIsProfileManagedFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateRequestFileSourceFunction>,
      DeveloperPrivateRequestFileSourceFunction::function_name(),
      DeveloperPrivateRequestFileSourceFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateOpenDevToolsFunction>,
      DeveloperPrivateOpenDevToolsFunction::function_name(),
      DeveloperPrivateOpenDevToolsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateDeleteExtensionErrorsFunction>,
      DeveloperPrivateDeleteExtensionErrorsFunction::function_name(),
      DeveloperPrivateDeleteExtensionErrorsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateRepairExtensionFunction>,
      DeveloperPrivateRepairExtensionFunction::function_name(),
      DeveloperPrivateRepairExtensionFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateShowOptionsFunction>,
      DeveloperPrivateShowOptionsFunction::function_name(),
      DeveloperPrivateShowOptionsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateShowPathFunction>,
      DeveloperPrivateShowPathFunction::function_name(),
      DeveloperPrivateShowPathFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateSetShortcutHandlingSuspendedFunction>,
      DeveloperPrivateSetShortcutHandlingSuspendedFunction::function_name(),
      DeveloperPrivateSetShortcutHandlingSuspendedFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateUpdateExtensionCommandFunction>,
      DeveloperPrivateUpdateExtensionCommandFunction::function_name(),
      DeveloperPrivateUpdateExtensionCommandFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateAddHostPermissionFunction>,
      DeveloperPrivateAddHostPermissionFunction::function_name(),
      DeveloperPrivateAddHostPermissionFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<DeveloperPrivateRemoveHostPermissionFunction>,
      DeveloperPrivateRemoveHostPermissionFunction::function_name(),
      DeveloperPrivateRemoveHostPermissionFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WindowsGetFunction>,
      WindowsGetFunction::function_name(),
      WindowsGetFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WindowsGetCurrentFunction>,
      WindowsGetCurrentFunction::function_name(),
      WindowsGetCurrentFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WindowsGetLastFocusedFunction>,
      WindowsGetLastFocusedFunction::function_name(),
      WindowsGetLastFocusedFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WindowsGetAllFunction>,
      WindowsGetAllFunction::function_name(),
      WindowsGetAllFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WindowsCreateFunction>,
      WindowsCreateFunction::function_name(),
      WindowsCreateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WindowsUpdateFunction>,
      WindowsUpdateFunction::function_name(),
      WindowsUpdateFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WindowsRemoveFunction>,
      WindowsRemoveFunction::function_name(),
      WindowsRemoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataSettingsFunction>,
      BrowsingDataSettingsFunction::function_name(),
      BrowsingDataSettingsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveFunction>,
      BrowsingDataRemoveFunction::function_name(),
      BrowsingDataRemoveFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveAppcacheFunction>,
      BrowsingDataRemoveAppcacheFunction::function_name(),
      BrowsingDataRemoveAppcacheFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveCacheFunction>,
      BrowsingDataRemoveCacheFunction::function_name(),
      BrowsingDataRemoveCacheFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveCacheStorageFunction>,
      BrowsingDataRemoveCacheStorageFunction::function_name(),
      BrowsingDataRemoveCacheStorageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveCookiesFunction>,
      BrowsingDataRemoveCookiesFunction::function_name(),
      BrowsingDataRemoveCookiesFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveDownloadsFunction>,
      BrowsingDataRemoveDownloadsFunction::function_name(),
      BrowsingDataRemoveDownloadsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveFileSystemsFunction>,
      BrowsingDataRemoveFileSystemsFunction::function_name(),
      BrowsingDataRemoveFileSystemsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveFormDataFunction>,
      BrowsingDataRemoveFormDataFunction::function_name(),
      BrowsingDataRemoveFormDataFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveHistoryFunction>,
      BrowsingDataRemoveHistoryFunction::function_name(),
      BrowsingDataRemoveHistoryFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveIndexedDBFunction>,
      BrowsingDataRemoveIndexedDBFunction::function_name(),
      BrowsingDataRemoveIndexedDBFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveLocalStorageFunction>,
      BrowsingDataRemoveLocalStorageFunction::function_name(),
      BrowsingDataRemoveLocalStorageFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemovePluginDataFunction>,
      BrowsingDataRemovePluginDataFunction::function_name(),
      BrowsingDataRemovePluginDataFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemovePasswordsFunction>,
      BrowsingDataRemovePasswordsFunction::function_name(),
      BrowsingDataRemovePasswordsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveServiceWorkersFunction>,
      BrowsingDataRemoveServiceWorkersFunction::function_name(),
      BrowsingDataRemoveServiceWorkersFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<BrowsingDataRemoveWebSQLFunction>,
      BrowsingDataRemoveWebSQLFunction::function_name(),
      BrowsingDataRemoveWebSQLFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<OmniboxSendSuggestionsFunction>,
      OmniboxSendSuggestionsFunction::function_name(),
      OmniboxSendSuggestionsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<OmniboxSetDefaultSuggestionFunction>,
      OmniboxSetDefaultSuggestionFunction::function_name(),
      OmniboxSetDefaultSuggestionFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ContentSettingsContentSettingClearFunction>,
      ContentSettingsContentSettingClearFunction::function_name(),
      ContentSettingsContentSettingClearFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ContentSettingsContentSettingGetFunction>,
      ContentSettingsContentSettingGetFunction::function_name(),
      ContentSettingsContentSettingGetFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ContentSettingsContentSettingSetFunction>,
      ContentSettingsContentSettingSetFunction::function_name(),
      ContentSettingsContentSettingSetFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<ContentSettingsContentSettingGetResourceIdentifiersFunction>,
      ContentSettingsContentSettingGetResourceIdentifiersFunction::function_name(),
      ContentSettingsContentSettingGetResourceIdentifiersFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateBeginInstallWithManifest3Function>,
      WebstorePrivateBeginInstallWithManifest3Function::function_name(),
      WebstorePrivateBeginInstallWithManifest3Function::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateCompleteInstallFunction>,
      WebstorePrivateCompleteInstallFunction::function_name(),
      WebstorePrivateCompleteInstallFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateEnableAppLauncherFunction>,
      WebstorePrivateEnableAppLauncherFunction::function_name(),
      WebstorePrivateEnableAppLauncherFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateGetBrowserLoginFunction>,
      WebstorePrivateGetBrowserLoginFunction::function_name(),
      WebstorePrivateGetBrowserLoginFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateGetStoreLoginFunction>,
      WebstorePrivateGetStoreLoginFunction::function_name(),
      WebstorePrivateGetStoreLoginFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateSetStoreLoginFunction>,
      WebstorePrivateSetStoreLoginFunction::function_name(),
      WebstorePrivateSetStoreLoginFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateGetWebGLStatusFunction>,
      WebstorePrivateGetWebGLStatusFunction::function_name(),
      WebstorePrivateGetWebGLStatusFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateGetIsLauncherEnabledFunction>,
      WebstorePrivateGetIsLauncherEnabledFunction::function_name(),
      WebstorePrivateGetIsLauncherEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateIsInIncognitoModeFunction>,
      WebstorePrivateIsInIncognitoModeFunction::function_name(),
      WebstorePrivateIsInIncognitoModeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateGetEphemeralAppsEnabledFunction>,
      WebstorePrivateGetEphemeralAppsEnabledFunction::function_name(),
      WebstorePrivateGetEphemeralAppsEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateLaunchEphemeralAppFunction>,
      WebstorePrivateLaunchEphemeralAppFunction::function_name(),
      WebstorePrivateLaunchEphemeralAppFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateIsPendingCustodianApprovalFunction>,
      WebstorePrivateIsPendingCustodianApprovalFunction::function_name(),
      WebstorePrivateIsPendingCustodianApprovalFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<WebstorePrivateGetReferrerChainFunction>,
      WebstorePrivateGetReferrerChainFunction::function_name(),
      WebstorePrivateGetReferrerChainFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PageCaptureSaveAsMHTMLFunction>,
      PageCaptureSaveAsMHTMLFunction::function_name(),
      PageCaptureSaveAsMHTMLFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateRecordPasswordsPageAccessInSettingsFunction>,
      PasswordsPrivateRecordPasswordsPageAccessInSettingsFunction::function_name(),
      PasswordsPrivateRecordPasswordsPageAccessInSettingsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateChangeSavedPasswordFunction>,
      PasswordsPrivateChangeSavedPasswordFunction::function_name(),
      PasswordsPrivateChangeSavedPasswordFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateRemoveSavedPasswordFunction>,
      PasswordsPrivateRemoveSavedPasswordFunction::function_name(),
      PasswordsPrivateRemoveSavedPasswordFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateRemovePasswordExceptionFunction>,
      PasswordsPrivateRemovePasswordExceptionFunction::function_name(),
      PasswordsPrivateRemovePasswordExceptionFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateUndoRemoveSavedPasswordOrExceptionFunction>,
      PasswordsPrivateUndoRemoveSavedPasswordOrExceptionFunction::function_name(),
      PasswordsPrivateUndoRemoveSavedPasswordOrExceptionFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateRequestPlaintextPasswordFunction>,
      PasswordsPrivateRequestPlaintextPasswordFunction::function_name(),
      PasswordsPrivateRequestPlaintextPasswordFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateGetSavedPasswordListFunction>,
      PasswordsPrivateGetSavedPasswordListFunction::function_name(),
      PasswordsPrivateGetSavedPasswordListFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateGetPasswordExceptionListFunction>,
      PasswordsPrivateGetPasswordExceptionListFunction::function_name(),
      PasswordsPrivateGetPasswordExceptionListFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateImportPasswordsFunction>,
      PasswordsPrivateImportPasswordsFunction::function_name(),
      PasswordsPrivateImportPasswordsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateExportPasswordsFunction>,
      PasswordsPrivateExportPasswordsFunction::function_name(),
      PasswordsPrivateExportPasswordsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateRequestExportProgressStatusFunction>,
      PasswordsPrivateRequestExportProgressStatusFunction::function_name(),
      PasswordsPrivateRequestExportProgressStatusFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<PasswordsPrivateCancelExportPasswordsFunction>,
      PasswordsPrivateCancelExportPasswordsFunction::function_name(),
      PasswordsPrivateCancelExportPasswordsFunction::histogram_value(),
    },
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateGetBatteryDescriptionFunction>,
      AccessibilityPrivateGetBatteryDescriptionFunction::function_name(),
      AccessibilityPrivateGetBatteryDescriptionFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateSetNativeAccessibilityEnabledFunction>,
      AccessibilityPrivateSetNativeAccessibilityEnabledFunction::function_name(),
      AccessibilityPrivateSetNativeAccessibilityEnabledFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AccessibilityPrivateSetFocusRingsFunction>,
      AccessibilityPrivateSetFocusRingsFunction::function_name(),
      AccessibilityPrivateSetFocusRingsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AccessibilityPrivateSetHighlightsFunction>,
      AccessibilityPrivateSetHighlightsFunction::function_name(),
      AccessibilityPrivateSetHighlightsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AccessibilityPrivateSetKeyboardListenerFunction>,
      AccessibilityPrivateSetKeyboardListenerFunction::function_name(),
      AccessibilityPrivateSetKeyboardListenerFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<AccessibilityPrivateDarkenScreenFunction>,
      AccessibilityPrivateDarkenScreenFunction::function_name(),
      AccessibilityPrivateDarkenScreenFunction::histogram_value(),
    },
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateSetSwitchAccessKeysFunction>,
      AccessibilityPrivateSetSwitchAccessKeysFunction::function_name(),
      AccessibilityPrivateSetSwitchAccessKeysFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateSetSwitchAccessMenuStateFunction>,
      AccessibilityPrivateSetSwitchAccessMenuStateFunction::function_name(),
      AccessibilityPrivateSetSwitchAccessMenuStateFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateForwardKeyEventsToSwitchAccessFunction>,
      AccessibilityPrivateForwardKeyEventsToSwitchAccessFunction::function_name(),
      AccessibilityPrivateForwardKeyEventsToSwitchAccessFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateSetNativeChromeVoxArcSupportForCurrentAppFunction>,
      AccessibilityPrivateSetNativeChromeVoxArcSupportForCurrentAppFunction::function_name(),
      AccessibilityPrivateSetNativeChromeVoxArcSupportForCurrentAppFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateSendSyntheticKeyEventFunction>,
      AccessibilityPrivateSendSyntheticKeyEventFunction::function_name(),
      AccessibilityPrivateSendSyntheticKeyEventFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateEnableChromeVoxMouseEventsFunction>,
      AccessibilityPrivateEnableChromeVoxMouseEventsFunction::function_name(),
      AccessibilityPrivateEnableChromeVoxMouseEventsFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateSendSyntheticMouseEventFunction>,
      AccessibilityPrivateSendSyntheticMouseEventFunction::function_name(),
      AccessibilityPrivateSendSyntheticMouseEventFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateOnSelectToSpeakStateChangedFunction>,
      AccessibilityPrivateOnSelectToSpeakStateChangedFunction::function_name(),
      AccessibilityPrivateOnSelectToSpeakStateChangedFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateToggleDictationFunction>,
      AccessibilityPrivateToggleDictationFunction::function_name(),
      AccessibilityPrivateToggleDictationFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    #if defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<AccessibilityPrivateSetVirtualKeyboardVisibleFunction>,
      AccessibilityPrivateSetVirtualKeyboardVisibleFunction::function_name(),
      AccessibilityPrivateSetVirtualKeyboardVisibleFunction::histogram_value(),
    },
    #endif  // defined(OS_CHROMEOS)
    {
      &NewExtensionFunction<HistorySearchFunction>,
      HistorySearchFunction::function_name(),
      HistorySearchFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<HistoryGetVisitsFunction>,
      HistoryGetVisitsFunction::function_name(),
      HistoryGetVisitsFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<HistoryAddUrlFunction>,
      HistoryAddUrlFunction::function_name(),
      HistoryAddUrlFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<HistoryDeleteUrlFunction>,
      HistoryDeleteUrlFunction::function_name(),
      HistoryDeleteUrlFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<HistoryDeleteRangeFunction>,
      HistoryDeleteRangeFunction::function_name(),
      HistoryDeleteRangeFunction::histogram_value(),
    },
    {
      &NewExtensionFunction<HistoryDeleteAllFunction>,
      HistoryDeleteAllFunction::function_name(),
      HistoryDeleteAllFunction::histogram_value(),
    },
  };
  for (const auto& entry : kEntries) {
      registry->Register(entry);
  }
}

}  // namespace api
}  // namespace extensions
