// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_PERFORMANCE_MANAGER_WEBUI_GRAPH_DUMP_MOJOM_SHARED_H_
#define CHROME_BROWSER_PERFORMANCE_MANAGER_WEBUI_GRAPH_DUMP_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/browser/performance_manager/webui_graph_dump.mojom-shared-internal.h"
#include "mojo/public/mojom/base/process_id.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom-shared.h"
#include "url/mojom/url.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace performance_manager {
namespace mojom {
class WebUIPageInfoDataView;

class WebUIFrameInfoDataView;

class WebUIProcessInfoDataView;

class WebUIFavIconInfoDataView;



}  // namespace mojom
}  // namespace performance_manager

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::performance_manager::mojom::WebUIPageInfoDataView> {
  using Data = ::performance_manager::mojom::internal::WebUIPageInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::performance_manager::mojom::WebUIFrameInfoDataView> {
  using Data = ::performance_manager::mojom::internal::WebUIFrameInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::performance_manager::mojom::WebUIProcessInfoDataView> {
  using Data = ::performance_manager::mojom::internal::WebUIProcessInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::performance_manager::mojom::WebUIFavIconInfoDataView> {
  using Data = ::performance_manager::mojom::internal::WebUIFavIconInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace performance_manager {
namespace mojom {
// Interface base classes. They are used for type safety check.
class WebUIGraphChangeStreamInterfaceBase {};

using WebUIGraphChangeStreamPtrDataView =
    mojo::InterfacePtrDataView<WebUIGraphChangeStreamInterfaceBase>;
using WebUIGraphChangeStreamRequestDataView =
    mojo::InterfaceRequestDataView<WebUIGraphChangeStreamInterfaceBase>;
using WebUIGraphChangeStreamAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<WebUIGraphChangeStreamInterfaceBase>;
using WebUIGraphChangeStreamAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<WebUIGraphChangeStreamInterfaceBase>;
class WebUIGraphDumpInterfaceBase {};

using WebUIGraphDumpPtrDataView =
    mojo::InterfacePtrDataView<WebUIGraphDumpInterfaceBase>;
using WebUIGraphDumpRequestDataView =
    mojo::InterfaceRequestDataView<WebUIGraphDumpInterfaceBase>;
using WebUIGraphDumpAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<WebUIGraphDumpInterfaceBase>;
using WebUIGraphDumpAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<WebUIGraphDumpInterfaceBase>;
class WebUIPageInfoDataView {
 public:
  WebUIPageInfoDataView() {}

  WebUIPageInfoDataView(
      internal::WebUIPageInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int64_t id() const {
    return data_->id;
  }
  inline void GetMainFrameUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMainFrameUrl(UserType* output) {
    auto* pointer = data_->main_frame_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::WebUIPageInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class WebUIFrameInfoDataView {
 public:
  WebUIFrameInfoDataView() {}

  WebUIFrameInfoDataView(
      internal::WebUIFrameInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int64_t id() const {
    return data_->id;
  }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  int64_t page_id() const {
    return data_->page_id;
  }
  int64_t parent_frame_id() const {
    return data_->parent_frame_id;
  }
  int64_t process_id() const {
    return data_->process_id;
  }
 private:
  internal::WebUIFrameInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class WebUIProcessInfoDataView {
 public:
  WebUIProcessInfoDataView() {}

  WebUIProcessInfoDataView(
      internal::WebUIProcessInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int64_t id() const {
    return data_->id;
  }
  inline void GetPidDataView(
      ::mojo_base::mojom::ProcessIdDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPid(UserType* output) {
    auto* pointer = data_->pid.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::ProcessIdDataView>(
        pointer, output, context_);
  }
  inline void GetCumulativeCpuUsageDataView(
      ::mojo_base::mojom::TimeDeltaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCumulativeCpuUsage(UserType* output) {
    auto* pointer = data_->cumulative_cpu_usage.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDeltaDataView>(
        pointer, output, context_);
  }
  uint64_t private_footprint_kb() const {
    return data_->private_footprint_kb;
  }
 private:
  internal::WebUIProcessInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class WebUIFavIconInfoDataView {
 public:
  WebUIFavIconInfoDataView() {}

  WebUIFavIconInfoDataView(
      internal::WebUIFavIconInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int64_t node_id() const {
    return data_->node_id;
  }
  inline void GetIconDataDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconData(UserType* output) {
    auto* pointer = data_->icon_data.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::WebUIFavIconInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace performance_manager

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::performance_manager::mojom::WebUIPageInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::performance_manager::mojom::WebUIPageInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::performance_manager::mojom::internal::WebUIPageInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    (*output)->id = Traits::id(input);
    decltype(Traits::main_frame_url(input)) in_main_frame_url = Traits::main_frame_url(input);
    typename decltype((*output)->main_frame_url)::BaseType::BufferWriter
        main_frame_url_writer;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_main_frame_url, buffer, &main_frame_url_writer, context);
    (*output)->main_frame_url.Set(
        main_frame_url_writer.is_null() ? nullptr : main_frame_url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->main_frame_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null main_frame_url in WebUIPageInfo struct");
  }

  static bool Deserialize(::performance_manager::mojom::internal::WebUIPageInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::performance_manager::mojom::WebUIPageInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::performance_manager::mojom::WebUIFrameInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::performance_manager::mojom::WebUIFrameInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::performance_manager::mojom::internal::WebUIFrameInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    (*output)->id = Traits::id(input);
    decltype(Traits::url(input)) in_url = Traits::url(input);
    typename decltype((*output)->url)::BaseType::BufferWriter
        url_writer;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_url, buffer, &url_writer, context);
    (*output)->url.Set(
        url_writer.is_null() ? nullptr : url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null url in WebUIFrameInfo struct");
    (*output)->page_id = Traits::page_id(input);
    (*output)->parent_frame_id = Traits::parent_frame_id(input);
    (*output)->process_id = Traits::process_id(input);
  }

  static bool Deserialize(::performance_manager::mojom::internal::WebUIFrameInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::performance_manager::mojom::WebUIFrameInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::performance_manager::mojom::WebUIProcessInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::performance_manager::mojom::WebUIProcessInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::performance_manager::mojom::internal::WebUIProcessInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    (*output)->id = Traits::id(input);
    decltype(Traits::pid(input)) in_pid = Traits::pid(input);
    typename decltype((*output)->pid)::BaseType::BufferWriter
        pid_writer;
    mojo::internal::Serialize<::mojo_base::mojom::ProcessIdDataView>(
        in_pid, buffer, &pid_writer, context);
    (*output)->pid.Set(
        pid_writer.is_null() ? nullptr : pid_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->pid.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null pid in WebUIProcessInfo struct");
    decltype(Traits::cumulative_cpu_usage(input)) in_cumulative_cpu_usage = Traits::cumulative_cpu_usage(input);
    typename decltype((*output)->cumulative_cpu_usage)::BaseType::BufferWriter
        cumulative_cpu_usage_writer;
    mojo::internal::Serialize<::mojo_base::mojom::TimeDeltaDataView>(
        in_cumulative_cpu_usage, buffer, &cumulative_cpu_usage_writer, context);
    (*output)->cumulative_cpu_usage.Set(
        cumulative_cpu_usage_writer.is_null() ? nullptr : cumulative_cpu_usage_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->cumulative_cpu_usage.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null cumulative_cpu_usage in WebUIProcessInfo struct");
    (*output)->private_footprint_kb = Traits::private_footprint_kb(input);
  }

  static bool Deserialize(::performance_manager::mojom::internal::WebUIProcessInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::performance_manager::mojom::WebUIProcessInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::performance_manager::mojom::WebUIFavIconInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::performance_manager::mojom::WebUIFavIconInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::performance_manager::mojom::internal::WebUIFavIconInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    (*output)->node_id = Traits::node_id(input);
    decltype(Traits::icon_data(input)) in_icon_data = Traits::icon_data(input);
    typename decltype((*output)->icon_data)::BaseType::BufferWriter
        icon_data_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_icon_data, buffer, &icon_data_writer, context);
    (*output)->icon_data.Set(
        icon_data_writer.is_null() ? nullptr : icon_data_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->icon_data.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null icon_data in WebUIFavIconInfo struct");
  }

  static bool Deserialize(::performance_manager::mojom::internal::WebUIFavIconInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::performance_manager::mojom::WebUIFavIconInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace performance_manager {
namespace mojom {

inline void WebUIPageInfoDataView::GetMainFrameUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->main_frame_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void WebUIFrameInfoDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void WebUIProcessInfoDataView::GetPidDataView(
    ::mojo_base::mojom::ProcessIdDataView* output) {
  auto pointer = data_->pid.Get();
  *output = ::mojo_base::mojom::ProcessIdDataView(pointer, context_);
}
inline void WebUIProcessInfoDataView::GetCumulativeCpuUsageDataView(
    ::mojo_base::mojom::TimeDeltaDataView* output) {
  auto pointer = data_->cumulative_cpu_usage.Get();
  *output = ::mojo_base::mojom::TimeDeltaDataView(pointer, context_);
}


inline void WebUIFavIconInfoDataView::GetIconDataDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->icon_data.Get();
  *output = mojo::StringDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace performance_manager

#endif  // CHROME_BROWSER_PERFORMANCE_MANAGER_WEBUI_GRAPH_DUMP_MOJOM_SHARED_H_