// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "chrome/browser/performance_manager/webui_graph_dump.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/hash/md5_constexpr.h"
#include "base/logging.h"
#include "base/run_loop.h"
#include "base/task/common/task_annotator.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "chrome/browser/performance_manager/webui_graph_dump.mojom-params-data.h"
#include "chrome/browser/performance_manager/webui_graph_dump.mojom-shared-message-ids.h"

#include "chrome/browser/performance_manager/webui_graph_dump.mojom-import-headers.h"


#ifndef CHROME_BROWSER_PERFORMANCE_MANAGER_WEBUI_GRAPH_DUMP_MOJOM_JUMBO_H_
#define CHROME_BROWSER_PERFORMANCE_MANAGER_WEBUI_GRAPH_DUMP_MOJOM_JUMBO_H_
#include "mojo/public/cpp/base/process_id_mojom_traits.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "url/mojom/url_gurl_mojom_traits.h"
#endif
namespace performance_manager {
namespace mojom {
WebUIPageInfo::WebUIPageInfo()
    : id(),
      main_frame_url() {}

WebUIPageInfo::WebUIPageInfo(
    int64_t id_in,
    const GURL& main_frame_url_in)
    : id(std::move(id_in)),
      main_frame_url(std::move(main_frame_url_in)) {}

WebUIPageInfo::~WebUIPageInfo() = default;

bool WebUIPageInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
WebUIFrameInfo::WebUIFrameInfo()
    : id(),
      url(),
      page_id(),
      parent_frame_id(),
      process_id() {}

WebUIFrameInfo::WebUIFrameInfo(
    int64_t id_in,
    const GURL& url_in,
    int64_t page_id_in,
    int64_t parent_frame_id_in,
    int64_t process_id_in)
    : id(std::move(id_in)),
      url(std::move(url_in)),
      page_id(std::move(page_id_in)),
      parent_frame_id(std::move(parent_frame_id_in)),
      process_id(std::move(process_id_in)) {}

WebUIFrameInfo::~WebUIFrameInfo() = default;

bool WebUIFrameInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
WebUIProcessInfo::WebUIProcessInfo()
    : id(),
      pid(),
      cumulative_cpu_usage(),
      private_footprint_kb() {}

WebUIProcessInfo::WebUIProcessInfo(
    int64_t id_in,
    base::ProcessId pid_in,
    base::TimeDelta cumulative_cpu_usage_in,
    uint64_t private_footprint_kb_in)
    : id(std::move(id_in)),
      pid(std::move(pid_in)),
      cumulative_cpu_usage(std::move(cumulative_cpu_usage_in)),
      private_footprint_kb(std::move(private_footprint_kb_in)) {}

WebUIProcessInfo::~WebUIProcessInfo() = default;

bool WebUIProcessInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
WebUIFavIconInfo::WebUIFavIconInfo()
    : node_id(),
      icon_data() {}

WebUIFavIconInfo::WebUIFavIconInfo(
    int64_t node_id_in,
    const std::string& icon_data_in)
    : node_id(std::move(node_id_in)),
      icon_data(std::move(icon_data_in)) {}

WebUIFavIconInfo::~WebUIFavIconInfo() = default;
size_t WebUIFavIconInfo::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->node_id);
  seed = mojo::internal::Hash(seed, this->icon_data);
  return seed;
}

bool WebUIFavIconInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char WebUIGraphChangeStream::Name_[] = "performance_manager.mojom.WebUIGraphChangeStream";

WebUIGraphChangeStreamProxy::WebUIGraphChangeStreamProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void WebUIGraphChangeStreamProxy::FrameCreated(
    WebUIFrameInfoPtr in_frame) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FrameCreated");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphChangeStream_FrameCreated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphChangeStream_FrameCreated_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->frame)::BaseType::BufferWriter
      frame_writer;
  mojo::internal::Serialize<::performance_manager::mojom::WebUIFrameInfoDataView>(
      in_frame, buffer, &frame_writer, &serialization_context);
  params->frame.Set(
      frame_writer.is_null() ? nullptr : frame_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->frame.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null frame in WebUIGraphChangeStream.FrameCreated request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphChangeStream::Name_);
  message.set_method_name("FrameCreated");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WebUIGraphChangeStreamProxy::PageCreated(
    WebUIPageInfoPtr in_pages) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::PageCreated");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphChangeStream_PageCreated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphChangeStream_PageCreated_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->pages)::BaseType::BufferWriter
      pages_writer;
  mojo::internal::Serialize<::performance_manager::mojom::WebUIPageInfoDataView>(
      in_pages, buffer, &pages_writer, &serialization_context);
  params->pages.Set(
      pages_writer.is_null() ? nullptr : pages_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->pages.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null pages in WebUIGraphChangeStream.PageCreated request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphChangeStream::Name_);
  message.set_method_name("PageCreated");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WebUIGraphChangeStreamProxy::ProcessCreated(
    WebUIProcessInfoPtr in_process) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::ProcessCreated");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphChangeStream_ProcessCreated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphChangeStream_ProcessCreated_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->process)::BaseType::BufferWriter
      process_writer;
  mojo::internal::Serialize<::performance_manager::mojom::WebUIProcessInfoDataView>(
      in_process, buffer, &process_writer, &serialization_context);
  params->process.Set(
      process_writer.is_null() ? nullptr : process_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->process.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null process in WebUIGraphChangeStream.ProcessCreated request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphChangeStream::Name_);
  message.set_method_name("ProcessCreated");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WebUIGraphChangeStreamProxy::FrameChanged(
    WebUIFrameInfoPtr in_frame) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FrameChanged");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphChangeStream_FrameChanged_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphChangeStream_FrameChanged_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->frame)::BaseType::BufferWriter
      frame_writer;
  mojo::internal::Serialize<::performance_manager::mojom::WebUIFrameInfoDataView>(
      in_frame, buffer, &frame_writer, &serialization_context);
  params->frame.Set(
      frame_writer.is_null() ? nullptr : frame_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->frame.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null frame in WebUIGraphChangeStream.FrameChanged request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphChangeStream::Name_);
  message.set_method_name("FrameChanged");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WebUIGraphChangeStreamProxy::PageChanged(
    WebUIPageInfoPtr in_page) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::PageChanged");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphChangeStream_PageChanged_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphChangeStream_PageChanged_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->page)::BaseType::BufferWriter
      page_writer;
  mojo::internal::Serialize<::performance_manager::mojom::WebUIPageInfoDataView>(
      in_page, buffer, &page_writer, &serialization_context);
  params->page.Set(
      page_writer.is_null() ? nullptr : page_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->page.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null page in WebUIGraphChangeStream.PageChanged request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphChangeStream::Name_);
  message.set_method_name("PageChanged");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WebUIGraphChangeStreamProxy::ProcessChanged(
    WebUIProcessInfoPtr in_process) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::ProcessChanged");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphChangeStream_ProcessChanged_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphChangeStream_ProcessChanged_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->process)::BaseType::BufferWriter
      process_writer;
  mojo::internal::Serialize<::performance_manager::mojom::WebUIProcessInfoDataView>(
      in_process, buffer, &process_writer, &serialization_context);
  params->process.Set(
      process_writer.is_null() ? nullptr : process_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->process.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null process in WebUIGraphChangeStream.ProcessChanged request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphChangeStream::Name_);
  message.set_method_name("ProcessChanged");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WebUIGraphChangeStreamProxy::FavIconDataAvailable(
    WebUIFavIconInfoPtr in_favicon) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FavIconDataAvailable");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphChangeStream_FavIconDataAvailable_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphChangeStream_FavIconDataAvailable_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->favicon)::BaseType::BufferWriter
      favicon_writer;
  mojo::internal::Serialize<::performance_manager::mojom::WebUIFavIconInfoDataView>(
      in_favicon, buffer, &favicon_writer, &serialization_context);
  params->favicon.Set(
      favicon_writer.is_null() ? nullptr : favicon_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->favicon.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null favicon in WebUIGraphChangeStream.FavIconDataAvailable request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphChangeStream::Name_);
  message.set_method_name("FavIconDataAvailable");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WebUIGraphChangeStreamProxy::NodeDeleted(
    int64_t in_node_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::NodeDeleted");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphChangeStream_NodeDeleted_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphChangeStream_NodeDeleted_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->node_id = in_node_id;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphChangeStream::Name_);
  message.set_method_name("NodeDeleted");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool WebUIGraphChangeStreamStubDispatch::Accept(
    WebUIGraphChangeStream* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kWebUIGraphChangeStream_FrameCreated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FrameCreated",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FrameCreated");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphChangeStream_FrameCreated_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphChangeStream_FrameCreated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIFrameInfoPtr p_frame{};
      WebUIGraphChangeStream_FrameCreated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadFrame(&p_frame))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphChangeStream::Name_, 0, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->FrameCreated(
std::move(p_frame));
      return true;
    }
    case internal::kWebUIGraphChangeStream_PageCreated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::PageCreated",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::PageCreated");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphChangeStream_PageCreated_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphChangeStream_PageCreated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIPageInfoPtr p_pages{};
      WebUIGraphChangeStream_PageCreated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPages(&p_pages))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphChangeStream::Name_, 1, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->PageCreated(
std::move(p_pages));
      return true;
    }
    case internal::kWebUIGraphChangeStream_ProcessCreated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::ProcessCreated",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::ProcessCreated");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphChangeStream_ProcessCreated_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphChangeStream_ProcessCreated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIProcessInfoPtr p_process{};
      WebUIGraphChangeStream_ProcessCreated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadProcess(&p_process))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphChangeStream::Name_, 2, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->ProcessCreated(
std::move(p_process));
      return true;
    }
    case internal::kWebUIGraphChangeStream_FrameChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FrameChanged",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FrameChanged");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphChangeStream_FrameChanged_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphChangeStream_FrameChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIFrameInfoPtr p_frame{};
      WebUIGraphChangeStream_FrameChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadFrame(&p_frame))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphChangeStream::Name_, 3, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->FrameChanged(
std::move(p_frame));
      return true;
    }
    case internal::kWebUIGraphChangeStream_PageChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::PageChanged",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::PageChanged");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphChangeStream_PageChanged_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphChangeStream_PageChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIPageInfoPtr p_page{};
      WebUIGraphChangeStream_PageChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPage(&p_page))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphChangeStream::Name_, 4, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->PageChanged(
std::move(p_page));
      return true;
    }
    case internal::kWebUIGraphChangeStream_ProcessChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::ProcessChanged",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::ProcessChanged");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphChangeStream_ProcessChanged_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphChangeStream_ProcessChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIProcessInfoPtr p_process{};
      WebUIGraphChangeStream_ProcessChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadProcess(&p_process))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphChangeStream::Name_, 5, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->ProcessChanged(
std::move(p_process));
      return true;
    }
    case internal::kWebUIGraphChangeStream_FavIconDataAvailable_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FavIconDataAvailable",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::FavIconDataAvailable");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphChangeStream_FavIconDataAvailable_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphChangeStream_FavIconDataAvailable_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIFavIconInfoPtr p_favicon{};
      WebUIGraphChangeStream_FavIconDataAvailable_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadFavicon(&p_favicon))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphChangeStream::Name_, 6, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->FavIconDataAvailable(
std::move(p_favicon));
      return true;
    }
    case internal::kWebUIGraphChangeStream_NodeDeleted_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::NodeDeleted",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphChangeStream::NodeDeleted");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphChangeStream_NodeDeleted_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphChangeStream_NodeDeleted_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_node_id{};
      WebUIGraphChangeStream_NodeDeleted_ParamsDataView input_data_view(params, &serialization_context);
      
      p_node_id = input_data_view.node_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphChangeStream::Name_, 7, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->NodeDeleted(
std::move(p_node_id));
      return true;
    }
  }
  return false;
}

// static
bool WebUIGraphChangeStreamStubDispatch::AcceptWithResponder(
    WebUIGraphChangeStream* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kWebUIGraphChangeStream_FrameCreated_Name: {
      break;
    }
    case internal::kWebUIGraphChangeStream_PageCreated_Name: {
      break;
    }
    case internal::kWebUIGraphChangeStream_ProcessCreated_Name: {
      break;
    }
    case internal::kWebUIGraphChangeStream_FrameChanged_Name: {
      break;
    }
    case internal::kWebUIGraphChangeStream_PageChanged_Name: {
      break;
    }
    case internal::kWebUIGraphChangeStream_ProcessChanged_Name: {
      break;
    }
    case internal::kWebUIGraphChangeStream_FavIconDataAvailable_Name: {
      break;
    }
    case internal::kWebUIGraphChangeStream_NodeDeleted_Name: {
      break;
    }
  }
  return false;
}

bool WebUIGraphChangeStreamRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WebUIGraphChangeStream RequestValidator");

  switch (message->header()->name) {
    case internal::kWebUIGraphChangeStream_FrameCreated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphChangeStream_FrameCreated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWebUIGraphChangeStream_PageCreated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphChangeStream_PageCreated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWebUIGraphChangeStream_ProcessCreated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphChangeStream_ProcessCreated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWebUIGraphChangeStream_FrameChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphChangeStream_FrameChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWebUIGraphChangeStream_PageChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphChangeStream_PageChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWebUIGraphChangeStream_ProcessChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphChangeStream_ProcessChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWebUIGraphChangeStream_FavIconDataAvailable_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphChangeStream_FavIconDataAvailable_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWebUIGraphChangeStream_NodeDeleted_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphChangeStream_NodeDeleted_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

const char WebUIGraphDump::Name_[] = "performance_manager.mojom.WebUIGraphDump";

WebUIGraphDumpProxy::WebUIGraphDumpProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void WebUIGraphDumpProxy::SubscribeToChanges(
    WebUIGraphChangeStreamPtr in_change_subscriber) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::WebUIGraphDump::SubscribeToChanges");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphDump_SubscribeToChanges_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::performance_manager::mojom::internal::WebUIGraphDump_SubscribeToChanges_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::performance_manager::mojom::WebUIGraphChangeStreamPtrDataView>(
      in_change_subscriber, &params->change_subscriber, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->change_subscriber),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid change_subscriber in WebUIGraphDump.SubscribeToChanges request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphDump::Name_);
  message.set_method_name("SubscribeToChanges");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool WebUIGraphDumpStubDispatch::Accept(
    WebUIGraphDump* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kWebUIGraphDump_SubscribeToChanges_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphDump::SubscribeToChanges",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::WebUIGraphDump::SubscribeToChanges");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WebUIGraphDump_SubscribeToChanges_Params_Data* params =
          reinterpret_cast<internal::WebUIGraphDump_SubscribeToChanges_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIGraphChangeStreamPtr p_change_subscriber{};
      WebUIGraphDump_SubscribeToChanges_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_subscriber =
          input_data_view.TakeChangeSubscriber<decltype(p_change_subscriber)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            WebUIGraphDump::Name_, 0, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->SubscribeToChanges(
std::move(p_change_subscriber));
      return true;
    }
  }
  return false;
}

// static
bool WebUIGraphDumpStubDispatch::AcceptWithResponder(
    WebUIGraphDump* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kWebUIGraphDump_SubscribeToChanges_Name: {
      break;
    }
  }
  return false;
}

bool WebUIGraphDumpRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WebUIGraphDump RequestValidator");

  switch (message->header()->name) {
    case internal::kWebUIGraphDump_SubscribeToChanges_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphDump_SubscribeToChanges_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace performance_manager

namespace mojo {


// static
bool StructTraits<::performance_manager::mojom::WebUIPageInfo::DataView, ::performance_manager::mojom::WebUIPageInfoPtr>::Read(
    ::performance_manager::mojom::WebUIPageInfo::DataView input,
    ::performance_manager::mojom::WebUIPageInfoPtr* output) {
  bool success = true;
  ::performance_manager::mojom::WebUIPageInfoPtr result(::performance_manager::mojom::WebUIPageInfo::New());
  
      result->id = input.id();
      if (!input.ReadMainFrameUrl(&result->main_frame_url))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::performance_manager::mojom::WebUIFrameInfo::DataView, ::performance_manager::mojom::WebUIFrameInfoPtr>::Read(
    ::performance_manager::mojom::WebUIFrameInfo::DataView input,
    ::performance_manager::mojom::WebUIFrameInfoPtr* output) {
  bool success = true;
  ::performance_manager::mojom::WebUIFrameInfoPtr result(::performance_manager::mojom::WebUIFrameInfo::New());
  
      result->id = input.id();
      if (!input.ReadUrl(&result->url))
        success = false;
      result->page_id = input.page_id();
      result->parent_frame_id = input.parent_frame_id();
      result->process_id = input.process_id();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::performance_manager::mojom::WebUIProcessInfo::DataView, ::performance_manager::mojom::WebUIProcessInfoPtr>::Read(
    ::performance_manager::mojom::WebUIProcessInfo::DataView input,
    ::performance_manager::mojom::WebUIProcessInfoPtr* output) {
  bool success = true;
  ::performance_manager::mojom::WebUIProcessInfoPtr result(::performance_manager::mojom::WebUIProcessInfo::New());
  
      result->id = input.id();
      if (!input.ReadPid(&result->pid))
        success = false;
      if (!input.ReadCumulativeCpuUsage(&result->cumulative_cpu_usage))
        success = false;
      result->private_footprint_kb = input.private_footprint_kb();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::performance_manager::mojom::WebUIFavIconInfo::DataView, ::performance_manager::mojom::WebUIFavIconInfoPtr>::Read(
    ::performance_manager::mojom::WebUIFavIconInfo::DataView input,
    ::performance_manager::mojom::WebUIFavIconInfoPtr* output) {
  bool success = true;
  ::performance_manager::mojom::WebUIFavIconInfoPtr result(::performance_manager::mojom::WebUIFavIconInfo::New());
  
      result->node_id = input.node_id();
      if (!input.ReadIconData(&result->icon_data))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif