// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_PERFORMANCE_MANAGER_WEBUI_GRAPH_DUMP_MOJOM_H_
#define CHROME_BROWSER_PERFORMANCE_MANAGER_WEBUI_GRAPH_DUMP_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/browser/performance_manager/webui_graph_dump.mojom-shared.h"
#include "chrome/browser/performance_manager/webui_graph_dump.mojom-forward.h"
#include "mojo/public/mojom/base/process_id.mojom.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include "url/mojom/url.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace performance_manager {
namespace mojom {

class WebUIGraphChangeStreamProxy;

template <typename ImplRefTraits>
class WebUIGraphChangeStreamStub;

class WebUIGraphChangeStreamRequestValidator;


class  WebUIGraphChangeStream
    : public WebUIGraphChangeStreamInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = WebUIGraphChangeStreamInterfaceBase;
  using Proxy_ = WebUIGraphChangeStreamProxy;

  template <typename ImplRefTraits>
  using Stub_ = WebUIGraphChangeStreamStub<ImplRefTraits>;

  using RequestValidator_ = WebUIGraphChangeStreamRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kFrameCreatedMinVersion = 0,
    kPageCreatedMinVersion = 0,
    kProcessCreatedMinVersion = 0,
    kFrameChangedMinVersion = 0,
    kPageChangedMinVersion = 0,
    kProcessChangedMinVersion = 0,
    kFavIconDataAvailableMinVersion = 0,
    kNodeDeletedMinVersion = 0,
  };
  virtual ~WebUIGraphChangeStream() {}

  
  virtual void FrameCreated(WebUIFrameInfoPtr frame) = 0;

  
  virtual void PageCreated(WebUIPageInfoPtr pages) = 0;

  
  virtual void ProcessCreated(WebUIProcessInfoPtr process) = 0;

  
  virtual void FrameChanged(WebUIFrameInfoPtr frame) = 0;

  
  virtual void PageChanged(WebUIPageInfoPtr page) = 0;

  
  virtual void ProcessChanged(WebUIProcessInfoPtr process) = 0;

  
  virtual void FavIconDataAvailable(WebUIFavIconInfoPtr favicon) = 0;

  
  virtual void NodeDeleted(int64_t node_id) = 0;
};

class WebUIGraphDumpProxy;

template <typename ImplRefTraits>
class WebUIGraphDumpStub;

class WebUIGraphDumpRequestValidator;


class  WebUIGraphDump
    : public WebUIGraphDumpInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = WebUIGraphDumpInterfaceBase;
  using Proxy_ = WebUIGraphDumpProxy;

  template <typename ImplRefTraits>
  using Stub_ = WebUIGraphDumpStub<ImplRefTraits>;

  using RequestValidator_ = WebUIGraphDumpRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSubscribeToChangesMinVersion = 0,
  };
  virtual ~WebUIGraphDump() {}

  
  virtual void SubscribeToChanges(WebUIGraphChangeStreamPtr change_subscriber) = 0;
};

class  WebUIGraphChangeStreamProxy
    : public WebUIGraphChangeStream {
 public:
  using InterfaceType = WebUIGraphChangeStream;

  explicit WebUIGraphChangeStreamProxy(mojo::MessageReceiverWithResponder* receiver);
  void FrameCreated(WebUIFrameInfoPtr frame) final;
  void PageCreated(WebUIPageInfoPtr pages) final;
  void ProcessCreated(WebUIProcessInfoPtr process) final;
  void FrameChanged(WebUIFrameInfoPtr frame) final;
  void PageChanged(WebUIPageInfoPtr page) final;
  void ProcessChanged(WebUIProcessInfoPtr process) final;
  void FavIconDataAvailable(WebUIFavIconInfoPtr favicon) final;
  void NodeDeleted(int64_t node_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  WebUIGraphDumpProxy
    : public WebUIGraphDump {
 public:
  using InterfaceType = WebUIGraphDump;

  explicit WebUIGraphDumpProxy(mojo::MessageReceiverWithResponder* receiver);
  void SubscribeToChanges(WebUIGraphChangeStreamPtr change_subscriber) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  WebUIGraphChangeStreamStubDispatch {
 public:
  static bool Accept(WebUIGraphChangeStream* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      WebUIGraphChangeStream* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<WebUIGraphChangeStream>>
class WebUIGraphChangeStreamStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  WebUIGraphChangeStreamStub() {}
  ~WebUIGraphChangeStreamStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebUIGraphChangeStreamStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebUIGraphChangeStreamStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  WebUIGraphDumpStubDispatch {
 public:
  static bool Accept(WebUIGraphDump* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      WebUIGraphDump* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<WebUIGraphDump>>
class WebUIGraphDumpStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  WebUIGraphDumpStub() {}
  ~WebUIGraphDumpStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebUIGraphDumpStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebUIGraphDumpStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  WebUIGraphChangeStreamRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  WebUIGraphDumpRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};








class  WebUIFavIconInfo {
 public:
  using DataView = WebUIFavIconInfoDataView;
  using Data_ = internal::WebUIFavIconInfo_Data;

  template <typename... Args>
  static WebUIFavIconInfoPtr New(Args&&... args) {
    return WebUIFavIconInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static WebUIFavIconInfoPtr From(const U& u) {
    return mojo::TypeConverter<WebUIFavIconInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebUIFavIconInfo>::Convert(*this);
  }


  WebUIFavIconInfo();

  WebUIFavIconInfo(
      int64_t node_id,
      const std::string& icon_data);

  ~WebUIFavIconInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebUIFavIconInfoPtr>
  WebUIFavIconInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebUIFavIconInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebUIFavIconInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebUIFavIconInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebUIFavIconInfo_UnserializedMessageContext<
            UserType, WebUIFavIconInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebUIFavIconInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebUIFavIconInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebUIFavIconInfo_UnserializedMessageContext<
            UserType, WebUIFavIconInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebUIFavIconInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int64_t node_id;
  
  std::string icon_data;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};







class  WebUIPageInfo {
 public:
  using DataView = WebUIPageInfoDataView;
  using Data_ = internal::WebUIPageInfo_Data;

  template <typename... Args>
  static WebUIPageInfoPtr New(Args&&... args) {
    return WebUIPageInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static WebUIPageInfoPtr From(const U& u) {
    return mojo::TypeConverter<WebUIPageInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebUIPageInfo>::Convert(*this);
  }


  WebUIPageInfo();

  WebUIPageInfo(
      int64_t id,
      const GURL& main_frame_url);

  ~WebUIPageInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebUIPageInfoPtr>
  WebUIPageInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebUIPageInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebUIPageInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebUIPageInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebUIPageInfo_UnserializedMessageContext<
            UserType, WebUIPageInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebUIPageInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebUIPageInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebUIPageInfo_UnserializedMessageContext<
            UserType, WebUIPageInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebUIPageInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int64_t id;
  
  GURL main_frame_url;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class  WebUIFrameInfo {
 public:
  using DataView = WebUIFrameInfoDataView;
  using Data_ = internal::WebUIFrameInfo_Data;

  template <typename... Args>
  static WebUIFrameInfoPtr New(Args&&... args) {
    return WebUIFrameInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static WebUIFrameInfoPtr From(const U& u) {
    return mojo::TypeConverter<WebUIFrameInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebUIFrameInfo>::Convert(*this);
  }


  WebUIFrameInfo();

  WebUIFrameInfo(
      int64_t id,
      const GURL& url,
      int64_t page_id,
      int64_t parent_frame_id,
      int64_t process_id);

  ~WebUIFrameInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebUIFrameInfoPtr>
  WebUIFrameInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebUIFrameInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebUIFrameInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebUIFrameInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebUIFrameInfo_UnserializedMessageContext<
            UserType, WebUIFrameInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebUIFrameInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebUIFrameInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebUIFrameInfo_UnserializedMessageContext<
            UserType, WebUIFrameInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebUIFrameInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int64_t id;
  
  GURL url;
  
  int64_t page_id;
  
  int64_t parent_frame_id;
  
  int64_t process_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class  WebUIProcessInfo {
 public:
  using DataView = WebUIProcessInfoDataView;
  using Data_ = internal::WebUIProcessInfo_Data;

  template <typename... Args>
  static WebUIProcessInfoPtr New(Args&&... args) {
    return WebUIProcessInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static WebUIProcessInfoPtr From(const U& u) {
    return mojo::TypeConverter<WebUIProcessInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebUIProcessInfo>::Convert(*this);
  }


  WebUIProcessInfo();

  WebUIProcessInfo(
      int64_t id,
      base::ProcessId pid,
      base::TimeDelta cumulative_cpu_usage,
      uint64_t private_footprint_kb);

  ~WebUIProcessInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebUIProcessInfoPtr>
  WebUIProcessInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebUIProcessInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebUIProcessInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebUIProcessInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebUIProcessInfo_UnserializedMessageContext<
            UserType, WebUIProcessInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebUIProcessInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebUIProcessInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebUIProcessInfo_UnserializedMessageContext<
            UserType, WebUIProcessInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebUIProcessInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int64_t id;
  
  base::ProcessId pid;
  
  base::TimeDelta cumulative_cpu_usage;
  
  uint64_t private_footprint_kb;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
WebUIPageInfoPtr WebUIPageInfo::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(main_frame_url)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebUIPageInfo>::value>::type*>
bool WebUIPageInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->main_frame_url, other_struct.main_frame_url))
    return false;
  return true;
}
template <typename StructPtrType>
WebUIFrameInfoPtr WebUIFrameInfo::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(url),
      mojo::Clone(page_id),
      mojo::Clone(parent_frame_id),
      mojo::Clone(process_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebUIFrameInfo>::value>::type*>
bool WebUIFrameInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->url, other_struct.url))
    return false;
  if (!mojo::Equals(this->page_id, other_struct.page_id))
    return false;
  if (!mojo::Equals(this->parent_frame_id, other_struct.parent_frame_id))
    return false;
  if (!mojo::Equals(this->process_id, other_struct.process_id))
    return false;
  return true;
}
template <typename StructPtrType>
WebUIProcessInfoPtr WebUIProcessInfo::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(pid),
      mojo::Clone(cumulative_cpu_usage),
      mojo::Clone(private_footprint_kb)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebUIProcessInfo>::value>::type*>
bool WebUIProcessInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->pid, other_struct.pid))
    return false;
  if (!mojo::Equals(this->cumulative_cpu_usage, other_struct.cumulative_cpu_usage))
    return false;
  if (!mojo::Equals(this->private_footprint_kb, other_struct.private_footprint_kb))
    return false;
  return true;
}
template <typename StructPtrType>
WebUIFavIconInfoPtr WebUIFavIconInfo::Clone() const {
  return New(
      mojo::Clone(node_id),
      mojo::Clone(icon_data)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebUIFavIconInfo>::value>::type*>
bool WebUIFavIconInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->node_id, other_struct.node_id))
    return false;
  if (!mojo::Equals(this->icon_data, other_struct.icon_data))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace performance_manager

namespace mojo {


template <>
struct  StructTraits<::performance_manager::mojom::WebUIPageInfo::DataView,
                                         ::performance_manager::mojom::WebUIPageInfoPtr> {
  static bool IsNull(const ::performance_manager::mojom::WebUIPageInfoPtr& input) { return !input; }
  static void SetToNull(::performance_manager::mojom::WebUIPageInfoPtr* output) { output->reset(); }

  static decltype(::performance_manager::mojom::WebUIPageInfo::id) id(
      const ::performance_manager::mojom::WebUIPageInfoPtr& input) {
    return input->id;
  }

  static const decltype(::performance_manager::mojom::WebUIPageInfo::main_frame_url)& main_frame_url(
      const ::performance_manager::mojom::WebUIPageInfoPtr& input) {
    return input->main_frame_url;
  }

  static bool Read(::performance_manager::mojom::WebUIPageInfo::DataView input, ::performance_manager::mojom::WebUIPageInfoPtr* output);
};


template <>
struct  StructTraits<::performance_manager::mojom::WebUIFrameInfo::DataView,
                                         ::performance_manager::mojom::WebUIFrameInfoPtr> {
  static bool IsNull(const ::performance_manager::mojom::WebUIFrameInfoPtr& input) { return !input; }
  static void SetToNull(::performance_manager::mojom::WebUIFrameInfoPtr* output) { output->reset(); }

  static decltype(::performance_manager::mojom::WebUIFrameInfo::id) id(
      const ::performance_manager::mojom::WebUIFrameInfoPtr& input) {
    return input->id;
  }

  static const decltype(::performance_manager::mojom::WebUIFrameInfo::url)& url(
      const ::performance_manager::mojom::WebUIFrameInfoPtr& input) {
    return input->url;
  }

  static decltype(::performance_manager::mojom::WebUIFrameInfo::page_id) page_id(
      const ::performance_manager::mojom::WebUIFrameInfoPtr& input) {
    return input->page_id;
  }

  static decltype(::performance_manager::mojom::WebUIFrameInfo::parent_frame_id) parent_frame_id(
      const ::performance_manager::mojom::WebUIFrameInfoPtr& input) {
    return input->parent_frame_id;
  }

  static decltype(::performance_manager::mojom::WebUIFrameInfo::process_id) process_id(
      const ::performance_manager::mojom::WebUIFrameInfoPtr& input) {
    return input->process_id;
  }

  static bool Read(::performance_manager::mojom::WebUIFrameInfo::DataView input, ::performance_manager::mojom::WebUIFrameInfoPtr* output);
};


template <>
struct  StructTraits<::performance_manager::mojom::WebUIProcessInfo::DataView,
                                         ::performance_manager::mojom::WebUIProcessInfoPtr> {
  static bool IsNull(const ::performance_manager::mojom::WebUIProcessInfoPtr& input) { return !input; }
  static void SetToNull(::performance_manager::mojom::WebUIProcessInfoPtr* output) { output->reset(); }

  static decltype(::performance_manager::mojom::WebUIProcessInfo::id) id(
      const ::performance_manager::mojom::WebUIProcessInfoPtr& input) {
    return input->id;
  }

  static const decltype(::performance_manager::mojom::WebUIProcessInfo::pid)& pid(
      const ::performance_manager::mojom::WebUIProcessInfoPtr& input) {
    return input->pid;
  }

  static const decltype(::performance_manager::mojom::WebUIProcessInfo::cumulative_cpu_usage)& cumulative_cpu_usage(
      const ::performance_manager::mojom::WebUIProcessInfoPtr& input) {
    return input->cumulative_cpu_usage;
  }

  static decltype(::performance_manager::mojom::WebUIProcessInfo::private_footprint_kb) private_footprint_kb(
      const ::performance_manager::mojom::WebUIProcessInfoPtr& input) {
    return input->private_footprint_kb;
  }

  static bool Read(::performance_manager::mojom::WebUIProcessInfo::DataView input, ::performance_manager::mojom::WebUIProcessInfoPtr* output);
};


template <>
struct  StructTraits<::performance_manager::mojom::WebUIFavIconInfo::DataView,
                                         ::performance_manager::mojom::WebUIFavIconInfoPtr> {
  static bool IsNull(const ::performance_manager::mojom::WebUIFavIconInfoPtr& input) { return !input; }
  static void SetToNull(::performance_manager::mojom::WebUIFavIconInfoPtr* output) { output->reset(); }

  static decltype(::performance_manager::mojom::WebUIFavIconInfo::node_id) node_id(
      const ::performance_manager::mojom::WebUIFavIconInfoPtr& input) {
    return input->node_id;
  }

  static const decltype(::performance_manager::mojom::WebUIFavIconInfo::icon_data)& icon_data(
      const ::performance_manager::mojom::WebUIFavIconInfoPtr& input) {
    return input->icon_data;
  }

  static bool Read(::performance_manager::mojom::WebUIFavIconInfo::DataView input, ::performance_manager::mojom::WebUIFavIconInfoPtr* output);
};

}  // namespace mojo

#endif  // CHROME_BROWSER_PERFORMANCE_MANAGER_WEBUI_GRAPH_DUMP_MOJOM_H_