// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_SHARED_H_
#define CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/browser/ui/webui/app_management/app_management.mojom-shared-internal.h"
#include "chrome/services/app_service/public/mojom/types.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace app_management {
namespace mojom {
class AppDataView;

class ExtensionAppPermissionMessageDataView;



}  // namespace mojom
}  // namespace app_management

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::app_management::mojom::AppDataView> {
  using Data = ::app_management::mojom::internal::App_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::app_management::mojom::ExtensionAppPermissionMessageDataView> {
  using Data = ::app_management::mojom::internal::ExtensionAppPermissionMessage_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace app_management {
namespace mojom {


enum class ArcPermissionType : int32_t {
  
  CAMERA = 0,
  
  LOCATION = 1,
  
  MICROPHONE = 2,
  
  NOTIFICATIONS = 3,
  kMinValue = 0,
  kMaxValue = 3,
};

 std::ostream& operator<<(std::ostream& os, ArcPermissionType value);
inline bool IsKnownEnumValue(ArcPermissionType value) {
  return internal::ArcPermissionType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}


enum class PwaPermissionType : int32_t {
  
  CONTENT_SETTINGS_TYPE_GEOLOCATION = 5,
  
  CONTENT_SETTINGS_TYPE_NOTIFICATIONS = 6,
  
  CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC = 9,
  
  CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA = 10,
  kMinValue = 5,
  kMaxValue = 10,
};

 std::ostream& operator<<(std::ostream& os, PwaPermissionType value);
inline bool IsKnownEnumValue(PwaPermissionType value) {
  return internal::PwaPermissionType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class PageHandlerFactoryInterfaceBase {};

using PageHandlerFactoryPtrDataView =
    mojo::InterfacePtrDataView<PageHandlerFactoryInterfaceBase>;
using PageHandlerFactoryRequestDataView =
    mojo::InterfaceRequestDataView<PageHandlerFactoryInterfaceBase>;
using PageHandlerFactoryAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PageHandlerFactoryInterfaceBase>;
using PageHandlerFactoryAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PageHandlerFactoryInterfaceBase>;
class PageHandlerInterfaceBase {};

using PageHandlerPtrDataView =
    mojo::InterfacePtrDataView<PageHandlerInterfaceBase>;
using PageHandlerRequestDataView =
    mojo::InterfaceRequestDataView<PageHandlerInterfaceBase>;
using PageHandlerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PageHandlerInterfaceBase>;
using PageHandlerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PageHandlerInterfaceBase>;
class PageInterfaceBase {};

using PagePtrDataView =
    mojo::InterfacePtrDataView<PageInterfaceBase>;
using PageRequestDataView =
    mojo::InterfaceRequestDataView<PageInterfaceBase>;
using PageAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PageInterfaceBase>;
using PageAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PageInterfaceBase>;
class AppDataView {
 public:
  AppDataView() {}

  AppDataView(
      internal::App_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::apps::mojom::AppType>(
        data_value, output);
  }

  ::apps::mojom::AppType type() const {
    return static_cast<::apps::mojom::AppType>(data_->type);
  }
  inline void GetTitleDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTitle(UserType* output) {
    auto* pointer = data_->title.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetDescriptionDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDescription(UserType* output) {
    auto* pointer = data_->description.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIsPinned(UserType* output) const {
    auto data_value = data_->is_pinned;
    return mojo::internal::Deserialize<::apps::mojom::OptionalBool>(
        data_value, output);
  }

  ::apps::mojom::OptionalBool is_pinned() const {
    return static_cast<::apps::mojom::OptionalBool>(data_->is_pinned);
  }
  inline void GetVersionDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadVersion(UserType* output) {
    auto* pointer = data_->version.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetSizeDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSize(UserType* output) {
    auto* pointer = data_->size.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetPermissionsDataView(
      mojo::MapDataView<uint32_t, ::apps::mojom::PermissionDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPermissions(UserType* output) {
    auto* pointer = data_->permissions.Get();
    return mojo::internal::Deserialize<mojo::MapDataView<uint32_t, ::apps::mojom::PermissionDataView>>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstallSource(UserType* output) const {
    auto data_value = data_->install_source;
    return mojo::internal::Deserialize<::apps::mojom::InstallSource>(
        data_value, output);
  }

  ::apps::mojom::InstallSource install_source() const {
    return static_cast<::apps::mojom::InstallSource>(data_->install_source);
  }
 private:
  internal::App_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ExtensionAppPermissionMessageDataView {
 public:
  ExtensionAppPermissionMessageDataView() {}

  ExtensionAppPermissionMessageDataView(
      internal::ExtensionAppPermissionMessage_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMessageDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMessage(UserType* output) {
    auto* pointer = data_->message.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetSubmessagesDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSubmessages(UserType* output) {
    auto* pointer = data_->submessages.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
 private:
  internal::ExtensionAppPermissionMessage_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace app_management

namespace std {

template <>
struct hash<::app_management::mojom::ArcPermissionType>
    : public mojo::internal::EnumHashImpl<::app_management::mojom::ArcPermissionType> {};

template <>
struct hash<::app_management::mojom::PwaPermissionType>
    : public mojo::internal::EnumHashImpl<::app_management::mojom::PwaPermissionType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::app_management::mojom::ArcPermissionType, ::app_management::mojom::ArcPermissionType> {
  static ::app_management::mojom::ArcPermissionType ToMojom(::app_management::mojom::ArcPermissionType input) { return input; }
  static bool FromMojom(::app_management::mojom::ArcPermissionType input, ::app_management::mojom::ArcPermissionType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::app_management::mojom::ArcPermissionType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::app_management::mojom::ArcPermissionType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::app_management::mojom::ArcPermissionType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::app_management::mojom::PwaPermissionType, ::app_management::mojom::PwaPermissionType> {
  static ::app_management::mojom::PwaPermissionType ToMojom(::app_management::mojom::PwaPermissionType input) { return input; }
  static bool FromMojom(::app_management::mojom::PwaPermissionType input, ::app_management::mojom::PwaPermissionType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::app_management::mojom::PwaPermissionType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::app_management::mojom::PwaPermissionType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::app_management::mojom::PwaPermissionType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::app_management::mojom::AppDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::app_management::mojom::AppDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::app_management::mojom::internal::App_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::id(input)) in_id = Traits::id(input);
    typename decltype((*output)->id)::BaseType::BufferWriter
        id_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_id, buffer, &id_writer, context);
    (*output)->id.Set(
        id_writer.is_null() ? nullptr : id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null id in App struct");
    mojo::internal::Serialize<::apps::mojom::AppType>(
        Traits::type(input), &(*output)->type);
    decltype(Traits::title(input)) in_title = Traits::title(input);
    typename decltype((*output)->title)::BaseType::BufferWriter
        title_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_title, buffer, &title_writer, context);
    (*output)->title.Set(
        title_writer.is_null() ? nullptr : title_writer.data());
    decltype(Traits::description(input)) in_description = Traits::description(input);
    typename decltype((*output)->description)::BaseType::BufferWriter
        description_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_description, buffer, &description_writer, context);
    (*output)->description.Set(
        description_writer.is_null() ? nullptr : description_writer.data());
    mojo::internal::Serialize<::apps::mojom::OptionalBool>(
        Traits::is_pinned(input), &(*output)->is_pinned);
    decltype(Traits::version(input)) in_version = Traits::version(input);
    typename decltype((*output)->version)::BaseType::BufferWriter
        version_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_version, buffer, &version_writer, context);
    (*output)->version.Set(
        version_writer.is_null() ? nullptr : version_writer.data());
    decltype(Traits::size(input)) in_size = Traits::size(input);
    typename decltype((*output)->size)::BaseType::BufferWriter
        size_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_size, buffer, &size_writer, context);
    (*output)->size.Set(
        size_writer.is_null() ? nullptr : size_writer.data());
    decltype(Traits::permissions(input)) in_permissions = Traits::permissions(input);
    typename decltype((*output)->permissions)::BaseType::BufferWriter
        permissions_writer;
    const mojo::internal::ContainerValidateParams permissions_validate_params(
        new mojo::internal::ContainerValidateParams(0, false, nullptr), new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::MapDataView<uint32_t, ::apps::mojom::PermissionDataView>>(
        in_permissions, buffer, &permissions_writer, &permissions_validate_params,
        context);
    (*output)->permissions.Set(
        permissions_writer.is_null() ? nullptr : permissions_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->permissions.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null permissions in App struct");
    mojo::internal::Serialize<::apps::mojom::InstallSource>(
        Traits::install_source(input), &(*output)->install_source);
  }

  static bool Deserialize(::app_management::mojom::internal::App_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::app_management::mojom::AppDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::app_management::mojom::ExtensionAppPermissionMessageDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::app_management::mojom::ExtensionAppPermissionMessageDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::app_management::mojom::internal::ExtensionAppPermissionMessage_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::message(input)) in_message = Traits::message(input);
    typename decltype((*output)->message)::BaseType::BufferWriter
        message_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_message, buffer, &message_writer, context);
    (*output)->message.Set(
        message_writer.is_null() ? nullptr : message_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->message.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null message in ExtensionAppPermissionMessage struct");
    decltype(Traits::submessages(input)) in_submessages = Traits::submessages(input);
    typename decltype((*output)->submessages)::BaseType::BufferWriter
        submessages_writer;
    const mojo::internal::ContainerValidateParams submessages_validate_params(
        0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_submessages, buffer, &submessages_writer, &submessages_validate_params,
        context);
    (*output)->submessages.Set(
        submessages_writer.is_null() ? nullptr : submessages_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->submessages.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null submessages in ExtensionAppPermissionMessage struct");
  }

  static bool Deserialize(::app_management::mojom::internal::ExtensionAppPermissionMessage_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::app_management::mojom::ExtensionAppPermissionMessageDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace app_management {
namespace mojom {

inline void AppDataView::GetIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetTitleDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->title.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetDescriptionDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->description.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetVersionDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->version.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetSizeDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->size.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetPermissionsDataView(
    mojo::MapDataView<uint32_t, ::apps::mojom::PermissionDataView>* output) {
  auto pointer = data_->permissions.Get();
  *output = mojo::MapDataView<uint32_t, ::apps::mojom::PermissionDataView>(pointer, context_);
}


inline void ExtensionAppPermissionMessageDataView::GetMessageDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->message.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void ExtensionAppPermissionMessageDataView::GetSubmessagesDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->submessages.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}



}  // namespace mojom
}  // namespace app_management

#endif  // CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_SHARED_H_