// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "chrome/browser/ui/webui/app_management/app_management.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/hash/md5_constexpr.h"
#include "base/logging.h"
#include "base/run_loop.h"
#include "base/task/common/task_annotator.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "chrome/browser/ui/webui/app_management/app_management.mojom-params-data.h"
#include "chrome/browser/ui/webui/app_management/app_management.mojom-shared-message-ids.h"

#include "chrome/browser/ui/webui/app_management/app_management.mojom-import-headers.h"


#ifndef CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_JUMBO_H_
#define CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_JUMBO_H_
#endif
namespace app_management {
namespace mojom {
App::App()
    : id(),
      type(),
      title(),
      description(),
      is_pinned(),
      version(),
      size(),
      permissions(),
      install_source() {}

App::App(
    const std::string& id_in,
    ::apps::mojom::AppType type_in,
    const base::Optional<std::string>& title_in,
    const base::Optional<std::string>& description_in,
    ::apps::mojom::OptionalBool is_pinned_in,
    const base::Optional<std::string>& version_in,
    const base::Optional<std::string>& size_in,
    base::flat_map<uint32_t, ::apps::mojom::PermissionPtr> permissions_in,
    ::apps::mojom::InstallSource install_source_in)
    : id(std::move(id_in)),
      type(std::move(type_in)),
      title(std::move(title_in)),
      description(std::move(description_in)),
      is_pinned(std::move(is_pinned_in)),
      version(std::move(version_in)),
      size(std::move(size_in)),
      permissions(std::move(permissions_in)),
      install_source(std::move(install_source_in)) {}

App::~App() = default;

bool App::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ExtensionAppPermissionMessage::ExtensionAppPermissionMessage()
    : message(),
      submessages() {}

ExtensionAppPermissionMessage::ExtensionAppPermissionMessage(
    const std::string& message_in,
    const std::vector<std::string>& submessages_in)
    : message(std::move(message_in)),
      submessages(std::move(submessages_in)) {}

ExtensionAppPermissionMessage::~ExtensionAppPermissionMessage() = default;

bool ExtensionAppPermissionMessage::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char PageHandlerFactory::Name_[] = "app_management.mojom.PageHandlerFactory";

PageHandlerFactoryProxy::PageHandlerFactoryProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PageHandlerFactoryProxy::CreatePageHandler(
    PagePtr in_page, PageHandlerRequest in_handler) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::PageHandlerFactory::CreatePageHandler");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandlerFactory_CreatePageHandler_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandlerFactory_CreatePageHandler_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::app_management::mojom::PagePtrDataView>(
      in_page, &params->page, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->page),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid page in PageHandlerFactory.CreatePageHandler request");
  mojo::internal::Serialize<::app_management::mojom::PageHandlerRequestDataView>(
      in_handler, &params->handler, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->handler),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid handler in PageHandlerFactory.CreatePageHandler request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandlerFactory::Name_);
  message.set_method_name("CreatePageHandler");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool PageHandlerFactoryStubDispatch::Accept(
    PageHandlerFactory* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPageHandlerFactory_CreatePageHandler_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::PageHandlerFactory::CreatePageHandler",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::PageHandlerFactory::CreatePageHandler");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageHandlerFactory_CreatePageHandler_Params_Data* params =
          reinterpret_cast<internal::PageHandlerFactory_CreatePageHandler_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      PagePtr p_page{};
      PageHandlerRequest p_handler{};
      PageHandlerFactory_CreatePageHandler_ParamsDataView input_data_view(params, &serialization_context);
      
      p_page =
          input_data_view.TakePage<decltype(p_page)>();
      p_handler =
          input_data_view.TakeHandler<decltype(p_handler)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            PageHandlerFactory::Name_, 0, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->CreatePageHandler(
std::move(p_page), 
std::move(p_handler));
      return true;
    }
  }
  return false;
}

// static
bool PageHandlerFactoryStubDispatch::AcceptWithResponder(
    PageHandlerFactory* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPageHandlerFactory_CreatePageHandler_Name: {
      break;
    }
  }
  return false;
}

bool PageHandlerFactoryRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PageHandlerFactory RequestValidator");

  switch (message->header()->name) {
    case internal::kPageHandlerFactory_CreatePageHandler_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandlerFactory_CreatePageHandler_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

const char PageHandler::Name_[] = "app_management.mojom.PageHandler";

class PageHandler_GetApps_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PageHandler_GetApps_ForwardToCallback(
      PageHandler::GetAppsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PageHandler::GetAppsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PageHandler_GetApps_ForwardToCallback);
};

class PageHandler_GetExtensionAppPermissionMessages_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PageHandler_GetExtensionAppPermissionMessages_ForwardToCallback(
      PageHandler::GetExtensionAppPermissionMessagesCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PageHandler::GetExtensionAppPermissionMessagesCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PageHandler_GetExtensionAppPermissionMessages_ForwardToCallback);
};

PageHandlerProxy::PageHandlerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PageHandlerProxy::GetApps(
    GetAppsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::PageHandler::GetApps");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandler_GetApps_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandler_GetApps_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandler::Name_);
  message.set_method_name("GetApps");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PageHandler_GetApps_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PageHandlerProxy::GetExtensionAppPermissionMessages(
    const std::string& in_app_id, GetExtensionAppPermissionMessagesCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::PageHandler::GetExtensionAppPermissionMessages");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandler_GetExtensionAppPermissionMessages_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandler_GetExtensionAppPermissionMessages_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app_id)::BaseType::BufferWriter
      app_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_app_id, buffer, &app_id_writer, &serialization_context);
  params->app_id.Set(
      app_id_writer.is_null() ? nullptr : app_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app_id in PageHandler.GetExtensionAppPermissionMessages request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandler::Name_);
  message.set_method_name("GetExtensionAppPermissionMessages");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PageHandler_GetExtensionAppPermissionMessages_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PageHandlerProxy::SetPinned(
    const std::string& in_app_id, ::apps::mojom::OptionalBool in_pinned) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::PageHandler::SetPinned");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandler_SetPinned_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandler_SetPinned_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app_id)::BaseType::BufferWriter
      app_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_app_id, buffer, &app_id_writer, &serialization_context);
  params->app_id.Set(
      app_id_writer.is_null() ? nullptr : app_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app_id in PageHandler.SetPinned request");
  mojo::internal::Serialize<::apps::mojom::OptionalBool>(
      in_pinned, &params->pinned);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandler::Name_);
  message.set_method_name("SetPinned");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageHandlerProxy::SetPermission(
    const std::string& in_app_id, ::apps::mojom::PermissionPtr in_permission) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::PageHandler::SetPermission");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandler_SetPermission_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandler_SetPermission_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app_id)::BaseType::BufferWriter
      app_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_app_id, buffer, &app_id_writer, &serialization_context);
  params->app_id.Set(
      app_id_writer.is_null() ? nullptr : app_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app_id in PageHandler.SetPermission request");
  typename decltype(params->permission)::BaseType::BufferWriter
      permission_writer;
  mojo::internal::Serialize<::apps::mojom::PermissionDataView>(
      in_permission, buffer, &permission_writer, &serialization_context);
  params->permission.Set(
      permission_writer.is_null() ? nullptr : permission_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->permission.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null permission in PageHandler.SetPermission request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandler::Name_);
  message.set_method_name("SetPermission");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageHandlerProxy::Uninstall(
    const std::string& in_app_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::PageHandler::Uninstall");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandler_Uninstall_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandler_Uninstall_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app_id)::BaseType::BufferWriter
      app_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_app_id, buffer, &app_id_writer, &serialization_context);
  params->app_id.Set(
      app_id_writer.is_null() ? nullptr : app_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app_id in PageHandler.Uninstall request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandler::Name_);
  message.set_method_name("Uninstall");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageHandlerProxy::OpenNativeSettings(
    const std::string& in_app_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::PageHandler::OpenNativeSettings");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandler_OpenNativeSettings_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandler_OpenNativeSettings_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app_id)::BaseType::BufferWriter
      app_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_app_id, buffer, &app_id_writer, &serialization_context);
  params->app_id.Set(
      app_id_writer.is_null() ? nullptr : app_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app_id in PageHandler.OpenNativeSettings request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandler::Name_);
  message.set_method_name("OpenNativeSettings");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class PageHandler_GetApps_ProxyToResponder {
 public:
  static PageHandler::GetAppsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PageHandler_GetApps_ProxyToResponder> proxy(
        new PageHandler_GetApps_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&PageHandler_GetApps_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~PageHandler_GetApps_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PageHandler_GetApps_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "PageHandler::GetAppsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      std::vector<AppPtr> in_apps);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PageHandler_GetApps_ProxyToResponder);
};

bool PageHandler_GetApps_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "<class 'jinja2::utils::Namespace'>::PageHandler::GetAppsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PageHandler_GetApps_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PageHandler_GetApps_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<AppPtr> p_apps{};
  PageHandler_GetApps_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadApps(&p_apps))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        PageHandler::Name_, 0, true);
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_apps));
  return true;
}

void PageHandler_GetApps_ProxyToResponder::Run(
    std::vector<AppPtr> in_apps) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandler_GetApps_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandler_GetApps_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->apps)::BaseType::BufferWriter
      apps_writer;
  const mojo::internal::ContainerValidateParams apps_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::app_management::mojom::AppDataView>>(
      in_apps, buffer, &apps_writer, &apps_validate_params,
      &serialization_context);
  params->apps.Set(
      apps_writer.is_null() ? nullptr : apps_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->apps.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null apps in ");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::GetAppsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandler::Name_);
  message.set_method_name("GetApps");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder {
 public:
  static PageHandler::GetExtensionAppPermissionMessagesCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder> proxy(
        new PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "PageHandler::GetExtensionAppPermissionMessagesCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      std::vector<ExtensionAppPermissionMessagePtr> in_messages);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder);
};

bool PageHandler_GetExtensionAppPermissionMessages_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "<class 'jinja2::utils::Namespace'>::PageHandler::GetExtensionAppPermissionMessagesCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PageHandler_GetExtensionAppPermissionMessages_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PageHandler_GetExtensionAppPermissionMessages_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<ExtensionAppPermissionMessagePtr> p_messages{};
  PageHandler_GetExtensionAppPermissionMessages_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadMessages(&p_messages))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        PageHandler::Name_, 1, true);
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_messages));
  return true;
}

void PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder::Run(
    std::vector<ExtensionAppPermissionMessagePtr> in_messages) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageHandler_GetExtensionAppPermissionMessages_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::PageHandler_GetExtensionAppPermissionMessages_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->messages)::BaseType::BufferWriter
      messages_writer;
  const mojo::internal::ContainerValidateParams messages_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::app_management::mojom::ExtensionAppPermissionMessageDataView>>(
      in_messages, buffer, &messages_writer, &messages_validate_params,
      &serialization_context);
  params->messages.Set(
      messages_writer.is_null() ? nullptr : messages_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->messages.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null messages in ");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::GetExtensionAppPermissionMessagesCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageHandler::Name_);
  message.set_method_name("GetExtensionAppPermissionMessages");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool PageHandlerStubDispatch::Accept(
    PageHandler* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPageHandler_GetApps_Name: {
      break;
    }
    case internal::kPageHandler_GetExtensionAppPermissionMessages_Name: {
      break;
    }
    case internal::kPageHandler_SetPinned_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::SetPinned",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::SetPinned");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageHandler_SetPinned_Params_Data* params =
          reinterpret_cast<internal::PageHandler_SetPinned_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_app_id{};
      ::apps::mojom::OptionalBool p_pinned{};
      PageHandler_SetPinned_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAppId(&p_app_id))
        success = false;
      if (!input_data_view.ReadPinned(&p_pinned))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            PageHandler::Name_, 2, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->SetPinned(
std::move(p_app_id), 
std::move(p_pinned));
      return true;
    }
    case internal::kPageHandler_SetPermission_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::SetPermission",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::SetPermission");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageHandler_SetPermission_Params_Data* params =
          reinterpret_cast<internal::PageHandler_SetPermission_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_app_id{};
      ::apps::mojom::PermissionPtr p_permission{};
      PageHandler_SetPermission_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAppId(&p_app_id))
        success = false;
      if (!input_data_view.ReadPermission(&p_permission))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            PageHandler::Name_, 3, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->SetPermission(
std::move(p_app_id), 
std::move(p_permission));
      return true;
    }
    case internal::kPageHandler_Uninstall_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::Uninstall",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::Uninstall");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageHandler_Uninstall_Params_Data* params =
          reinterpret_cast<internal::PageHandler_Uninstall_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_app_id{};
      PageHandler_Uninstall_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAppId(&p_app_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            PageHandler::Name_, 4, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->Uninstall(
std::move(p_app_id));
      return true;
    }
    case internal::kPageHandler_OpenNativeSettings_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::OpenNativeSettings",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::OpenNativeSettings");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageHandler_OpenNativeSettings_Params_Data* params =
          reinterpret_cast<internal::PageHandler_OpenNativeSettings_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_app_id{};
      PageHandler_OpenNativeSettings_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAppId(&p_app_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            PageHandler::Name_, 5, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->OpenNativeSettings(
std::move(p_app_id));
      return true;
    }
  }
  return false;
}

// static
bool PageHandlerStubDispatch::AcceptWithResponder(
    PageHandler* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPageHandler_GetApps_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::GetApps",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::GetApps");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      internal::PageHandler_GetApps_Params_Data* params =
          reinterpret_cast<
              internal::PageHandler_GetApps_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      PageHandler_GetApps_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            PageHandler::Name_, 0, false);
        return false;
      }
      PageHandler::GetAppsCallback callback =
          PageHandler_GetApps_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->GetApps(std::move(callback));
      return true;
    }
    case internal::kPageHandler_GetExtensionAppPermissionMessages_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::GetExtensionAppPermissionMessages",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::PageHandler::GetExtensionAppPermissionMessages");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      internal::PageHandler_GetExtensionAppPermissionMessages_Params_Data* params =
          reinterpret_cast<
              internal::PageHandler_GetExtensionAppPermissionMessages_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_app_id{};
      PageHandler_GetExtensionAppPermissionMessages_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAppId(&p_app_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            PageHandler::Name_, 1, false);
        return false;
      }
      PageHandler::GetExtensionAppPermissionMessagesCallback callback =
          PageHandler_GetExtensionAppPermissionMessages_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->GetExtensionAppPermissionMessages(
std::move(p_app_id), std::move(callback));
      return true;
    }
    case internal::kPageHandler_SetPinned_Name: {
      break;
    }
    case internal::kPageHandler_SetPermission_Name: {
      break;
    }
    case internal::kPageHandler_Uninstall_Name: {
      break;
    }
    case internal::kPageHandler_OpenNativeSettings_Name: {
      break;
    }
  }
  return false;
}

bool PageHandlerRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PageHandler RequestValidator");

  switch (message->header()->name) {
    case internal::kPageHandler_GetApps_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandler_GetApps_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageHandler_GetExtensionAppPermissionMessages_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandler_GetExtensionAppPermissionMessages_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageHandler_SetPinned_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandler_SetPinned_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageHandler_SetPermission_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandler_SetPermission_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageHandler_Uninstall_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandler_Uninstall_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageHandler_OpenNativeSettings_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandler_OpenNativeSettings_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool PageHandlerResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PageHandler ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kPageHandler_GetApps_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandler_GetApps_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageHandler_GetExtensionAppPermissionMessages_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageHandler_GetExtensionAppPermissionMessages_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char Page::Name_[] = "app_management.mojom.Page";

PageProxy::PageProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PageProxy::OnAppAdded(
    AppPtr in_app) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::Page::OnAppAdded");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPage_OnAppAdded_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::Page_OnAppAdded_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app)::BaseType::BufferWriter
      app_writer;
  mojo::internal::Serialize<::app_management::mojom::AppDataView>(
      in_app, buffer, &app_writer, &serialization_context);
  params->app.Set(
      app_writer.is_null() ? nullptr : app_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app in Page.OnAppAdded request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(Page::Name_);
  message.set_method_name("OnAppAdded");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageProxy::OnAppChanged(
    AppPtr in_update) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::Page::OnAppChanged");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPage_OnAppChanged_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::Page_OnAppChanged_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->update)::BaseType::BufferWriter
      update_writer;
  mojo::internal::Serialize<::app_management::mojom::AppDataView>(
      in_update, buffer, &update_writer, &serialization_context);
  params->update.Set(
      update_writer.is_null() ? nullptr : update_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->update.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null update in Page.OnAppChanged request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(Page::Name_);
  message.set_method_name("OnAppChanged");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageProxy::OnAppRemoved(
    const std::string& in_app_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::Page::OnAppRemoved");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPage_OnAppRemoved_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::app_management::mojom::internal::Page_OnAppRemoved_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app_id)::BaseType::BufferWriter
      app_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_app_id, buffer, &app_id_writer, &serialization_context);
  params->app_id.Set(
      app_id_writer.is_null() ? nullptr : app_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app_id in Page.OnAppRemoved request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(Page::Name_);
  message.set_method_name("OnAppRemoved");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool PageStubDispatch::Accept(
    Page* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPage_OnAppAdded_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::Page::OnAppAdded",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::Page::OnAppAdded");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Page_OnAppAdded_Params_Data* params =
          reinterpret_cast<internal::Page_OnAppAdded_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      AppPtr p_app{};
      Page_OnAppAdded_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadApp(&p_app))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            Page::Name_, 0, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->OnAppAdded(
std::move(p_app));
      return true;
    }
    case internal::kPage_OnAppChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::Page::OnAppChanged",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::Page::OnAppChanged");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Page_OnAppChanged_Params_Data* params =
          reinterpret_cast<internal::Page_OnAppChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      AppPtr p_update{};
      Page_OnAppChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadUpdate(&p_update))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            Page::Name_, 1, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->OnAppChanged(
std::move(p_update));
      return true;
    }
    case internal::kPage_OnAppRemoved_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::Page::OnAppRemoved",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::Page::OnAppRemoved");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Page_OnAppRemoved_Params_Data* params =
          reinterpret_cast<internal::Page_OnAppRemoved_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_app_id{};
      Page_OnAppRemoved_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAppId(&p_app_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            Page::Name_, 2, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->OnAppRemoved(
std::move(p_app_id));
      return true;
    }
  }
  return false;
}

// static
bool PageStubDispatch::AcceptWithResponder(
    Page* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPage_OnAppAdded_Name: {
      break;
    }
    case internal::kPage_OnAppChanged_Name: {
      break;
    }
    case internal::kPage_OnAppRemoved_Name: {
      break;
    }
  }
  return false;
}

bool PageRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "Page RequestValidator");

  switch (message->header()->name) {
    case internal::kPage_OnAppAdded_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Page_OnAppAdded_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPage_OnAppChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Page_OnAppChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPage_OnAppRemoved_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Page_OnAppRemoved_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace app_management

namespace mojo {


// static
bool StructTraits<::app_management::mojom::App::DataView, ::app_management::mojom::AppPtr>::Read(
    ::app_management::mojom::App::DataView input,
    ::app_management::mojom::AppPtr* output) {
  bool success = true;
  ::app_management::mojom::AppPtr result(::app_management::mojom::App::New());
  
      if (!input.ReadId(&result->id))
        success = false;
      if (!input.ReadType(&result->type))
        success = false;
      if (!input.ReadTitle(&result->title))
        success = false;
      if (!input.ReadDescription(&result->description))
        success = false;
      if (!input.ReadIsPinned(&result->is_pinned))
        success = false;
      if (!input.ReadVersion(&result->version))
        success = false;
      if (!input.ReadSize(&result->size))
        success = false;
      if (!input.ReadPermissions(&result->permissions))
        success = false;
      if (!input.ReadInstallSource(&result->install_source))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::app_management::mojom::ExtensionAppPermissionMessage::DataView, ::app_management::mojom::ExtensionAppPermissionMessagePtr>::Read(
    ::app_management::mojom::ExtensionAppPermissionMessage::DataView input,
    ::app_management::mojom::ExtensionAppPermissionMessagePtr* output) {
  bool success = true;
  ::app_management::mojom::ExtensionAppPermissionMessagePtr result(::app_management::mojom::ExtensionAppPermissionMessage::New());
  
      if (!input.ReadMessage(&result->message))
        success = false;
      if (!input.ReadSubmessages(&result->submessages))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif