// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_H_
#define CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/browser/ui/webui/app_management/app_management.mojom-shared.h"
#include "chrome/browser/ui/webui/app_management/app_management.mojom-forward.h"
#include "chrome/services/app_service/public/mojom/types.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace app_management {
namespace mojom {

class PageHandlerFactoryProxy;

template <typename ImplRefTraits>
class PageHandlerFactoryStub;

class PageHandlerFactoryRequestValidator;


class  PageHandlerFactory
    : public PageHandlerFactoryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = PageHandlerFactoryInterfaceBase;
  using Proxy_ = PageHandlerFactoryProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageHandlerFactoryStub<ImplRefTraits>;

  using RequestValidator_ = PageHandlerFactoryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreatePageHandlerMinVersion = 0,
  };
  virtual ~PageHandlerFactory() {}

  
  virtual void CreatePageHandler(PagePtr page, PageHandlerRequest handler) = 0;
};

class PageHandlerProxy;

template <typename ImplRefTraits>
class PageHandlerStub;

class PageHandlerRequestValidator;
class PageHandlerResponseValidator;


class  PageHandler
    : public PageHandlerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = PageHandlerInterfaceBase;
  using Proxy_ = PageHandlerProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageHandlerStub<ImplRefTraits>;

  using RequestValidator_ = PageHandlerRequestValidator;
  using ResponseValidator_ = PageHandlerResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetAppsMinVersion = 0,
    kGetExtensionAppPermissionMessagesMinVersion = 0,
    kSetPinnedMinVersion = 0,
    kSetPermissionMinVersion = 0,
    kUninstallMinVersion = 0,
    kOpenNativeSettingsMinVersion = 0,
  };
  virtual ~PageHandler() {}


  using GetAppsCallback = base::OnceCallback<void(std::vector<AppPtr>)>;
  
  virtual void GetApps(GetAppsCallback callback) = 0;


  using GetExtensionAppPermissionMessagesCallback = base::OnceCallback<void(std::vector<ExtensionAppPermissionMessagePtr>)>;
  
  virtual void GetExtensionAppPermissionMessages(const std::string& app_id, GetExtensionAppPermissionMessagesCallback callback) = 0;

  
  virtual void SetPinned(const std::string& app_id, ::apps::mojom::OptionalBool pinned) = 0;

  
  virtual void SetPermission(const std::string& app_id, ::apps::mojom::PermissionPtr permission) = 0;

  
  virtual void Uninstall(const std::string& app_id) = 0;

  
  virtual void OpenNativeSettings(const std::string& app_id) = 0;
};

class PageProxy;

template <typename ImplRefTraits>
class PageStub;

class PageRequestValidator;


class  Page
    : public PageInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = PageInterfaceBase;
  using Proxy_ = PageProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageStub<ImplRefTraits>;

  using RequestValidator_ = PageRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnAppAddedMinVersion = 0,
    kOnAppChangedMinVersion = 0,
    kOnAppRemovedMinVersion = 0,
  };
  virtual ~Page() {}

  
  virtual void OnAppAdded(AppPtr app) = 0;

  
  virtual void OnAppChanged(AppPtr update) = 0;

  
  virtual void OnAppRemoved(const std::string& app_id) = 0;
};

class  PageHandlerFactoryProxy
    : public PageHandlerFactory {
 public:
  using InterfaceType = PageHandlerFactory;

  explicit PageHandlerFactoryProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreatePageHandler(PagePtr page, PageHandlerRequest handler) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  PageHandlerProxy
    : public PageHandler {
 public:
  using InterfaceType = PageHandler;

  explicit PageHandlerProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetApps(GetAppsCallback callback) final;
  void GetExtensionAppPermissionMessages(const std::string& app_id, GetExtensionAppPermissionMessagesCallback callback) final;
  void SetPinned(const std::string& app_id, ::apps::mojom::OptionalBool pinned) final;
  void SetPermission(const std::string& app_id, ::apps::mojom::PermissionPtr permission) final;
  void Uninstall(const std::string& app_id) final;
  void OpenNativeSettings(const std::string& app_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  PageProxy
    : public Page {
 public:
  using InterfaceType = Page;

  explicit PageProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnAppAdded(AppPtr app) final;
  void OnAppChanged(AppPtr update) final;
  void OnAppRemoved(const std::string& app_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  PageHandlerFactoryStubDispatch {
 public:
  static bool Accept(PageHandlerFactory* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PageHandlerFactory* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PageHandlerFactory>>
class PageHandlerFactoryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageHandlerFactoryStub() {}
  ~PageHandlerFactoryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerFactoryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerFactoryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PageHandlerStubDispatch {
 public:
  static bool Accept(PageHandler* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PageHandler* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PageHandler>>
class PageHandlerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageHandlerStub() {}
  ~PageHandlerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PageStubDispatch {
 public:
  static bool Accept(Page* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Page* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Page>>
class PageStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageStub() {}
  ~PageStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PageHandlerFactoryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PageHandlerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PageRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PageHandlerResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};









class  App {
 public:
  using DataView = AppDataView;
  using Data_ = internal::App_Data;

  template <typename... Args>
  static AppPtr New(Args&&... args) {
    return AppPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AppPtr From(const U& u) {
    return mojo::TypeConverter<AppPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, App>::Convert(*this);
  }


  App();

  App(
      const std::string& id,
      ::apps::mojom::AppType type,
      const base::Optional<std::string>& title,
      const base::Optional<std::string>& description,
      ::apps::mojom::OptionalBool is_pinned,
      const base::Optional<std::string>& version,
      const base::Optional<std::string>& size,
      base::flat_map<uint32_t, ::apps::mojom::PermissionPtr> permissions,
      ::apps::mojom::InstallSource install_source);

  ~App();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AppPtr>
  AppPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, App>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        App::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        App::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::App_UnserializedMessageContext<
            UserType, App::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<App::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return App::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::App_UnserializedMessageContext<
            UserType, App::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<App::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string id;
  
  ::apps::mojom::AppType type;
  
  base::Optional<std::string> title;
  
  base::Optional<std::string> description;
  
  ::apps::mojom::OptionalBool is_pinned;
  
  base::Optional<std::string> version;
  
  base::Optional<std::string> size;
  
  base::flat_map<uint32_t, ::apps::mojom::PermissionPtr> permissions;
  
  ::apps::mojom::InstallSource install_source;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(App);
};





class  ExtensionAppPermissionMessage {
 public:
  using DataView = ExtensionAppPermissionMessageDataView;
  using Data_ = internal::ExtensionAppPermissionMessage_Data;

  template <typename... Args>
  static ExtensionAppPermissionMessagePtr New(Args&&... args) {
    return ExtensionAppPermissionMessagePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static ExtensionAppPermissionMessagePtr From(const U& u) {
    return mojo::TypeConverter<ExtensionAppPermissionMessagePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ExtensionAppPermissionMessage>::Convert(*this);
  }


  ExtensionAppPermissionMessage();

  ExtensionAppPermissionMessage(
      const std::string& message,
      const std::vector<std::string>& submessages);

  ~ExtensionAppPermissionMessage();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ExtensionAppPermissionMessagePtr>
  ExtensionAppPermissionMessagePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ExtensionAppPermissionMessage>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ExtensionAppPermissionMessage::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ExtensionAppPermissionMessage::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ExtensionAppPermissionMessage_UnserializedMessageContext<
            UserType, ExtensionAppPermissionMessage::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ExtensionAppPermissionMessage::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ExtensionAppPermissionMessage::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ExtensionAppPermissionMessage_UnserializedMessageContext<
            UserType, ExtensionAppPermissionMessage::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ExtensionAppPermissionMessage::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string message;
  
  std::vector<std::string> submessages;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
AppPtr App::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(type),
      mojo::Clone(title),
      mojo::Clone(description),
      mojo::Clone(is_pinned),
      mojo::Clone(version),
      mojo::Clone(size),
      mojo::Clone(permissions),
      mojo::Clone(install_source)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, App>::value>::type*>
bool App::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->type, other_struct.type))
    return false;
  if (!mojo::Equals(this->title, other_struct.title))
    return false;
  if (!mojo::Equals(this->description, other_struct.description))
    return false;
  if (!mojo::Equals(this->is_pinned, other_struct.is_pinned))
    return false;
  if (!mojo::Equals(this->version, other_struct.version))
    return false;
  if (!mojo::Equals(this->size, other_struct.size))
    return false;
  if (!mojo::Equals(this->permissions, other_struct.permissions))
    return false;
  if (!mojo::Equals(this->install_source, other_struct.install_source))
    return false;
  return true;
}
template <typename StructPtrType>
ExtensionAppPermissionMessagePtr ExtensionAppPermissionMessage::Clone() const {
  return New(
      mojo::Clone(message),
      mojo::Clone(submessages)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ExtensionAppPermissionMessage>::value>::type*>
bool ExtensionAppPermissionMessage::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->message, other_struct.message))
    return false;
  if (!mojo::Equals(this->submessages, other_struct.submessages))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace app_management

namespace mojo {


template <>
struct  StructTraits<::app_management::mojom::App::DataView,
                                         ::app_management::mojom::AppPtr> {
  static bool IsNull(const ::app_management::mojom::AppPtr& input) { return !input; }
  static void SetToNull(::app_management::mojom::AppPtr* output) { output->reset(); }

  static const decltype(::app_management::mojom::App::id)& id(
      const ::app_management::mojom::AppPtr& input) {
    return input->id;
  }

  static decltype(::app_management::mojom::App::type) type(
      const ::app_management::mojom::AppPtr& input) {
    return input->type;
  }

  static const decltype(::app_management::mojom::App::title)& title(
      const ::app_management::mojom::AppPtr& input) {
    return input->title;
  }

  static const decltype(::app_management::mojom::App::description)& description(
      const ::app_management::mojom::AppPtr& input) {
    return input->description;
  }

  static decltype(::app_management::mojom::App::is_pinned) is_pinned(
      const ::app_management::mojom::AppPtr& input) {
    return input->is_pinned;
  }

  static const decltype(::app_management::mojom::App::version)& version(
      const ::app_management::mojom::AppPtr& input) {
    return input->version;
  }

  static const decltype(::app_management::mojom::App::size)& size(
      const ::app_management::mojom::AppPtr& input) {
    return input->size;
  }

  static const decltype(::app_management::mojom::App::permissions)& permissions(
      const ::app_management::mojom::AppPtr& input) {
    return input->permissions;
  }

  static decltype(::app_management::mojom::App::install_source) install_source(
      const ::app_management::mojom::AppPtr& input) {
    return input->install_source;
  }

  static bool Read(::app_management::mojom::App::DataView input, ::app_management::mojom::AppPtr* output);
};


template <>
struct  StructTraits<::app_management::mojom::ExtensionAppPermissionMessage::DataView,
                                         ::app_management::mojom::ExtensionAppPermissionMessagePtr> {
  static bool IsNull(const ::app_management::mojom::ExtensionAppPermissionMessagePtr& input) { return !input; }
  static void SetToNull(::app_management::mojom::ExtensionAppPermissionMessagePtr* output) { output->reset(); }

  static const decltype(::app_management::mojom::ExtensionAppPermissionMessage::message)& message(
      const ::app_management::mojom::ExtensionAppPermissionMessagePtr& input) {
    return input->message;
  }

  static const decltype(::app_management::mojom::ExtensionAppPermissionMessage::submessages)& submessages(
      const ::app_management::mojom::ExtensionAppPermissionMessagePtr& input) {
    return input->submessages;
  }

  static bool Read(::app_management::mojom::ExtensionAppPermissionMessage::DataView input, ::app_management::mojom::ExtensionAppPermissionMessagePtr* output);
};

}  // namespace mojo

#endif  // CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_H_