// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_UI_WEBUI_SNIPPETS_INTERNALS_SNIPPETS_INTERNALS_MOJOM_TEST_UTILS_H_
#define CHROME_BROWSER_UI_WEBUI_SNIPPETS_INTERNALS_SNIPPETS_INTERNALS_MOJOM_TEST_UTILS_H_

#include "chrome/browser/ui/webui/snippets_internals/snippets_internals.mojom.h"


namespace snippets_internals {
namespace mojom {


class  PageHandlerFactoryInterceptorForTesting : public PageHandlerFactory {
  virtual PageHandlerFactory* GetForwardingInterface() = 0;
  void CreatePageHandler(PagePtr page, CreatePageHandlerCallback callback) override;
};
class  PageHandlerFactoryAsyncWaiter {
 public:
  explicit PageHandlerFactoryAsyncWaiter(PageHandlerFactory* proxy);
  ~PageHandlerFactoryAsyncWaiter();
  void CreatePageHandler(
      PagePtr page, PageHandlerPtr* out_handler);

 private:
  PageHandlerFactory* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageHandlerFactoryAsyncWaiter);
};


class  PageHandlerInterceptorForTesting : public PageHandler {
  virtual PageHandler* GetForwardingInterface() = 0;
  void GetGeneralProperties(GetGeneralPropertiesCallback callback) override;
  void GetUserClassifierProperties(GetUserClassifierPropertiesCallback callback) override;
  void ClearUserClassifierProperties() override;
  void GetCategoryRankerProperties(GetCategoryRankerPropertiesCallback callback) override;
  void ReloadSuggestions() override;
  void ClearCachedSuggestions() override;
  void GetRemoteContentSuggestionsProperties(GetRemoteContentSuggestionsPropertiesCallback callback) override;
  void FetchSuggestionsInBackground(int64_t delaySeconds, FetchSuggestionsInBackgroundCallback callback) override;
  void GetLastJson(GetLastJsonCallback callback) override;
  void GetSuggestionsByCategory(GetSuggestionsByCategoryCallback callback) override;
  void ClearDismissedSuggestions(int64_t category) override;
};
class  PageHandlerAsyncWaiter {
 public:
  explicit PageHandlerAsyncWaiter(PageHandler* proxy);
  ~PageHandlerAsyncWaiter();
  void GetGeneralProperties(
      base::flat_map<std::string, std::string>* out_properties);
  void GetUserClassifierProperties(
      base::flat_map<std::string, std::string>* out_properties);
  void GetCategoryRankerProperties(
      base::flat_map<std::string, std::string>* out_properties);
  void GetRemoteContentSuggestionsProperties(
      base::flat_map<std::string, std::string>* out_properties);
  void FetchSuggestionsInBackground(
      int64_t delaySeconds);
  void GetLastJson(
      std::string* out_json);
  void GetSuggestionsByCategory(
      std::vector<SuggestionCategoryPtr>* out_categories);

 private:
  PageHandler* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageHandlerAsyncWaiter);
};


class  PageInterceptorForTesting : public Page {
  virtual Page* GetForwardingInterface() = 0;
  void OnSuggestionsChanged() override;
};
class  PageAsyncWaiter {
 public:
  explicit PageAsyncWaiter(Page* proxy);
  ~PageAsyncWaiter();

 private:
  Page* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageAsyncWaiter);
};




}  // namespace mojom
}  // namespace snippets_internals

#endif  // CHROME_BROWSER_UI_WEBUI_SNIPPETS_INTERNALS_SNIPPETS_INTERNALS_MOJOM_TEST_UTILS_H_