// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_COMMON_CHROME_RENDER_FRAME_MOJOM_H_
#define CHROME_COMMON_CHROME_RENDER_FRAME_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/common/chrome_render_frame.mojom-shared.h"
#include "chrome/common/chrome_render_frame.mojom-forward.h"
#include "mojo/public/mojom/base/string16.mojom-forward.h"
#include "third_party/blink/public/mojom/window_features/window_features.mojom-forward.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-forward.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "chrome/common/web_application_info.h"
#include "content/public/common/browser_controls_state.h"




namespace chrome {
namespace mojom {

class ChromeRenderFrameProxy;

template <typename ImplRefTraits>
class ChromeRenderFrameStub;

class ChromeRenderFrameRequestValidator;
class ChromeRenderFrameResponseValidator;


class  ChromeRenderFrame
    : public ChromeRenderFrameInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = ChromeRenderFrameInterfaceBase;
  using Proxy_ = ChromeRenderFrameProxy;

  template <typename ImplRefTraits>
  using Stub_ = ChromeRenderFrameStub<ImplRefTraits>;

  using RequestValidator_ = ChromeRenderFrameRequestValidator;
  using ResponseValidator_ = ChromeRenderFrameResponseValidator;
  enum MethodMinVersions : uint32_t {
    kSetWindowFeaturesMinVersion = 0,
    kRequestReloadImageForContextNodeMinVersion = 0,
    kRequestThumbnailForContextNodeMinVersion = 0,
    kExecuteWebUIJavaScriptMinVersion = 0,
    kSetClientSidePhishingDetectionMinVersion = 0,
    kGetWebApplicationInfoMinVersion = 0,
    kUpdateBrowserControlsStateMinVersion = 0,
  };
  virtual ~ChromeRenderFrame() {}

  
  virtual void SetWindowFeatures(::blink::mojom::WindowFeaturesPtr window_features) = 0;

  
  virtual void RequestReloadImageForContextNode() = 0;


  using RequestThumbnailForContextNodeCallback = base::OnceCallback<void(const std::vector<uint8_t>&, const gfx::Size&)>;
  
  virtual void RequestThumbnailForContextNode(int32_t thumbnail_min_area_pixels, const gfx::Size& thumbnail_max_size_pixels, ImageFormat image_format, RequestThumbnailForContextNodeCallback callback) = 0;

  
  virtual void ExecuteWebUIJavaScript(const base::string16& javascript) = 0;

  
  virtual void SetClientSidePhishingDetection(bool enable_phishing_detection) = 0;


  using GetWebApplicationInfoCallback = base::OnceCallback<void(const ::WebApplicationInfo&)>;
  
  virtual void GetWebApplicationInfo(GetWebApplicationInfoCallback callback) = 0;

  
  virtual void UpdateBrowserControlsState(content::BrowserControlsState constraints, content::BrowserControlsState current, bool animate) = 0;
};

class  ChromeRenderFrameProxy
    : public ChromeRenderFrame {
 public:
  using InterfaceType = ChromeRenderFrame;

  explicit ChromeRenderFrameProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetWindowFeatures(::blink::mojom::WindowFeaturesPtr window_features) final;
  void RequestReloadImageForContextNode() final;
  void RequestThumbnailForContextNode(int32_t thumbnail_min_area_pixels, const gfx::Size& thumbnail_max_size_pixels, ImageFormat image_format, RequestThumbnailForContextNodeCallback callback) final;
  void ExecuteWebUIJavaScript(const base::string16& javascript) final;
  void SetClientSidePhishingDetection(bool enable_phishing_detection) final;
  void GetWebApplicationInfo(GetWebApplicationInfoCallback callback) final;
  void UpdateBrowserControlsState(content::BrowserControlsState constraints, content::BrowserControlsState current, bool animate) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  ChromeRenderFrameStubDispatch {
 public:
  static bool Accept(ChromeRenderFrame* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ChromeRenderFrame* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ChromeRenderFrame>>
class ChromeRenderFrameStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ChromeRenderFrameStub() {}
  ~ChromeRenderFrameStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ChromeRenderFrameStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ChromeRenderFrameStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ChromeRenderFrameRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ChromeRenderFrameResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};







}  // namespace mojom
}  // namespace chrome

namespace mojo {

}  // namespace mojo

#endif  // CHROME_COMMON_CHROME_RENDER_FRAME_MOJOM_H_