// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_COMMON_PLUGIN_MOJOM_TEST_UTILS_H_
#define CHROME_COMMON_PLUGIN_MOJOM_TEST_UTILS_H_

#include "chrome/common/plugin.mojom.h"


namespace chrome {
namespace mojom {


class  PluginHostInterceptorForTesting : public PluginHost {
  virtual PluginHost* GetForwardingInterface() = 0;
  void ShowFlashPermissionBubble() override;
  void CouldNotLoadPlugin(const base::FilePath& file_path) override;
  void BlockedOutdatedPlugin(PluginRendererPtr plugin_renderer, const std::string& group_id) override;
  void BlockedComponentUpdatedPlugin(PluginRendererPtr plugin_renderer, const std::string& group_id) override;
};
class  PluginHostAsyncWaiter {
 public:
  explicit PluginHostAsyncWaiter(PluginHost* proxy);
  ~PluginHostAsyncWaiter();

 private:
  PluginHost* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PluginHostAsyncWaiter);
};


class  PluginAuthHostInterceptorForTesting : public PluginAuthHost {
  virtual PluginAuthHost* GetForwardingInterface() = 0;
  void BlockedUnauthorizedPlugin(const base::string16& name, const std::string& group_id) override;
};
class  PluginAuthHostAsyncWaiter {
 public:
  explicit PluginAuthHostAsyncWaiter(PluginAuthHost* proxy);
  ~PluginAuthHostAsyncWaiter();

 private:
  PluginAuthHost* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PluginAuthHostAsyncWaiter);
};


class  PluginInfoHostInterceptorForTesting : public PluginInfoHost {
  virtual PluginInfoHost* GetForwardingInterface() = 0;
  void GetPluginInfo(int32_t render_frame_id, const GURL& url, const url::Origin& origin, const std::string& mime_type, GetPluginInfoCallback callback) override;
};
class  PluginInfoHostAsyncWaiter {
 public:
  explicit PluginInfoHostAsyncWaiter(PluginInfoHost* proxy);
  ~PluginInfoHostAsyncWaiter();
  void GetPluginInfo(
      int32_t render_frame_id, const GURL& url, const url::Origin& origin, const std::string& mime_type, PluginInfoPtr* out_plugin_info);

 private:
  PluginInfoHost* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PluginInfoHostAsyncWaiter);
};


class  PluginRendererInterceptorForTesting : public PluginRenderer {
  virtual PluginRenderer* GetForwardingInterface() = 0;
  void FinishedDownloading() override;
  void UpdateSuccess() override;
  void UpdateFailure() override;
  void UpdateDownloading() override;
};
class  PluginRendererAsyncWaiter {
 public:
  explicit PluginRendererAsyncWaiter(PluginRenderer* proxy);
  ~PluginRendererAsyncWaiter();

 private:
  PluginRenderer* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PluginRendererAsyncWaiter);
};




}  // namespace mojom
}  // namespace chrome

#endif  // CHROME_COMMON_PLUGIN_MOJOM_TEST_UTILS_H_