// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "chrome/common/renderer_configuration.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/hash/md5_constexpr.h"
#include "base/logging.h"
#include "base/run_loop.h"
#include "base/task/common/task_annotator.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "chrome/common/renderer_configuration.mojom-params-data.h"
#include "chrome/common/renderer_configuration.mojom-shared-message-ids.h"

#include "chrome/common/renderer_configuration.mojom-import-headers.h"


#ifndef CHROME_COMMON_RENDERER_CONFIGURATION_MOJOM_JUMBO_H_
#define CHROME_COMMON_RENDERER_CONFIGURATION_MOJOM_JUMBO_H_
#include "components/content_settings/core/common/content_settings_struct_traits.h"
#include "mojo/public/cpp/base/values_mojom_traits.h"
#endif
namespace chrome {
namespace mojom {
DynamicParams::DynamicParams()
    : force_safe_search(true),
      youtube_restrict(0),
      allowed_domains_for_apps(),
      variation_ids_header() {}

DynamicParams::DynamicParams(
    bool force_safe_search_in,
    int32_t youtube_restrict_in,
    const std::string& allowed_domains_for_apps_in,
    const std::string& variation_ids_header_in)
    : force_safe_search(std::move(force_safe_search_in)),
      youtube_restrict(std::move(youtube_restrict_in)),
      allowed_domains_for_apps(std::move(allowed_domains_for_apps_in)),
      variation_ids_header(std::move(variation_ids_header_in)) {}

DynamicParams::~DynamicParams() = default;
size_t DynamicParams::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->force_safe_search);
  seed = mojo::internal::Hash(seed, this->youtube_restrict);
  seed = mojo::internal::Hash(seed, this->allowed_domains_for_apps);
  seed = mojo::internal::Hash(seed, this->variation_ids_header);
  return seed;
}

bool DynamicParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char ChromeOSListener::Name_[] = "chrome.mojom.ChromeOSListener";

ChromeOSListenerProxy::ChromeOSListenerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

// static
bool ChromeOSListenerStubDispatch::Accept(
    ChromeOSListener* impl,
    mojo::Message* message) {
  return false;
}

// static
bool ChromeOSListenerStubDispatch::AcceptWithResponder(
    ChromeOSListener* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  return false;
}

bool ChromeOSListenerRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ChromeOSListener RequestValidator");

  switch (message->header()->name) {
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

const char RendererConfiguration::Name_[] = "chrome.mojom.RendererConfiguration";

RendererConfigurationProxy::RendererConfigurationProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void RendererConfigurationProxy::SetInitialConfiguration(
    bool in_is_incognito_process, ChromeOSListenerRequest in_chromeos_listener) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetInitialConfiguration");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kRendererConfiguration_SetInitialConfiguration_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::chrome::mojom::internal::RendererConfiguration_SetInitialConfiguration_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->is_incognito_process = in_is_incognito_process;
  mojo::internal::Serialize<::chrome::mojom::ChromeOSListenerRequestDataView>(
      in_chromeos_listener, &params->chromeos_listener, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(RendererConfiguration::Name_);
  message.set_method_name("SetInitialConfiguration");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererConfigurationProxy::SetConfiguration(
    DynamicParamsPtr in_params) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetConfiguration");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kRendererConfiguration_SetConfiguration_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::chrome::mojom::internal::RendererConfiguration_SetConfiguration_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->params)::BaseType::BufferWriter
      params_writer;
  mojo::internal::Serialize<::chrome::mojom::DynamicParamsDataView>(
      in_params, buffer, &params_writer, &serialization_context);
  params->params.Set(
      params_writer.is_null() ? nullptr : params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in RendererConfiguration.SetConfiguration request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(RendererConfiguration::Name_);
  message.set_method_name("SetConfiguration");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererConfigurationProxy::SetContentSettingRules(
    const ::RendererContentSettingRules& in_rules) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetContentSettingRules");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kRendererConfiguration_SetContentSettingRules_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::chrome::mojom::internal::RendererConfiguration_SetContentSettingRules_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->rules)::BaseType::BufferWriter
      rules_writer;
  mojo::internal::Serialize<::content_settings::mojom::RendererContentSettingRulesDataView>(
      in_rules, buffer, &rules_writer, &serialization_context);
  params->rules.Set(
      rules_writer.is_null() ? nullptr : rules_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->rules.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null rules in RendererConfiguration.SetContentSettingRules request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(RendererConfiguration::Name_);
  message.set_method_name("SetContentSettingRules");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void RendererConfigurationProxy::SetFieldTrialGroup(
    const std::string& in_trial_name, const std::string& in_group_name) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetFieldTrialGroup");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kRendererConfiguration_SetFieldTrialGroup_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::chrome::mojom::internal::RendererConfiguration_SetFieldTrialGroup_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->trial_name)::BaseType::BufferWriter
      trial_name_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_trial_name, buffer, &trial_name_writer, &serialization_context);
  params->trial_name.Set(
      trial_name_writer.is_null() ? nullptr : trial_name_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->trial_name.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null trial_name in RendererConfiguration.SetFieldTrialGroup request");
  typename decltype(params->group_name)::BaseType::BufferWriter
      group_name_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_group_name, buffer, &group_name_writer, &serialization_context);
  params->group_name.Set(
      group_name_writer.is_null() ? nullptr : group_name_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->group_name.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null group_name in RendererConfiguration.SetFieldTrialGroup request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(RendererConfiguration::Name_);
  message.set_method_name("SetFieldTrialGroup");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool RendererConfigurationStubDispatch::Accept(
    RendererConfiguration* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kRendererConfiguration_SetInitialConfiguration_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetInitialConfiguration",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetInitialConfiguration");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::RendererConfiguration_SetInitialConfiguration_Params_Data* params =
          reinterpret_cast<internal::RendererConfiguration_SetInitialConfiguration_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_is_incognito_process{};
      ChromeOSListenerRequest p_chromeos_listener{};
      RendererConfiguration_SetInitialConfiguration_ParamsDataView input_data_view(params, &serialization_context);
      
      p_is_incognito_process = input_data_view.is_incognito_process();
      p_chromeos_listener =
          input_data_view.TakeChromeosListener<decltype(p_chromeos_listener)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            RendererConfiguration::Name_, 0, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->SetInitialConfiguration(
std::move(p_is_incognito_process), 
std::move(p_chromeos_listener));
      return true;
    }
    case internal::kRendererConfiguration_SetConfiguration_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetConfiguration",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetConfiguration");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::RendererConfiguration_SetConfiguration_Params_Data* params =
          reinterpret_cast<internal::RendererConfiguration_SetConfiguration_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      DynamicParamsPtr p_params{};
      RendererConfiguration_SetConfiguration_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadParams(&p_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            RendererConfiguration::Name_, 1, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->SetConfiguration(
std::move(p_params));
      return true;
    }
    case internal::kRendererConfiguration_SetContentSettingRules_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetContentSettingRules",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetContentSettingRules");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::RendererConfiguration_SetContentSettingRules_Params_Data* params =
          reinterpret_cast<internal::RendererConfiguration_SetContentSettingRules_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::RendererContentSettingRules p_rules{};
      RendererConfiguration_SetContentSettingRules_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadRules(&p_rules))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            RendererConfiguration::Name_, 2, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->SetContentSettingRules(
std::move(p_rules));
      return true;
    }
    case internal::kRendererConfiguration_SetFieldTrialGroup_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
      TRACE_EVENT1(
          "mojom",
          "(Impl)<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetFieldTrialGroup",
          "message", message->name());
#endif
      static constexpr uint32_t kMessageHash = base::MD5Hash32Constexpr(
              "(Impl)<class 'jinja2::utils::Namespace'>::RendererConfiguration::SetFieldTrialGroup");
      base::TaskAnnotator::ScopedSetIpcHash scoped_ipc_hash(kMessageHash);
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::RendererConfiguration_SetFieldTrialGroup_Params_Data* params =
          reinterpret_cast<internal::RendererConfiguration_SetFieldTrialGroup_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_trial_name{};
      std::string p_group_name{};
      RendererConfiguration_SetFieldTrialGroup_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadTrialName(&p_trial_name))
        success = false;
      if (!input_data_view.ReadGroupName(&p_group_name))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            RendererConfiguration::Name_, 3, false);
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->SetFieldTrialGroup(
std::move(p_trial_name), 
std::move(p_group_name));
      return true;
    }
  }
  return false;
}

// static
bool RendererConfigurationStubDispatch::AcceptWithResponder(
    RendererConfiguration* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kRendererConfiguration_SetInitialConfiguration_Name: {
      break;
    }
    case internal::kRendererConfiguration_SetConfiguration_Name: {
      break;
    }
    case internal::kRendererConfiguration_SetContentSettingRules_Name: {
      break;
    }
    case internal::kRendererConfiguration_SetFieldTrialGroup_Name: {
      break;
    }
  }
  return false;
}

bool RendererConfigurationRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "RendererConfiguration RequestValidator");

  switch (message->header()->name) {
    case internal::kRendererConfiguration_SetInitialConfiguration_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::RendererConfiguration_SetInitialConfiguration_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRendererConfiguration_SetConfiguration_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::RendererConfiguration_SetConfiguration_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRendererConfiguration_SetContentSettingRules_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::RendererConfiguration_SetContentSettingRules_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kRendererConfiguration_SetFieldTrialGroup_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::RendererConfiguration_SetFieldTrialGroup_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace chrome

namespace mojo {


// static
bool StructTraits<::chrome::mojom::DynamicParams::DataView, ::chrome::mojom::DynamicParamsPtr>::Read(
    ::chrome::mojom::DynamicParams::DataView input,
    ::chrome::mojom::DynamicParamsPtr* output) {
  bool success = true;
  ::chrome::mojom::DynamicParamsPtr result(::chrome::mojom::DynamicParams::New());
  
      result->force_safe_search = input.force_safe_search();
      result->youtube_restrict = input.youtube_restrict();
      if (!input.ReadAllowedDomainsForApps(&result->allowed_domains_for_apps))
        success = false;
      if (!input.ReadVariationIdsHeader(&result->variation_ids_header))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif