// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_H_
#define CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/services/app_service/public/mojom/app_service.mojom-shared.h"
#include "chrome/services/app_service/public/mojom/app_service.mojom-forward.h"
#include "chrome/services/app_service/public/mojom/types.mojom-forward.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace apps {
namespace mojom {

class AppServiceProxy;

template <typename ImplRefTraits>
class AppServiceStub;

class AppServiceRequestValidator;
class AppServiceResponseValidator;


class  AppService
    : public AppServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = AppServiceInterfaceBase;
  using Proxy_ = AppServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = AppServiceStub<ImplRefTraits>;

  using RequestValidator_ = AppServiceRequestValidator;
  using ResponseValidator_ = AppServiceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRegisterPublisherMinVersion = 0,
    kRegisterSubscriberMinVersion = 0,
    kLoadIconMinVersion = 0,
    kLaunchMinVersion = 0,
    kSetPermissionMinVersion = 0,
    kUninstallMinVersion = 0,
    kOpenNativeSettingsMinVersion = 0,
  };
  virtual ~AppService() {}

  
  virtual void RegisterPublisher(PublisherPtr publisher, ::apps::mojom::AppType app_type) = 0;

  
  virtual void RegisterSubscriber(SubscriberPtr subscriber, ConnectOptionsPtr opts) = 0;


  using LoadIconCallback = base::OnceCallback<void(::apps::mojom::IconValuePtr)>;
  
  virtual void LoadIcon(::apps::mojom::AppType app_type, const std::string& app_id, ::apps::mojom::IconKeyPtr icon_key, ::apps::mojom::IconCompression icon_compression, int32_t size_hint_in_dip, bool allow_placeholder_icon, LoadIconCallback callback) = 0;

  
  virtual void Launch(::apps::mojom::AppType app_type, const std::string& app_id, int32_t event_flags, ::apps::mojom::LaunchSource launch_source, int64_t display_id) = 0;

  
  virtual void SetPermission(::apps::mojom::AppType app_type, const std::string& app_id, ::apps::mojom::PermissionPtr permission) = 0;

  
  virtual void Uninstall(::apps::mojom::AppType app_type, const std::string& app_id) = 0;

  
  virtual void OpenNativeSettings(::apps::mojom::AppType app_type, const std::string& app_id) = 0;
};

class PublisherProxy;

template <typename ImplRefTraits>
class PublisherStub;

class PublisherRequestValidator;
class PublisherResponseValidator;


class  Publisher
    : public PublisherInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = PublisherInterfaceBase;
  using Proxy_ = PublisherProxy;

  template <typename ImplRefTraits>
  using Stub_ = PublisherStub<ImplRefTraits>;

  using RequestValidator_ = PublisherRequestValidator;
  using ResponseValidator_ = PublisherResponseValidator;
  enum MethodMinVersions : uint32_t {
    kConnectMinVersion = 0,
    kLoadIconMinVersion = 0,
    kLaunchMinVersion = 0,
    kSetPermissionMinVersion = 0,
    kUninstallMinVersion = 0,
    kOpenNativeSettingsMinVersion = 0,
  };
  virtual ~Publisher() {}

  
  virtual void Connect(SubscriberPtr subscriber, ConnectOptionsPtr opts) = 0;


  using LoadIconCallback = base::OnceCallback<void(::apps::mojom::IconValuePtr)>;
  
  virtual void LoadIcon(const std::string& app_id, ::apps::mojom::IconKeyPtr icon_key, ::apps::mojom::IconCompression icon_compression, int32_t size_hint_in_dip, bool allow_placeholder_icon, LoadIconCallback callback) = 0;

  
  virtual void Launch(const std::string& app_id, int32_t event_flags, ::apps::mojom::LaunchSource launch_source, int64_t display_id) = 0;

  
  virtual void SetPermission(const std::string& app_id, ::apps::mojom::PermissionPtr permission) = 0;

  
  virtual void Uninstall(const std::string& app_id) = 0;

  
  virtual void OpenNativeSettings(const std::string& app_id) = 0;
};

class SubscriberProxy;

template <typename ImplRefTraits>
class SubscriberStub;

class SubscriberRequestValidator;


class  Subscriber
    : public SubscriberInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = SubscriberInterfaceBase;
  using Proxy_ = SubscriberProxy;

  template <typename ImplRefTraits>
  using Stub_ = SubscriberStub<ImplRefTraits>;

  using RequestValidator_ = SubscriberRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnAppsMinVersion = 0,
    kCloneMinVersion = 0,
  };
  virtual ~Subscriber() {}

  
  virtual void OnApps(std::vector<::apps::mojom::AppPtr> deltas) = 0;

  
  virtual void Clone(SubscriberRequest request) = 0;
};

class  AppServiceProxy
    : public AppService {
 public:
  using InterfaceType = AppService;

  explicit AppServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void RegisterPublisher(PublisherPtr publisher, ::apps::mojom::AppType app_type) final;
  void RegisterSubscriber(SubscriberPtr subscriber, ConnectOptionsPtr opts) final;
  void LoadIcon(::apps::mojom::AppType app_type, const std::string& app_id, ::apps::mojom::IconKeyPtr icon_key, ::apps::mojom::IconCompression icon_compression, int32_t size_hint_in_dip, bool allow_placeholder_icon, LoadIconCallback callback) final;
  void Launch(::apps::mojom::AppType app_type, const std::string& app_id, int32_t event_flags, ::apps::mojom::LaunchSource launch_source, int64_t display_id) final;
  void SetPermission(::apps::mojom::AppType app_type, const std::string& app_id, ::apps::mojom::PermissionPtr permission) final;
  void Uninstall(::apps::mojom::AppType app_type, const std::string& app_id) final;
  void OpenNativeSettings(::apps::mojom::AppType app_type, const std::string& app_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  PublisherProxy
    : public Publisher {
 public:
  using InterfaceType = Publisher;

  explicit PublisherProxy(mojo::MessageReceiverWithResponder* receiver);
  void Connect(SubscriberPtr subscriber, ConnectOptionsPtr opts) final;
  void LoadIcon(const std::string& app_id, ::apps::mojom::IconKeyPtr icon_key, ::apps::mojom::IconCompression icon_compression, int32_t size_hint_in_dip, bool allow_placeholder_icon, LoadIconCallback callback) final;
  void Launch(const std::string& app_id, int32_t event_flags, ::apps::mojom::LaunchSource launch_source, int64_t display_id) final;
  void SetPermission(const std::string& app_id, ::apps::mojom::PermissionPtr permission) final;
  void Uninstall(const std::string& app_id) final;
  void OpenNativeSettings(const std::string& app_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  SubscriberProxy
    : public Subscriber {
 public:
  using InterfaceType = Subscriber;

  explicit SubscriberProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnApps(std::vector<::apps::mojom::AppPtr> deltas) final;
  void Clone(SubscriberRequest request) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  AppServiceStubDispatch {
 public:
  static bool Accept(AppService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AppService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AppService>>
class AppServiceStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AppServiceStub() {}
  ~AppServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AppServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AppServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PublisherStubDispatch {
 public:
  static bool Accept(Publisher* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Publisher* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Publisher>>
class PublisherStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PublisherStub() {}
  ~PublisherStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PublisherStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PublisherStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  SubscriberStubDispatch {
 public:
  static bool Accept(Subscriber* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Subscriber* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Subscriber>>
class SubscriberStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  SubscriberStub() {}
  ~SubscriberStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return SubscriberStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return SubscriberStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  AppServiceRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PublisherRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  SubscriberRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  AppServiceResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PublisherResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class  ConnectOptions {
 public:
  using DataView = ConnectOptionsDataView;
  using Data_ = internal::ConnectOptions_Data;

  template <typename... Args>
  static ConnectOptionsPtr New(Args&&... args) {
    return ConnectOptionsPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static ConnectOptionsPtr From(const U& u) {
    return mojo::TypeConverter<ConnectOptionsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ConnectOptions>::Convert(*this);
  }


  ConnectOptions();

  ~ConnectOptions();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ConnectOptionsPtr>
  ConnectOptionsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ConnectOptions>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ConnectOptions::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ConnectOptions::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ConnectOptions_UnserializedMessageContext<
            UserType, ConnectOptions::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ConnectOptions::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ConnectOptions::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ConnectOptions_UnserializedMessageContext<
            UserType, ConnectOptions::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ConnectOptions::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }


 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
ConnectOptionsPtr ConnectOptions::Clone() const {
  return New(
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ConnectOptions>::value>::type*>
bool ConnectOptions::Equals(const T& other_struct) const {
  return true;
}


}  // namespace mojom
}  // namespace apps

namespace mojo {


template <>
struct  StructTraits<::apps::mojom::ConnectOptions::DataView,
                                         ::apps::mojom::ConnectOptionsPtr> {
  static bool IsNull(const ::apps::mojom::ConnectOptionsPtr& input) { return !input; }
  static void SetToNull(::apps::mojom::ConnectOptionsPtr* output) { output->reset(); }

  static bool Read(::apps::mojom::ConnectOptions::DataView input, ::apps::mojom::ConnectOptionsPtr* output);
};

}  // namespace mojo

#endif  // CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_H_