// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "chrome/services/app_service/public/mojom/types.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "base/stl_util.h"  // for base::size()
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"

#include "chrome/services/app_service/public/mojom/types.mojom-params-data.h"
namespace apps {
namespace mojom {

std::ostream& operator<<(std::ostream& os, AppType value) {
  switch(value) {
    case AppType::kUnknown:
      return os << "AppType::kUnknown";
    case AppType::kArc:
      return os << "AppType::kArc";
    case AppType::kBuiltIn:
      return os << "AppType::kBuiltIn";
    case AppType::kCrostini:
      return os << "AppType::kCrostini";
    case AppType::kExtension:
      return os << "AppType::kExtension";
    case AppType::kWeb:
      return os << "AppType::kWeb";
    default:
      return os << "Unknown AppType value: " << static_cast<int32_t>(value);
  }
}

std::ostream& operator<<(std::ostream& os, Readiness value) {
  switch(value) {
    case Readiness::kUnknown:
      return os << "Readiness::kUnknown";
    case Readiness::kReady:
      return os << "Readiness::kReady";
    case Readiness::kDisabledByBlacklist:
      return os << "Readiness::kDisabledByBlacklist";
    case Readiness::kDisabledByPolicy:
      return os << "Readiness::kDisabledByPolicy";
    case Readiness::kDisabledByUser:
      return os << "Readiness::kDisabledByUser";
    case Readiness::kTerminated:
      return os << "Readiness::kTerminated";
    case Readiness::kUninstalledByUser:
      return os << "Readiness::kUninstalledByUser";
    default:
      return os << "Unknown Readiness value: " << static_cast<int32_t>(value);
  }
}

std::ostream& operator<<(std::ostream& os, InstallSource value) {
  switch(value) {
    case InstallSource::kUnknown:
      return os << "InstallSource::kUnknown";
    case InstallSource::kSystem:
      return os << "InstallSource::kSystem";
    case InstallSource::kPolicy:
      return os << "InstallSource::kPolicy";
    case InstallSource::kOem:
      return os << "InstallSource::kOem";
    case InstallSource::kDefault:
      return os << "InstallSource::kDefault";
    case InstallSource::kSync:
      return os << "InstallSource::kSync";
    case InstallSource::kUser:
      return os << "InstallSource::kUser";
    default:
      return os << "Unknown InstallSource value: " << static_cast<int32_t>(value);
  }
}

std::ostream& operator<<(std::ostream& os, OptionalBool value) {
  switch(value) {
    case OptionalBool::kUnknown:
      return os << "OptionalBool::kUnknown";
    case OptionalBool::kFalse:
      return os << "OptionalBool::kFalse";
    case OptionalBool::kTrue:
      return os << "OptionalBool::kTrue";
    default:
      return os << "Unknown OptionalBool value: " << static_cast<int32_t>(value);
  }
}

std::ostream& operator<<(std::ostream& os, IconCompression value) {
  switch(value) {
    case IconCompression::kUnknown:
      return os << "IconCompression::kUnknown";
    case IconCompression::kUncompressed:
      return os << "IconCompression::kUncompressed";
    case IconCompression::kCompressed:
      return os << "IconCompression::kCompressed";
    default:
      return os << "Unknown IconCompression value: " << static_cast<int32_t>(value);
  }
}

std::ostream& operator<<(std::ostream& os, LaunchSource value) {
  switch(value) {
    case LaunchSource::kUnknown:
      return os << "LaunchSource::kUnknown";
    case LaunchSource::kFromAppListGrid:
      return os << "LaunchSource::kFromAppListGrid";
    case LaunchSource::kFromAppListGridContextMenu:
      return os << "LaunchSource::kFromAppListGridContextMenu";
    case LaunchSource::kFromAppListQuery:
      return os << "LaunchSource::kFromAppListQuery";
    case LaunchSource::kFromAppListQueryContextMenu:
      return os << "LaunchSource::kFromAppListQueryContextMenu";
    case LaunchSource::kFromAppListRecommendation:
      return os << "LaunchSource::kFromAppListRecommendation";
    case LaunchSource::kFromKioskNextHome:
      return os << "LaunchSource::kFromKioskNextHome";
    case LaunchSource::kFromParentalControls:
      return os << "LaunchSource::kFromParentalControls";
    default:
      return os << "Unknown LaunchSource value: " << static_cast<int32_t>(value);
  }
}

std::ostream& operator<<(std::ostream& os, TriState value) {
  switch(value) {
    case TriState::kAllow:
      return os << "TriState::kAllow";
    case TriState::kBlock:
      return os << "TriState::kBlock";
    case TriState::kAsk:
      return os << "TriState::kAsk";
    default:
      return os << "Unknown TriState value: " << static_cast<int32_t>(value);
  }
}

std::ostream& operator<<(std::ostream& os, PermissionValueType value) {
  switch(value) {
    case PermissionValueType::kBool:
      return os << "PermissionValueType::kBool";
    case PermissionValueType::kTriState:
      return os << "PermissionValueType::kTriState";
    default:
      return os << "Unknown PermissionValueType value: " << static_cast<int32_t>(value);
  }
}

namespace internal {


// static
bool App_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const App_Data* object = static_cast<const App_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 120 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::apps::mojom::internal::AppType_Data
        ::Validate(object->app_type, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->app_id, 2, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams app_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->app_id, validation_context,
                                         &app_id_validate_params)) {
    return false;
  }


  if (!::apps::mojom::internal::Readiness_Data
        ::Validate(object->readiness, validation_context))
    return false;

  const mojo::internal::ContainerValidateParams name_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->name, validation_context,
                                         &name_validate_params)) {
    return false;
  }

  const mojo::internal::ContainerValidateParams short_name_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->short_name, validation_context,
                                         &short_name_validate_params)) {
    return false;
  }

  const mojo::internal::ContainerValidateParams description_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->description, validation_context,
                                         &description_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->additional_search_terms, 7, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams additional_search_terms_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  if (!mojo::internal::ValidateContainer(object->additional_search_terms, validation_context,
                                         &additional_search_terms_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->icon_key, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->last_launch_time, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->install_time, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->permissions, 11, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams permissions_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->permissions, validation_context,
                                         &permissions_validate_params)) {
    return false;
  }


  if (!::apps::mojom::internal::InstallSource_Data
        ::Validate(object->install_source, validation_context))
    return false;


  if (!::apps::mojom::internal::OptionalBool_Data
        ::Validate(object->is_platform_app, validation_context))
    return false;


  if (!::apps::mojom::internal::OptionalBool_Data
        ::Validate(object->recommendable, validation_context))
    return false;


  if (!::apps::mojom::internal::OptionalBool_Data
        ::Validate(object->searchable, validation_context))
    return false;


  if (!::apps::mojom::internal::OptionalBool_Data
        ::Validate(object->show_in_launcher, validation_context))
    return false;


  if (!::apps::mojom::internal::OptionalBool_Data
        ::Validate(object->show_in_search, validation_context))
    return false;


  if (!::apps::mojom::internal::OptionalBool_Data
        ::Validate(object->show_in_management, validation_context))
    return false;

  return true;
}

App_Data::App_Data()
    : header_({sizeof(*this), 0}) {}


// static
bool Permission_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Permission_Data* object = static_cast<const Permission_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::apps::mojom::internal::PermissionValueType_Data
        ::Validate(object->value_type, validation_context))
    return false;

  return true;
}

Permission_Data::Permission_Data()
    : header_({sizeof(*this), 0}) {}


// static
bool IconKey_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const IconKey_Data* object = static_cast<const IconKey_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

IconKey_Data::IconKey_Data()
    : header_({sizeof(*this), 0}) {}


// static
bool IconValue_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const IconValue_Data* object = static_cast<const IconValue_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::apps::mojom::internal::IconCompression_Data
        ::Validate(object->icon_compression, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->uncompressed, validation_context))
    return false;

  const mojo::internal::ContainerValidateParams compressed_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->compressed, validation_context,
                                         &compressed_validate_params)) {
    return false;
  }

  return true;
}

IconValue_Data::IconValue_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace apps

#if defined(_MSC_VER)
#pragma warning(pop)
#endif