// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_
#define CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/services/app_service/public/mojom/types.mojom-shared-internal.h"
#include "mojo/public/mojom/base/time.mojom-shared.h"
#include "ui/gfx/image/mojo/image.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace apps {
namespace mojom {
class AppDataView;

class PermissionDataView;

class IconKeyDataView;

class IconValueDataView;



}  // namespace mojom
}  // namespace apps

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::apps::mojom::AppDataView> {
  using Data = ::apps::mojom::internal::App_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::apps::mojom::PermissionDataView> {
  using Data = ::apps::mojom::internal::Permission_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::apps::mojom::IconKeyDataView> {
  using Data = ::apps::mojom::internal::IconKey_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::apps::mojom::IconValueDataView> {
  using Data = ::apps::mojom::internal::IconValue_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace apps {
namespace mojom {


enum class AppType : int32_t {
  
  kUnknown = 0,
  
  kArc,
  
  kBuiltIn,
  
  kCrostini,
  
  kExtension,
  
  kWeb,
  kMinValue = 0,
  kMaxValue = 5,
};

 std::ostream& operator<<(std::ostream& os, AppType value);
inline bool IsKnownEnumValue(AppType value) {
  return internal::AppType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}


enum class Readiness : int32_t {
  
  kUnknown = 0,
  
  kReady,
  
  kDisabledByBlacklist,
  
  kDisabledByPolicy,
  
  kDisabledByUser,
  
  kTerminated,
  
  kUninstalledByUser,
  kMinValue = 0,
  kMaxValue = 6,
};

 std::ostream& operator<<(std::ostream& os, Readiness value);
inline bool IsKnownEnumValue(Readiness value) {
  return internal::Readiness_Data::IsKnownValue(
      static_cast<int32_t>(value));
}


enum class InstallSource : int32_t {
  
  kUnknown = 0,
  
  kSystem,
  
  kPolicy,
  
  kOem,
  
  kDefault,
  
  kSync,
  
  kUser,
  kMinValue = 0,
  kMaxValue = 6,
};

 std::ostream& operator<<(std::ostream& os, InstallSource value);
inline bool IsKnownEnumValue(InstallSource value) {
  return internal::InstallSource_Data::IsKnownValue(
      static_cast<int32_t>(value));
}


enum class OptionalBool : int32_t {
  
  kUnknown = 0,
  
  kFalse,
  
  kTrue,
  kMinValue = 0,
  kMaxValue = 2,
};

 std::ostream& operator<<(std::ostream& os, OptionalBool value);
inline bool IsKnownEnumValue(OptionalBool value) {
  return internal::OptionalBool_Data::IsKnownValue(
      static_cast<int32_t>(value));
}


enum class IconCompression : int32_t {
  
  kUnknown,
  
  kUncompressed,
  
  kCompressed,
  kMinValue = 0,
  kMaxValue = 2,
};

 std::ostream& operator<<(std::ostream& os, IconCompression value);
inline bool IsKnownEnumValue(IconCompression value) {
  return internal::IconCompression_Data::IsKnownValue(
      static_cast<int32_t>(value));
}


enum class LaunchSource : int32_t {
  
  kUnknown,
  
  kFromAppListGrid,
  
  kFromAppListGridContextMenu,
  
  kFromAppListQuery,
  
  kFromAppListQueryContextMenu,
  
  kFromAppListRecommendation,
  
  kFromKioskNextHome,
  
  kFromParentalControls,
  kMinValue = 0,
  kMaxValue = 7,
};

 std::ostream& operator<<(std::ostream& os, LaunchSource value);
inline bool IsKnownEnumValue(LaunchSource value) {
  return internal::LaunchSource_Data::IsKnownValue(
      static_cast<int32_t>(value));
}


enum class TriState : int32_t {
  
  kAllow,
  
  kBlock,
  
  kAsk,
  kMinValue = 0,
  kMaxValue = 2,
};

 std::ostream& operator<<(std::ostream& os, TriState value);
inline bool IsKnownEnumValue(TriState value) {
  return internal::TriState_Data::IsKnownValue(
      static_cast<int32_t>(value));
}


enum class PermissionValueType : int32_t {
  
  kBool,
  
  kTriState,
  kMinValue = 0,
  kMaxValue = 1,
};

 std::ostream& operator<<(std::ostream& os, PermissionValueType value);
inline bool IsKnownEnumValue(PermissionValueType value) {
  return internal::PermissionValueType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class AppDataView {
 public:
  AppDataView() {}

  AppDataView(
      internal::App_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppType(UserType* output) const {
    auto data_value = data_->app_type;
    return mojo::internal::Deserialize<::apps::mojom::AppType>(
        data_value, output);
  }

  AppType app_type() const {
    return static_cast<AppType>(data_->app_type);
  }
  inline void GetAppIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppId(UserType* output) {
    auto* pointer = data_->app_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadReadiness(UserType* output) const {
    auto data_value = data_->readiness;
    return mojo::internal::Deserialize<::apps::mojom::Readiness>(
        data_value, output);
  }

  Readiness readiness() const {
    return static_cast<Readiness>(data_->readiness);
  }
  inline void GetNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetShortNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadShortName(UserType* output) {
    auto* pointer = data_->short_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetDescriptionDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDescription(UserType* output) {
    auto* pointer = data_->description.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetAdditionalSearchTermsDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAdditionalSearchTerms(UserType* output) {
    auto* pointer = data_->additional_search_terms.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
  inline void GetIconKeyDataView(
      IconKeyDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconKey(UserType* output) {
    auto* pointer = data_->icon_key.Get();
    return mojo::internal::Deserialize<::apps::mojom::IconKeyDataView>(
        pointer, output, context_);
  }
  inline void GetLastLaunchTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLastLaunchTime(UserType* output) {
    auto* pointer = data_->last_launch_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetInstallTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstallTime(UserType* output) {
    auto* pointer = data_->install_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetPermissionsDataView(
      mojo::ArrayDataView<PermissionDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPermissions(UserType* output) {
    auto* pointer = data_->permissions.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::apps::mojom::PermissionDataView>>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstallSource(UserType* output) const {
    auto data_value = data_->install_source;
    return mojo::internal::Deserialize<::apps::mojom::InstallSource>(
        data_value, output);
  }

  InstallSource install_source() const {
    return static_cast<InstallSource>(data_->install_source);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIsPlatformApp(UserType* output) const {
    auto data_value = data_->is_platform_app;
    return mojo::internal::Deserialize<::apps::mojom::OptionalBool>(
        data_value, output);
  }

  OptionalBool is_platform_app() const {
    return static_cast<OptionalBool>(data_->is_platform_app);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRecommendable(UserType* output) const {
    auto data_value = data_->recommendable;
    return mojo::internal::Deserialize<::apps::mojom::OptionalBool>(
        data_value, output);
  }

  OptionalBool recommendable() const {
    return static_cast<OptionalBool>(data_->recommendable);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSearchable(UserType* output) const {
    auto data_value = data_->searchable;
    return mojo::internal::Deserialize<::apps::mojom::OptionalBool>(
        data_value, output);
  }

  OptionalBool searchable() const {
    return static_cast<OptionalBool>(data_->searchable);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadShowInLauncher(UserType* output) const {
    auto data_value = data_->show_in_launcher;
    return mojo::internal::Deserialize<::apps::mojom::OptionalBool>(
        data_value, output);
  }

  OptionalBool show_in_launcher() const {
    return static_cast<OptionalBool>(data_->show_in_launcher);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadShowInSearch(UserType* output) const {
    auto data_value = data_->show_in_search;
    return mojo::internal::Deserialize<::apps::mojom::OptionalBool>(
        data_value, output);
  }

  OptionalBool show_in_search() const {
    return static_cast<OptionalBool>(data_->show_in_search);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadShowInManagement(UserType* output) const {
    auto data_value = data_->show_in_management;
    return mojo::internal::Deserialize<::apps::mojom::OptionalBool>(
        data_value, output);
  }

  OptionalBool show_in_management() const {
    return static_cast<OptionalBool>(data_->show_in_management);
  }
 private:
  internal::App_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PermissionDataView {
 public:
  PermissionDataView() {}

  PermissionDataView(
      internal::Permission_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t permission_id() const {
    return data_->permission_id;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadValueType(UserType* output) const {
    auto data_value = data_->value_type;
    return mojo::internal::Deserialize<::apps::mojom::PermissionValueType>(
        data_value, output);
  }

  PermissionValueType value_type() const {
    return static_cast<PermissionValueType>(data_->value_type);
  }
  uint32_t value() const {
    return data_->value;
  }
 private:
  internal::Permission_Data* data_ = nullptr;
};

class IconKeyDataView {
 public:
  IconKeyDataView() {}

  IconKeyDataView(
      internal::IconKey_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint64_t timeline() const {
    return data_->timeline;
  }
  int32_t resource_id() const {
    return data_->resource_id;
  }
  uint32_t icon_effects() const {
    return data_->icon_effects;
  }
 private:
  internal::IconKey_Data* data_ = nullptr;
};

class IconValueDataView {
 public:
  IconValueDataView() {}

  IconValueDataView(
      internal::IconValue_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconCompression(UserType* output) const {
    auto data_value = data_->icon_compression;
    return mojo::internal::Deserialize<::apps::mojom::IconCompression>(
        data_value, output);
  }

  IconCompression icon_compression() const {
    return static_cast<IconCompression>(data_->icon_compression);
  }
  inline void GetUncompressedDataView(
      ::gfx::mojom::ImageSkiaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUncompressed(UserType* output) {
    auto* pointer = data_->uncompressed.Get();
    return mojo::internal::Deserialize<::gfx::mojom::ImageSkiaDataView>(
        pointer, output, context_);
  }
  inline void GetCompressedDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCompressed(UserType* output) {
    auto* pointer = data_->compressed.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  bool is_placeholder_icon() const {
    return data_->is_placeholder_icon;
  }
 private:
  internal::IconValue_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace apps

namespace std {

template <>
struct hash<::apps::mojom::AppType>
    : public mojo::internal::EnumHashImpl<::apps::mojom::AppType> {};

template <>
struct hash<::apps::mojom::Readiness>
    : public mojo::internal::EnumHashImpl<::apps::mojom::Readiness> {};

template <>
struct hash<::apps::mojom::InstallSource>
    : public mojo::internal::EnumHashImpl<::apps::mojom::InstallSource> {};

template <>
struct hash<::apps::mojom::OptionalBool>
    : public mojo::internal::EnumHashImpl<::apps::mojom::OptionalBool> {};

template <>
struct hash<::apps::mojom::IconCompression>
    : public mojo::internal::EnumHashImpl<::apps::mojom::IconCompression> {};

template <>
struct hash<::apps::mojom::LaunchSource>
    : public mojo::internal::EnumHashImpl<::apps::mojom::LaunchSource> {};

template <>
struct hash<::apps::mojom::TriState>
    : public mojo::internal::EnumHashImpl<::apps::mojom::TriState> {};

template <>
struct hash<::apps::mojom::PermissionValueType>
    : public mojo::internal::EnumHashImpl<::apps::mojom::PermissionValueType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::apps::mojom::AppType, ::apps::mojom::AppType> {
  static ::apps::mojom::AppType ToMojom(::apps::mojom::AppType input) { return input; }
  static bool FromMojom(::apps::mojom::AppType input, ::apps::mojom::AppType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::AppType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::AppType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::AppType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::Readiness, ::apps::mojom::Readiness> {
  static ::apps::mojom::Readiness ToMojom(::apps::mojom::Readiness input) { return input; }
  static bool FromMojom(::apps::mojom::Readiness input, ::apps::mojom::Readiness* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::Readiness, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::Readiness, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::Readiness>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::InstallSource, ::apps::mojom::InstallSource> {
  static ::apps::mojom::InstallSource ToMojom(::apps::mojom::InstallSource input) { return input; }
  static bool FromMojom(::apps::mojom::InstallSource input, ::apps::mojom::InstallSource* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::InstallSource, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::InstallSource, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::InstallSource>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::OptionalBool, ::apps::mojom::OptionalBool> {
  static ::apps::mojom::OptionalBool ToMojom(::apps::mojom::OptionalBool input) { return input; }
  static bool FromMojom(::apps::mojom::OptionalBool input, ::apps::mojom::OptionalBool* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::OptionalBool, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::OptionalBool, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::OptionalBool>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::IconCompression, ::apps::mojom::IconCompression> {
  static ::apps::mojom::IconCompression ToMojom(::apps::mojom::IconCompression input) { return input; }
  static bool FromMojom(::apps::mojom::IconCompression input, ::apps::mojom::IconCompression* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::IconCompression, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::IconCompression, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::IconCompression>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::LaunchSource, ::apps::mojom::LaunchSource> {
  static ::apps::mojom::LaunchSource ToMojom(::apps::mojom::LaunchSource input) { return input; }
  static bool FromMojom(::apps::mojom::LaunchSource input, ::apps::mojom::LaunchSource* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::LaunchSource, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::LaunchSource, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::LaunchSource>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::TriState, ::apps::mojom::TriState> {
  static ::apps::mojom::TriState ToMojom(::apps::mojom::TriState input) { return input; }
  static bool FromMojom(::apps::mojom::TriState input, ::apps::mojom::TriState* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::TriState, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::TriState, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::TriState>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::PermissionValueType, ::apps::mojom::PermissionValueType> {
  static ::apps::mojom::PermissionValueType ToMojom(::apps::mojom::PermissionValueType input) { return input; }
  static bool FromMojom(::apps::mojom::PermissionValueType input, ::apps::mojom::PermissionValueType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::PermissionValueType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::PermissionValueType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::PermissionValueType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::AppDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::AppDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::App_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::apps::mojom::AppType>(
        Traits::app_type(input), &(*output)->app_type);
    decltype(Traits::app_id(input)) in_app_id = Traits::app_id(input);
    typename decltype((*output)->app_id)::BaseType::BufferWriter
        app_id_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_app_id, buffer, &app_id_writer, context);
    (*output)->app_id.Set(
        app_id_writer.is_null() ? nullptr : app_id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->app_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null app_id in App struct");
    mojo::internal::Serialize<::apps::mojom::Readiness>(
        Traits::readiness(input), &(*output)->readiness);
    decltype(Traits::name(input)) in_name = Traits::name(input);
    typename decltype((*output)->name)::BaseType::BufferWriter
        name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_name, buffer, &name_writer, context);
    (*output)->name.Set(
        name_writer.is_null() ? nullptr : name_writer.data());
    decltype(Traits::short_name(input)) in_short_name = Traits::short_name(input);
    typename decltype((*output)->short_name)::BaseType::BufferWriter
        short_name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_short_name, buffer, &short_name_writer, context);
    (*output)->short_name.Set(
        short_name_writer.is_null() ? nullptr : short_name_writer.data());
    decltype(Traits::description(input)) in_description = Traits::description(input);
    typename decltype((*output)->description)::BaseType::BufferWriter
        description_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_description, buffer, &description_writer, context);
    (*output)->description.Set(
        description_writer.is_null() ? nullptr : description_writer.data());
    decltype(Traits::additional_search_terms(input)) in_additional_search_terms = Traits::additional_search_terms(input);
    typename decltype((*output)->additional_search_terms)::BaseType::BufferWriter
        additional_search_terms_writer;
    const mojo::internal::ContainerValidateParams additional_search_terms_validate_params(
        0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_additional_search_terms, buffer, &additional_search_terms_writer, &additional_search_terms_validate_params,
        context);
    (*output)->additional_search_terms.Set(
        additional_search_terms_writer.is_null() ? nullptr : additional_search_terms_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->additional_search_terms.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null additional_search_terms in App struct");
    decltype(Traits::icon_key(input)) in_icon_key = Traits::icon_key(input);
    typename decltype((*output)->icon_key)::BaseType::BufferWriter
        icon_key_writer;
    mojo::internal::Serialize<::apps::mojom::IconKeyDataView>(
        in_icon_key, buffer, &icon_key_writer, context);
    (*output)->icon_key.Set(
        icon_key_writer.is_null() ? nullptr : icon_key_writer.data());
    decltype(Traits::last_launch_time(input)) in_last_launch_time = Traits::last_launch_time(input);
    typename decltype((*output)->last_launch_time)::BaseType::BufferWriter
        last_launch_time_writer;
    mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
        in_last_launch_time, buffer, &last_launch_time_writer, context);
    (*output)->last_launch_time.Set(
        last_launch_time_writer.is_null() ? nullptr : last_launch_time_writer.data());
    decltype(Traits::install_time(input)) in_install_time = Traits::install_time(input);
    typename decltype((*output)->install_time)::BaseType::BufferWriter
        install_time_writer;
    mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
        in_install_time, buffer, &install_time_writer, context);
    (*output)->install_time.Set(
        install_time_writer.is_null() ? nullptr : install_time_writer.data());
    decltype(Traits::permissions(input)) in_permissions = Traits::permissions(input);
    typename decltype((*output)->permissions)::BaseType::BufferWriter
        permissions_writer;
    const mojo::internal::ContainerValidateParams permissions_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::apps::mojom::PermissionDataView>>(
        in_permissions, buffer, &permissions_writer, &permissions_validate_params,
        context);
    (*output)->permissions.Set(
        permissions_writer.is_null() ? nullptr : permissions_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->permissions.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null permissions in App struct");
    mojo::internal::Serialize<::apps::mojom::InstallSource>(
        Traits::install_source(input), &(*output)->install_source);
    mojo::internal::Serialize<::apps::mojom::OptionalBool>(
        Traits::is_platform_app(input), &(*output)->is_platform_app);
    mojo::internal::Serialize<::apps::mojom::OptionalBool>(
        Traits::recommendable(input), &(*output)->recommendable);
    mojo::internal::Serialize<::apps::mojom::OptionalBool>(
        Traits::searchable(input), &(*output)->searchable);
    mojo::internal::Serialize<::apps::mojom::OptionalBool>(
        Traits::show_in_launcher(input), &(*output)->show_in_launcher);
    mojo::internal::Serialize<::apps::mojom::OptionalBool>(
        Traits::show_in_search(input), &(*output)->show_in_search);
    mojo::internal::Serialize<::apps::mojom::OptionalBool>(
        Traits::show_in_management(input), &(*output)->show_in_management);
  }

  static bool Deserialize(::apps::mojom::internal::App_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::AppDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::PermissionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::PermissionDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::Permission_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    (*output)->permission_id = Traits::permission_id(input);
    mojo::internal::Serialize<::apps::mojom::PermissionValueType>(
        Traits::value_type(input), &(*output)->value_type);
    (*output)->value = Traits::value(input);
  }

  static bool Deserialize(::apps::mojom::internal::Permission_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::PermissionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::IconKeyDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::IconKeyDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::IconKey_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    (*output)->timeline = Traits::timeline(input);
    (*output)->resource_id = Traits::resource_id(input);
    (*output)->icon_effects = Traits::icon_effects(input);
  }

  static bool Deserialize(::apps::mojom::internal::IconKey_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::IconKeyDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::IconValueDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::IconValueDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::IconValue_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::apps::mojom::IconCompression>(
        Traits::icon_compression(input), &(*output)->icon_compression);
    decltype(Traits::uncompressed(input)) in_uncompressed = Traits::uncompressed(input);
    typename decltype((*output)->uncompressed)::BaseType::BufferWriter
        uncompressed_writer;
    mojo::internal::Serialize<::gfx::mojom::ImageSkiaDataView>(
        in_uncompressed, buffer, &uncompressed_writer, context);
    (*output)->uncompressed.Set(
        uncompressed_writer.is_null() ? nullptr : uncompressed_writer.data());
    decltype(Traits::compressed(input)) in_compressed = Traits::compressed(input);
    typename decltype((*output)->compressed)::BaseType::BufferWriter
        compressed_writer;
    const mojo::internal::ContainerValidateParams compressed_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_compressed, buffer, &compressed_writer, &compressed_validate_params,
        context);
    (*output)->compressed.Set(
        compressed_writer.is_null() ? nullptr : compressed_writer.data());
    (*output)->is_placeholder_icon = Traits::is_placeholder_icon(input);
  }

  static bool Deserialize(::apps::mojom::internal::IconValue_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::IconValueDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace apps {
namespace mojom {

inline void AppDataView::GetAppIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->app_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetShortNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->short_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetDescriptionDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->description.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetAdditionalSearchTermsDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->additional_search_terms.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}
inline void AppDataView::GetIconKeyDataView(
    IconKeyDataView* output) {
  auto pointer = data_->icon_key.Get();
  *output = IconKeyDataView(pointer, context_);
}
inline void AppDataView::GetLastLaunchTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->last_launch_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void AppDataView::GetInstallTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->install_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void AppDataView::GetPermissionsDataView(
    mojo::ArrayDataView<PermissionDataView>* output) {
  auto pointer = data_->permissions.Get();
  *output = mojo::ArrayDataView<PermissionDataView>(pointer, context_);
}






inline void IconValueDataView::GetUncompressedDataView(
    ::gfx::mojom::ImageSkiaDataView* output) {
  auto pointer = data_->uncompressed.Get();
  *output = ::gfx::mojom::ImageSkiaDataView(pointer, context_);
}
inline void IconValueDataView::GetCompressedDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->compressed.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}



}  // namespace mojom
}  // namespace apps

#endif  // CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_