// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "chrome/services/app_service/public/mojom/types.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/hash/md5_constexpr.h"
#include "base/logging.h"
#include "base/run_loop.h"
#include "base/task/common/task_annotator.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "chrome/services/app_service/public/mojom/types.mojom-params-data.h"
#include "chrome/services/app_service/public/mojom/types.mojom-shared-message-ids.h"

#include "chrome/services/app_service/public/mojom/types.mojom-import-headers.h"


#ifndef CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_JUMBO_H_
#define CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_JUMBO_H_
#include "mojo/public/cpp/base/big_buffer_mojom_traits.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "skia/public/interfaces/bitmap_skbitmap_struct_traits.h"
#include "skia/public/interfaces/image_info_struct_traits.h"
#include "ui/gfx/image/mojo/image_skia_struct_traits.h"
#endif
namespace apps {
namespace mojom {
constexpr uint64_t IconKey::kDoesNotChangeOverTime;
constexpr int32_t IconKey::kInvalidResourceId;
App::App()
    : app_type(),
      app_id(),
      readiness(),
      name(),
      short_name(),
      description(),
      additional_search_terms(),
      icon_key(),
      last_launch_time(),
      install_time(),
      permissions(),
      install_source(),
      is_platform_app(),
      recommendable(),
      searchable(),
      show_in_launcher(),
      show_in_search(),
      show_in_management() {}

App::App(
    AppType app_type_in,
    const std::string& app_id_in,
    Readiness readiness_in,
    const base::Optional<std::string>& name_in,
    const base::Optional<std::string>& short_name_in,
    const base::Optional<std::string>& description_in,
    const std::vector<std::string>& additional_search_terms_in,
    IconKeyPtr icon_key_in,
    base::Optional<base::Time> last_launch_time_in,
    base::Optional<base::Time> install_time_in,
    std::vector<PermissionPtr> permissions_in,
    InstallSource install_source_in,
    OptionalBool is_platform_app_in,
    OptionalBool recommendable_in,
    OptionalBool searchable_in,
    OptionalBool show_in_launcher_in,
    OptionalBool show_in_search_in,
    OptionalBool show_in_management_in)
    : app_type(std::move(app_type_in)),
      app_id(std::move(app_id_in)),
      readiness(std::move(readiness_in)),
      name(std::move(name_in)),
      short_name(std::move(short_name_in)),
      description(std::move(description_in)),
      additional_search_terms(std::move(additional_search_terms_in)),
      icon_key(std::move(icon_key_in)),
      last_launch_time(std::move(last_launch_time_in)),
      install_time(std::move(install_time_in)),
      permissions(std::move(permissions_in)),
      install_source(std::move(install_source_in)),
      is_platform_app(std::move(is_platform_app_in)),
      recommendable(std::move(recommendable_in)),
      searchable(std::move(searchable_in)),
      show_in_launcher(std::move(show_in_launcher_in)),
      show_in_search(std::move(show_in_search_in)),
      show_in_management(std::move(show_in_management_in)) {}

App::~App() = default;

bool App::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
Permission::Permission()
    : permission_id(),
      value_type(),
      value() {}

Permission::Permission(
    uint32_t permission_id_in,
    PermissionValueType value_type_in,
    uint32_t value_in)
    : permission_id(std::move(permission_id_in)),
      value_type(std::move(value_type_in)),
      value(std::move(value_in)) {}

Permission::~Permission() = default;
size_t Permission::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->permission_id);
  seed = mojo::internal::Hash(seed, this->value_type);
  seed = mojo::internal::Hash(seed, this->value);
  return seed;
}

bool Permission::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
IconKey::IconKey()
    : timeline(),
      resource_id(),
      icon_effects() {}

IconKey::IconKey(
    uint64_t timeline_in,
    int32_t resource_id_in,
    uint32_t icon_effects_in)
    : timeline(std::move(timeline_in)),
      resource_id(std::move(resource_id_in)),
      icon_effects(std::move(icon_effects_in)) {}

IconKey::~IconKey() = default;
size_t IconKey::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->timeline);
  seed = mojo::internal::Hash(seed, this->resource_id);
  seed = mojo::internal::Hash(seed, this->icon_effects);
  return seed;
}

bool IconKey::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
IconValue::IconValue()
    : icon_compression(),
      uncompressed(),
      compressed(),
      is_placeholder_icon() {}

IconValue::IconValue(
    IconCompression icon_compression_in,
    const gfx::ImageSkia& uncompressed_in,
    const base::Optional<std::vector<uint8_t>>& compressed_in,
    bool is_placeholder_icon_in)
    : icon_compression(std::move(icon_compression_in)),
      uncompressed(std::move(uncompressed_in)),
      compressed(std::move(compressed_in)),
      is_placeholder_icon(std::move(is_placeholder_icon_in)) {}

IconValue::~IconValue() = default;

bool IconValue::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
}  // namespace mojom
}  // namespace apps

namespace mojo {


// static
bool StructTraits<::apps::mojom::App::DataView, ::apps::mojom::AppPtr>::Read(
    ::apps::mojom::App::DataView input,
    ::apps::mojom::AppPtr* output) {
  bool success = true;
  ::apps::mojom::AppPtr result(::apps::mojom::App::New());
  
      if (!input.ReadAppType(&result->app_type))
        success = false;
      if (!input.ReadAppId(&result->app_id))
        success = false;
      if (!input.ReadReadiness(&result->readiness))
        success = false;
      if (!input.ReadName(&result->name))
        success = false;
      if (!input.ReadShortName(&result->short_name))
        success = false;
      if (!input.ReadDescription(&result->description))
        success = false;
      if (!input.ReadAdditionalSearchTerms(&result->additional_search_terms))
        success = false;
      if (!input.ReadIconKey(&result->icon_key))
        success = false;
      if (!input.ReadLastLaunchTime(&result->last_launch_time))
        success = false;
      if (!input.ReadInstallTime(&result->install_time))
        success = false;
      if (!input.ReadPermissions(&result->permissions))
        success = false;
      if (!input.ReadInstallSource(&result->install_source))
        success = false;
      if (!input.ReadIsPlatformApp(&result->is_platform_app))
        success = false;
      if (!input.ReadRecommendable(&result->recommendable))
        success = false;
      if (!input.ReadSearchable(&result->searchable))
        success = false;
      if (!input.ReadShowInLauncher(&result->show_in_launcher))
        success = false;
      if (!input.ReadShowInSearch(&result->show_in_search))
        success = false;
      if (!input.ReadShowInManagement(&result->show_in_management))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::apps::mojom::Permission::DataView, ::apps::mojom::PermissionPtr>::Read(
    ::apps::mojom::Permission::DataView input,
    ::apps::mojom::PermissionPtr* output) {
  bool success = true;
  ::apps::mojom::PermissionPtr result(::apps::mojom::Permission::New());
  
      result->permission_id = input.permission_id();
      if (!input.ReadValueType(&result->value_type))
        success = false;
      result->value = input.value();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::apps::mojom::IconKey::DataView, ::apps::mojom::IconKeyPtr>::Read(
    ::apps::mojom::IconKey::DataView input,
    ::apps::mojom::IconKeyPtr* output) {
  bool success = true;
  ::apps::mojom::IconKeyPtr result(::apps::mojom::IconKey::New());
  
      result->timeline = input.timeline();
      result->resource_id = input.resource_id();
      result->icon_effects = input.icon_effects();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::apps::mojom::IconValue::DataView, ::apps::mojom::IconValuePtr>::Read(
    ::apps::mojom::IconValue::DataView input,
    ::apps::mojom::IconValuePtr* output) {
  bool success = true;
  ::apps::mojom::IconValuePtr result(::apps::mojom::IconValue::New());
  
      if (!input.ReadIconCompression(&result->icon_compression))
        success = false;
      if (!input.ReadUncompressed(&result->uncompressed))
        success = false;
      if (!input.ReadCompressed(&result->compressed))
        success = false;
      result->is_placeholder_icon = input.is_placeholder_icon();
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif