// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "chrome/services/printing/public/mojom/pdf_nup_converter.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "base/stl_util.h"  // for base::size()
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"

#include "chrome/services/printing/public/mojom/pdf_nup_converter.mojom-params-data.h"
namespace printing {
namespace mojom {

std::ostream& operator<<(std::ostream& os, PdfNupConverter_Status value) {
  switch(value) {
    case PdfNupConverter_Status::SUCCESS:
      return os << "PdfNupConverter_Status::SUCCESS";
    case PdfNupConverter_Status::CONVERSION_FAILURE:
      return os << "PdfNupConverter_Status::CONVERSION_FAILURE";
    case PdfNupConverter_Status::HANDLE_MAP_ERROR:
      return os << "PdfNupConverter_Status::HANDLE_MAP_ERROR";
    default:
      return os << "Unknown PdfNupConverter_Status value: " << static_cast<int32_t>(value);
  }
}

namespace internal {


// static
bool PdfNupConverter_NupPageConvert_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PdfNupConverter_NupPageConvert_Params_Data* object = static_cast<const PdfNupConverter_NupPageConvert_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->page_size, 2, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->page_size, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->printable_area, 3, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->printable_area, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->pdf_page_regions, 4, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams pdf_page_regions_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->pdf_page_regions, validation_context,
                                         &pdf_page_regions_validate_params)) {
    return false;
  }

  return true;
}

PdfNupConverter_NupPageConvert_Params_Data::PdfNupConverter_NupPageConvert_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
bool PdfNupConverter_NupPageConvert_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PdfNupConverter_NupPageConvert_ResponseParams_Data* object = static_cast<const PdfNupConverter_NupPageConvert_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::printing::mojom::internal::PdfNupConverter_Status_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->pdf_region, validation_context))
    return false;

  return true;
}

PdfNupConverter_NupPageConvert_ResponseParams_Data::PdfNupConverter_NupPageConvert_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
bool PdfNupConverter_NupDocumentConvert_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PdfNupConverter_NupDocumentConvert_Params_Data* object = static_cast<const PdfNupConverter_NupDocumentConvert_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->page_size, 2, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->page_size, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->printable_area, 3, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->printable_area, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->src_pdf_region, 4, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->src_pdf_region, validation_context))
    return false;

  return true;
}

PdfNupConverter_NupDocumentConvert_Params_Data::PdfNupConverter_NupDocumentConvert_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
bool PdfNupConverter_NupDocumentConvert_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PdfNupConverter_NupDocumentConvert_ResponseParams_Data* object = static_cast<const PdfNupConverter_NupDocumentConvert_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::printing::mojom::internal::PdfNupConverter_Status_Data
        ::Validate(object->status, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->pdf_region, validation_context))
    return false;

  return true;
}

PdfNupConverter_NupDocumentConvert_ResponseParams_Data::PdfNupConverter_NupDocumentConvert_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
bool PdfNupConverter_SetWebContentsURL_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PdfNupConverter_SetWebContentsURL_Params_Data* object = static_cast<const PdfNupConverter_SetWebContentsURL_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->url, 1, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->url, validation_context))
    return false;

  return true;
}

PdfNupConverter_SetWebContentsURL_Params_Data::PdfNupConverter_SetWebContentsURL_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace printing

#if defined(_MSC_VER)
#pragma warning(pop)
#endif