// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_SERVICES_PRINTING_PUBLIC_MOJOM_PDF_NUP_CONVERTER_MOJOM_H_
#define CHROME_SERVICES_PRINTING_PUBLIC_MOJOM_PDF_NUP_CONVERTER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/services/printing/public/mojom/pdf_nup_converter.mojom-shared.h"
#include "chrome/services/printing/public/mojom/pdf_nup_converter.mojom-forward.h"
#include "mojo/public/mojom/base/shared_memory.mojom-forward.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-forward.h"
#include "url/mojom/url.mojom-forward.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace printing {
namespace mojom {

class PdfNupConverterProxy;

template <typename ImplRefTraits>
class PdfNupConverterStub;

class PdfNupConverterRequestValidator;
class PdfNupConverterResponseValidator;


class  PdfNupConverter
    : public PdfNupConverterInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = PdfNupConverterInterfaceBase;
  using Proxy_ = PdfNupConverterProxy;

  template <typename ImplRefTraits>
  using Stub_ = PdfNupConverterStub<ImplRefTraits>;

  using RequestValidator_ = PdfNupConverterRequestValidator;
  using ResponseValidator_ = PdfNupConverterResponseValidator;
  enum MethodMinVersions : uint32_t {
    kNupPageConvertMinVersion = 0,
    kNupDocumentConvertMinVersion = 0,
    kSetWebContentsURLMinVersion = 0,
  };
  
  using Status = PdfNupConverter_Status;
  virtual ~PdfNupConverter() {}


  using NupPageConvertCallback = base::OnceCallback<void(PdfNupConverter::Status, base::ReadOnlySharedMemoryRegion)>;
  
  virtual void NupPageConvert(uint32_t pages_per_sheet, const gfx::Size& page_size, const gfx::Rect& printable_area, std::vector<base::ReadOnlySharedMemoryRegion> pdf_page_regions, NupPageConvertCallback callback) = 0;


  using NupDocumentConvertCallback = base::OnceCallback<void(PdfNupConverter::Status, base::ReadOnlySharedMemoryRegion)>;
  
  virtual void NupDocumentConvert(uint32_t pages_per_sheet, const gfx::Size& page_size, const gfx::Rect& printable_area, base::ReadOnlySharedMemoryRegion src_pdf_region, NupDocumentConvertCallback callback) = 0;

  
  virtual void SetWebContentsURL(const GURL& url) = 0;
};

class  PdfNupConverterProxy
    : public PdfNupConverter {
 public:
  using InterfaceType = PdfNupConverter;

  explicit PdfNupConverterProxy(mojo::MessageReceiverWithResponder* receiver);
  void NupPageConvert(uint32_t pages_per_sheet, const gfx::Size& page_size, const gfx::Rect& printable_area, std::vector<base::ReadOnlySharedMemoryRegion> pdf_page_regions, NupPageConvertCallback callback) final;
  void NupDocumentConvert(uint32_t pages_per_sheet, const gfx::Size& page_size, const gfx::Rect& printable_area, base::ReadOnlySharedMemoryRegion src_pdf_region, NupDocumentConvertCallback callback) final;
  void SetWebContentsURL(const GURL& url) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  PdfNupConverterStubDispatch {
 public:
  static bool Accept(PdfNupConverter* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PdfNupConverter* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PdfNupConverter>>
class PdfNupConverterStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PdfNupConverterStub() {}
  ~PdfNupConverterStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PdfNupConverterStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PdfNupConverterStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PdfNupConverterRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PdfNupConverterResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace printing

namespace mojo {

}  // namespace mojo

#endif  // CHROME_SERVICES_PRINTING_PUBLIC_MOJOM_PDF_NUP_CONVERTER_MOJOM_H_