// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_TEST_DATA_WEBUI_WEB_UI_TEST_MOJOM_H_
#define CHROME_TEST_DATA_WEBUI_WEB_UI_TEST_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/test/data/webui/web_ui_test.mojom-shared.h"
#include "chrome/test/data/webui/web_ui_test.mojom-forward.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace web_ui_test {
namespace mojom {

class TestRunnerProxy;

template <typename ImplRefTraits>
class TestRunnerStub;

class TestRunnerRequestValidator;


class  TestRunner
    : public TestRunnerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = TestRunnerInterfaceBase;
  using Proxy_ = TestRunnerProxy;

  template <typename ImplRefTraits>
  using Stub_ = TestRunnerStub<ImplRefTraits>;

  using RequestValidator_ = TestRunnerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kTestCompleteMinVersion = 0,
  };
  virtual ~TestRunner() {}

  
  virtual void TestComplete(const base::Optional<std::string>& failureMessage) = 0;
};

class  TestRunnerProxy
    : public TestRunner {
 public:
  using InterfaceType = TestRunner;

  explicit TestRunnerProxy(mojo::MessageReceiverWithResponder* receiver);
  void TestComplete(const base::Optional<std::string>& failureMessage) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  TestRunnerStubDispatch {
 public:
  static bool Accept(TestRunner* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      TestRunner* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<TestRunner>>
class TestRunnerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  TestRunnerStub() {}
  ~TestRunnerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TestRunnerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TestRunnerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  TestRunnerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace web_ui_test

namespace mojo {

}  // namespace mojo

#endif  // CHROME_TEST_DATA_WEBUI_WEB_UI_TEST_MOJOM_H_