// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_CONTENT_CAPTURE_COMMON_CONTENT_CAPTURE_DATA_MOJOM_H_
#define COMPONENTS_CONTENT_CAPTURE_COMMON_CONTENT_CAPTURE_DATA_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "components/content_capture/common/content_capture_data.mojom-shared.h"
#include "components/content_capture/common/content_capture_data.mojom-forward.h"
#include "mojo/public/mojom/base/string16.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "components/content_capture/common/content_capture_data.h"




namespace content_capture {
namespace mojom {








class  ContentCaptureData {
 public:
  using DataView = ContentCaptureDataDataView;
  using Data_ = internal::ContentCaptureData_Data;

  template <typename... Args>
  static ContentCaptureDataPtr New(Args&&... args) {
    return ContentCaptureDataPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static ContentCaptureDataPtr From(const U& u) {
    return mojo::TypeConverter<ContentCaptureDataPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ContentCaptureData>::Convert(*this);
  }


  ContentCaptureData();

  ContentCaptureData(
      int64_t id,
      const base::string16& value,
      const gfx::Rect& bounds,
      const std::vector<content_capture::ContentCaptureData>& children);

  ~ContentCaptureData();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ContentCaptureDataPtr>
  ContentCaptureDataPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ContentCaptureData>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ContentCaptureData::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ContentCaptureData::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ContentCaptureData_UnserializedMessageContext<
            UserType, ContentCaptureData::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ContentCaptureData::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ContentCaptureData::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ContentCaptureData_UnserializedMessageContext<
            UserType, ContentCaptureData::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ContentCaptureData::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int64_t id;
  
  base::string16 value;
  
  gfx::Rect bounds;
  
  std::vector<content_capture::ContentCaptureData> children;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
ContentCaptureDataPtr ContentCaptureData::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(value),
      mojo::Clone(bounds),
      mojo::Clone(children)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ContentCaptureData>::value>::type*>
bool ContentCaptureData::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->value, other_struct.value))
    return false;
  if (!mojo::Equals(this->bounds, other_struct.bounds))
    return false;
  if (!mojo::Equals(this->children, other_struct.children))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content_capture

namespace mojo {


template <>
struct  StructTraits<::content_capture::mojom::ContentCaptureData::DataView,
                                         ::content_capture::mojom::ContentCaptureDataPtr> {
  static bool IsNull(const ::content_capture::mojom::ContentCaptureDataPtr& input) { return !input; }
  static void SetToNull(::content_capture::mojom::ContentCaptureDataPtr* output) { output->reset(); }

  static decltype(::content_capture::mojom::ContentCaptureData::id) id(
      const ::content_capture::mojom::ContentCaptureDataPtr& input) {
    return input->id;
  }

  static const decltype(::content_capture::mojom::ContentCaptureData::value)& value(
      const ::content_capture::mojom::ContentCaptureDataPtr& input) {
    return input->value;
  }

  static const decltype(::content_capture::mojom::ContentCaptureData::bounds)& bounds(
      const ::content_capture::mojom::ContentCaptureDataPtr& input) {
    return input->bounds;
  }

  static const decltype(::content_capture::mojom::ContentCaptureData::children)& children(
      const ::content_capture::mojom::ContentCaptureDataPtr& input) {
    return input->children;
  }

  static bool Read(::content_capture::mojom::ContentCaptureData::DataView input, ::content_capture::mojom::ContentCaptureDataPtr* output);
};

}  // namespace mojo

#endif  // COMPONENTS_CONTENT_CAPTURE_COMMON_CONTENT_CAPTURE_DATA_MOJOM_H_