// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_PDF_COMMON_PDF_MOJOM_TEST_UTILS_H_
#define COMPONENTS_PDF_COMMON_PDF_MOJOM_TEST_UTILS_H_

#include "components/pdf/common/pdf.mojom.h"


namespace pdf {
namespace mojom {


class  PdfListenerInterceptorForTesting : public PdfListener {
  virtual PdfListener* GetForwardingInterface() = 0;
  void SetCaretPosition(const gfx::PointF& position) override;
  void MoveRangeSelectionExtent(const gfx::PointF& extent) override;
  void SetSelectionBounds(const gfx::PointF& base, const gfx::PointF& extent) override;
};
class  PdfListenerAsyncWaiter {
 public:
  explicit PdfListenerAsyncWaiter(PdfListener* proxy);
  ~PdfListenerAsyncWaiter();

 private:
  PdfListener* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PdfListenerAsyncWaiter);
};


class  PdfServiceInterceptorForTesting : public PdfService {
  virtual PdfService* GetForwardingInterface() = 0;
  void SetListener(PdfListenerPtr client) override;
  void UpdateContentRestrictions(int32_t restrictions) override;
  void HasUnsupportedFeature() override;
  void SaveUrlAs(const GURL& url, ::blink::mojom::ReferrerPtr referrer) override;
  void SelectionChanged(const gfx::PointF& left, int32_t left_height, const gfx::PointF& right, int32_t right_height) override;
  void SetPluginCanSave(bool can_save) override;
};
class  PdfServiceAsyncWaiter {
 public:
  explicit PdfServiceAsyncWaiter(PdfService* proxy);
  ~PdfServiceAsyncWaiter();

 private:
  PdfService* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PdfServiceAsyncWaiter);
};




}  // namespace mojom
}  // namespace pdf

#endif  // COMPONENTS_PDF_COMMON_PDF_MOJOM_TEST_UTILS_H_