// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_IDENTITY_PUBLIC_MOJOM_CORE_ACCOUNT_INFO_MOJOM_SHARED_H_
#define SERVICES_IDENTITY_PUBLIC_MOJOM_CORE_ACCOUNT_INFO_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/identity/public/mojom/core_account_info.mojom-shared-internal.h"
#include "services/identity/public/mojom/core_account_id.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace identity {
namespace mojom {
class CoreAccountInfoDataView;



}  // namespace mojom
}  // namespace identity

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::identity::mojom::CoreAccountInfoDataView> {
  using Data = ::identity::mojom::internal::CoreAccountInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace identity {
namespace mojom {
class CoreAccountInfoDataView {
 public:
  CoreAccountInfoDataView() {}

  CoreAccountInfoDataView(
      internal::CoreAccountInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetAccountIdDataView(
      ::identity::mojom::CoreAccountIdDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAccountId(UserType* output) {
    auto* pointer = data_->account_id.Get();
    return mojo::internal::Deserialize<::identity::mojom::CoreAccountIdDataView>(
        pointer, output, context_);
  }
  inline void GetGaiaDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadGaia(UserType* output) {
    auto* pointer = data_->gaia.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetEmailDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEmail(UserType* output) {
    auto* pointer = data_->email.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::CoreAccountInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace identity

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::identity::mojom::CoreAccountInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::identity::mojom::CoreAccountInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::identity::mojom::internal::CoreAccountInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::account_id(input)) in_account_id = Traits::account_id(input);
    typename decltype((*output)->account_id)::BaseType::BufferWriter
        account_id_writer;
    mojo::internal::Serialize<::identity::mojom::CoreAccountIdDataView>(
        in_account_id, buffer, &account_id_writer, context);
    (*output)->account_id.Set(
        account_id_writer.is_null() ? nullptr : account_id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->account_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null account_id in CoreAccountInfo struct");
    decltype(Traits::gaia(input)) in_gaia = Traits::gaia(input);
    typename decltype((*output)->gaia)::BaseType::BufferWriter
        gaia_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_gaia, buffer, &gaia_writer, context);
    (*output)->gaia.Set(
        gaia_writer.is_null() ? nullptr : gaia_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->gaia.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null gaia in CoreAccountInfo struct");
    decltype(Traits::email(input)) in_email = Traits::email(input);
    typename decltype((*output)->email)::BaseType::BufferWriter
        email_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_email, buffer, &email_writer, context);
    (*output)->email.Set(
        email_writer.is_null() ? nullptr : email_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->email.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null email in CoreAccountInfo struct");
  }

  static bool Deserialize(::identity::mojom::internal::CoreAccountInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::identity::mojom::CoreAccountInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace identity {
namespace mojom {

inline void CoreAccountInfoDataView::GetAccountIdDataView(
    ::identity::mojom::CoreAccountIdDataView* output) {
  auto pointer = data_->account_id.Get();
  *output = ::identity::mojom::CoreAccountIdDataView(pointer, context_);
}
inline void CoreAccountInfoDataView::GetGaiaDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->gaia.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void CoreAccountInfoDataView::GetEmailDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->email.Get();
  *output = mojo::StringDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace identity

#endif  // SERVICES_IDENTITY_PUBLIC_MOJOM_CORE_ACCOUNT_INFO_MOJOM_SHARED_H_