// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_IDENTITY_PUBLIC_MOJOM_CORE_ACCOUNT_INFO_MOJOM_H_
#define SERVICES_IDENTITY_PUBLIC_MOJOM_CORE_ACCOUNT_INFO_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/identity/public/mojom/core_account_info.mojom-shared.h"
#include "services/identity/public/mojom/core_account_info.mojom-forward.h"
#include "services/identity/public/mojom/core_account_id.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "components/signin/core/browser/account_info.h"




namespace identity {
namespace mojom {








class  CoreAccountInfo {
 public:
  using DataView = CoreAccountInfoDataView;
  using Data_ = internal::CoreAccountInfo_Data;

  template <typename... Args>
  static CoreAccountInfoPtr New(Args&&... args) {
    return CoreAccountInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static CoreAccountInfoPtr From(const U& u) {
    return mojo::TypeConverter<CoreAccountInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CoreAccountInfo>::Convert(*this);
  }


  CoreAccountInfo();

  CoreAccountInfo(
      const ::CoreAccountId& account_id,
      const std::string& gaia,
      const std::string& email);

  ~CoreAccountInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CoreAccountInfoPtr>
  CoreAccountInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CoreAccountInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        CoreAccountInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        CoreAccountInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::CoreAccountInfo_UnserializedMessageContext<
            UserType, CoreAccountInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<CoreAccountInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return CoreAccountInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::CoreAccountInfo_UnserializedMessageContext<
            UserType, CoreAccountInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<CoreAccountInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  ::CoreAccountId account_id;
  
  std::string gaia;
  
  std::string email;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
CoreAccountInfoPtr CoreAccountInfo::Clone() const {
  return New(
      mojo::Clone(account_id),
      mojo::Clone(gaia),
      mojo::Clone(email)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CoreAccountInfo>::value>::type*>
bool CoreAccountInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->account_id, other_struct.account_id))
    return false;
  if (!mojo::Equals(this->gaia, other_struct.gaia))
    return false;
  if (!mojo::Equals(this->email, other_struct.email))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace identity

namespace mojo {


template <>
struct  StructTraits<::identity::mojom::CoreAccountInfo::DataView,
                                         ::identity::mojom::CoreAccountInfoPtr> {
  static bool IsNull(const ::identity::mojom::CoreAccountInfoPtr& input) { return !input; }
  static void SetToNull(::identity::mojom::CoreAccountInfoPtr* output) { output->reset(); }

  static const decltype(::identity::mojom::CoreAccountInfo::account_id)& account_id(
      const ::identity::mojom::CoreAccountInfoPtr& input) {
    return input->account_id;
  }

  static const decltype(::identity::mojom::CoreAccountInfo::gaia)& gaia(
      const ::identity::mojom::CoreAccountInfoPtr& input) {
    return input->gaia;
  }

  static const decltype(::identity::mojom::CoreAccountInfo::email)& email(
      const ::identity::mojom::CoreAccountInfoPtr& input) {
    return input->email;
  }

  static bool Read(::identity::mojom::CoreAccountInfo::DataView input, ::identity::mojom::CoreAccountInfoPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_IDENTITY_PUBLIC_MOJOM_CORE_ACCOUNT_INFO_MOJOM_H_