// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_PREFERENCES_PUBLIC_MOJOM_PREFERENCES_MOJOM_TEST_UTILS_H_
#define SERVICES_PREFERENCES_PUBLIC_MOJOM_PREFERENCES_MOJOM_TEST_UTILS_H_

#include "services/preferences/public/mojom/preferences.mojom.h"


namespace prefs {
namespace mojom {


class  PrefStoreObserverInterceptorForTesting : public PrefStoreObserver {
  virtual PrefStoreObserver* GetForwardingInterface() = 0;
  void OnPrefsChanged(std::vector<PrefUpdatePtr> updates) override;
  void OnInitializationCompleted(bool succeeded) override;
  void OnPrefChangeAck() override;
};
class  PrefStoreObserverAsyncWaiter {
 public:
  explicit PrefStoreObserverAsyncWaiter(PrefStoreObserver* proxy);
  ~PrefStoreObserverAsyncWaiter();

 private:
  PrefStoreObserver* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PrefStoreObserverAsyncWaiter);
};


class  PrefStoreConnectorInterceptorForTesting : public PrefStoreConnector {
  virtual PrefStoreConnector* GetForwardingInterface() = 0;
  void Connect(PrefRegistryPtr pref_registry, ConnectCallback callback) override;
};
class  PrefStoreConnectorAsyncWaiter {
 public:
  explicit PrefStoreConnectorAsyncWaiter(PrefStoreConnector* proxy);
  ~PrefStoreConnectorAsyncWaiter();
  void Connect(
      PrefRegistryPtr pref_registry, PersistentPrefStoreConnectionPtr* out_connection, IncognitoPersistentPrefStoreConnectionPtr* out_underlay, std::vector<PrefRegistrationPtr>* out_remote_defaults, base::flat_map<::PrefValueStore::PrefStoreType, PrefStoreConnectionPtr>* out_connections);

 private:
  PrefStoreConnector* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PrefStoreConnectorAsyncWaiter);
};


class  PersistentPrefStoreInterceptorForTesting : public PersistentPrefStore {
  virtual PersistentPrefStore* GetForwardingInterface() = 0;
  void SetValues(std::vector<PrefUpdatePtr> updates) override;
  void RequestValue(const std::string& key, const std::vector<std::string>& sub_pref_path) override;
  void CommitPendingWrite(CommitPendingWriteCallback callback) override;
  void SchedulePendingLossyWrites() override;
  void ClearMutableValues() override;
  void OnStoreDeletionFromDisk() override;
};
class  PersistentPrefStoreAsyncWaiter {
 public:
  explicit PersistentPrefStoreAsyncWaiter(PersistentPrefStore* proxy);
  ~PersistentPrefStoreAsyncWaiter();
  void CommitPendingWrite(
      );

 private:
  PersistentPrefStore* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PersistentPrefStoreAsyncWaiter);
};


class  ResetOnLoadObserverInterceptorForTesting : public ResetOnLoadObserver {
  virtual ResetOnLoadObserver* GetForwardingInterface() = 0;
  void OnResetOnLoad() override;
};
class  ResetOnLoadObserverAsyncWaiter {
 public:
  explicit ResetOnLoadObserverAsyncWaiter(ResetOnLoadObserver* proxy);
  ~ResetOnLoadObserverAsyncWaiter();

 private:
  ResetOnLoadObserver* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ResetOnLoadObserverAsyncWaiter);
};




}  // namespace mojom
}  // namespace prefs

#endif  // SERVICES_PREFERENCES_PUBLIC_MOJOM_PREFERENCES_MOJOM_TEST_UTILS_H_