// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_

#include <stddef.h>
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/platform/wtf/allocator.h"
#include "third_party/blink/renderer/platform/wtf/assertions.h"

namespace WTF {
class AtomicString;
class String;
}

namespace blink {

enum class CSSPropertyID {
    kInvalid = 0,
    kVariable = 1,
    kAnimationDelay = 2,
    kAnimationDirection = 3,
    kAnimationDuration = 4,
    kAnimationFillMode = 5,
    kAnimationIterationCount = 6,
    kAnimationName = 7,
    kAnimationPlayState = 8,
    kAnimationTimingFunction = 9,
    kTransitionDelay = 10,
    kTransitionDuration = 11,
    kTransitionProperty = 12,
    kTransitionTimingFunction = 13,
    kColor = 14,
    kDirection = 15,
    kFontFamily = 16,
    kFontFeatureSettings = 17,
    kFontKerning = 18,
    kFontSize = 19,
    kFontSizeAdjust = 20,
    kFontStretch = 21,
    kFontStyle = 22,
    kFontVariantCaps = 23,
    kFontVariantEastAsian = 24,
    kFontVariantLigatures = 25,
    kFontVariantNumeric = 26,
    kFontVariationSettings = 27,
    kFontWeight = 28,
    kTextOrientation = 29,
    kTextRendering = 30,
    kWebkitFontSmoothing = 31,
    kWebkitLocale = 32,
    kWebkitTextOrientation = 33,
    kWebkitWritingMode = 34,
    kWritingMode = 35,
    kZoom = 36,
    kAlignContent = 37,
    kAlignItems = 38,
    kAlignSelf = 39,
    kAlignmentBaseline = 40,
    kAll = 41,
    kBackdropFilter = 42,
    kBackfaceVisibility = 43,
    kBackgroundAttachment = 44,
    kBackgroundBlendMode = 45,
    kBackgroundClip = 46,
    kBackgroundColor = 47,
    kBackgroundImage = 48,
    kBackgroundOrigin = 49,
    kBackgroundPositionX = 50,
    kBackgroundPositionY = 51,
    kBackgroundRepeatX = 52,
    kBackgroundRepeatY = 53,
    kBackgroundSize = 54,
    kBaselineShift = 55,
    kBlockSize = 56,
    kBorderBlockEndColor = 57,
    kBorderBlockEndStyle = 58,
    kBorderBlockEndWidth = 59,
    kBorderBlockStartColor = 60,
    kBorderBlockStartStyle = 61,
    kBorderBlockStartWidth = 62,
    kBorderBottomColor = 63,
    kBorderBottomLeftRadius = 64,
    kBorderBottomRightRadius = 65,
    kBorderBottomStyle = 66,
    kBorderBottomWidth = 67,
    kBorderCollapse = 68,
    kBorderImageOutset = 69,
    kBorderImageRepeat = 70,
    kBorderImageSlice = 71,
    kBorderImageSource = 72,
    kBorderImageWidth = 73,
    kBorderInlineEndColor = 74,
    kBorderInlineEndStyle = 75,
    kBorderInlineEndWidth = 76,
    kBorderInlineStartColor = 77,
    kBorderInlineStartStyle = 78,
    kBorderInlineStartWidth = 79,
    kBorderLeftColor = 80,
    kBorderLeftStyle = 81,
    kBorderLeftWidth = 82,
    kBorderRightColor = 83,
    kBorderRightStyle = 84,
    kBorderRightWidth = 85,
    kBorderTopColor = 86,
    kBorderTopLeftRadius = 87,
    kBorderTopRightRadius = 88,
    kBorderTopStyle = 89,
    kBorderTopWidth = 90,
    kBottom = 91,
    kBoxShadow = 92,
    kBoxSizing = 93,
    kBreakAfter = 94,
    kBreakBefore = 95,
    kBreakInside = 96,
    kBufferedRendering = 97,
    kCaptionSide = 98,
    kCaretColor = 99,
    kClear = 100,
    kClip = 101,
    kClipPath = 102,
    kClipRule = 103,
    kColorInterpolation = 104,
    kColorInterpolationFilters = 105,
    kColorRendering = 106,
    kColorScheme = 107,
    kColumnCount = 108,
    kColumnFill = 109,
    kColumnGap = 110,
    kColumnRuleColor = 111,
    kColumnRuleStyle = 112,
    kColumnRuleWidth = 113,
    kColumnSpan = 114,
    kColumnWidth = 115,
    kContain = 116,
    kContent = 117,
    kCounterIncrement = 118,
    kCounterReset = 119,
    kCursor = 120,
    kCx = 121,
    kCy = 122,
    kD = 123,
    kDisplay = 124,
    kDominantBaseline = 125,
    kEmptyCells = 126,
    kFill = 127,
    kFillOpacity = 128,
    kFillRule = 129,
    kFilter = 130,
    kFlexBasis = 131,
    kFlexDirection = 132,
    kFlexGrow = 133,
    kFlexShrink = 134,
    kFlexWrap = 135,
    kFloat = 136,
    kFloodColor = 137,
    kFloodOpacity = 138,
    kFontDisplay = 139,
    kGridAutoColumns = 140,
    kGridAutoFlow = 141,
    kGridAutoRows = 142,
    kGridColumnEnd = 143,
    kGridColumnStart = 144,
    kGridRowEnd = 145,
    kGridRowStart = 146,
    kGridTemplateAreas = 147,
    kGridTemplateColumns = 148,
    kGridTemplateRows = 149,
    kHeight = 150,
    kHyphens = 151,
    kImageOrientation = 152,
    kImageRendering = 153,
    kInlineSize = 154,
    kInsetBlockEnd = 155,
    kInsetBlockStart = 156,
    kInsetInlineEnd = 157,
    kInsetInlineStart = 158,
    kIsolation = 159,
    kJustifyContent = 160,
    kJustifyItems = 161,
    kJustifySelf = 162,
    kLeft = 163,
    kLetterSpacing = 164,
    kLightingColor = 165,
    kLineBreak = 166,
    kLineHeight = 167,
    kLineHeightStep = 168,
    kListStyleImage = 169,
    kListStylePosition = 170,
    kListStyleType = 171,
    kMarginBlockEnd = 172,
    kMarginBlockStart = 173,
    kMarginBottom = 174,
    kMarginInlineEnd = 175,
    kMarginInlineStart = 176,
    kMarginLeft = 177,
    kMarginRight = 178,
    kMarginTop = 179,
    kMarkerEnd = 180,
    kMarkerMid = 181,
    kMarkerStart = 182,
    kMask = 183,
    kMaskSourceType = 184,
    kMaskType = 185,
    kMaxBlockSize = 186,
    kMaxHeight = 187,
    kMaxInlineSize = 188,
    kMaxWidth = 189,
    kMaxZoom = 190,
    kMinBlockSize = 191,
    kMinHeight = 192,
    kMinInlineSize = 193,
    kMinWidth = 194,
    kMinZoom = 195,
    kMixBlendMode = 196,
    kObjectFit = 197,
    kObjectPosition = 198,
    kOffsetAnchor = 199,
    kOffsetDistance = 200,
    kOffsetPath = 201,
    kOffsetPosition = 202,
    kOffsetRotate = 203,
    kOpacity = 204,
    kOrder = 205,
    kOrientation = 206,
    kOrphans = 207,
    kOutlineColor = 208,
    kOutlineOffset = 209,
    kOutlineStyle = 210,
    kOutlineWidth = 211,
    kOverflowAnchor = 212,
    kOverflowWrap = 213,
    kOverflowX = 214,
    kOverflowY = 215,
    kOverscrollBehaviorX = 216,
    kOverscrollBehaviorY = 217,
    kPaddingBlockEnd = 218,
    kPaddingBlockStart = 219,
    kPaddingBottom = 220,
    kPaddingInlineEnd = 221,
    kPaddingInlineStart = 222,
    kPaddingLeft = 223,
    kPaddingRight = 224,
    kPaddingTop = 225,
    kPage = 226,
    kPaintOrder = 227,
    kPerspective = 228,
    kPerspectiveOrigin = 229,
    kPointerEvents = 230,
    kPosition = 231,
    kQuotes = 232,
    kR = 233,
    kResize = 234,
    kRight = 235,
    kRotate = 236,
    kRowGap = 237,
    kRx = 238,
    kRy = 239,
    kScale = 240,
    kScrollBehavior = 241,
    kScrollCustomization = 242,
    kScrollMarginBlockEnd = 243,
    kScrollMarginBlockStart = 244,
    kScrollMarginBottom = 245,
    kScrollMarginInlineEnd = 246,
    kScrollMarginInlineStart = 247,
    kScrollMarginLeft = 248,
    kScrollMarginRight = 249,
    kScrollMarginTop = 250,
    kScrollPaddingBlockEnd = 251,
    kScrollPaddingBlockStart = 252,
    kScrollPaddingBottom = 253,
    kScrollPaddingInlineEnd = 254,
    kScrollPaddingInlineStart = 255,
    kScrollPaddingLeft = 256,
    kScrollPaddingRight = 257,
    kScrollPaddingTop = 258,
    kScrollSnapAlign = 259,
    kScrollSnapStop = 260,
    kScrollSnapType = 261,
    kShapeImageThreshold = 262,
    kShapeMargin = 263,
    kShapeOutside = 264,
    kShapeRendering = 265,
    kSize = 266,
    kSpeak = 267,
    kSrc = 268,
    kStopColor = 269,
    kStopOpacity = 270,
    kStroke = 271,
    kStrokeDasharray = 272,
    kStrokeDashoffset = 273,
    kStrokeLinecap = 274,
    kStrokeLinejoin = 275,
    kStrokeMiterlimit = 276,
    kStrokeOpacity = 277,
    kStrokeWidth = 278,
    kTabSize = 279,
    kTableLayout = 280,
    kTextAlign = 281,
    kTextAlignLast = 282,
    kTextAnchor = 283,
    kTextCombineUpright = 284,
    kTextDecorationColor = 285,
    kTextDecorationLine = 286,
    kTextDecorationSkipInk = 287,
    kTextDecorationStyle = 288,
    kTextIndent = 289,
    kTextJustify = 290,
    kTextOverflow = 291,
    kTextShadow = 292,
    kTextSizeAdjust = 293,
    kTextTransform = 294,
    kTextUnderlinePosition = 295,
    kTop = 296,
    kTouchAction = 297,
    kTransform = 298,
    kTransformBox = 299,
    kTransformOrigin = 300,
    kTransformStyle = 301,
    kTranslate = 302,
    kUnicodeBidi = 303,
    kUnicodeRange = 304,
    kUserSelect = 305,
    kUserZoom = 306,
    kVectorEffect = 307,
    kVerticalAlign = 308,
    kViewportFit = 309,
    kVisibility = 310,
    kWebkitAppRegion = 311,
    kWebkitAppearance = 312,
    kWebkitBorderHorizontalSpacing = 313,
    kWebkitBorderImage = 314,
    kWebkitBorderVerticalSpacing = 315,
    kWebkitBoxAlign = 316,
    kWebkitBoxDecorationBreak = 317,
    kWebkitBoxDirection = 318,
    kWebkitBoxFlex = 319,
    kWebkitBoxOrdinalGroup = 320,
    kWebkitBoxOrient = 321,
    kWebkitBoxPack = 322,
    kWebkitBoxReflect = 323,
    kWebkitFontSizeDelta = 324,
    kWebkitHighlight = 325,
    kWebkitHyphenateCharacter = 326,
    kWebkitLineBreak = 327,
    kWebkitLineClamp = 328,
    kWebkitMarginAfterCollapse = 329,
    kWebkitMarginBeforeCollapse = 330,
    kWebkitMarginBottomCollapse = 331,
    kWebkitMarginTopCollapse = 332,
    kWebkitMaskBoxImageOutset = 333,
    kWebkitMaskBoxImageRepeat = 334,
    kWebkitMaskBoxImageSlice = 335,
    kWebkitMaskBoxImageSource = 336,
    kWebkitMaskBoxImageWidth = 337,
    kWebkitMaskClip = 338,
    kWebkitMaskComposite = 339,
    kWebkitMaskImage = 340,
    kWebkitMaskOrigin = 341,
    kWebkitMaskPositionX = 342,
    kWebkitMaskPositionY = 343,
    kWebkitMaskRepeatX = 344,
    kWebkitMaskRepeatY = 345,
    kWebkitMaskSize = 346,
    kWebkitPerspectiveOriginX = 347,
    kWebkitPerspectiveOriginY = 348,
    kWebkitPrintColorAdjust = 349,
    kWebkitRtlOrdering = 350,
    kWebkitRubyPosition = 351,
    kWebkitTapHighlightColor = 352,
    kWebkitTextCombine = 353,
    kWebkitTextDecorationsInEffect = 354,
    kWebkitTextEmphasisColor = 355,
    kWebkitTextEmphasisPosition = 356,
    kWebkitTextEmphasisStyle = 357,
    kWebkitTextFillColor = 358,
    kWebkitTextSecurity = 359,
    kWebkitTextStrokeColor = 360,
    kWebkitTextStrokeWidth = 361,
    kWebkitTransformOriginX = 362,
    kWebkitTransformOriginY = 363,
    kWebkitTransformOriginZ = 364,
    kWebkitUserDrag = 365,
    kWebkitUserModify = 366,
    kWhiteSpace = 367,
    kWidows = 368,
    kWidth = 369,
    kWillChange = 370,
    kWordBreak = 371,
    kWordSpacing = 372,
    kX = 373,
    kY = 374,
    kZIndex = 375,
    kAnimation = 376,
    kBackground = 377,
    kBackgroundPosition = 378,
    kBackgroundRepeat = 379,
    kBorder = 380,
    kBorderBlock = 381,
    kBorderBlockColor = 382,
    kBorderBlockEnd = 383,
    kBorderBlockStart = 384,
    kBorderBlockStyle = 385,
    kBorderBlockWidth = 386,
    kBorderBottom = 387,
    kBorderColor = 388,
    kBorderImage = 389,
    kBorderInline = 390,
    kBorderInlineColor = 391,
    kBorderInlineEnd = 392,
    kBorderInlineStart = 393,
    kBorderInlineStyle = 394,
    kBorderInlineWidth = 395,
    kBorderLeft = 396,
    kBorderRadius = 397,
    kBorderRight = 398,
    kBorderSpacing = 399,
    kBorderStyle = 400,
    kBorderTop = 401,
    kBorderWidth = 402,
    kColumnRule = 403,
    kColumns = 404,
    kFlex = 405,
    kFlexFlow = 406,
    kFont = 407,
    kFontVariant = 408,
    kGap = 409,
    kGrid = 410,
    kGridArea = 411,
    kGridColumn = 412,
    kGridColumnGap = 413,
    kGridGap = 414,
    kGridRow = 415,
    kGridRowGap = 416,
    kGridTemplate = 417,
    kInset = 418,
    kInsetBlock = 419,
    kInsetInline = 420,
    kListStyle = 421,
    kMargin = 422,
    kMarginBlock = 423,
    kMarginInline = 424,
    kMarker = 425,
    kOffset = 426,
    kOutline = 427,
    kOverflow = 428,
    kOverscrollBehavior = 429,
    kPadding = 430,
    kPaddingBlock = 431,
    kPaddingInline = 432,
    kPageBreakAfter = 433,
    kPageBreakBefore = 434,
    kPageBreakInside = 435,
    kPlaceContent = 436,
    kPlaceItems = 437,
    kPlaceSelf = 438,
    kScrollMargin = 439,
    kScrollMarginBlock = 440,
    kScrollMarginInline = 441,
    kScrollPadding = 442,
    kScrollPaddingBlock = 443,
    kScrollPaddingInline = 444,
    kTextDecoration = 445,
    kTransition = 446,
    kWebkitColumnBreakAfter = 447,
    kWebkitColumnBreakBefore = 448,
    kWebkitColumnBreakInside = 449,
    kWebkitMarginCollapse = 450,
    kWebkitMask = 451,
    kWebkitMaskBoxImage = 452,
    kWebkitMaskPosition = 453,
    kWebkitMaskRepeat = 454,
    kWebkitTextEmphasis = 455,
    kWebkitTextStroke = 456,
    kAliasWebkitBorderEndColor = 586,
    kAliasWebkitBorderEndStyle = 587,
    kAliasWebkitBorderEndWidth = 588,
    kAliasWebkitBorderStartColor = 589,
    kAliasWebkitBorderStartStyle = 590,
    kAliasWebkitBorderStartWidth = 591,
    kAliasWebkitBorderBeforeColor = 572,
    kAliasWebkitBorderBeforeStyle = 573,
    kAliasWebkitBorderBeforeWidth = 574,
    kAliasWebkitBorderAfterColor = 569,
    kAliasWebkitBorderAfterStyle = 570,
    kAliasWebkitBorderAfterWidth = 571,
    kAliasWebkitMarginEnd = 687,
    kAliasWebkitMarginStart = 688,
    kAliasWebkitMarginBefore = 685,
    kAliasWebkitMarginAfter = 684,
    kAliasWebkitPaddingEnd = 733,
    kAliasWebkitPaddingStart = 734,
    kAliasWebkitPaddingBefore = 731,
    kAliasWebkitPaddingAfter = 730,
    kAliasWebkitLogicalWidth = 666,
    kAliasWebkitLogicalHeight = 568,
    kAliasWebkitMinLogicalWidth = 705,
    kAliasWebkitMinLogicalHeight = 703,
    kAliasWebkitMaxLogicalWidth = 700,
    kAliasWebkitMaxLogicalHeight = 698,
    kAliasWebkitBorderAfter = 895,
    kAliasWebkitBorderBefore = 896,
    kAliasWebkitBorderEnd = 904,
    kAliasWebkitBorderStart = 905,
    kAliasEpubCaptionSide = 610,
    kAliasEpubTextCombine = 865,
    kAliasEpubTextEmphasis = 967,
    kAliasEpubTextEmphasisColor = 867,
    kAliasEpubTextEmphasisStyle = 869,
    kAliasEpubTextOrientation = 545,
    kAliasEpubTextTransform = 806,
    kAliasEpubWordBreak = 883,
    kAliasEpubWritingMode = 546,
    kAliasWebkitAlignContent = 549,
    kAliasWebkitAlignItems = 550,
    kAliasWebkitAlignSelf = 551,
    kAliasWebkitAnimation = 888,
    kAliasWebkitAnimationDelay = 514,
    kAliasWebkitAnimationDirection = 515,
    kAliasWebkitAnimationDuration = 516,
    kAliasWebkitAnimationFillMode = 517,
    kAliasWebkitAnimationIterationCount = 518,
    kAliasWebkitAnimationName = 519,
    kAliasWebkitAnimationPlayState = 520,
    kAliasWebkitAnimationTimingFunction = 521,
    kAliasWebkitBackfaceVisibility = 555,
    kAliasWebkitBackgroundClip = 558,
    kAliasWebkitBackgroundOrigin = 561,
    kAliasWebkitBackgroundSize = 566,
    kAliasWebkitBorderBottomLeftRadius = 576,
    kAliasWebkitBorderBottomRightRadius = 577,
    kAliasWebkitBorderRadius = 909,
    kAliasWebkitBorderTopLeftRadius = 599,
    kAliasWebkitBorderTopRightRadius = 600,
    kAliasWebkitBoxShadow = 604,
    kAliasWebkitBoxSizing = 605,
    kAliasWebkitClipPath = 614,
    kAliasWebkitColumnCount = 620,
    kAliasWebkitColumnGap = 622,
    kAliasWebkitColumnRule = 915,
    kAliasWebkitColumnRuleColor = 623,
    kAliasWebkitColumnRuleStyle = 624,
    kAliasWebkitColumnRuleWidth = 625,
    kAliasWebkitColumnSpan = 626,
    kAliasWebkitColumnWidth = 627,
    kAliasWebkitColumns = 916,
    kAliasWebkitFilter = 642,
    kAliasWebkitFlex = 917,
    kAliasWebkitFlexBasis = 643,
    kAliasWebkitFlexDirection = 644,
    kAliasWebkitFlexFlow = 918,
    kAliasWebkitFlexGrow = 645,
    kAliasWebkitFlexShrink = 646,
    kAliasWebkitFlexWrap = 647,
    kAliasWebkitFontFeatureSettings = 529,
    kAliasWebkitJustifyContent = 672,
    kAliasWebkitOpacity = 716,
    kAliasWebkitOrder = 717,
    kAliasWebkitPerspective = 740,
    kAliasWebkitPerspectiveOrigin = 741,
    kAliasWebkitShapeImageThreshold = 774,
    kAliasWebkitShapeMargin = 775,
    kAliasWebkitShapeOutside = 776,
    kAliasWebkitTextSizeAdjust = 805,
    kAliasWebkitTransform = 810,
    kAliasWebkitTransformOrigin = 812,
    kAliasWebkitTransformStyle = 813,
    kAliasWebkitTransition = 958,
    kAliasWebkitTransitionDelay = 522,
    kAliasWebkitTransitionDuration = 523,
    kAliasWebkitTransitionProperty = 524,
    kAliasWebkitTransitionTimingFunction = 525,
    kAliasWebkitUserSelect = 817,
    kAliasWordWrap = 725,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    CSSPropertyID::kAliasWebkitBorderEndColor,
    CSSPropertyID::kAliasWebkitBorderEndStyle,
    CSSPropertyID::kAliasWebkitBorderEndWidth,
    CSSPropertyID::kAliasWebkitBorderStartColor,
    CSSPropertyID::kAliasWebkitBorderStartStyle,
    CSSPropertyID::kAliasWebkitBorderStartWidth,
    CSSPropertyID::kAliasWebkitBorderBeforeColor,
    CSSPropertyID::kAliasWebkitBorderBeforeStyle,
    CSSPropertyID::kAliasWebkitBorderBeforeWidth,
    CSSPropertyID::kAliasWebkitBorderAfterColor,
    CSSPropertyID::kAliasWebkitBorderAfterStyle,
    CSSPropertyID::kAliasWebkitBorderAfterWidth,
    CSSPropertyID::kAliasWebkitMarginEnd,
    CSSPropertyID::kAliasWebkitMarginStart,
    CSSPropertyID::kAliasWebkitMarginBefore,
    CSSPropertyID::kAliasWebkitMarginAfter,
    CSSPropertyID::kAliasWebkitPaddingEnd,
    CSSPropertyID::kAliasWebkitPaddingStart,
    CSSPropertyID::kAliasWebkitPaddingBefore,
    CSSPropertyID::kAliasWebkitPaddingAfter,
    CSSPropertyID::kAliasWebkitLogicalWidth,
    CSSPropertyID::kAliasWebkitLogicalHeight,
    CSSPropertyID::kAliasWebkitMinLogicalWidth,
    CSSPropertyID::kAliasWebkitMinLogicalHeight,
    CSSPropertyID::kAliasWebkitMaxLogicalWidth,
    CSSPropertyID::kAliasWebkitMaxLogicalHeight,
    CSSPropertyID::kAliasWebkitBorderAfter,
    CSSPropertyID::kAliasWebkitBorderBefore,
    CSSPropertyID::kAliasWebkitBorderEnd,
    CSSPropertyID::kAliasWebkitBorderStart,
    CSSPropertyID::kAliasEpubCaptionSide,
    CSSPropertyID::kAliasEpubTextCombine,
    CSSPropertyID::kAliasEpubTextEmphasis,
    CSSPropertyID::kAliasEpubTextEmphasisColor,
    CSSPropertyID::kAliasEpubTextEmphasisStyle,
    CSSPropertyID::kAliasEpubTextOrientation,
    CSSPropertyID::kAliasEpubTextTransform,
    CSSPropertyID::kAliasEpubWordBreak,
    CSSPropertyID::kAliasEpubWritingMode,
    CSSPropertyID::kAliasWebkitAlignContent,
    CSSPropertyID::kAliasWebkitAlignItems,
    CSSPropertyID::kAliasWebkitAlignSelf,
    CSSPropertyID::kAliasWebkitAnimation,
    CSSPropertyID::kAliasWebkitAnimationDelay,
    CSSPropertyID::kAliasWebkitAnimationDirection,
    CSSPropertyID::kAliasWebkitAnimationDuration,
    CSSPropertyID::kAliasWebkitAnimationFillMode,
    CSSPropertyID::kAliasWebkitAnimationIterationCount,
    CSSPropertyID::kAliasWebkitAnimationName,
    CSSPropertyID::kAliasWebkitAnimationPlayState,
    CSSPropertyID::kAliasWebkitAnimationTimingFunction,
    CSSPropertyID::kAliasWebkitBackfaceVisibility,
    CSSPropertyID::kAliasWebkitBackgroundClip,
    CSSPropertyID::kAliasWebkitBackgroundOrigin,
    CSSPropertyID::kAliasWebkitBackgroundSize,
    CSSPropertyID::kAliasWebkitBorderBottomLeftRadius,
    CSSPropertyID::kAliasWebkitBorderBottomRightRadius,
    CSSPropertyID::kAliasWebkitBorderRadius,
    CSSPropertyID::kAliasWebkitBorderTopLeftRadius,
    CSSPropertyID::kAliasWebkitBorderTopRightRadius,
    CSSPropertyID::kAliasWebkitBoxShadow,
    CSSPropertyID::kAliasWebkitBoxSizing,
    CSSPropertyID::kAliasWebkitClipPath,
    CSSPropertyID::kAliasWebkitColumnCount,
    CSSPropertyID::kAliasWebkitColumnGap,
    CSSPropertyID::kAliasWebkitColumnRule,
    CSSPropertyID::kAliasWebkitColumnRuleColor,
    CSSPropertyID::kAliasWebkitColumnRuleStyle,
    CSSPropertyID::kAliasWebkitColumnRuleWidth,
    CSSPropertyID::kAliasWebkitColumnSpan,
    CSSPropertyID::kAliasWebkitColumnWidth,
    CSSPropertyID::kAliasWebkitColumns,
    CSSPropertyID::kAliasWebkitFilter,
    CSSPropertyID::kAliasWebkitFlex,
    CSSPropertyID::kAliasWebkitFlexBasis,
    CSSPropertyID::kAliasWebkitFlexDirection,
    CSSPropertyID::kAliasWebkitFlexFlow,
    CSSPropertyID::kAliasWebkitFlexGrow,
    CSSPropertyID::kAliasWebkitFlexShrink,
    CSSPropertyID::kAliasWebkitFlexWrap,
    CSSPropertyID::kAliasWebkitFontFeatureSettings,
    CSSPropertyID::kAliasWebkitJustifyContent,
    CSSPropertyID::kAliasWebkitOpacity,
    CSSPropertyID::kAliasWebkitOrder,
    CSSPropertyID::kAliasWebkitPerspective,
    CSSPropertyID::kAliasWebkitPerspectiveOrigin,
    CSSPropertyID::kAliasWebkitShapeImageThreshold,
    CSSPropertyID::kAliasWebkitShapeMargin,
    CSSPropertyID::kAliasWebkitShapeOutside,
    CSSPropertyID::kAliasWebkitTextSizeAdjust,
    CSSPropertyID::kAliasWebkitTransform,
    CSSPropertyID::kAliasWebkitTransformOrigin,
    CSSPropertyID::kAliasWebkitTransformStyle,
    CSSPropertyID::kAliasWebkitTransition,
    CSSPropertyID::kAliasWebkitTransitionDelay,
    CSSPropertyID::kAliasWebkitTransitionDuration,
    CSSPropertyID::kAliasWebkitTransitionProperty,
    CSSPropertyID::kAliasWebkitTransitionTimingFunction,
    CSSPropertyID::kAliasWebkitUserSelect,
    CSSPropertyID::kAliasWordWrap,
};

const int kIntFirstCSSProperty = 2;
const CSSPropertyID firstCSSProperty = static_cast<CSSPropertyID>(kIntFirstCSSProperty);
const int numCSSProperties = 555;
const int kIntLastCSSProperty = 456;
const CSSPropertyID lastCSSProperty = static_cast<CSSPropertyID>(kIntLastCSSProperty);
const CSSPropertyID lastUnresolvedCSSProperty = static_cast<CSSPropertyID>(967);
const int numCSSPropertyIDs = static_cast<int>(lastUnresolvedCSSProperty) + 1;
const size_t maxCSSPropertyNameLength = 40;

inline int GetCSSPropertyIDIndex(CSSPropertyID id) {
    DCHECK_GE(id, firstCSSProperty);
    DCHECK_LE(id, lastCSSProperty);
    return static_cast<int>(id) - kIntFirstCSSProperty;
}

inline bool isCSSPropertyIDWithName(CSSPropertyID id)
{
    return id >= firstCSSProperty && id <= lastUnresolvedCSSProperty;
}

inline bool isValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyID::kInvalid;
}

inline CSSPropertyID convertToCSSPropertyID(int value)
{
    DCHECK_GE(value, static_cast<int>(CSSPropertyID::kInvalid));
    DCHECK_LE(value, kIntLastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

inline CSSPropertyID resolveCSSPropertyID(CSSPropertyID id)
{
    return convertToCSSPropertyID(static_cast<int>(id) & ~512);
}

inline bool isPropertyAlias(CSSPropertyID id) { return static_cast<int>(id) & 512; }

CSSPropertyID CORE_EXPORT unresolvedCSSPropertyID(const WTF::String&);

CSSPropertyID CORE_EXPORT cssPropertyID(const WTF::String&);

class CSSPropertyIDList {
  STACK_ALLOCATED();
 public:
    class Iterator {
      STACK_ALLOCATED();
     public:
      Iterator(int id) : id_(id) {}
      CSSPropertyID operator*() const { return convertToCSSPropertyID(id_); }
      Iterator& operator++() { id_++; return *this; }
      bool operator!=(const Iterator& i) const { return id_ != i.id_; }
     private:
      int id_;
    };
  Iterator begin() const { return Iterator(kIntFirstCSSProperty); }
  Iterator end() const { return Iterator(kIntLastCSSProperty + 1); }
};

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
