// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/element_type_helpers.h.tmpl
// and input files:
//   ../../third_party/blink/renderer/core/html/html_tag_names.json5


#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_HTML_ELEMENT_TYPE_HELPERS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_HTML_ELEMENT_TYPE_HELPERS_H_

#include "third_party/blink/renderer/core/html/html_element.h"
#include "third_party/blink/renderer/core/html_names.h"
#include "third_party/blink/renderer/platform/runtime_enabled_features.h"

namespace blink {
// Type checking.
class HTMLAnchorElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLAnchorElement(const HTMLAnchorElement&);
void IsHTMLAnchorElement(const HTMLAnchorElement*);

inline bool IsHTMLAnchorElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kATag);
}
inline bool IsHTMLAnchorElement(const HTMLElement* element) {
  return element && IsHTMLAnchorElement(*element);
}
inline bool IsHTMLAnchorElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLAnchorElement(To<HTMLElement>(node));
}
inline bool IsHTMLAnchorElement(const Node* node) {
  return node && IsHTMLAnchorElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLAnchorElement>(const Node& node) {
  return IsHTMLAnchorElement(node);
}
template <>
inline bool IsElementOfType<const HTMLAnchorElement>(const HTMLElement& element) {
  return IsHTMLAnchorElement(element);
}

class HTMLAreaElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLAreaElement(const HTMLAreaElement&);
void IsHTMLAreaElement(const HTMLAreaElement*);

inline bool IsHTMLAreaElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kAreaTag);
}
inline bool IsHTMLAreaElement(const HTMLElement* element) {
  return element && IsHTMLAreaElement(*element);
}
inline bool IsHTMLAreaElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLAreaElement(To<HTMLElement>(node));
}
inline bool IsHTMLAreaElement(const Node* node) {
  return node && IsHTMLAreaElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLAreaElement>(const Node& node) {
  return IsHTMLAreaElement(node);
}
template <>
inline bool IsElementOfType<const HTMLAreaElement>(const HTMLElement& element) {
  return IsHTMLAreaElement(element);
}

class HTMLAudioElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLAudioElement(const HTMLAudioElement&);
void IsHTMLAudioElement(const HTMLAudioElement*);

inline bool IsHTMLAudioElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kAudioTag);
}
inline bool IsHTMLAudioElement(const HTMLElement* element) {
  return element && IsHTMLAudioElement(*element);
}
inline bool IsHTMLAudioElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLAudioElement(To<HTMLElement>(node));
}
inline bool IsHTMLAudioElement(const Node* node) {
  return node && IsHTMLAudioElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLAudioElement>(const Node& node) {
  return IsHTMLAudioElement(node);
}
template <>
inline bool IsElementOfType<const HTMLAudioElement>(const HTMLElement& element) {
  return IsHTMLAudioElement(element);
}

class HTMLBRElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLBRElement(const HTMLBRElement&);
void IsHTMLBRElement(const HTMLBRElement*);

inline bool IsHTMLBRElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kBrTag);
}
inline bool IsHTMLBRElement(const HTMLElement* element) {
  return element && IsHTMLBRElement(*element);
}
inline bool IsHTMLBRElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLBRElement(To<HTMLElement>(node));
}
inline bool IsHTMLBRElement(const Node* node) {
  return node && IsHTMLBRElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLBRElement>(const Node& node) {
  return IsHTMLBRElement(node);
}
template <>
inline bool IsElementOfType<const HTMLBRElement>(const HTMLElement& element) {
  return IsHTMLBRElement(element);
}

class HTMLBaseElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLBaseElement(const HTMLBaseElement&);
void IsHTMLBaseElement(const HTMLBaseElement*);

inline bool IsHTMLBaseElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kBaseTag);
}
inline bool IsHTMLBaseElement(const HTMLElement* element) {
  return element && IsHTMLBaseElement(*element);
}
inline bool IsHTMLBaseElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLBaseElement(To<HTMLElement>(node));
}
inline bool IsHTMLBaseElement(const Node* node) {
  return node && IsHTMLBaseElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLBaseElement>(const Node& node) {
  return IsHTMLBaseElement(node);
}
template <>
inline bool IsElementOfType<const HTMLBaseElement>(const HTMLElement& element) {
  return IsHTMLBaseElement(element);
}

class HTMLBodyElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLBodyElement(const HTMLBodyElement&);
void IsHTMLBodyElement(const HTMLBodyElement*);

inline bool IsHTMLBodyElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kBodyTag);
}
inline bool IsHTMLBodyElement(const HTMLElement* element) {
  return element && IsHTMLBodyElement(*element);
}
inline bool IsHTMLBodyElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLBodyElement(To<HTMLElement>(node));
}
inline bool IsHTMLBodyElement(const Node* node) {
  return node && IsHTMLBodyElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLBodyElement>(const Node& node) {
  return IsHTMLBodyElement(node);
}
template <>
inline bool IsElementOfType<const HTMLBodyElement>(const HTMLElement& element) {
  return IsHTMLBodyElement(element);
}

class HTMLButtonElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLButtonElement(const HTMLButtonElement&);
void IsHTMLButtonElement(const HTMLButtonElement*);

inline bool IsHTMLButtonElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kButtonTag);
}
inline bool IsHTMLButtonElement(const HTMLElement* element) {
  return element && IsHTMLButtonElement(*element);
}
inline bool IsHTMLButtonElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLButtonElement(To<HTMLElement>(node));
}
inline bool IsHTMLButtonElement(const Node* node) {
  return node && IsHTMLButtonElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLButtonElement>(const Node& node) {
  return IsHTMLButtonElement(node);
}
template <>
inline bool IsElementOfType<const HTMLButtonElement>(const HTMLElement& element) {
  return IsHTMLButtonElement(element);
}

class HTMLCanvasElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLCanvasElement(const HTMLCanvasElement&);
void IsHTMLCanvasElement(const HTMLCanvasElement*);

inline bool IsHTMLCanvasElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kCanvasTag);
}
inline bool IsHTMLCanvasElement(const HTMLElement* element) {
  return element && IsHTMLCanvasElement(*element);
}
inline bool IsHTMLCanvasElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLCanvasElement(To<HTMLElement>(node));
}
inline bool IsHTMLCanvasElement(const Node* node) {
  return node && IsHTMLCanvasElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLCanvasElement>(const Node& node) {
  return IsHTMLCanvasElement(node);
}
template <>
inline bool IsElementOfType<const HTMLCanvasElement>(const HTMLElement& element) {
  return IsHTMLCanvasElement(element);
}

class HTMLContentElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLContentElement(const HTMLContentElement&);
void IsHTMLContentElement(const HTMLContentElement*);

inline bool IsHTMLContentElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kContentTag);
}
inline bool IsHTMLContentElement(const HTMLElement* element) {
  return element && IsHTMLContentElement(*element);
}
inline bool IsHTMLContentElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLContentElement(To<HTMLElement>(node));
}
inline bool IsHTMLContentElement(const Node* node) {
  return node && IsHTMLContentElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLContentElement>(const Node& node) {
  return IsHTMLContentElement(node);
}
template <>
inline bool IsElementOfType<const HTMLContentElement>(const HTMLElement& element) {
  return IsHTMLContentElement(element);
}

class HTMLDListElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLDListElement(const HTMLDListElement&);
void IsHTMLDListElement(const HTMLDListElement*);

inline bool IsHTMLDListElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kDlTag);
}
inline bool IsHTMLDListElement(const HTMLElement* element) {
  return element && IsHTMLDListElement(*element);
}
inline bool IsHTMLDListElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLDListElement(To<HTMLElement>(node));
}
inline bool IsHTMLDListElement(const Node* node) {
  return node && IsHTMLDListElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLDListElement>(const Node& node) {
  return IsHTMLDListElement(node);
}
template <>
inline bool IsElementOfType<const HTMLDListElement>(const HTMLElement& element) {
  return IsHTMLDListElement(element);
}

class HTMLDataElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLDataElement(const HTMLDataElement&);
void IsHTMLDataElement(const HTMLDataElement*);

inline bool IsHTMLDataElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kDataTag);
}
inline bool IsHTMLDataElement(const HTMLElement* element) {
  return element && IsHTMLDataElement(*element);
}
inline bool IsHTMLDataElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLDataElement(To<HTMLElement>(node));
}
inline bool IsHTMLDataElement(const Node* node) {
  return node && IsHTMLDataElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLDataElement>(const Node& node) {
  return IsHTMLDataElement(node);
}
template <>
inline bool IsElementOfType<const HTMLDataElement>(const HTMLElement& element) {
  return IsHTMLDataElement(element);
}

class HTMLDataListElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLDataListElement(const HTMLDataListElement&);
void IsHTMLDataListElement(const HTMLDataListElement*);

inline bool IsHTMLDataListElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kDatalistTag);
}
inline bool IsHTMLDataListElement(const HTMLElement* element) {
  return element && IsHTMLDataListElement(*element);
}
inline bool IsHTMLDataListElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLDataListElement(To<HTMLElement>(node));
}
inline bool IsHTMLDataListElement(const Node* node) {
  return node && IsHTMLDataListElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLDataListElement>(const Node& node) {
  return IsHTMLDataListElement(node);
}
template <>
inline bool IsElementOfType<const HTMLDataListElement>(const HTMLElement& element) {
  return IsHTMLDataListElement(element);
}

class HTMLDetailsElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLDetailsElement(const HTMLDetailsElement&);
void IsHTMLDetailsElement(const HTMLDetailsElement*);

inline bool IsHTMLDetailsElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kDetailsTag);
}
inline bool IsHTMLDetailsElement(const HTMLElement* element) {
  return element && IsHTMLDetailsElement(*element);
}
inline bool IsHTMLDetailsElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLDetailsElement(To<HTMLElement>(node));
}
inline bool IsHTMLDetailsElement(const Node* node) {
  return node && IsHTMLDetailsElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLDetailsElement>(const Node& node) {
  return IsHTMLDetailsElement(node);
}
template <>
inline bool IsElementOfType<const HTMLDetailsElement>(const HTMLElement& element) {
  return IsHTMLDetailsElement(element);
}

class HTMLDialogElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLDialogElement(const HTMLDialogElement&);
void IsHTMLDialogElement(const HTMLDialogElement*);

inline bool IsHTMLDialogElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kDialogTag);
}
inline bool IsHTMLDialogElement(const HTMLElement* element) {
  return element && IsHTMLDialogElement(*element);
}
inline bool IsHTMLDialogElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLDialogElement(To<HTMLElement>(node));
}
inline bool IsHTMLDialogElement(const Node* node) {
  return node && IsHTMLDialogElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLDialogElement>(const Node& node) {
  return IsHTMLDialogElement(node);
}
template <>
inline bool IsElementOfType<const HTMLDialogElement>(const HTMLElement& element) {
  return IsHTMLDialogElement(element);
}

class HTMLDirectoryElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLDirectoryElement(const HTMLDirectoryElement&);
void IsHTMLDirectoryElement(const HTMLDirectoryElement*);

inline bool IsHTMLDirectoryElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kDirTag);
}
inline bool IsHTMLDirectoryElement(const HTMLElement* element) {
  return element && IsHTMLDirectoryElement(*element);
}
inline bool IsHTMLDirectoryElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLDirectoryElement(To<HTMLElement>(node));
}
inline bool IsHTMLDirectoryElement(const Node* node) {
  return node && IsHTMLDirectoryElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLDirectoryElement>(const Node& node) {
  return IsHTMLDirectoryElement(node);
}
template <>
inline bool IsElementOfType<const HTMLDirectoryElement>(const HTMLElement& element) {
  return IsHTMLDirectoryElement(element);
}

class HTMLDivElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLDivElement(const HTMLDivElement&);
void IsHTMLDivElement(const HTMLDivElement*);

inline bool IsHTMLDivElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kDivTag);
}
inline bool IsHTMLDivElement(const HTMLElement* element) {
  return element && IsHTMLDivElement(*element);
}
inline bool IsHTMLDivElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLDivElement(To<HTMLElement>(node));
}
inline bool IsHTMLDivElement(const Node* node) {
  return node && IsHTMLDivElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLDivElement>(const Node& node) {
  return IsHTMLDivElement(node);
}
template <>
inline bool IsElementOfType<const HTMLDivElement>(const HTMLElement& element) {
  return IsHTMLDivElement(element);
}

class HTMLFieldSetElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLFieldSetElement(const HTMLFieldSetElement&);
void IsHTMLFieldSetElement(const HTMLFieldSetElement*);

inline bool IsHTMLFieldSetElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kFieldsetTag);
}
inline bool IsHTMLFieldSetElement(const HTMLElement* element) {
  return element && IsHTMLFieldSetElement(*element);
}
inline bool IsHTMLFieldSetElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLFieldSetElement(To<HTMLElement>(node));
}
inline bool IsHTMLFieldSetElement(const Node* node) {
  return node && IsHTMLFieldSetElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLFieldSetElement>(const Node& node) {
  return IsHTMLFieldSetElement(node);
}
template <>
inline bool IsElementOfType<const HTMLFieldSetElement>(const HTMLElement& element) {
  return IsHTMLFieldSetElement(element);
}

class HTMLFontElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLFontElement(const HTMLFontElement&);
void IsHTMLFontElement(const HTMLFontElement*);

inline bool IsHTMLFontElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kFontTag);
}
inline bool IsHTMLFontElement(const HTMLElement* element) {
  return element && IsHTMLFontElement(*element);
}
inline bool IsHTMLFontElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLFontElement(To<HTMLElement>(node));
}
inline bool IsHTMLFontElement(const Node* node) {
  return node && IsHTMLFontElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLFontElement>(const Node& node) {
  return IsHTMLFontElement(node);
}
template <>
inline bool IsElementOfType<const HTMLFontElement>(const HTMLElement& element) {
  return IsHTMLFontElement(element);
}

class HTMLFormElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLFormElement(const HTMLFormElement&);
void IsHTMLFormElement(const HTMLFormElement*);

inline bool IsHTMLFormElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kFormTag);
}
inline bool IsHTMLFormElement(const HTMLElement* element) {
  return element && IsHTMLFormElement(*element);
}
inline bool IsHTMLFormElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLFormElement(To<HTMLElement>(node));
}
inline bool IsHTMLFormElement(const Node* node) {
  return node && IsHTMLFormElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLFormElement>(const Node& node) {
  return IsHTMLFormElement(node);
}
template <>
inline bool IsElementOfType<const HTMLFormElement>(const HTMLElement& element) {
  return IsHTMLFormElement(element);
}

class HTMLFrameElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLFrameElement(const HTMLFrameElement&);
void IsHTMLFrameElement(const HTMLFrameElement*);

inline bool IsHTMLFrameElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kFrameTag);
}
inline bool IsHTMLFrameElement(const HTMLElement* element) {
  return element && IsHTMLFrameElement(*element);
}
inline bool IsHTMLFrameElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLFrameElement(To<HTMLElement>(node));
}
inline bool IsHTMLFrameElement(const Node* node) {
  return node && IsHTMLFrameElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLFrameElement>(const Node& node) {
  return IsHTMLFrameElement(node);
}
template <>
inline bool IsElementOfType<const HTMLFrameElement>(const HTMLElement& element) {
  return IsHTMLFrameElement(element);
}

class HTMLFrameSetElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLFrameSetElement(const HTMLFrameSetElement&);
void IsHTMLFrameSetElement(const HTMLFrameSetElement*);

inline bool IsHTMLFrameSetElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kFramesetTag);
}
inline bool IsHTMLFrameSetElement(const HTMLElement* element) {
  return element && IsHTMLFrameSetElement(*element);
}
inline bool IsHTMLFrameSetElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLFrameSetElement(To<HTMLElement>(node));
}
inline bool IsHTMLFrameSetElement(const Node* node) {
  return node && IsHTMLFrameSetElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLFrameSetElement>(const Node& node) {
  return IsHTMLFrameSetElement(node);
}
template <>
inline bool IsElementOfType<const HTMLFrameSetElement>(const HTMLElement& element) {
  return IsHTMLFrameSetElement(element);
}

class HTMLHRElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLHRElement(const HTMLHRElement&);
void IsHTMLHRElement(const HTMLHRElement*);

inline bool IsHTMLHRElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kHrTag);
}
inline bool IsHTMLHRElement(const HTMLElement* element) {
  return element && IsHTMLHRElement(*element);
}
inline bool IsHTMLHRElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLHRElement(To<HTMLElement>(node));
}
inline bool IsHTMLHRElement(const Node* node) {
  return node && IsHTMLHRElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLHRElement>(const Node& node) {
  return IsHTMLHRElement(node);
}
template <>
inline bool IsElementOfType<const HTMLHRElement>(const HTMLElement& element) {
  return IsHTMLHRElement(element);
}

class HTMLHeadElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLHeadElement(const HTMLHeadElement&);
void IsHTMLHeadElement(const HTMLHeadElement*);

inline bool IsHTMLHeadElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kHeadTag);
}
inline bool IsHTMLHeadElement(const HTMLElement* element) {
  return element && IsHTMLHeadElement(*element);
}
inline bool IsHTMLHeadElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLHeadElement(To<HTMLElement>(node));
}
inline bool IsHTMLHeadElement(const Node* node) {
  return node && IsHTMLHeadElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLHeadElement>(const Node& node) {
  return IsHTMLHeadElement(node);
}
template <>
inline bool IsElementOfType<const HTMLHeadElement>(const HTMLElement& element) {
  return IsHTMLHeadElement(element);
}

class HTMLHtmlElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLHtmlElement(const HTMLHtmlElement&);
void IsHTMLHtmlElement(const HTMLHtmlElement*);

inline bool IsHTMLHtmlElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kHTMLTag);
}
inline bool IsHTMLHtmlElement(const HTMLElement* element) {
  return element && IsHTMLHtmlElement(*element);
}
inline bool IsHTMLHtmlElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLHtmlElement(To<HTMLElement>(node));
}
inline bool IsHTMLHtmlElement(const Node* node) {
  return node && IsHTMLHtmlElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLHtmlElement>(const Node& node) {
  return IsHTMLHtmlElement(node);
}
template <>
inline bool IsElementOfType<const HTMLHtmlElement>(const HTMLElement& element) {
  return IsHTMLHtmlElement(element);
}

class HTMLIFrameElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLIFrameElement(const HTMLIFrameElement&);
void IsHTMLIFrameElement(const HTMLIFrameElement*);

inline bool IsHTMLIFrameElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kIFrameTag);
}
inline bool IsHTMLIFrameElement(const HTMLElement* element) {
  return element && IsHTMLIFrameElement(*element);
}
inline bool IsHTMLIFrameElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLIFrameElement(To<HTMLElement>(node));
}
inline bool IsHTMLIFrameElement(const Node* node) {
  return node && IsHTMLIFrameElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLIFrameElement>(const Node& node) {
  return IsHTMLIFrameElement(node);
}
template <>
inline bool IsElementOfType<const HTMLIFrameElement>(const HTMLElement& element) {
  return IsHTMLIFrameElement(element);
}

class HTMLLIElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLLIElement(const HTMLLIElement&);
void IsHTMLLIElement(const HTMLLIElement*);

inline bool IsHTMLLIElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kLiTag);
}
inline bool IsHTMLLIElement(const HTMLElement* element) {
  return element && IsHTMLLIElement(*element);
}
inline bool IsHTMLLIElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLLIElement(To<HTMLElement>(node));
}
inline bool IsHTMLLIElement(const Node* node) {
  return node && IsHTMLLIElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLLIElement>(const Node& node) {
  return IsHTMLLIElement(node);
}
template <>
inline bool IsElementOfType<const HTMLLIElement>(const HTMLElement& element) {
  return IsHTMLLIElement(element);
}

class HTMLLabelElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLLabelElement(const HTMLLabelElement&);
void IsHTMLLabelElement(const HTMLLabelElement*);

inline bool IsHTMLLabelElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kLabelTag);
}
inline bool IsHTMLLabelElement(const HTMLElement* element) {
  return element && IsHTMLLabelElement(*element);
}
inline bool IsHTMLLabelElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLLabelElement(To<HTMLElement>(node));
}
inline bool IsHTMLLabelElement(const Node* node) {
  return node && IsHTMLLabelElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLLabelElement>(const Node& node) {
  return IsHTMLLabelElement(node);
}
template <>
inline bool IsElementOfType<const HTMLLabelElement>(const HTMLElement& element) {
  return IsHTMLLabelElement(element);
}

class HTMLLegendElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLLegendElement(const HTMLLegendElement&);
void IsHTMLLegendElement(const HTMLLegendElement*);

inline bool IsHTMLLegendElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kLegendTag);
}
inline bool IsHTMLLegendElement(const HTMLElement* element) {
  return element && IsHTMLLegendElement(*element);
}
inline bool IsHTMLLegendElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLLegendElement(To<HTMLElement>(node));
}
inline bool IsHTMLLegendElement(const Node* node) {
  return node && IsHTMLLegendElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLLegendElement>(const Node& node) {
  return IsHTMLLegendElement(node);
}
template <>
inline bool IsElementOfType<const HTMLLegendElement>(const HTMLElement& element) {
  return IsHTMLLegendElement(element);
}

class HTMLMapElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLMapElement(const HTMLMapElement&);
void IsHTMLMapElement(const HTMLMapElement*);

inline bool IsHTMLMapElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kMapTag);
}
inline bool IsHTMLMapElement(const HTMLElement* element) {
  return element && IsHTMLMapElement(*element);
}
inline bool IsHTMLMapElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLMapElement(To<HTMLElement>(node));
}
inline bool IsHTMLMapElement(const Node* node) {
  return node && IsHTMLMapElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLMapElement>(const Node& node) {
  return IsHTMLMapElement(node);
}
template <>
inline bool IsElementOfType<const HTMLMapElement>(const HTMLElement& element) {
  return IsHTMLMapElement(element);
}

class HTMLMarqueeElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLMarqueeElement(const HTMLMarqueeElement&);
void IsHTMLMarqueeElement(const HTMLMarqueeElement*);

inline bool IsHTMLMarqueeElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kMarqueeTag);
}
inline bool IsHTMLMarqueeElement(const HTMLElement* element) {
  return element && IsHTMLMarqueeElement(*element);
}
inline bool IsHTMLMarqueeElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLMarqueeElement(To<HTMLElement>(node));
}
inline bool IsHTMLMarqueeElement(const Node* node) {
  return node && IsHTMLMarqueeElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLMarqueeElement>(const Node& node) {
  return IsHTMLMarqueeElement(node);
}
template <>
inline bool IsElementOfType<const HTMLMarqueeElement>(const HTMLElement& element) {
  return IsHTMLMarqueeElement(element);
}

class HTMLMenuElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLMenuElement(const HTMLMenuElement&);
void IsHTMLMenuElement(const HTMLMenuElement*);

inline bool IsHTMLMenuElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kMenuTag);
}
inline bool IsHTMLMenuElement(const HTMLElement* element) {
  return element && IsHTMLMenuElement(*element);
}
inline bool IsHTMLMenuElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLMenuElement(To<HTMLElement>(node));
}
inline bool IsHTMLMenuElement(const Node* node) {
  return node && IsHTMLMenuElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLMenuElement>(const Node& node) {
  return IsHTMLMenuElement(node);
}
template <>
inline bool IsElementOfType<const HTMLMenuElement>(const HTMLElement& element) {
  return IsHTMLMenuElement(element);
}

class HTMLMetaElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLMetaElement(const HTMLMetaElement&);
void IsHTMLMetaElement(const HTMLMetaElement*);

inline bool IsHTMLMetaElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kMetaTag);
}
inline bool IsHTMLMetaElement(const HTMLElement* element) {
  return element && IsHTMLMetaElement(*element);
}
inline bool IsHTMLMetaElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLMetaElement(To<HTMLElement>(node));
}
inline bool IsHTMLMetaElement(const Node* node) {
  return node && IsHTMLMetaElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLMetaElement>(const Node& node) {
  return IsHTMLMetaElement(node);
}
template <>
inline bool IsElementOfType<const HTMLMetaElement>(const HTMLElement& element) {
  return IsHTMLMetaElement(element);
}

class HTMLMeterElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLMeterElement(const HTMLMeterElement&);
void IsHTMLMeterElement(const HTMLMeterElement*);

inline bool IsHTMLMeterElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kMeterTag);
}
inline bool IsHTMLMeterElement(const HTMLElement* element) {
  return element && IsHTMLMeterElement(*element);
}
inline bool IsHTMLMeterElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLMeterElement(To<HTMLElement>(node));
}
inline bool IsHTMLMeterElement(const Node* node) {
  return node && IsHTMLMeterElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLMeterElement>(const Node& node) {
  return IsHTMLMeterElement(node);
}
template <>
inline bool IsElementOfType<const HTMLMeterElement>(const HTMLElement& element) {
  return IsHTMLMeterElement(element);
}

class HTMLOListElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLOListElement(const HTMLOListElement&);
void IsHTMLOListElement(const HTMLOListElement*);

inline bool IsHTMLOListElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kOlTag);
}
inline bool IsHTMLOListElement(const HTMLElement* element) {
  return element && IsHTMLOListElement(*element);
}
inline bool IsHTMLOListElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLOListElement(To<HTMLElement>(node));
}
inline bool IsHTMLOListElement(const Node* node) {
  return node && IsHTMLOListElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLOListElement>(const Node& node) {
  return IsHTMLOListElement(node);
}
template <>
inline bool IsElementOfType<const HTMLOListElement>(const HTMLElement& element) {
  return IsHTMLOListElement(element);
}

class HTMLOptGroupElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLOptGroupElement(const HTMLOptGroupElement&);
void IsHTMLOptGroupElement(const HTMLOptGroupElement*);

inline bool IsHTMLOptGroupElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kOptgroupTag);
}
inline bool IsHTMLOptGroupElement(const HTMLElement* element) {
  return element && IsHTMLOptGroupElement(*element);
}
inline bool IsHTMLOptGroupElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLOptGroupElement(To<HTMLElement>(node));
}
inline bool IsHTMLOptGroupElement(const Node* node) {
  return node && IsHTMLOptGroupElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLOptGroupElement>(const Node& node) {
  return IsHTMLOptGroupElement(node);
}
template <>
inline bool IsElementOfType<const HTMLOptGroupElement>(const HTMLElement& element) {
  return IsHTMLOptGroupElement(element);
}

class HTMLOptionElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLOptionElement(const HTMLOptionElement&);
void IsHTMLOptionElement(const HTMLOptionElement*);

inline bool IsHTMLOptionElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kOptionTag);
}
inline bool IsHTMLOptionElement(const HTMLElement* element) {
  return element && IsHTMLOptionElement(*element);
}
inline bool IsHTMLOptionElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLOptionElement(To<HTMLElement>(node));
}
inline bool IsHTMLOptionElement(const Node* node) {
  return node && IsHTMLOptionElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLOptionElement>(const Node& node) {
  return IsHTMLOptionElement(node);
}
template <>
inline bool IsElementOfType<const HTMLOptionElement>(const HTMLElement& element) {
  return IsHTMLOptionElement(element);
}

class HTMLOutputElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLOutputElement(const HTMLOutputElement&);
void IsHTMLOutputElement(const HTMLOutputElement*);

inline bool IsHTMLOutputElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kOutputTag);
}
inline bool IsHTMLOutputElement(const HTMLElement* element) {
  return element && IsHTMLOutputElement(*element);
}
inline bool IsHTMLOutputElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLOutputElement(To<HTMLElement>(node));
}
inline bool IsHTMLOutputElement(const Node* node) {
  return node && IsHTMLOutputElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLOutputElement>(const Node& node) {
  return IsHTMLOutputElement(node);
}
template <>
inline bool IsElementOfType<const HTMLOutputElement>(const HTMLElement& element) {
  return IsHTMLOutputElement(element);
}

class HTMLParagraphElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLParagraphElement(const HTMLParagraphElement&);
void IsHTMLParagraphElement(const HTMLParagraphElement*);

inline bool IsHTMLParagraphElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kPTag);
}
inline bool IsHTMLParagraphElement(const HTMLElement* element) {
  return element && IsHTMLParagraphElement(*element);
}
inline bool IsHTMLParagraphElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLParagraphElement(To<HTMLElement>(node));
}
inline bool IsHTMLParagraphElement(const Node* node) {
  return node && IsHTMLParagraphElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLParagraphElement>(const Node& node) {
  return IsHTMLParagraphElement(node);
}
template <>
inline bool IsElementOfType<const HTMLParagraphElement>(const HTMLElement& element) {
  return IsHTMLParagraphElement(element);
}

class HTMLParamElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLParamElement(const HTMLParamElement&);
void IsHTMLParamElement(const HTMLParamElement*);

inline bool IsHTMLParamElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kParamTag);
}
inline bool IsHTMLParamElement(const HTMLElement* element) {
  return element && IsHTMLParamElement(*element);
}
inline bool IsHTMLParamElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLParamElement(To<HTMLElement>(node));
}
inline bool IsHTMLParamElement(const Node* node) {
  return node && IsHTMLParamElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLParamElement>(const Node& node) {
  return IsHTMLParamElement(node);
}
template <>
inline bool IsElementOfType<const HTMLParamElement>(const HTMLElement& element) {
  return IsHTMLParamElement(element);
}

class HTMLPictureElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLPictureElement(const HTMLPictureElement&);
void IsHTMLPictureElement(const HTMLPictureElement*);

inline bool IsHTMLPictureElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kPictureTag);
}
inline bool IsHTMLPictureElement(const HTMLElement* element) {
  return element && IsHTMLPictureElement(*element);
}
inline bool IsHTMLPictureElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLPictureElement(To<HTMLElement>(node));
}
inline bool IsHTMLPictureElement(const Node* node) {
  return node && IsHTMLPictureElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLPictureElement>(const Node& node) {
  return IsHTMLPictureElement(node);
}
template <>
inline bool IsElementOfType<const HTMLPictureElement>(const HTMLElement& element) {
  return IsHTMLPictureElement(element);
}

class HTMLPortalElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLPortalElement(const HTMLPortalElement&);
void IsHTMLPortalElement(const HTMLPortalElement*);

inline bool IsHTMLPortalElement(const HTMLElement& element) {
  if (!RuntimeEnabledFeatures::PortalsEnabled())
    return false;
  return element.HasTagName(html_names::kPortalTag);
}
inline bool IsHTMLPortalElement(const HTMLElement* element) {
  return element && IsHTMLPortalElement(*element);
}
inline bool IsHTMLPortalElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLPortalElement(To<HTMLElement>(node));
}
inline bool IsHTMLPortalElement(const Node* node) {
  return node && IsHTMLPortalElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLPortalElement>(const Node& node) {
  return IsHTMLPortalElement(node);
}
template <>
inline bool IsElementOfType<const HTMLPortalElement>(const HTMLElement& element) {
  return IsHTMLPortalElement(element);
}

class HTMLProgressElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLProgressElement(const HTMLProgressElement&);
void IsHTMLProgressElement(const HTMLProgressElement*);

inline bool IsHTMLProgressElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kProgressTag);
}
inline bool IsHTMLProgressElement(const HTMLElement* element) {
  return element && IsHTMLProgressElement(*element);
}
inline bool IsHTMLProgressElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLProgressElement(To<HTMLElement>(node));
}
inline bool IsHTMLProgressElement(const Node* node) {
  return node && IsHTMLProgressElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLProgressElement>(const Node& node) {
  return IsHTMLProgressElement(node);
}
template <>
inline bool IsElementOfType<const HTMLProgressElement>(const HTMLElement& element) {
  return IsHTMLProgressElement(element);
}

class HTMLSelectElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLSelectElement(const HTMLSelectElement&);
void IsHTMLSelectElement(const HTMLSelectElement*);

inline bool IsHTMLSelectElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kSelectTag);
}
inline bool IsHTMLSelectElement(const HTMLElement* element) {
  return element && IsHTMLSelectElement(*element);
}
inline bool IsHTMLSelectElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLSelectElement(To<HTMLElement>(node));
}
inline bool IsHTMLSelectElement(const Node* node) {
  return node && IsHTMLSelectElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLSelectElement>(const Node& node) {
  return IsHTMLSelectElement(node);
}
template <>
inline bool IsElementOfType<const HTMLSelectElement>(const HTMLElement& element) {
  return IsHTMLSelectElement(element);
}

class HTMLShadowElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLShadowElement(const HTMLShadowElement&);
void IsHTMLShadowElement(const HTMLShadowElement*);

inline bool IsHTMLShadowElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kShadowTag);
}
inline bool IsHTMLShadowElement(const HTMLElement* element) {
  return element && IsHTMLShadowElement(*element);
}
inline bool IsHTMLShadowElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLShadowElement(To<HTMLElement>(node));
}
inline bool IsHTMLShadowElement(const Node* node) {
  return node && IsHTMLShadowElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLShadowElement>(const Node& node) {
  return IsHTMLShadowElement(node);
}
template <>
inline bool IsElementOfType<const HTMLShadowElement>(const HTMLElement& element) {
  return IsHTMLShadowElement(element);
}

class HTMLSlotElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLSlotElement(const HTMLSlotElement&);
void IsHTMLSlotElement(const HTMLSlotElement*);

inline bool IsHTMLSlotElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kSlotTag);
}
inline bool IsHTMLSlotElement(const HTMLElement* element) {
  return element && IsHTMLSlotElement(*element);
}
inline bool IsHTMLSlotElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLSlotElement(To<HTMLElement>(node));
}
inline bool IsHTMLSlotElement(const Node* node) {
  return node && IsHTMLSlotElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLSlotElement>(const Node& node) {
  return IsHTMLSlotElement(node);
}
template <>
inline bool IsElementOfType<const HTMLSlotElement>(const HTMLElement& element) {
  return IsHTMLSlotElement(element);
}

class HTMLSourceElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLSourceElement(const HTMLSourceElement&);
void IsHTMLSourceElement(const HTMLSourceElement*);

inline bool IsHTMLSourceElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kSourceTag);
}
inline bool IsHTMLSourceElement(const HTMLElement* element) {
  return element && IsHTMLSourceElement(*element);
}
inline bool IsHTMLSourceElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLSourceElement(To<HTMLElement>(node));
}
inline bool IsHTMLSourceElement(const Node* node) {
  return node && IsHTMLSourceElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLSourceElement>(const Node& node) {
  return IsHTMLSourceElement(node);
}
template <>
inline bool IsElementOfType<const HTMLSourceElement>(const HTMLElement& element) {
  return IsHTMLSourceElement(element);
}

class HTMLSpanElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLSpanElement(const HTMLSpanElement&);
void IsHTMLSpanElement(const HTMLSpanElement*);

inline bool IsHTMLSpanElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kSpanTag);
}
inline bool IsHTMLSpanElement(const HTMLElement* element) {
  return element && IsHTMLSpanElement(*element);
}
inline bool IsHTMLSpanElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLSpanElement(To<HTMLElement>(node));
}
inline bool IsHTMLSpanElement(const Node* node) {
  return node && IsHTMLSpanElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLSpanElement>(const Node& node) {
  return IsHTMLSpanElement(node);
}
template <>
inline bool IsElementOfType<const HTMLSpanElement>(const HTMLElement& element) {
  return IsHTMLSpanElement(element);
}

class HTMLTableCaptionElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLTableCaptionElement(const HTMLTableCaptionElement&);
void IsHTMLTableCaptionElement(const HTMLTableCaptionElement*);

inline bool IsHTMLTableCaptionElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kCaptionTag);
}
inline bool IsHTMLTableCaptionElement(const HTMLElement* element) {
  return element && IsHTMLTableCaptionElement(*element);
}
inline bool IsHTMLTableCaptionElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLTableCaptionElement(To<HTMLElement>(node));
}
inline bool IsHTMLTableCaptionElement(const Node* node) {
  return node && IsHTMLTableCaptionElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLTableCaptionElement>(const Node& node) {
  return IsHTMLTableCaptionElement(node);
}
template <>
inline bool IsElementOfType<const HTMLTableCaptionElement>(const HTMLElement& element) {
  return IsHTMLTableCaptionElement(element);
}

class HTMLTableElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLTableElement(const HTMLTableElement&);
void IsHTMLTableElement(const HTMLTableElement*);

inline bool IsHTMLTableElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kTableTag);
}
inline bool IsHTMLTableElement(const HTMLElement* element) {
  return element && IsHTMLTableElement(*element);
}
inline bool IsHTMLTableElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLTableElement(To<HTMLElement>(node));
}
inline bool IsHTMLTableElement(const Node* node) {
  return node && IsHTMLTableElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLTableElement>(const Node& node) {
  return IsHTMLTableElement(node);
}
template <>
inline bool IsElementOfType<const HTMLTableElement>(const HTMLElement& element) {
  return IsHTMLTableElement(element);
}

class HTMLTableRowElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLTableRowElement(const HTMLTableRowElement&);
void IsHTMLTableRowElement(const HTMLTableRowElement*);

inline bool IsHTMLTableRowElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kTrTag);
}
inline bool IsHTMLTableRowElement(const HTMLElement* element) {
  return element && IsHTMLTableRowElement(*element);
}
inline bool IsHTMLTableRowElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLTableRowElement(To<HTMLElement>(node));
}
inline bool IsHTMLTableRowElement(const Node* node) {
  return node && IsHTMLTableRowElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLTableRowElement>(const Node& node) {
  return IsHTMLTableRowElement(node);
}
template <>
inline bool IsElementOfType<const HTMLTableRowElement>(const HTMLElement& element) {
  return IsHTMLTableRowElement(element);
}

class HTMLTemplateElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLTemplateElement(const HTMLTemplateElement&);
void IsHTMLTemplateElement(const HTMLTemplateElement*);

inline bool IsHTMLTemplateElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kTemplateTag);
}
inline bool IsHTMLTemplateElement(const HTMLElement* element) {
  return element && IsHTMLTemplateElement(*element);
}
inline bool IsHTMLTemplateElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLTemplateElement(To<HTMLElement>(node));
}
inline bool IsHTMLTemplateElement(const Node* node) {
  return node && IsHTMLTemplateElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLTemplateElement>(const Node& node) {
  return IsHTMLTemplateElement(node);
}
template <>
inline bool IsElementOfType<const HTMLTemplateElement>(const HTMLElement& element) {
  return IsHTMLTemplateElement(element);
}

class HTMLTextAreaElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLTextAreaElement(const HTMLTextAreaElement&);
void IsHTMLTextAreaElement(const HTMLTextAreaElement*);

inline bool IsHTMLTextAreaElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kTextareaTag);
}
inline bool IsHTMLTextAreaElement(const HTMLElement* element) {
  return element && IsHTMLTextAreaElement(*element);
}
inline bool IsHTMLTextAreaElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLTextAreaElement(To<HTMLElement>(node));
}
inline bool IsHTMLTextAreaElement(const Node* node) {
  return node && IsHTMLTextAreaElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLTextAreaElement>(const Node& node) {
  return IsHTMLTextAreaElement(node);
}
template <>
inline bool IsElementOfType<const HTMLTextAreaElement>(const HTMLElement& element) {
  return IsHTMLTextAreaElement(element);
}

class HTMLTimeElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLTimeElement(const HTMLTimeElement&);
void IsHTMLTimeElement(const HTMLTimeElement*);

inline bool IsHTMLTimeElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kTimeTag);
}
inline bool IsHTMLTimeElement(const HTMLElement* element) {
  return element && IsHTMLTimeElement(*element);
}
inline bool IsHTMLTimeElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLTimeElement(To<HTMLElement>(node));
}
inline bool IsHTMLTimeElement(const Node* node) {
  return node && IsHTMLTimeElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLTimeElement>(const Node& node) {
  return IsHTMLTimeElement(node);
}
template <>
inline bool IsElementOfType<const HTMLTimeElement>(const HTMLElement& element) {
  return IsHTMLTimeElement(element);
}

class HTMLTitleElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLTitleElement(const HTMLTitleElement&);
void IsHTMLTitleElement(const HTMLTitleElement*);

inline bool IsHTMLTitleElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kTitleTag);
}
inline bool IsHTMLTitleElement(const HTMLElement* element) {
  return element && IsHTMLTitleElement(*element);
}
inline bool IsHTMLTitleElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLTitleElement(To<HTMLElement>(node));
}
inline bool IsHTMLTitleElement(const Node* node) {
  return node && IsHTMLTitleElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLTitleElement>(const Node& node) {
  return IsHTMLTitleElement(node);
}
template <>
inline bool IsElementOfType<const HTMLTitleElement>(const HTMLElement& element) {
  return IsHTMLTitleElement(element);
}

class HTMLTrackElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLTrackElement(const HTMLTrackElement&);
void IsHTMLTrackElement(const HTMLTrackElement*);

inline bool IsHTMLTrackElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kTrackTag);
}
inline bool IsHTMLTrackElement(const HTMLElement* element) {
  return element && IsHTMLTrackElement(*element);
}
inline bool IsHTMLTrackElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLTrackElement(To<HTMLElement>(node));
}
inline bool IsHTMLTrackElement(const Node* node) {
  return node && IsHTMLTrackElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLTrackElement>(const Node& node) {
  return IsHTMLTrackElement(node);
}
template <>
inline bool IsElementOfType<const HTMLTrackElement>(const HTMLElement& element) {
  return IsHTMLTrackElement(element);
}

class HTMLUListElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLUListElement(const HTMLUListElement&);
void IsHTMLUListElement(const HTMLUListElement*);

inline bool IsHTMLUListElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kUlTag);
}
inline bool IsHTMLUListElement(const HTMLElement* element) {
  return element && IsHTMLUListElement(*element);
}
inline bool IsHTMLUListElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLUListElement(To<HTMLElement>(node));
}
inline bool IsHTMLUListElement(const Node* node) {
  return node && IsHTMLUListElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLUListElement>(const Node& node) {
  return IsHTMLUListElement(node);
}
template <>
inline bool IsElementOfType<const HTMLUListElement>(const HTMLElement& element) {
  return IsHTMLUListElement(element);
}

class HTMLVideoElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLVideoElement(const HTMLVideoElement&);
void IsHTMLVideoElement(const HTMLVideoElement*);

inline bool IsHTMLVideoElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kVideoTag);
}
inline bool IsHTMLVideoElement(const HTMLElement* element) {
  return element && IsHTMLVideoElement(*element);
}
inline bool IsHTMLVideoElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLVideoElement(To<HTMLElement>(node));
}
inline bool IsHTMLVideoElement(const Node* node) {
  return node && IsHTMLVideoElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLVideoElement>(const Node& node) {
  return IsHTMLVideoElement(node);
}
template <>
inline bool IsElementOfType<const HTMLVideoElement>(const HTMLElement& element) {
  return IsHTMLVideoElement(element);
}

class HTMLEmbedElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLEmbedElement(const HTMLEmbedElement&);
void IsHTMLEmbedElement(const HTMLEmbedElement*);

inline bool IsHTMLEmbedElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kEmbedTag);
}
inline bool IsHTMLEmbedElement(const HTMLElement* element) {
  return element && IsHTMLEmbedElement(*element);
}
inline bool IsHTMLEmbedElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLEmbedElement(To<HTMLElement>(node));
}
inline bool IsHTMLEmbedElement(const Node* node) {
  return node && IsHTMLEmbedElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLEmbedElement>(const Node& node) {
  return IsHTMLEmbedElement(node);
}
template <>
inline bool IsElementOfType<const HTMLEmbedElement>(const HTMLElement& element) {
  return IsHTMLEmbedElement(element);
}

class HTMLImageElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLImageElement(const HTMLImageElement&);
void IsHTMLImageElement(const HTMLImageElement*);

inline bool IsHTMLImageElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kImgTag);
}
inline bool IsHTMLImageElement(const HTMLElement* element) {
  return element && IsHTMLImageElement(*element);
}
inline bool IsHTMLImageElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLImageElement(To<HTMLElement>(node));
}
inline bool IsHTMLImageElement(const Node* node) {
  return node && IsHTMLImageElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLImageElement>(const Node& node) {
  return IsHTMLImageElement(node);
}
template <>
inline bool IsElementOfType<const HTMLImageElement>(const HTMLElement& element) {
  return IsHTMLImageElement(element);
}

class HTMLInputElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLInputElement(const HTMLInputElement&);
void IsHTMLInputElement(const HTMLInputElement*);

inline bool IsHTMLInputElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kInputTag);
}
inline bool IsHTMLInputElement(const HTMLElement* element) {
  return element && IsHTMLInputElement(*element);
}
inline bool IsHTMLInputElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLInputElement(To<HTMLElement>(node));
}
inline bool IsHTMLInputElement(const Node* node) {
  return node && IsHTMLInputElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLInputElement>(const Node& node) {
  return IsHTMLInputElement(node);
}
template <>
inline bool IsElementOfType<const HTMLInputElement>(const HTMLElement& element) {
  return IsHTMLInputElement(element);
}

class HTMLLinkElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLLinkElement(const HTMLLinkElement&);
void IsHTMLLinkElement(const HTMLLinkElement*);

inline bool IsHTMLLinkElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kLinkTag);
}
inline bool IsHTMLLinkElement(const HTMLElement* element) {
  return element && IsHTMLLinkElement(*element);
}
inline bool IsHTMLLinkElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLLinkElement(To<HTMLElement>(node));
}
inline bool IsHTMLLinkElement(const Node* node) {
  return node && IsHTMLLinkElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLLinkElement>(const Node& node) {
  return IsHTMLLinkElement(node);
}
template <>
inline bool IsElementOfType<const HTMLLinkElement>(const HTMLElement& element) {
  return IsHTMLLinkElement(element);
}

class HTMLObjectElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLObjectElement(const HTMLObjectElement&);
void IsHTMLObjectElement(const HTMLObjectElement*);

inline bool IsHTMLObjectElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kObjectTag);
}
inline bool IsHTMLObjectElement(const HTMLElement* element) {
  return element && IsHTMLObjectElement(*element);
}
inline bool IsHTMLObjectElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLObjectElement(To<HTMLElement>(node));
}
inline bool IsHTMLObjectElement(const Node* node) {
  return node && IsHTMLObjectElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLObjectElement>(const Node& node) {
  return IsHTMLObjectElement(node);
}
template <>
inline bool IsElementOfType<const HTMLObjectElement>(const HTMLElement& element) {
  return IsHTMLObjectElement(element);
}

class HTMLScriptElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLScriptElement(const HTMLScriptElement&);
void IsHTMLScriptElement(const HTMLScriptElement*);

inline bool IsHTMLScriptElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kScriptTag);
}
inline bool IsHTMLScriptElement(const HTMLElement* element) {
  return element && IsHTMLScriptElement(*element);
}
inline bool IsHTMLScriptElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLScriptElement(To<HTMLElement>(node));
}
inline bool IsHTMLScriptElement(const Node* node) {
  return node && IsHTMLScriptElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLScriptElement>(const Node& node) {
  return IsHTMLScriptElement(node);
}
template <>
inline bool IsElementOfType<const HTMLScriptElement>(const HTMLElement& element) {
  return IsHTMLScriptElement(element);
}

class HTMLStyleElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLStyleElement(const HTMLStyleElement&);
void IsHTMLStyleElement(const HTMLStyleElement*);

inline bool IsHTMLStyleElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kStyleTag);
}
inline bool IsHTMLStyleElement(const HTMLElement* element) {
  return element && IsHTMLStyleElement(*element);
}
inline bool IsHTMLStyleElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLStyleElement(To<HTMLElement>(node));
}
inline bool IsHTMLStyleElement(const Node* node) {
  return node && IsHTMLStyleElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLStyleElement>(const Node& node) {
  return IsHTMLStyleElement(node);
}
template <>
inline bool IsElementOfType<const HTMLStyleElement>(const HTMLElement& element) {
  return IsHTMLStyleElement(element);
}

class HTMLBDIElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLBDIElement(const HTMLBDIElement&);
void IsHTMLBDIElement(const HTMLBDIElement*);

inline bool IsHTMLBDIElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kBdiTag);
}
inline bool IsHTMLBDIElement(const HTMLElement* element) {
  return element && IsHTMLBDIElement(*element);
}
inline bool IsHTMLBDIElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLBDIElement(To<HTMLElement>(node));
}
inline bool IsHTMLBDIElement(const Node* node) {
  return node && IsHTMLBDIElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLBDIElement>(const Node& node) {
  return IsHTMLBDIElement(node);
}
template <>
inline bool IsElementOfType<const HTMLBDIElement>(const HTMLElement& element) {
  return IsHTMLBDIElement(element);
}

class HTMLNoEmbedElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLNoEmbedElement(const HTMLNoEmbedElement&);
void IsHTMLNoEmbedElement(const HTMLNoEmbedElement*);

inline bool IsHTMLNoEmbedElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kNoembedTag);
}
inline bool IsHTMLNoEmbedElement(const HTMLElement* element) {
  return element && IsHTMLNoEmbedElement(*element);
}
inline bool IsHTMLNoEmbedElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLNoEmbedElement(To<HTMLElement>(node));
}
inline bool IsHTMLNoEmbedElement(const Node* node) {
  return node && IsHTMLNoEmbedElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLNoEmbedElement>(const Node& node) {
  return IsHTMLNoEmbedElement(node);
}
template <>
inline bool IsElementOfType<const HTMLNoEmbedElement>(const HTMLElement& element) {
  return IsHTMLNoEmbedElement(element);
}

class HTMLNoScriptElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLNoScriptElement(const HTMLNoScriptElement&);
void IsHTMLNoScriptElement(const HTMLNoScriptElement*);

inline bool IsHTMLNoScriptElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kNoscriptTag);
}
inline bool IsHTMLNoScriptElement(const HTMLElement* element) {
  return element && IsHTMLNoScriptElement(*element);
}
inline bool IsHTMLNoScriptElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLNoScriptElement(To<HTMLElement>(node));
}
inline bool IsHTMLNoScriptElement(const Node* node) {
  return node && IsHTMLNoScriptElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLNoScriptElement>(const Node& node) {
  return IsHTMLNoScriptElement(node);
}
template <>
inline bool IsElementOfType<const HTMLNoScriptElement>(const HTMLElement& element) {
  return IsHTMLNoScriptElement(element);
}

class HTMLRTElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLRTElement(const HTMLRTElement&);
void IsHTMLRTElement(const HTMLRTElement*);

inline bool IsHTMLRTElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kRtTag);
}
inline bool IsHTMLRTElement(const HTMLElement* element) {
  return element && IsHTMLRTElement(*element);
}
inline bool IsHTMLRTElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLRTElement(To<HTMLElement>(node));
}
inline bool IsHTMLRTElement(const Node* node) {
  return node && IsHTMLRTElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLRTElement>(const Node& node) {
  return IsHTMLRTElement(node);
}
template <>
inline bool IsElementOfType<const HTMLRTElement>(const HTMLElement& element) {
  return IsHTMLRTElement(element);
}

class HTMLRubyElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLRubyElement(const HTMLRubyElement&);
void IsHTMLRubyElement(const HTMLRubyElement*);

inline bool IsHTMLRubyElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kRubyTag);
}
inline bool IsHTMLRubyElement(const HTMLElement* element) {
  return element && IsHTMLRubyElement(*element);
}
inline bool IsHTMLRubyElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLRubyElement(To<HTMLElement>(node));
}
inline bool IsHTMLRubyElement(const Node* node) {
  return node && IsHTMLRubyElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLRubyElement>(const Node& node) {
  return IsHTMLRubyElement(node);
}
template <>
inline bool IsElementOfType<const HTMLRubyElement>(const HTMLElement& element) {
  return IsHTMLRubyElement(element);
}

class HTMLSummaryElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLSummaryElement(const HTMLSummaryElement&);
void IsHTMLSummaryElement(const HTMLSummaryElement*);

inline bool IsHTMLSummaryElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kSummaryTag);
}
inline bool IsHTMLSummaryElement(const HTMLElement* element) {
  return element && IsHTMLSummaryElement(*element);
}
inline bool IsHTMLSummaryElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLSummaryElement(To<HTMLElement>(node));
}
inline bool IsHTMLSummaryElement(const Node* node) {
  return node && IsHTMLSummaryElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLSummaryElement>(const Node& node) {
  return IsHTMLSummaryElement(node);
}
template <>
inline bool IsElementOfType<const HTMLSummaryElement>(const HTMLElement& element) {
  return IsHTMLSummaryElement(element);
}

class HTMLWBRElement;
// Catch unnecessary runtime check of type known at compile time.
void IsHTMLWBRElement(const HTMLWBRElement&);
void IsHTMLWBRElement(const HTMLWBRElement*);

inline bool IsHTMLWBRElement(const HTMLElement& element) {
  return element.HasTagName(html_names::kWbrTag);
}
inline bool IsHTMLWBRElement(const HTMLElement* element) {
  return element && IsHTMLWBRElement(*element);
}
inline bool IsHTMLWBRElement(const Node& node) {
  return node.IsHTMLElement() && IsHTMLWBRElement(To<HTMLElement>(node));
}
inline bool IsHTMLWBRElement(const Node* node) {
  return node && IsHTMLWBRElement(*node);
}
template <>
inline bool IsElementOfType<const HTMLWBRElement>(const Node& node) {
  return IsHTMLWBRElement(node);
}
template <>
inline bool IsElementOfType<const HTMLWBRElement>(const HTMLElement& element) {
  return IsHTMLWBRElement(element);
}

// Using macros because the types are forward-declared and we don't want to use
// reinterpret_cast in the casting functions above. reinterpret_cast would be
// unsafe due to multiple inheritence.

// TODO(abhijeet@igalia.com): Remove below type casting macros.
template <>
struct DowncastTraits<HTMLAnchorElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kATag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLAnchorElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLAnchorElement(x) blink::To<blink::HTMLAnchorElement>(x)
#define ToHTMLAnchorElementOrNull(x) blink::DynamicTo<blink::HTMLAnchorElement>(x)

template <>
struct DowncastTraits<HTMLAreaElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kAreaTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLAreaElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLAreaElement(x) blink::To<blink::HTMLAreaElement>(x)
#define ToHTMLAreaElementOrNull(x) blink::DynamicTo<blink::HTMLAreaElement>(x)

template <>
struct DowncastTraits<HTMLAudioElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kAudioTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLAudioElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLAudioElement(x) blink::To<blink::HTMLAudioElement>(x)
#define ToHTMLAudioElementOrNull(x) blink::DynamicTo<blink::HTMLAudioElement>(x)

template <>
struct DowncastTraits<HTMLBRElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kBrTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLBRElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLBRElement(x) blink::To<blink::HTMLBRElement>(x)
#define ToHTMLBRElementOrNull(x) blink::DynamicTo<blink::HTMLBRElement>(x)

template <>
struct DowncastTraits<HTMLBaseElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kBaseTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLBaseElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLBaseElement(x) blink::To<blink::HTMLBaseElement>(x)
#define ToHTMLBaseElementOrNull(x) blink::DynamicTo<blink::HTMLBaseElement>(x)

template <>
struct DowncastTraits<HTMLBodyElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kBodyTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLBodyElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLBodyElement(x) blink::To<blink::HTMLBodyElement>(x)
#define ToHTMLBodyElementOrNull(x) blink::DynamicTo<blink::HTMLBodyElement>(x)

template <>
struct DowncastTraits<HTMLButtonElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kButtonTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLButtonElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLButtonElement(x) blink::To<blink::HTMLButtonElement>(x)
#define ToHTMLButtonElementOrNull(x) blink::DynamicTo<blink::HTMLButtonElement>(x)

template <>
struct DowncastTraits<HTMLCanvasElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kCanvasTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLCanvasElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLCanvasElement(x) blink::To<blink::HTMLCanvasElement>(x)
#define ToHTMLCanvasElementOrNull(x) blink::DynamicTo<blink::HTMLCanvasElement>(x)

template <>
struct DowncastTraits<HTMLContentElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kContentTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLContentElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLContentElement(x) blink::To<blink::HTMLContentElement>(x)
#define ToHTMLContentElementOrNull(x) blink::DynamicTo<blink::HTMLContentElement>(x)

template <>
struct DowncastTraits<HTMLDListElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kDlTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLDListElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLDListElement(x) blink::To<blink::HTMLDListElement>(x)
#define ToHTMLDListElementOrNull(x) blink::DynamicTo<blink::HTMLDListElement>(x)

template <>
struct DowncastTraits<HTMLDataElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kDataTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLDataElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLDataElement(x) blink::To<blink::HTMLDataElement>(x)
#define ToHTMLDataElementOrNull(x) blink::DynamicTo<blink::HTMLDataElement>(x)

template <>
struct DowncastTraits<HTMLDataListElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kDatalistTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLDataListElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLDataListElement(x) blink::To<blink::HTMLDataListElement>(x)
#define ToHTMLDataListElementOrNull(x) blink::DynamicTo<blink::HTMLDataListElement>(x)

template <>
struct DowncastTraits<HTMLDetailsElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kDetailsTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLDetailsElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLDetailsElement(x) blink::To<blink::HTMLDetailsElement>(x)
#define ToHTMLDetailsElementOrNull(x) blink::DynamicTo<blink::HTMLDetailsElement>(x)

template <>
struct DowncastTraits<HTMLDialogElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kDialogTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLDialogElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLDialogElement(x) blink::To<blink::HTMLDialogElement>(x)
#define ToHTMLDialogElementOrNull(x) blink::DynamicTo<blink::HTMLDialogElement>(x)

template <>
struct DowncastTraits<HTMLDirectoryElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kDirTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLDirectoryElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLDirectoryElement(x) blink::To<blink::HTMLDirectoryElement>(x)
#define ToHTMLDirectoryElementOrNull(x) blink::DynamicTo<blink::HTMLDirectoryElement>(x)

template <>
struct DowncastTraits<HTMLDivElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kDivTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLDivElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLDivElement(x) blink::To<blink::HTMLDivElement>(x)
#define ToHTMLDivElementOrNull(x) blink::DynamicTo<blink::HTMLDivElement>(x)

template <>
struct DowncastTraits<HTMLFieldSetElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kFieldsetTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLFieldSetElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLFieldSetElement(x) blink::To<blink::HTMLFieldSetElement>(x)
#define ToHTMLFieldSetElementOrNull(x) blink::DynamicTo<blink::HTMLFieldSetElement>(x)

template <>
struct DowncastTraits<HTMLFontElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kFontTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLFontElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLFontElement(x) blink::To<blink::HTMLFontElement>(x)
#define ToHTMLFontElementOrNull(x) blink::DynamicTo<blink::HTMLFontElement>(x)

template <>
struct DowncastTraits<HTMLFormElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kFormTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLFormElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLFormElement(x) blink::To<blink::HTMLFormElement>(x)
#define ToHTMLFormElementOrNull(x) blink::DynamicTo<blink::HTMLFormElement>(x)

template <>
struct DowncastTraits<HTMLFrameElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kFrameTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLFrameElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLFrameElement(x) blink::To<blink::HTMLFrameElement>(x)
#define ToHTMLFrameElementOrNull(x) blink::DynamicTo<blink::HTMLFrameElement>(x)

template <>
struct DowncastTraits<HTMLFrameSetElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kFramesetTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLFrameSetElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLFrameSetElement(x) blink::To<blink::HTMLFrameSetElement>(x)
#define ToHTMLFrameSetElementOrNull(x) blink::DynamicTo<blink::HTMLFrameSetElement>(x)

template <>
struct DowncastTraits<HTMLHRElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kHrTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLHRElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLHRElement(x) blink::To<blink::HTMLHRElement>(x)
#define ToHTMLHRElementOrNull(x) blink::DynamicTo<blink::HTMLHRElement>(x)

template <>
struct DowncastTraits<HTMLHeadElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kHeadTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLHeadElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLHeadElement(x) blink::To<blink::HTMLHeadElement>(x)
#define ToHTMLHeadElementOrNull(x) blink::DynamicTo<blink::HTMLHeadElement>(x)

template <>
struct DowncastTraits<HTMLHtmlElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kHTMLTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLHtmlElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLHtmlElement(x) blink::To<blink::HTMLHtmlElement>(x)
#define ToHTMLHtmlElementOrNull(x) blink::DynamicTo<blink::HTMLHtmlElement>(x)

template <>
struct DowncastTraits<HTMLIFrameElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kIFrameTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLIFrameElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLIFrameElement(x) blink::To<blink::HTMLIFrameElement>(x)
#define ToHTMLIFrameElementOrNull(x) blink::DynamicTo<blink::HTMLIFrameElement>(x)

template <>
struct DowncastTraits<HTMLLIElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kLiTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLLIElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLLIElement(x) blink::To<blink::HTMLLIElement>(x)
#define ToHTMLLIElementOrNull(x) blink::DynamicTo<blink::HTMLLIElement>(x)

template <>
struct DowncastTraits<HTMLLabelElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kLabelTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLLabelElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLLabelElement(x) blink::To<blink::HTMLLabelElement>(x)
#define ToHTMLLabelElementOrNull(x) blink::DynamicTo<blink::HTMLLabelElement>(x)

template <>
struct DowncastTraits<HTMLLegendElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kLegendTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLLegendElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLLegendElement(x) blink::To<blink::HTMLLegendElement>(x)
#define ToHTMLLegendElementOrNull(x) blink::DynamicTo<blink::HTMLLegendElement>(x)

template <>
struct DowncastTraits<HTMLMapElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kMapTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLMapElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLMapElement(x) blink::To<blink::HTMLMapElement>(x)
#define ToHTMLMapElementOrNull(x) blink::DynamicTo<blink::HTMLMapElement>(x)

template <>
struct DowncastTraits<HTMLMarqueeElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kMarqueeTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLMarqueeElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLMarqueeElement(x) blink::To<blink::HTMLMarqueeElement>(x)
#define ToHTMLMarqueeElementOrNull(x) blink::DynamicTo<blink::HTMLMarqueeElement>(x)

template <>
struct DowncastTraits<HTMLMenuElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kMenuTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLMenuElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLMenuElement(x) blink::To<blink::HTMLMenuElement>(x)
#define ToHTMLMenuElementOrNull(x) blink::DynamicTo<blink::HTMLMenuElement>(x)

template <>
struct DowncastTraits<HTMLMetaElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kMetaTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLMetaElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLMetaElement(x) blink::To<blink::HTMLMetaElement>(x)
#define ToHTMLMetaElementOrNull(x) blink::DynamicTo<blink::HTMLMetaElement>(x)

template <>
struct DowncastTraits<HTMLMeterElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kMeterTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLMeterElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLMeterElement(x) blink::To<blink::HTMLMeterElement>(x)
#define ToHTMLMeterElementOrNull(x) blink::DynamicTo<blink::HTMLMeterElement>(x)

template <>
struct DowncastTraits<HTMLOListElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kOlTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLOListElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLOListElement(x) blink::To<blink::HTMLOListElement>(x)
#define ToHTMLOListElementOrNull(x) blink::DynamicTo<blink::HTMLOListElement>(x)

template <>
struct DowncastTraits<HTMLOptGroupElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kOptgroupTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLOptGroupElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLOptGroupElement(x) blink::To<blink::HTMLOptGroupElement>(x)
#define ToHTMLOptGroupElementOrNull(x) blink::DynamicTo<blink::HTMLOptGroupElement>(x)

template <>
struct DowncastTraits<HTMLOptionElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kOptionTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLOptionElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLOptionElement(x) blink::To<blink::HTMLOptionElement>(x)
#define ToHTMLOptionElementOrNull(x) blink::DynamicTo<blink::HTMLOptionElement>(x)

template <>
struct DowncastTraits<HTMLOutputElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kOutputTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLOutputElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLOutputElement(x) blink::To<blink::HTMLOutputElement>(x)
#define ToHTMLOutputElementOrNull(x) blink::DynamicTo<blink::HTMLOutputElement>(x)

template <>
struct DowncastTraits<HTMLParagraphElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kPTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLParagraphElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLParagraphElement(x) blink::To<blink::HTMLParagraphElement>(x)
#define ToHTMLParagraphElementOrNull(x) blink::DynamicTo<blink::HTMLParagraphElement>(x)

template <>
struct DowncastTraits<HTMLParamElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kParamTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLParamElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLParamElement(x) blink::To<blink::HTMLParamElement>(x)
#define ToHTMLParamElementOrNull(x) blink::DynamicTo<blink::HTMLParamElement>(x)

template <>
struct DowncastTraits<HTMLPictureElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kPictureTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLPictureElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLPictureElement(x) blink::To<blink::HTMLPictureElement>(x)
#define ToHTMLPictureElementOrNull(x) blink::DynamicTo<blink::HTMLPictureElement>(x)

template <>
struct DowncastTraits<HTMLPortalElement> {
  static bool AllowFrom(const Element& element) {
    if (!RuntimeEnabledFeatures::PortalsEnabled())
      return false;
    return element.HasTagName(html_names::kPortalTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLPortalElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLPortalElement(x) blink::To<blink::HTMLPortalElement>(x)
#define ToHTMLPortalElementOrNull(x) blink::DynamicTo<blink::HTMLPortalElement>(x)

template <>
struct DowncastTraits<HTMLProgressElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kProgressTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLProgressElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLProgressElement(x) blink::To<blink::HTMLProgressElement>(x)
#define ToHTMLProgressElementOrNull(x) blink::DynamicTo<blink::HTMLProgressElement>(x)

template <>
struct DowncastTraits<HTMLSelectElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kSelectTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLSelectElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLSelectElement(x) blink::To<blink::HTMLSelectElement>(x)
#define ToHTMLSelectElementOrNull(x) blink::DynamicTo<blink::HTMLSelectElement>(x)

template <>
struct DowncastTraits<HTMLShadowElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kShadowTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLShadowElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLShadowElement(x) blink::To<blink::HTMLShadowElement>(x)
#define ToHTMLShadowElementOrNull(x) blink::DynamicTo<blink::HTMLShadowElement>(x)

template <>
struct DowncastTraits<HTMLSlotElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kSlotTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLSlotElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLSlotElement(x) blink::To<blink::HTMLSlotElement>(x)
#define ToHTMLSlotElementOrNull(x) blink::DynamicTo<blink::HTMLSlotElement>(x)

template <>
struct DowncastTraits<HTMLSourceElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kSourceTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLSourceElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLSourceElement(x) blink::To<blink::HTMLSourceElement>(x)
#define ToHTMLSourceElementOrNull(x) blink::DynamicTo<blink::HTMLSourceElement>(x)

template <>
struct DowncastTraits<HTMLSpanElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kSpanTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLSpanElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLSpanElement(x) blink::To<blink::HTMLSpanElement>(x)
#define ToHTMLSpanElementOrNull(x) blink::DynamicTo<blink::HTMLSpanElement>(x)

template <>
struct DowncastTraits<HTMLTableCaptionElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kCaptionTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLTableCaptionElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLTableCaptionElement(x) blink::To<blink::HTMLTableCaptionElement>(x)
#define ToHTMLTableCaptionElementOrNull(x) blink::DynamicTo<blink::HTMLTableCaptionElement>(x)

template <>
struct DowncastTraits<HTMLTableElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kTableTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLTableElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLTableElement(x) blink::To<blink::HTMLTableElement>(x)
#define ToHTMLTableElementOrNull(x) blink::DynamicTo<blink::HTMLTableElement>(x)

template <>
struct DowncastTraits<HTMLTableRowElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kTrTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLTableRowElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLTableRowElement(x) blink::To<blink::HTMLTableRowElement>(x)
#define ToHTMLTableRowElementOrNull(x) blink::DynamicTo<blink::HTMLTableRowElement>(x)

template <>
struct DowncastTraits<HTMLTemplateElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kTemplateTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLTemplateElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLTemplateElement(x) blink::To<blink::HTMLTemplateElement>(x)
#define ToHTMLTemplateElementOrNull(x) blink::DynamicTo<blink::HTMLTemplateElement>(x)

template <>
struct DowncastTraits<HTMLTextAreaElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kTextareaTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLTextAreaElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLTextAreaElement(x) blink::To<blink::HTMLTextAreaElement>(x)
#define ToHTMLTextAreaElementOrNull(x) blink::DynamicTo<blink::HTMLTextAreaElement>(x)

template <>
struct DowncastTraits<HTMLTimeElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kTimeTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLTimeElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLTimeElement(x) blink::To<blink::HTMLTimeElement>(x)
#define ToHTMLTimeElementOrNull(x) blink::DynamicTo<blink::HTMLTimeElement>(x)

template <>
struct DowncastTraits<HTMLTitleElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kTitleTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLTitleElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLTitleElement(x) blink::To<blink::HTMLTitleElement>(x)
#define ToHTMLTitleElementOrNull(x) blink::DynamicTo<blink::HTMLTitleElement>(x)

template <>
struct DowncastTraits<HTMLTrackElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kTrackTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLTrackElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLTrackElement(x) blink::To<blink::HTMLTrackElement>(x)
#define ToHTMLTrackElementOrNull(x) blink::DynamicTo<blink::HTMLTrackElement>(x)

template <>
struct DowncastTraits<HTMLUListElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kUlTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLUListElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLUListElement(x) blink::To<blink::HTMLUListElement>(x)
#define ToHTMLUListElementOrNull(x) blink::DynamicTo<blink::HTMLUListElement>(x)

template <>
struct DowncastTraits<HTMLVideoElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kVideoTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLVideoElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLVideoElement(x) blink::To<blink::HTMLVideoElement>(x)
#define ToHTMLVideoElementOrNull(x) blink::DynamicTo<blink::HTMLVideoElement>(x)

template <>
struct DowncastTraits<HTMLEmbedElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kEmbedTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLEmbedElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLEmbedElement(x) blink::To<blink::HTMLEmbedElement>(x)
#define ToHTMLEmbedElementOrNull(x) blink::DynamicTo<blink::HTMLEmbedElement>(x)

template <>
struct DowncastTraits<HTMLImageElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kImgTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLImageElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLImageElement(x) blink::To<blink::HTMLImageElement>(x)
#define ToHTMLImageElementOrNull(x) blink::DynamicTo<blink::HTMLImageElement>(x)

template <>
struct DowncastTraits<HTMLInputElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kInputTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLInputElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLInputElement(x) blink::To<blink::HTMLInputElement>(x)
#define ToHTMLInputElementOrNull(x) blink::DynamicTo<blink::HTMLInputElement>(x)

template <>
struct DowncastTraits<HTMLLinkElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kLinkTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLLinkElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLLinkElement(x) blink::To<blink::HTMLLinkElement>(x)
#define ToHTMLLinkElementOrNull(x) blink::DynamicTo<blink::HTMLLinkElement>(x)

template <>
struct DowncastTraits<HTMLObjectElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kObjectTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLObjectElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLObjectElement(x) blink::To<blink::HTMLObjectElement>(x)
#define ToHTMLObjectElementOrNull(x) blink::DynamicTo<blink::HTMLObjectElement>(x)

template <>
struct DowncastTraits<HTMLScriptElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kScriptTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLScriptElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLScriptElement(x) blink::To<blink::HTMLScriptElement>(x)
#define ToHTMLScriptElementOrNull(x) blink::DynamicTo<blink::HTMLScriptElement>(x)

template <>
struct DowncastTraits<HTMLStyleElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kStyleTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLStyleElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLStyleElement(x) blink::To<blink::HTMLStyleElement>(x)
#define ToHTMLStyleElementOrNull(x) blink::DynamicTo<blink::HTMLStyleElement>(x)

template <>
struct DowncastTraits<HTMLBDIElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kBdiTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLBDIElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLBDIElement(x) blink::To<blink::HTMLBDIElement>(x)
#define ToHTMLBDIElementOrNull(x) blink::DynamicTo<blink::HTMLBDIElement>(x)

template <>
struct DowncastTraits<HTMLNoEmbedElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kNoembedTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLNoEmbedElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLNoEmbedElement(x) blink::To<blink::HTMLNoEmbedElement>(x)
#define ToHTMLNoEmbedElementOrNull(x) blink::DynamicTo<blink::HTMLNoEmbedElement>(x)

template <>
struct DowncastTraits<HTMLNoScriptElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kNoscriptTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLNoScriptElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLNoScriptElement(x) blink::To<blink::HTMLNoScriptElement>(x)
#define ToHTMLNoScriptElementOrNull(x) blink::DynamicTo<blink::HTMLNoScriptElement>(x)

template <>
struct DowncastTraits<HTMLRTElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kRtTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLRTElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLRTElement(x) blink::To<blink::HTMLRTElement>(x)
#define ToHTMLRTElementOrNull(x) blink::DynamicTo<blink::HTMLRTElement>(x)

template <>
struct DowncastTraits<HTMLRubyElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kRubyTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLRubyElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLRubyElement(x) blink::To<blink::HTMLRubyElement>(x)
#define ToHTMLRubyElementOrNull(x) blink::DynamicTo<blink::HTMLRubyElement>(x)

template <>
struct DowncastTraits<HTMLSummaryElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kSummaryTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLSummaryElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLSummaryElement(x) blink::To<blink::HTMLSummaryElement>(x)
#define ToHTMLSummaryElementOrNull(x) blink::DynamicTo<blink::HTMLSummaryElement>(x)

template <>
struct DowncastTraits<HTMLWBRElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(html_names::kWbrTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsHTMLElement() && IsA<HTMLWBRElement>(To<HTMLElement>(node));
  }
};
#define ToHTMLWBRElement(x) blink::To<blink::HTMLWBRElement>(x)
#define ToHTMLWBRElementOrNull(x) blink::DynamicTo<blink::HTMLWBRElement>(x)



enum class HTMLElementType {
kHTMLAnchorElement,
kHTMLAreaElement,
kHTMLAudioElement,
kHTMLBaseElement,
kHTMLBodyElement,
kHTMLBRElement,
kHTMLButtonElement,
kHTMLCanvasElement,
kHTMLContentElement,
kHTMLDataElement,
kHTMLDataListElement,
kHTMLDetailsElement,
kHTMLDialogElement,
kHTMLDirectoryElement,
kHTMLDivElement,
kHTMLDListElement,
kHTMLElement,
kHTMLEmbedElement,
kHTMLFieldSetElement,
kHTMLFontElement,
kHTMLFormElement,
kHTMLFrameElement,
kHTMLFrameSetElement,
kHTMLHeadElement,
kHTMLHeadingElement,
kHTMLHRElement,
kHTMLHtmlElement,
kHTMLIFrameElement,
kHTMLImageElement,
kHTMLInputElement,
kHTMLLabelElement,
kHTMLLegendElement,
kHTMLLIElement,
kHTMLLinkElement,
kHTMLMapElement,
kHTMLMarqueeElement,
kHTMLMenuElement,
kHTMLMetaElement,
kHTMLMeterElement,
kHTMLModElement,
kHTMLObjectElement,
kHTMLOListElement,
kHTMLOptGroupElement,
kHTMLOptionElement,
kHTMLOutputElement,
kHTMLParagraphElement,
kHTMLParamElement,
kHTMLPictureElement,
kHTMLPortalElement,
kHTMLPreElement,
kHTMLProgressElement,
kHTMLQuoteElement,
kHTMLScriptElement,
kHTMLSelectElement,
kHTMLShadowElement,
kHTMLSlotElement,
kHTMLSourceElement,
kHTMLSpanElement,
kHTMLStyleElement,
kHTMLTableCaptionElement,
kHTMLTableCellElement,
kHTMLTableColElement,
kHTMLTableElement,
kHTMLTableRowElement,
kHTMLTableSectionElement,
kHTMLTemplateElement,
kHTMLTextAreaElement,
kHTMLTimeElement,
kHTMLTitleElement,
kHTMLTrackElement,
kHTMLUListElement,
kHTMLUnknownElement,
kHTMLVideoElement,
};

// Tag checking.
// tagName is the local name for an html element in lowercase
// The corresponding HTMLElement type for the tag name will be returned
// Do NOT use this function with SVG tag names and SVGElements
// If tagName is an undefined html tag name HTMLUnknownElement is returned
HTMLElementType htmlElementTypeForTag(const AtomicString& tagName);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_HTML_ELEMENT_TYPE_HELPERS_H_
