// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/element_type_helpers.h.tmpl
// and input files:
//   ../../third_party/blink/renderer/core/svg/svg_tag_names.json5


#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_SVG_ELEMENT_TYPE_HELPERS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_SVG_ELEMENT_TYPE_HELPERS_H_

#include "third_party/blink/renderer/core/svg/svg_element.h"
#include "third_party/blink/renderer/core/svg_names.h"
#include "third_party/blink/renderer/platform/runtime_enabled_features.h"

namespace blink {
// Type checking.
class SVGAElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGAElement(const SVGAElement&);
void IsSVGAElement(const SVGAElement*);

inline bool IsSVGAElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kATag);
}
inline bool IsSVGAElement(const SVGElement* element) {
  return element && IsSVGAElement(*element);
}
inline bool IsSVGAElement(const Node& node) {
  return node.IsSVGElement() && IsSVGAElement(To<SVGElement>(node));
}
inline bool IsSVGAElement(const Node* node) {
  return node && IsSVGAElement(*node);
}
template <>
inline bool IsElementOfType<const SVGAElement>(const Node& node) {
  return IsSVGAElement(node);
}
template <>
inline bool IsElementOfType<const SVGAElement>(const SVGElement& element) {
  return IsSVGAElement(element);
}

class SVGAnimateMotionElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGAnimateMotionElement(const SVGAnimateMotionElement&);
void IsSVGAnimateMotionElement(const SVGAnimateMotionElement*);

inline bool IsSVGAnimateMotionElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kAnimateMotionTag);
}
inline bool IsSVGAnimateMotionElement(const SVGElement* element) {
  return element && IsSVGAnimateMotionElement(*element);
}
inline bool IsSVGAnimateMotionElement(const Node& node) {
  return node.IsSVGElement() && IsSVGAnimateMotionElement(To<SVGElement>(node));
}
inline bool IsSVGAnimateMotionElement(const Node* node) {
  return node && IsSVGAnimateMotionElement(*node);
}
template <>
inline bool IsElementOfType<const SVGAnimateMotionElement>(const Node& node) {
  return IsSVGAnimateMotionElement(node);
}
template <>
inline bool IsElementOfType<const SVGAnimateMotionElement>(const SVGElement& element) {
  return IsSVGAnimateMotionElement(element);
}

class SVGAnimateTransformElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGAnimateTransformElement(const SVGAnimateTransformElement&);
void IsSVGAnimateTransformElement(const SVGAnimateTransformElement*);

inline bool IsSVGAnimateTransformElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kAnimateTransformTag);
}
inline bool IsSVGAnimateTransformElement(const SVGElement* element) {
  return element && IsSVGAnimateTransformElement(*element);
}
inline bool IsSVGAnimateTransformElement(const Node& node) {
  return node.IsSVGElement() && IsSVGAnimateTransformElement(To<SVGElement>(node));
}
inline bool IsSVGAnimateTransformElement(const Node* node) {
  return node && IsSVGAnimateTransformElement(*node);
}
template <>
inline bool IsElementOfType<const SVGAnimateTransformElement>(const Node& node) {
  return IsSVGAnimateTransformElement(node);
}
template <>
inline bool IsElementOfType<const SVGAnimateTransformElement>(const SVGElement& element) {
  return IsSVGAnimateTransformElement(element);
}

class SVGCircleElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGCircleElement(const SVGCircleElement&);
void IsSVGCircleElement(const SVGCircleElement*);

inline bool IsSVGCircleElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kCircleTag);
}
inline bool IsSVGCircleElement(const SVGElement* element) {
  return element && IsSVGCircleElement(*element);
}
inline bool IsSVGCircleElement(const Node& node) {
  return node.IsSVGElement() && IsSVGCircleElement(To<SVGElement>(node));
}
inline bool IsSVGCircleElement(const Node* node) {
  return node && IsSVGCircleElement(*node);
}
template <>
inline bool IsElementOfType<const SVGCircleElement>(const Node& node) {
  return IsSVGCircleElement(node);
}
template <>
inline bool IsElementOfType<const SVGCircleElement>(const SVGElement& element) {
  return IsSVGCircleElement(element);
}

class SVGClipPathElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGClipPathElement(const SVGClipPathElement&);
void IsSVGClipPathElement(const SVGClipPathElement*);

inline bool IsSVGClipPathElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kClipPathTag);
}
inline bool IsSVGClipPathElement(const SVGElement* element) {
  return element && IsSVGClipPathElement(*element);
}
inline bool IsSVGClipPathElement(const Node& node) {
  return node.IsSVGElement() && IsSVGClipPathElement(To<SVGElement>(node));
}
inline bool IsSVGClipPathElement(const Node* node) {
  return node && IsSVGClipPathElement(*node);
}
template <>
inline bool IsElementOfType<const SVGClipPathElement>(const Node& node) {
  return IsSVGClipPathElement(node);
}
template <>
inline bool IsElementOfType<const SVGClipPathElement>(const SVGElement& element) {
  return IsSVGClipPathElement(element);
}

class SVGDefsElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGDefsElement(const SVGDefsElement&);
void IsSVGDefsElement(const SVGDefsElement*);

inline bool IsSVGDefsElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kDefsTag);
}
inline bool IsSVGDefsElement(const SVGElement* element) {
  return element && IsSVGDefsElement(*element);
}
inline bool IsSVGDefsElement(const Node& node) {
  return node.IsSVGElement() && IsSVGDefsElement(To<SVGElement>(node));
}
inline bool IsSVGDefsElement(const Node* node) {
  return node && IsSVGDefsElement(*node);
}
template <>
inline bool IsElementOfType<const SVGDefsElement>(const Node& node) {
  return IsSVGDefsElement(node);
}
template <>
inline bool IsElementOfType<const SVGDefsElement>(const SVGElement& element) {
  return IsSVGDefsElement(element);
}

class SVGDescElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGDescElement(const SVGDescElement&);
void IsSVGDescElement(const SVGDescElement*);

inline bool IsSVGDescElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kDescTag);
}
inline bool IsSVGDescElement(const SVGElement* element) {
  return element && IsSVGDescElement(*element);
}
inline bool IsSVGDescElement(const Node& node) {
  return node.IsSVGElement() && IsSVGDescElement(To<SVGElement>(node));
}
inline bool IsSVGDescElement(const Node* node) {
  return node && IsSVGDescElement(*node);
}
template <>
inline bool IsElementOfType<const SVGDescElement>(const Node& node) {
  return IsSVGDescElement(node);
}
template <>
inline bool IsElementOfType<const SVGDescElement>(const SVGElement& element) {
  return IsSVGDescElement(element);
}

class SVGDiscardElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGDiscardElement(const SVGDiscardElement&);
void IsSVGDiscardElement(const SVGDiscardElement*);

inline bool IsSVGDiscardElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kDiscardTag);
}
inline bool IsSVGDiscardElement(const SVGElement* element) {
  return element && IsSVGDiscardElement(*element);
}
inline bool IsSVGDiscardElement(const Node& node) {
  return node.IsSVGElement() && IsSVGDiscardElement(To<SVGElement>(node));
}
inline bool IsSVGDiscardElement(const Node* node) {
  return node && IsSVGDiscardElement(*node);
}
template <>
inline bool IsElementOfType<const SVGDiscardElement>(const Node& node) {
  return IsSVGDiscardElement(node);
}
template <>
inline bool IsElementOfType<const SVGDiscardElement>(const SVGElement& element) {
  return IsSVGDiscardElement(element);
}

class SVGEllipseElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGEllipseElement(const SVGEllipseElement&);
void IsSVGEllipseElement(const SVGEllipseElement*);

inline bool IsSVGEllipseElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kEllipseTag);
}
inline bool IsSVGEllipseElement(const SVGElement* element) {
  return element && IsSVGEllipseElement(*element);
}
inline bool IsSVGEllipseElement(const Node& node) {
  return node.IsSVGElement() && IsSVGEllipseElement(To<SVGElement>(node));
}
inline bool IsSVGEllipseElement(const Node* node) {
  return node && IsSVGEllipseElement(*node);
}
template <>
inline bool IsElementOfType<const SVGEllipseElement>(const Node& node) {
  return IsSVGEllipseElement(node);
}
template <>
inline bool IsElementOfType<const SVGEllipseElement>(const SVGElement& element) {
  return IsSVGEllipseElement(element);
}

class SVGFEBlendElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEBlendElement(const SVGFEBlendElement&);
void IsSVGFEBlendElement(const SVGFEBlendElement*);

inline bool IsSVGFEBlendElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEBlendTag);
}
inline bool IsSVGFEBlendElement(const SVGElement* element) {
  return element && IsSVGFEBlendElement(*element);
}
inline bool IsSVGFEBlendElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEBlendElement(To<SVGElement>(node));
}
inline bool IsSVGFEBlendElement(const Node* node) {
  return node && IsSVGFEBlendElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEBlendElement>(const Node& node) {
  return IsSVGFEBlendElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEBlendElement>(const SVGElement& element) {
  return IsSVGFEBlendElement(element);
}

class SVGFEColorMatrixElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEColorMatrixElement(const SVGFEColorMatrixElement&);
void IsSVGFEColorMatrixElement(const SVGFEColorMatrixElement*);

inline bool IsSVGFEColorMatrixElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEColorMatrixTag);
}
inline bool IsSVGFEColorMatrixElement(const SVGElement* element) {
  return element && IsSVGFEColorMatrixElement(*element);
}
inline bool IsSVGFEColorMatrixElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEColorMatrixElement(To<SVGElement>(node));
}
inline bool IsSVGFEColorMatrixElement(const Node* node) {
  return node && IsSVGFEColorMatrixElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEColorMatrixElement>(const Node& node) {
  return IsSVGFEColorMatrixElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEColorMatrixElement>(const SVGElement& element) {
  return IsSVGFEColorMatrixElement(element);
}

class SVGFEComponentTransferElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEComponentTransferElement(const SVGFEComponentTransferElement&);
void IsSVGFEComponentTransferElement(const SVGFEComponentTransferElement*);

inline bool IsSVGFEComponentTransferElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEComponentTransferTag);
}
inline bool IsSVGFEComponentTransferElement(const SVGElement* element) {
  return element && IsSVGFEComponentTransferElement(*element);
}
inline bool IsSVGFEComponentTransferElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEComponentTransferElement(To<SVGElement>(node));
}
inline bool IsSVGFEComponentTransferElement(const Node* node) {
  return node && IsSVGFEComponentTransferElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEComponentTransferElement>(const Node& node) {
  return IsSVGFEComponentTransferElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEComponentTransferElement>(const SVGElement& element) {
  return IsSVGFEComponentTransferElement(element);
}

class SVGFECompositeElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFECompositeElement(const SVGFECompositeElement&);
void IsSVGFECompositeElement(const SVGFECompositeElement*);

inline bool IsSVGFECompositeElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFECompositeTag);
}
inline bool IsSVGFECompositeElement(const SVGElement* element) {
  return element && IsSVGFECompositeElement(*element);
}
inline bool IsSVGFECompositeElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFECompositeElement(To<SVGElement>(node));
}
inline bool IsSVGFECompositeElement(const Node* node) {
  return node && IsSVGFECompositeElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFECompositeElement>(const Node& node) {
  return IsSVGFECompositeElement(node);
}
template <>
inline bool IsElementOfType<const SVGFECompositeElement>(const SVGElement& element) {
  return IsSVGFECompositeElement(element);
}

class SVGFEConvolveMatrixElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEConvolveMatrixElement(const SVGFEConvolveMatrixElement&);
void IsSVGFEConvolveMatrixElement(const SVGFEConvolveMatrixElement*);

inline bool IsSVGFEConvolveMatrixElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEConvolveMatrixTag);
}
inline bool IsSVGFEConvolveMatrixElement(const SVGElement* element) {
  return element && IsSVGFEConvolveMatrixElement(*element);
}
inline bool IsSVGFEConvolveMatrixElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEConvolveMatrixElement(To<SVGElement>(node));
}
inline bool IsSVGFEConvolveMatrixElement(const Node* node) {
  return node && IsSVGFEConvolveMatrixElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEConvolveMatrixElement>(const Node& node) {
  return IsSVGFEConvolveMatrixElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEConvolveMatrixElement>(const SVGElement& element) {
  return IsSVGFEConvolveMatrixElement(element);
}

class SVGFEDiffuseLightingElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEDiffuseLightingElement(const SVGFEDiffuseLightingElement&);
void IsSVGFEDiffuseLightingElement(const SVGFEDiffuseLightingElement*);

inline bool IsSVGFEDiffuseLightingElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEDiffuseLightingTag);
}
inline bool IsSVGFEDiffuseLightingElement(const SVGElement* element) {
  return element && IsSVGFEDiffuseLightingElement(*element);
}
inline bool IsSVGFEDiffuseLightingElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEDiffuseLightingElement(To<SVGElement>(node));
}
inline bool IsSVGFEDiffuseLightingElement(const Node* node) {
  return node && IsSVGFEDiffuseLightingElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEDiffuseLightingElement>(const Node& node) {
  return IsSVGFEDiffuseLightingElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEDiffuseLightingElement>(const SVGElement& element) {
  return IsSVGFEDiffuseLightingElement(element);
}

class SVGFEDisplacementMapElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEDisplacementMapElement(const SVGFEDisplacementMapElement&);
void IsSVGFEDisplacementMapElement(const SVGFEDisplacementMapElement*);

inline bool IsSVGFEDisplacementMapElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEDisplacementMapTag);
}
inline bool IsSVGFEDisplacementMapElement(const SVGElement* element) {
  return element && IsSVGFEDisplacementMapElement(*element);
}
inline bool IsSVGFEDisplacementMapElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEDisplacementMapElement(To<SVGElement>(node));
}
inline bool IsSVGFEDisplacementMapElement(const Node* node) {
  return node && IsSVGFEDisplacementMapElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEDisplacementMapElement>(const Node& node) {
  return IsSVGFEDisplacementMapElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEDisplacementMapElement>(const SVGElement& element) {
  return IsSVGFEDisplacementMapElement(element);
}

class SVGFEDistantLightElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEDistantLightElement(const SVGFEDistantLightElement&);
void IsSVGFEDistantLightElement(const SVGFEDistantLightElement*);

inline bool IsSVGFEDistantLightElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEDistantLightTag);
}
inline bool IsSVGFEDistantLightElement(const SVGElement* element) {
  return element && IsSVGFEDistantLightElement(*element);
}
inline bool IsSVGFEDistantLightElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEDistantLightElement(To<SVGElement>(node));
}
inline bool IsSVGFEDistantLightElement(const Node* node) {
  return node && IsSVGFEDistantLightElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEDistantLightElement>(const Node& node) {
  return IsSVGFEDistantLightElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEDistantLightElement>(const SVGElement& element) {
  return IsSVGFEDistantLightElement(element);
}

class SVGFEDropShadowElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEDropShadowElement(const SVGFEDropShadowElement&);
void IsSVGFEDropShadowElement(const SVGFEDropShadowElement*);

inline bool IsSVGFEDropShadowElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEDropShadowTag);
}
inline bool IsSVGFEDropShadowElement(const SVGElement* element) {
  return element && IsSVGFEDropShadowElement(*element);
}
inline bool IsSVGFEDropShadowElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEDropShadowElement(To<SVGElement>(node));
}
inline bool IsSVGFEDropShadowElement(const Node* node) {
  return node && IsSVGFEDropShadowElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEDropShadowElement>(const Node& node) {
  return IsSVGFEDropShadowElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEDropShadowElement>(const SVGElement& element) {
  return IsSVGFEDropShadowElement(element);
}

class SVGFEFloodElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEFloodElement(const SVGFEFloodElement&);
void IsSVGFEFloodElement(const SVGFEFloodElement*);

inline bool IsSVGFEFloodElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEFloodTag);
}
inline bool IsSVGFEFloodElement(const SVGElement* element) {
  return element && IsSVGFEFloodElement(*element);
}
inline bool IsSVGFEFloodElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEFloodElement(To<SVGElement>(node));
}
inline bool IsSVGFEFloodElement(const Node* node) {
  return node && IsSVGFEFloodElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFloodElement>(const Node& node) {
  return IsSVGFEFloodElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFloodElement>(const SVGElement& element) {
  return IsSVGFEFloodElement(element);
}

class SVGFEFuncAElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEFuncAElement(const SVGFEFuncAElement&);
void IsSVGFEFuncAElement(const SVGFEFuncAElement*);

inline bool IsSVGFEFuncAElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEFuncATag);
}
inline bool IsSVGFEFuncAElement(const SVGElement* element) {
  return element && IsSVGFEFuncAElement(*element);
}
inline bool IsSVGFEFuncAElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEFuncAElement(To<SVGElement>(node));
}
inline bool IsSVGFEFuncAElement(const Node* node) {
  return node && IsSVGFEFuncAElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncAElement>(const Node& node) {
  return IsSVGFEFuncAElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncAElement>(const SVGElement& element) {
  return IsSVGFEFuncAElement(element);
}

class SVGFEFuncBElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEFuncBElement(const SVGFEFuncBElement&);
void IsSVGFEFuncBElement(const SVGFEFuncBElement*);

inline bool IsSVGFEFuncBElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEFuncBTag);
}
inline bool IsSVGFEFuncBElement(const SVGElement* element) {
  return element && IsSVGFEFuncBElement(*element);
}
inline bool IsSVGFEFuncBElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEFuncBElement(To<SVGElement>(node));
}
inline bool IsSVGFEFuncBElement(const Node* node) {
  return node && IsSVGFEFuncBElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncBElement>(const Node& node) {
  return IsSVGFEFuncBElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncBElement>(const SVGElement& element) {
  return IsSVGFEFuncBElement(element);
}

class SVGFEFuncGElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEFuncGElement(const SVGFEFuncGElement&);
void IsSVGFEFuncGElement(const SVGFEFuncGElement*);

inline bool IsSVGFEFuncGElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEFuncGTag);
}
inline bool IsSVGFEFuncGElement(const SVGElement* element) {
  return element && IsSVGFEFuncGElement(*element);
}
inline bool IsSVGFEFuncGElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEFuncGElement(To<SVGElement>(node));
}
inline bool IsSVGFEFuncGElement(const Node* node) {
  return node && IsSVGFEFuncGElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncGElement>(const Node& node) {
  return IsSVGFEFuncGElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncGElement>(const SVGElement& element) {
  return IsSVGFEFuncGElement(element);
}

class SVGFEFuncRElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEFuncRElement(const SVGFEFuncRElement&);
void IsSVGFEFuncRElement(const SVGFEFuncRElement*);

inline bool IsSVGFEFuncRElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEFuncRTag);
}
inline bool IsSVGFEFuncRElement(const SVGElement* element) {
  return element && IsSVGFEFuncRElement(*element);
}
inline bool IsSVGFEFuncRElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEFuncRElement(To<SVGElement>(node));
}
inline bool IsSVGFEFuncRElement(const Node* node) {
  return node && IsSVGFEFuncRElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncRElement>(const Node& node) {
  return IsSVGFEFuncRElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEFuncRElement>(const SVGElement& element) {
  return IsSVGFEFuncRElement(element);
}

class SVGFEGaussianBlurElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEGaussianBlurElement(const SVGFEGaussianBlurElement&);
void IsSVGFEGaussianBlurElement(const SVGFEGaussianBlurElement*);

inline bool IsSVGFEGaussianBlurElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEGaussianBlurTag);
}
inline bool IsSVGFEGaussianBlurElement(const SVGElement* element) {
  return element && IsSVGFEGaussianBlurElement(*element);
}
inline bool IsSVGFEGaussianBlurElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEGaussianBlurElement(To<SVGElement>(node));
}
inline bool IsSVGFEGaussianBlurElement(const Node* node) {
  return node && IsSVGFEGaussianBlurElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEGaussianBlurElement>(const Node& node) {
  return IsSVGFEGaussianBlurElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEGaussianBlurElement>(const SVGElement& element) {
  return IsSVGFEGaussianBlurElement(element);
}

class SVGFEImageElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEImageElement(const SVGFEImageElement&);
void IsSVGFEImageElement(const SVGFEImageElement*);

inline bool IsSVGFEImageElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEImageTag);
}
inline bool IsSVGFEImageElement(const SVGElement* element) {
  return element && IsSVGFEImageElement(*element);
}
inline bool IsSVGFEImageElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEImageElement(To<SVGElement>(node));
}
inline bool IsSVGFEImageElement(const Node* node) {
  return node && IsSVGFEImageElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEImageElement>(const Node& node) {
  return IsSVGFEImageElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEImageElement>(const SVGElement& element) {
  return IsSVGFEImageElement(element);
}

class SVGFEMergeElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEMergeElement(const SVGFEMergeElement&);
void IsSVGFEMergeElement(const SVGFEMergeElement*);

inline bool IsSVGFEMergeElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEMergeTag);
}
inline bool IsSVGFEMergeElement(const SVGElement* element) {
  return element && IsSVGFEMergeElement(*element);
}
inline bool IsSVGFEMergeElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEMergeElement(To<SVGElement>(node));
}
inline bool IsSVGFEMergeElement(const Node* node) {
  return node && IsSVGFEMergeElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEMergeElement>(const Node& node) {
  return IsSVGFEMergeElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEMergeElement>(const SVGElement& element) {
  return IsSVGFEMergeElement(element);
}

class SVGFEMergeNodeElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEMergeNodeElement(const SVGFEMergeNodeElement&);
void IsSVGFEMergeNodeElement(const SVGFEMergeNodeElement*);

inline bool IsSVGFEMergeNodeElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEMergeNodeTag);
}
inline bool IsSVGFEMergeNodeElement(const SVGElement* element) {
  return element && IsSVGFEMergeNodeElement(*element);
}
inline bool IsSVGFEMergeNodeElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEMergeNodeElement(To<SVGElement>(node));
}
inline bool IsSVGFEMergeNodeElement(const Node* node) {
  return node && IsSVGFEMergeNodeElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEMergeNodeElement>(const Node& node) {
  return IsSVGFEMergeNodeElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEMergeNodeElement>(const SVGElement& element) {
  return IsSVGFEMergeNodeElement(element);
}

class SVGFEMorphologyElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEMorphologyElement(const SVGFEMorphologyElement&);
void IsSVGFEMorphologyElement(const SVGFEMorphologyElement*);

inline bool IsSVGFEMorphologyElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEMorphologyTag);
}
inline bool IsSVGFEMorphologyElement(const SVGElement* element) {
  return element && IsSVGFEMorphologyElement(*element);
}
inline bool IsSVGFEMorphologyElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEMorphologyElement(To<SVGElement>(node));
}
inline bool IsSVGFEMorphologyElement(const Node* node) {
  return node && IsSVGFEMorphologyElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEMorphologyElement>(const Node& node) {
  return IsSVGFEMorphologyElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEMorphologyElement>(const SVGElement& element) {
  return IsSVGFEMorphologyElement(element);
}

class SVGFEOffsetElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEOffsetElement(const SVGFEOffsetElement&);
void IsSVGFEOffsetElement(const SVGFEOffsetElement*);

inline bool IsSVGFEOffsetElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEOffsetTag);
}
inline bool IsSVGFEOffsetElement(const SVGElement* element) {
  return element && IsSVGFEOffsetElement(*element);
}
inline bool IsSVGFEOffsetElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEOffsetElement(To<SVGElement>(node));
}
inline bool IsSVGFEOffsetElement(const Node* node) {
  return node && IsSVGFEOffsetElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEOffsetElement>(const Node& node) {
  return IsSVGFEOffsetElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEOffsetElement>(const SVGElement& element) {
  return IsSVGFEOffsetElement(element);
}

class SVGFEPointLightElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFEPointLightElement(const SVGFEPointLightElement&);
void IsSVGFEPointLightElement(const SVGFEPointLightElement*);

inline bool IsSVGFEPointLightElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFEPointLightTag);
}
inline bool IsSVGFEPointLightElement(const SVGElement* element) {
  return element && IsSVGFEPointLightElement(*element);
}
inline bool IsSVGFEPointLightElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFEPointLightElement(To<SVGElement>(node));
}
inline bool IsSVGFEPointLightElement(const Node* node) {
  return node && IsSVGFEPointLightElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFEPointLightElement>(const Node& node) {
  return IsSVGFEPointLightElement(node);
}
template <>
inline bool IsElementOfType<const SVGFEPointLightElement>(const SVGElement& element) {
  return IsSVGFEPointLightElement(element);
}

class SVGFESpecularLightingElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFESpecularLightingElement(const SVGFESpecularLightingElement&);
void IsSVGFESpecularLightingElement(const SVGFESpecularLightingElement*);

inline bool IsSVGFESpecularLightingElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFESpecularLightingTag);
}
inline bool IsSVGFESpecularLightingElement(const SVGElement* element) {
  return element && IsSVGFESpecularLightingElement(*element);
}
inline bool IsSVGFESpecularLightingElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFESpecularLightingElement(To<SVGElement>(node));
}
inline bool IsSVGFESpecularLightingElement(const Node* node) {
  return node && IsSVGFESpecularLightingElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFESpecularLightingElement>(const Node& node) {
  return IsSVGFESpecularLightingElement(node);
}
template <>
inline bool IsElementOfType<const SVGFESpecularLightingElement>(const SVGElement& element) {
  return IsSVGFESpecularLightingElement(element);
}

class SVGFESpotLightElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFESpotLightElement(const SVGFESpotLightElement&);
void IsSVGFESpotLightElement(const SVGFESpotLightElement*);

inline bool IsSVGFESpotLightElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFESpotLightTag);
}
inline bool IsSVGFESpotLightElement(const SVGElement* element) {
  return element && IsSVGFESpotLightElement(*element);
}
inline bool IsSVGFESpotLightElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFESpotLightElement(To<SVGElement>(node));
}
inline bool IsSVGFESpotLightElement(const Node* node) {
  return node && IsSVGFESpotLightElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFESpotLightElement>(const Node& node) {
  return IsSVGFESpotLightElement(node);
}
template <>
inline bool IsElementOfType<const SVGFESpotLightElement>(const SVGElement& element) {
  return IsSVGFESpotLightElement(element);
}

class SVGFETileElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFETileElement(const SVGFETileElement&);
void IsSVGFETileElement(const SVGFETileElement*);

inline bool IsSVGFETileElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFETileTag);
}
inline bool IsSVGFETileElement(const SVGElement* element) {
  return element && IsSVGFETileElement(*element);
}
inline bool IsSVGFETileElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFETileElement(To<SVGElement>(node));
}
inline bool IsSVGFETileElement(const Node* node) {
  return node && IsSVGFETileElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFETileElement>(const Node& node) {
  return IsSVGFETileElement(node);
}
template <>
inline bool IsElementOfType<const SVGFETileElement>(const SVGElement& element) {
  return IsSVGFETileElement(element);
}

class SVGFETurbulenceElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFETurbulenceElement(const SVGFETurbulenceElement&);
void IsSVGFETurbulenceElement(const SVGFETurbulenceElement*);

inline bool IsSVGFETurbulenceElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFETurbulenceTag);
}
inline bool IsSVGFETurbulenceElement(const SVGElement* element) {
  return element && IsSVGFETurbulenceElement(*element);
}
inline bool IsSVGFETurbulenceElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFETurbulenceElement(To<SVGElement>(node));
}
inline bool IsSVGFETurbulenceElement(const Node* node) {
  return node && IsSVGFETurbulenceElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFETurbulenceElement>(const Node& node) {
  return IsSVGFETurbulenceElement(node);
}
template <>
inline bool IsElementOfType<const SVGFETurbulenceElement>(const SVGElement& element) {
  return IsSVGFETurbulenceElement(element);
}

class SVGFilterElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGFilterElement(const SVGFilterElement&);
void IsSVGFilterElement(const SVGFilterElement*);

inline bool IsSVGFilterElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kFilterTag);
}
inline bool IsSVGFilterElement(const SVGElement* element) {
  return element && IsSVGFilterElement(*element);
}
inline bool IsSVGFilterElement(const Node& node) {
  return node.IsSVGElement() && IsSVGFilterElement(To<SVGElement>(node));
}
inline bool IsSVGFilterElement(const Node* node) {
  return node && IsSVGFilterElement(*node);
}
template <>
inline bool IsElementOfType<const SVGFilterElement>(const Node& node) {
  return IsSVGFilterElement(node);
}
template <>
inline bool IsElementOfType<const SVGFilterElement>(const SVGElement& element) {
  return IsSVGFilterElement(element);
}

class SVGForeignObjectElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGForeignObjectElement(const SVGForeignObjectElement&);
void IsSVGForeignObjectElement(const SVGForeignObjectElement*);

inline bool IsSVGForeignObjectElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kForeignObjectTag);
}
inline bool IsSVGForeignObjectElement(const SVGElement* element) {
  return element && IsSVGForeignObjectElement(*element);
}
inline bool IsSVGForeignObjectElement(const Node& node) {
  return node.IsSVGElement() && IsSVGForeignObjectElement(To<SVGElement>(node));
}
inline bool IsSVGForeignObjectElement(const Node* node) {
  return node && IsSVGForeignObjectElement(*node);
}
template <>
inline bool IsElementOfType<const SVGForeignObjectElement>(const Node& node) {
  return IsSVGForeignObjectElement(node);
}
template <>
inline bool IsElementOfType<const SVGForeignObjectElement>(const SVGElement& element) {
  return IsSVGForeignObjectElement(element);
}

class SVGGElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGGElement(const SVGGElement&);
void IsSVGGElement(const SVGGElement*);

inline bool IsSVGGElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kGTag);
}
inline bool IsSVGGElement(const SVGElement* element) {
  return element && IsSVGGElement(*element);
}
inline bool IsSVGGElement(const Node& node) {
  return node.IsSVGElement() && IsSVGGElement(To<SVGElement>(node));
}
inline bool IsSVGGElement(const Node* node) {
  return node && IsSVGGElement(*node);
}
template <>
inline bool IsElementOfType<const SVGGElement>(const Node& node) {
  return IsSVGGElement(node);
}
template <>
inline bool IsElementOfType<const SVGGElement>(const SVGElement& element) {
  return IsSVGGElement(element);
}

class SVGImageElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGImageElement(const SVGImageElement&);
void IsSVGImageElement(const SVGImageElement*);

inline bool IsSVGImageElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kImageTag);
}
inline bool IsSVGImageElement(const SVGElement* element) {
  return element && IsSVGImageElement(*element);
}
inline bool IsSVGImageElement(const Node& node) {
  return node.IsSVGElement() && IsSVGImageElement(To<SVGElement>(node));
}
inline bool IsSVGImageElement(const Node* node) {
  return node && IsSVGImageElement(*node);
}
template <>
inline bool IsElementOfType<const SVGImageElement>(const Node& node) {
  return IsSVGImageElement(node);
}
template <>
inline bool IsElementOfType<const SVGImageElement>(const SVGElement& element) {
  return IsSVGImageElement(element);
}

class SVGLineElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGLineElement(const SVGLineElement&);
void IsSVGLineElement(const SVGLineElement*);

inline bool IsSVGLineElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kLineTag);
}
inline bool IsSVGLineElement(const SVGElement* element) {
  return element && IsSVGLineElement(*element);
}
inline bool IsSVGLineElement(const Node& node) {
  return node.IsSVGElement() && IsSVGLineElement(To<SVGElement>(node));
}
inline bool IsSVGLineElement(const Node* node) {
  return node && IsSVGLineElement(*node);
}
template <>
inline bool IsElementOfType<const SVGLineElement>(const Node& node) {
  return IsSVGLineElement(node);
}
template <>
inline bool IsElementOfType<const SVGLineElement>(const SVGElement& element) {
  return IsSVGLineElement(element);
}

class SVGLinearGradientElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGLinearGradientElement(const SVGLinearGradientElement&);
void IsSVGLinearGradientElement(const SVGLinearGradientElement*);

inline bool IsSVGLinearGradientElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kLinearGradientTag);
}
inline bool IsSVGLinearGradientElement(const SVGElement* element) {
  return element && IsSVGLinearGradientElement(*element);
}
inline bool IsSVGLinearGradientElement(const Node& node) {
  return node.IsSVGElement() && IsSVGLinearGradientElement(To<SVGElement>(node));
}
inline bool IsSVGLinearGradientElement(const Node* node) {
  return node && IsSVGLinearGradientElement(*node);
}
template <>
inline bool IsElementOfType<const SVGLinearGradientElement>(const Node& node) {
  return IsSVGLinearGradientElement(node);
}
template <>
inline bool IsElementOfType<const SVGLinearGradientElement>(const SVGElement& element) {
  return IsSVGLinearGradientElement(element);
}

class SVGMPathElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGMPathElement(const SVGMPathElement&);
void IsSVGMPathElement(const SVGMPathElement*);

inline bool IsSVGMPathElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kMPathTag);
}
inline bool IsSVGMPathElement(const SVGElement* element) {
  return element && IsSVGMPathElement(*element);
}
inline bool IsSVGMPathElement(const Node& node) {
  return node.IsSVGElement() && IsSVGMPathElement(To<SVGElement>(node));
}
inline bool IsSVGMPathElement(const Node* node) {
  return node && IsSVGMPathElement(*node);
}
template <>
inline bool IsElementOfType<const SVGMPathElement>(const Node& node) {
  return IsSVGMPathElement(node);
}
template <>
inline bool IsElementOfType<const SVGMPathElement>(const SVGElement& element) {
  return IsSVGMPathElement(element);
}

class SVGMarkerElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGMarkerElement(const SVGMarkerElement&);
void IsSVGMarkerElement(const SVGMarkerElement*);

inline bool IsSVGMarkerElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kMarkerTag);
}
inline bool IsSVGMarkerElement(const SVGElement* element) {
  return element && IsSVGMarkerElement(*element);
}
inline bool IsSVGMarkerElement(const Node& node) {
  return node.IsSVGElement() && IsSVGMarkerElement(To<SVGElement>(node));
}
inline bool IsSVGMarkerElement(const Node* node) {
  return node && IsSVGMarkerElement(*node);
}
template <>
inline bool IsElementOfType<const SVGMarkerElement>(const Node& node) {
  return IsSVGMarkerElement(node);
}
template <>
inline bool IsElementOfType<const SVGMarkerElement>(const SVGElement& element) {
  return IsSVGMarkerElement(element);
}

class SVGMaskElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGMaskElement(const SVGMaskElement&);
void IsSVGMaskElement(const SVGMaskElement*);

inline bool IsSVGMaskElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kMaskTag);
}
inline bool IsSVGMaskElement(const SVGElement* element) {
  return element && IsSVGMaskElement(*element);
}
inline bool IsSVGMaskElement(const Node& node) {
  return node.IsSVGElement() && IsSVGMaskElement(To<SVGElement>(node));
}
inline bool IsSVGMaskElement(const Node* node) {
  return node && IsSVGMaskElement(*node);
}
template <>
inline bool IsElementOfType<const SVGMaskElement>(const Node& node) {
  return IsSVGMaskElement(node);
}
template <>
inline bool IsElementOfType<const SVGMaskElement>(const SVGElement& element) {
  return IsSVGMaskElement(element);
}

class SVGMetadataElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGMetadataElement(const SVGMetadataElement&);
void IsSVGMetadataElement(const SVGMetadataElement*);

inline bool IsSVGMetadataElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kMetadataTag);
}
inline bool IsSVGMetadataElement(const SVGElement* element) {
  return element && IsSVGMetadataElement(*element);
}
inline bool IsSVGMetadataElement(const Node& node) {
  return node.IsSVGElement() && IsSVGMetadataElement(To<SVGElement>(node));
}
inline bool IsSVGMetadataElement(const Node* node) {
  return node && IsSVGMetadataElement(*node);
}
template <>
inline bool IsElementOfType<const SVGMetadataElement>(const Node& node) {
  return IsSVGMetadataElement(node);
}
template <>
inline bool IsElementOfType<const SVGMetadataElement>(const SVGElement& element) {
  return IsSVGMetadataElement(element);
}

class SVGPathElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGPathElement(const SVGPathElement&);
void IsSVGPathElement(const SVGPathElement*);

inline bool IsSVGPathElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kPathTag);
}
inline bool IsSVGPathElement(const SVGElement* element) {
  return element && IsSVGPathElement(*element);
}
inline bool IsSVGPathElement(const Node& node) {
  return node.IsSVGElement() && IsSVGPathElement(To<SVGElement>(node));
}
inline bool IsSVGPathElement(const Node* node) {
  return node && IsSVGPathElement(*node);
}
template <>
inline bool IsElementOfType<const SVGPathElement>(const Node& node) {
  return IsSVGPathElement(node);
}
template <>
inline bool IsElementOfType<const SVGPathElement>(const SVGElement& element) {
  return IsSVGPathElement(element);
}

class SVGPatternElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGPatternElement(const SVGPatternElement&);
void IsSVGPatternElement(const SVGPatternElement*);

inline bool IsSVGPatternElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kPatternTag);
}
inline bool IsSVGPatternElement(const SVGElement* element) {
  return element && IsSVGPatternElement(*element);
}
inline bool IsSVGPatternElement(const Node& node) {
  return node.IsSVGElement() && IsSVGPatternElement(To<SVGElement>(node));
}
inline bool IsSVGPatternElement(const Node* node) {
  return node && IsSVGPatternElement(*node);
}
template <>
inline bool IsElementOfType<const SVGPatternElement>(const Node& node) {
  return IsSVGPatternElement(node);
}
template <>
inline bool IsElementOfType<const SVGPatternElement>(const SVGElement& element) {
  return IsSVGPatternElement(element);
}

class SVGPolygonElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGPolygonElement(const SVGPolygonElement&);
void IsSVGPolygonElement(const SVGPolygonElement*);

inline bool IsSVGPolygonElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kPolygonTag);
}
inline bool IsSVGPolygonElement(const SVGElement* element) {
  return element && IsSVGPolygonElement(*element);
}
inline bool IsSVGPolygonElement(const Node& node) {
  return node.IsSVGElement() && IsSVGPolygonElement(To<SVGElement>(node));
}
inline bool IsSVGPolygonElement(const Node* node) {
  return node && IsSVGPolygonElement(*node);
}
template <>
inline bool IsElementOfType<const SVGPolygonElement>(const Node& node) {
  return IsSVGPolygonElement(node);
}
template <>
inline bool IsElementOfType<const SVGPolygonElement>(const SVGElement& element) {
  return IsSVGPolygonElement(element);
}

class SVGPolylineElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGPolylineElement(const SVGPolylineElement&);
void IsSVGPolylineElement(const SVGPolylineElement*);

inline bool IsSVGPolylineElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kPolylineTag);
}
inline bool IsSVGPolylineElement(const SVGElement* element) {
  return element && IsSVGPolylineElement(*element);
}
inline bool IsSVGPolylineElement(const Node& node) {
  return node.IsSVGElement() && IsSVGPolylineElement(To<SVGElement>(node));
}
inline bool IsSVGPolylineElement(const Node* node) {
  return node && IsSVGPolylineElement(*node);
}
template <>
inline bool IsElementOfType<const SVGPolylineElement>(const Node& node) {
  return IsSVGPolylineElement(node);
}
template <>
inline bool IsElementOfType<const SVGPolylineElement>(const SVGElement& element) {
  return IsSVGPolylineElement(element);
}

class SVGRadialGradientElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGRadialGradientElement(const SVGRadialGradientElement&);
void IsSVGRadialGradientElement(const SVGRadialGradientElement*);

inline bool IsSVGRadialGradientElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kRadialGradientTag);
}
inline bool IsSVGRadialGradientElement(const SVGElement* element) {
  return element && IsSVGRadialGradientElement(*element);
}
inline bool IsSVGRadialGradientElement(const Node& node) {
  return node.IsSVGElement() && IsSVGRadialGradientElement(To<SVGElement>(node));
}
inline bool IsSVGRadialGradientElement(const Node* node) {
  return node && IsSVGRadialGradientElement(*node);
}
template <>
inline bool IsElementOfType<const SVGRadialGradientElement>(const Node& node) {
  return IsSVGRadialGradientElement(node);
}
template <>
inline bool IsElementOfType<const SVGRadialGradientElement>(const SVGElement& element) {
  return IsSVGRadialGradientElement(element);
}

class SVGRectElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGRectElement(const SVGRectElement&);
void IsSVGRectElement(const SVGRectElement*);

inline bool IsSVGRectElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kRectTag);
}
inline bool IsSVGRectElement(const SVGElement* element) {
  return element && IsSVGRectElement(*element);
}
inline bool IsSVGRectElement(const Node& node) {
  return node.IsSVGElement() && IsSVGRectElement(To<SVGElement>(node));
}
inline bool IsSVGRectElement(const Node* node) {
  return node && IsSVGRectElement(*node);
}
template <>
inline bool IsElementOfType<const SVGRectElement>(const Node& node) {
  return IsSVGRectElement(node);
}
template <>
inline bool IsElementOfType<const SVGRectElement>(const SVGElement& element) {
  return IsSVGRectElement(element);
}

class SVGSVGElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGSVGElement(const SVGSVGElement&);
void IsSVGSVGElement(const SVGSVGElement*);

inline bool IsSVGSVGElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kSVGTag);
}
inline bool IsSVGSVGElement(const SVGElement* element) {
  return element && IsSVGSVGElement(*element);
}
inline bool IsSVGSVGElement(const Node& node) {
  return node.IsSVGElement() && IsSVGSVGElement(To<SVGElement>(node));
}
inline bool IsSVGSVGElement(const Node* node) {
  return node && IsSVGSVGElement(*node);
}
template <>
inline bool IsElementOfType<const SVGSVGElement>(const Node& node) {
  return IsSVGSVGElement(node);
}
template <>
inline bool IsElementOfType<const SVGSVGElement>(const SVGElement& element) {
  return IsSVGSVGElement(element);
}

class SVGSetElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGSetElement(const SVGSetElement&);
void IsSVGSetElement(const SVGSetElement*);

inline bool IsSVGSetElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kSetTag);
}
inline bool IsSVGSetElement(const SVGElement* element) {
  return element && IsSVGSetElement(*element);
}
inline bool IsSVGSetElement(const Node& node) {
  return node.IsSVGElement() && IsSVGSetElement(To<SVGElement>(node));
}
inline bool IsSVGSetElement(const Node* node) {
  return node && IsSVGSetElement(*node);
}
template <>
inline bool IsElementOfType<const SVGSetElement>(const Node& node) {
  return IsSVGSetElement(node);
}
template <>
inline bool IsElementOfType<const SVGSetElement>(const SVGElement& element) {
  return IsSVGSetElement(element);
}

class SVGStopElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGStopElement(const SVGStopElement&);
void IsSVGStopElement(const SVGStopElement*);

inline bool IsSVGStopElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kStopTag);
}
inline bool IsSVGStopElement(const SVGElement* element) {
  return element && IsSVGStopElement(*element);
}
inline bool IsSVGStopElement(const Node& node) {
  return node.IsSVGElement() && IsSVGStopElement(To<SVGElement>(node));
}
inline bool IsSVGStopElement(const Node* node) {
  return node && IsSVGStopElement(*node);
}
template <>
inline bool IsElementOfType<const SVGStopElement>(const Node& node) {
  return IsSVGStopElement(node);
}
template <>
inline bool IsElementOfType<const SVGStopElement>(const SVGElement& element) {
  return IsSVGStopElement(element);
}

class SVGSwitchElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGSwitchElement(const SVGSwitchElement&);
void IsSVGSwitchElement(const SVGSwitchElement*);

inline bool IsSVGSwitchElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kSwitchTag);
}
inline bool IsSVGSwitchElement(const SVGElement* element) {
  return element && IsSVGSwitchElement(*element);
}
inline bool IsSVGSwitchElement(const Node& node) {
  return node.IsSVGElement() && IsSVGSwitchElement(To<SVGElement>(node));
}
inline bool IsSVGSwitchElement(const Node* node) {
  return node && IsSVGSwitchElement(*node);
}
template <>
inline bool IsElementOfType<const SVGSwitchElement>(const Node& node) {
  return IsSVGSwitchElement(node);
}
template <>
inline bool IsElementOfType<const SVGSwitchElement>(const SVGElement& element) {
  return IsSVGSwitchElement(element);
}

class SVGSymbolElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGSymbolElement(const SVGSymbolElement&);
void IsSVGSymbolElement(const SVGSymbolElement*);

inline bool IsSVGSymbolElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kSymbolTag);
}
inline bool IsSVGSymbolElement(const SVGElement* element) {
  return element && IsSVGSymbolElement(*element);
}
inline bool IsSVGSymbolElement(const Node& node) {
  return node.IsSVGElement() && IsSVGSymbolElement(To<SVGElement>(node));
}
inline bool IsSVGSymbolElement(const Node* node) {
  return node && IsSVGSymbolElement(*node);
}
template <>
inline bool IsElementOfType<const SVGSymbolElement>(const Node& node) {
  return IsSVGSymbolElement(node);
}
template <>
inline bool IsElementOfType<const SVGSymbolElement>(const SVGElement& element) {
  return IsSVGSymbolElement(element);
}

class SVGTSpanElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGTSpanElement(const SVGTSpanElement&);
void IsSVGTSpanElement(const SVGTSpanElement*);

inline bool IsSVGTSpanElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kTSpanTag);
}
inline bool IsSVGTSpanElement(const SVGElement* element) {
  return element && IsSVGTSpanElement(*element);
}
inline bool IsSVGTSpanElement(const Node& node) {
  return node.IsSVGElement() && IsSVGTSpanElement(To<SVGElement>(node));
}
inline bool IsSVGTSpanElement(const Node* node) {
  return node && IsSVGTSpanElement(*node);
}
template <>
inline bool IsElementOfType<const SVGTSpanElement>(const Node& node) {
  return IsSVGTSpanElement(node);
}
template <>
inline bool IsElementOfType<const SVGTSpanElement>(const SVGElement& element) {
  return IsSVGTSpanElement(element);
}

class SVGTextElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGTextElement(const SVGTextElement&);
void IsSVGTextElement(const SVGTextElement*);

inline bool IsSVGTextElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kTextTag);
}
inline bool IsSVGTextElement(const SVGElement* element) {
  return element && IsSVGTextElement(*element);
}
inline bool IsSVGTextElement(const Node& node) {
  return node.IsSVGElement() && IsSVGTextElement(To<SVGElement>(node));
}
inline bool IsSVGTextElement(const Node* node) {
  return node && IsSVGTextElement(*node);
}
template <>
inline bool IsElementOfType<const SVGTextElement>(const Node& node) {
  return IsSVGTextElement(node);
}
template <>
inline bool IsElementOfType<const SVGTextElement>(const SVGElement& element) {
  return IsSVGTextElement(element);
}

class SVGTextPathElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGTextPathElement(const SVGTextPathElement&);
void IsSVGTextPathElement(const SVGTextPathElement*);

inline bool IsSVGTextPathElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kTextPathTag);
}
inline bool IsSVGTextPathElement(const SVGElement* element) {
  return element && IsSVGTextPathElement(*element);
}
inline bool IsSVGTextPathElement(const Node& node) {
  return node.IsSVGElement() && IsSVGTextPathElement(To<SVGElement>(node));
}
inline bool IsSVGTextPathElement(const Node* node) {
  return node && IsSVGTextPathElement(*node);
}
template <>
inline bool IsElementOfType<const SVGTextPathElement>(const Node& node) {
  return IsSVGTextPathElement(node);
}
template <>
inline bool IsElementOfType<const SVGTextPathElement>(const SVGElement& element) {
  return IsSVGTextPathElement(element);
}

class SVGTitleElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGTitleElement(const SVGTitleElement&);
void IsSVGTitleElement(const SVGTitleElement*);

inline bool IsSVGTitleElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kTitleTag);
}
inline bool IsSVGTitleElement(const SVGElement* element) {
  return element && IsSVGTitleElement(*element);
}
inline bool IsSVGTitleElement(const Node& node) {
  return node.IsSVGElement() && IsSVGTitleElement(To<SVGElement>(node));
}
inline bool IsSVGTitleElement(const Node* node) {
  return node && IsSVGTitleElement(*node);
}
template <>
inline bool IsElementOfType<const SVGTitleElement>(const Node& node) {
  return IsSVGTitleElement(node);
}
template <>
inline bool IsElementOfType<const SVGTitleElement>(const SVGElement& element) {
  return IsSVGTitleElement(element);
}

class SVGUseElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGUseElement(const SVGUseElement&);
void IsSVGUseElement(const SVGUseElement*);

inline bool IsSVGUseElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kUseTag);
}
inline bool IsSVGUseElement(const SVGElement* element) {
  return element && IsSVGUseElement(*element);
}
inline bool IsSVGUseElement(const Node& node) {
  return node.IsSVGElement() && IsSVGUseElement(To<SVGElement>(node));
}
inline bool IsSVGUseElement(const Node* node) {
  return node && IsSVGUseElement(*node);
}
template <>
inline bool IsElementOfType<const SVGUseElement>(const Node& node) {
  return IsSVGUseElement(node);
}
template <>
inline bool IsElementOfType<const SVGUseElement>(const SVGElement& element) {
  return IsSVGUseElement(element);
}

class SVGViewElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGViewElement(const SVGViewElement&);
void IsSVGViewElement(const SVGViewElement*);

inline bool IsSVGViewElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kViewTag);
}
inline bool IsSVGViewElement(const SVGElement* element) {
  return element && IsSVGViewElement(*element);
}
inline bool IsSVGViewElement(const Node& node) {
  return node.IsSVGElement() && IsSVGViewElement(To<SVGElement>(node));
}
inline bool IsSVGViewElement(const Node* node) {
  return node && IsSVGViewElement(*node);
}
template <>
inline bool IsElementOfType<const SVGViewElement>(const Node& node) {
  return IsSVGViewElement(node);
}
template <>
inline bool IsElementOfType<const SVGViewElement>(const SVGElement& element) {
  return IsSVGViewElement(element);
}

class SVGScriptElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGScriptElement(const SVGScriptElement&);
void IsSVGScriptElement(const SVGScriptElement*);

inline bool IsSVGScriptElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kScriptTag);
}
inline bool IsSVGScriptElement(const SVGElement* element) {
  return element && IsSVGScriptElement(*element);
}
inline bool IsSVGScriptElement(const Node& node) {
  return node.IsSVGElement() && IsSVGScriptElement(To<SVGElement>(node));
}
inline bool IsSVGScriptElement(const Node* node) {
  return node && IsSVGScriptElement(*node);
}
template <>
inline bool IsElementOfType<const SVGScriptElement>(const Node& node) {
  return IsSVGScriptElement(node);
}
template <>
inline bool IsElementOfType<const SVGScriptElement>(const SVGElement& element) {
  return IsSVGScriptElement(element);
}

class SVGStyleElement;
// Catch unnecessary runtime check of type known at compile time.
void IsSVGStyleElement(const SVGStyleElement&);
void IsSVGStyleElement(const SVGStyleElement*);

inline bool IsSVGStyleElement(const SVGElement& element) {
  return element.HasTagName(svg_names::kStyleTag);
}
inline bool IsSVGStyleElement(const SVGElement* element) {
  return element && IsSVGStyleElement(*element);
}
inline bool IsSVGStyleElement(const Node& node) {
  return node.IsSVGElement() && IsSVGStyleElement(To<SVGElement>(node));
}
inline bool IsSVGStyleElement(const Node* node) {
  return node && IsSVGStyleElement(*node);
}
template <>
inline bool IsElementOfType<const SVGStyleElement>(const Node& node) {
  return IsSVGStyleElement(node);
}
template <>
inline bool IsElementOfType<const SVGStyleElement>(const SVGElement& element) {
  return IsSVGStyleElement(element);
}

// Using macros because the types are forward-declared and we don't want to use
// reinterpret_cast in the casting functions above. reinterpret_cast would be
// unsafe due to multiple inheritence.

// TODO(abhijeet@igalia.com): Remove below type casting macros.
template <>
struct DowncastTraits<SVGAElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kATag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGAElement>(To<SVGElement>(node));
  }
};
#define ToSVGAElement(x) blink::To<blink::SVGAElement>(x)
#define ToSVGAElementOrNull(x) blink::DynamicTo<blink::SVGAElement>(x)

template <>
struct DowncastTraits<SVGAnimateMotionElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kAnimateMotionTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGAnimateMotionElement>(To<SVGElement>(node));
  }
};
#define ToSVGAnimateMotionElement(x) blink::To<blink::SVGAnimateMotionElement>(x)
#define ToSVGAnimateMotionElementOrNull(x) blink::DynamicTo<blink::SVGAnimateMotionElement>(x)

template <>
struct DowncastTraits<SVGAnimateTransformElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kAnimateTransformTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGAnimateTransformElement>(To<SVGElement>(node));
  }
};
#define ToSVGAnimateTransformElement(x) blink::To<blink::SVGAnimateTransformElement>(x)
#define ToSVGAnimateTransformElementOrNull(x) blink::DynamicTo<blink::SVGAnimateTransformElement>(x)

template <>
struct DowncastTraits<SVGCircleElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kCircleTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGCircleElement>(To<SVGElement>(node));
  }
};
#define ToSVGCircleElement(x) blink::To<blink::SVGCircleElement>(x)
#define ToSVGCircleElementOrNull(x) blink::DynamicTo<blink::SVGCircleElement>(x)

template <>
struct DowncastTraits<SVGClipPathElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kClipPathTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGClipPathElement>(To<SVGElement>(node));
  }
};
#define ToSVGClipPathElement(x) blink::To<blink::SVGClipPathElement>(x)
#define ToSVGClipPathElementOrNull(x) blink::DynamicTo<blink::SVGClipPathElement>(x)

template <>
struct DowncastTraits<SVGDefsElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kDefsTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGDefsElement>(To<SVGElement>(node));
  }
};
#define ToSVGDefsElement(x) blink::To<blink::SVGDefsElement>(x)
#define ToSVGDefsElementOrNull(x) blink::DynamicTo<blink::SVGDefsElement>(x)

template <>
struct DowncastTraits<SVGDescElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kDescTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGDescElement>(To<SVGElement>(node));
  }
};
#define ToSVGDescElement(x) blink::To<blink::SVGDescElement>(x)
#define ToSVGDescElementOrNull(x) blink::DynamicTo<blink::SVGDescElement>(x)

template <>
struct DowncastTraits<SVGDiscardElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kDiscardTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGDiscardElement>(To<SVGElement>(node));
  }
};
#define ToSVGDiscardElement(x) blink::To<blink::SVGDiscardElement>(x)
#define ToSVGDiscardElementOrNull(x) blink::DynamicTo<blink::SVGDiscardElement>(x)

template <>
struct DowncastTraits<SVGEllipseElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kEllipseTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGEllipseElement>(To<SVGElement>(node));
  }
};
#define ToSVGEllipseElement(x) blink::To<blink::SVGEllipseElement>(x)
#define ToSVGEllipseElementOrNull(x) blink::DynamicTo<blink::SVGEllipseElement>(x)

template <>
struct DowncastTraits<SVGFEBlendElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEBlendTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEBlendElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEBlendElement(x) blink::To<blink::SVGFEBlendElement>(x)
#define ToSVGFEBlendElementOrNull(x) blink::DynamicTo<blink::SVGFEBlendElement>(x)

template <>
struct DowncastTraits<SVGFEColorMatrixElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEColorMatrixTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEColorMatrixElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEColorMatrixElement(x) blink::To<blink::SVGFEColorMatrixElement>(x)
#define ToSVGFEColorMatrixElementOrNull(x) blink::DynamicTo<blink::SVGFEColorMatrixElement>(x)

template <>
struct DowncastTraits<SVGFEComponentTransferElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEComponentTransferTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEComponentTransferElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEComponentTransferElement(x) blink::To<blink::SVGFEComponentTransferElement>(x)
#define ToSVGFEComponentTransferElementOrNull(x) blink::DynamicTo<blink::SVGFEComponentTransferElement>(x)

template <>
struct DowncastTraits<SVGFECompositeElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFECompositeTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFECompositeElement>(To<SVGElement>(node));
  }
};
#define ToSVGFECompositeElement(x) blink::To<blink::SVGFECompositeElement>(x)
#define ToSVGFECompositeElementOrNull(x) blink::DynamicTo<blink::SVGFECompositeElement>(x)

template <>
struct DowncastTraits<SVGFEConvolveMatrixElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEConvolveMatrixTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEConvolveMatrixElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEConvolveMatrixElement(x) blink::To<blink::SVGFEConvolveMatrixElement>(x)
#define ToSVGFEConvolveMatrixElementOrNull(x) blink::DynamicTo<blink::SVGFEConvolveMatrixElement>(x)

template <>
struct DowncastTraits<SVGFEDiffuseLightingElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEDiffuseLightingTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEDiffuseLightingElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEDiffuseLightingElement(x) blink::To<blink::SVGFEDiffuseLightingElement>(x)
#define ToSVGFEDiffuseLightingElementOrNull(x) blink::DynamicTo<blink::SVGFEDiffuseLightingElement>(x)

template <>
struct DowncastTraits<SVGFEDisplacementMapElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEDisplacementMapTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEDisplacementMapElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEDisplacementMapElement(x) blink::To<blink::SVGFEDisplacementMapElement>(x)
#define ToSVGFEDisplacementMapElementOrNull(x) blink::DynamicTo<blink::SVGFEDisplacementMapElement>(x)

template <>
struct DowncastTraits<SVGFEDistantLightElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEDistantLightTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEDistantLightElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEDistantLightElement(x) blink::To<blink::SVGFEDistantLightElement>(x)
#define ToSVGFEDistantLightElementOrNull(x) blink::DynamicTo<blink::SVGFEDistantLightElement>(x)

template <>
struct DowncastTraits<SVGFEDropShadowElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEDropShadowTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEDropShadowElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEDropShadowElement(x) blink::To<blink::SVGFEDropShadowElement>(x)
#define ToSVGFEDropShadowElementOrNull(x) blink::DynamicTo<blink::SVGFEDropShadowElement>(x)

template <>
struct DowncastTraits<SVGFEFloodElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEFloodTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEFloodElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEFloodElement(x) blink::To<blink::SVGFEFloodElement>(x)
#define ToSVGFEFloodElementOrNull(x) blink::DynamicTo<blink::SVGFEFloodElement>(x)

template <>
struct DowncastTraits<SVGFEFuncAElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEFuncATag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEFuncAElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEFuncAElement(x) blink::To<blink::SVGFEFuncAElement>(x)
#define ToSVGFEFuncAElementOrNull(x) blink::DynamicTo<blink::SVGFEFuncAElement>(x)

template <>
struct DowncastTraits<SVGFEFuncBElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEFuncBTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEFuncBElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEFuncBElement(x) blink::To<blink::SVGFEFuncBElement>(x)
#define ToSVGFEFuncBElementOrNull(x) blink::DynamicTo<blink::SVGFEFuncBElement>(x)

template <>
struct DowncastTraits<SVGFEFuncGElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEFuncGTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEFuncGElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEFuncGElement(x) blink::To<blink::SVGFEFuncGElement>(x)
#define ToSVGFEFuncGElementOrNull(x) blink::DynamicTo<blink::SVGFEFuncGElement>(x)

template <>
struct DowncastTraits<SVGFEFuncRElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEFuncRTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEFuncRElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEFuncRElement(x) blink::To<blink::SVGFEFuncRElement>(x)
#define ToSVGFEFuncRElementOrNull(x) blink::DynamicTo<blink::SVGFEFuncRElement>(x)

template <>
struct DowncastTraits<SVGFEGaussianBlurElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEGaussianBlurTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEGaussianBlurElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEGaussianBlurElement(x) blink::To<blink::SVGFEGaussianBlurElement>(x)
#define ToSVGFEGaussianBlurElementOrNull(x) blink::DynamicTo<blink::SVGFEGaussianBlurElement>(x)

template <>
struct DowncastTraits<SVGFEImageElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEImageTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEImageElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEImageElement(x) blink::To<blink::SVGFEImageElement>(x)
#define ToSVGFEImageElementOrNull(x) blink::DynamicTo<blink::SVGFEImageElement>(x)

template <>
struct DowncastTraits<SVGFEMergeElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEMergeTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEMergeElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEMergeElement(x) blink::To<blink::SVGFEMergeElement>(x)
#define ToSVGFEMergeElementOrNull(x) blink::DynamicTo<blink::SVGFEMergeElement>(x)

template <>
struct DowncastTraits<SVGFEMergeNodeElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEMergeNodeTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEMergeNodeElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEMergeNodeElement(x) blink::To<blink::SVGFEMergeNodeElement>(x)
#define ToSVGFEMergeNodeElementOrNull(x) blink::DynamicTo<blink::SVGFEMergeNodeElement>(x)

template <>
struct DowncastTraits<SVGFEMorphologyElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEMorphologyTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEMorphologyElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEMorphologyElement(x) blink::To<blink::SVGFEMorphologyElement>(x)
#define ToSVGFEMorphologyElementOrNull(x) blink::DynamicTo<blink::SVGFEMorphologyElement>(x)

template <>
struct DowncastTraits<SVGFEOffsetElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEOffsetTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEOffsetElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEOffsetElement(x) blink::To<blink::SVGFEOffsetElement>(x)
#define ToSVGFEOffsetElementOrNull(x) blink::DynamicTo<blink::SVGFEOffsetElement>(x)

template <>
struct DowncastTraits<SVGFEPointLightElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFEPointLightTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFEPointLightElement>(To<SVGElement>(node));
  }
};
#define ToSVGFEPointLightElement(x) blink::To<blink::SVGFEPointLightElement>(x)
#define ToSVGFEPointLightElementOrNull(x) blink::DynamicTo<blink::SVGFEPointLightElement>(x)

template <>
struct DowncastTraits<SVGFESpecularLightingElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFESpecularLightingTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFESpecularLightingElement>(To<SVGElement>(node));
  }
};
#define ToSVGFESpecularLightingElement(x) blink::To<blink::SVGFESpecularLightingElement>(x)
#define ToSVGFESpecularLightingElementOrNull(x) blink::DynamicTo<blink::SVGFESpecularLightingElement>(x)

template <>
struct DowncastTraits<SVGFESpotLightElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFESpotLightTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFESpotLightElement>(To<SVGElement>(node));
  }
};
#define ToSVGFESpotLightElement(x) blink::To<blink::SVGFESpotLightElement>(x)
#define ToSVGFESpotLightElementOrNull(x) blink::DynamicTo<blink::SVGFESpotLightElement>(x)

template <>
struct DowncastTraits<SVGFETileElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFETileTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFETileElement>(To<SVGElement>(node));
  }
};
#define ToSVGFETileElement(x) blink::To<blink::SVGFETileElement>(x)
#define ToSVGFETileElementOrNull(x) blink::DynamicTo<blink::SVGFETileElement>(x)

template <>
struct DowncastTraits<SVGFETurbulenceElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFETurbulenceTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFETurbulenceElement>(To<SVGElement>(node));
  }
};
#define ToSVGFETurbulenceElement(x) blink::To<blink::SVGFETurbulenceElement>(x)
#define ToSVGFETurbulenceElementOrNull(x) blink::DynamicTo<blink::SVGFETurbulenceElement>(x)

template <>
struct DowncastTraits<SVGFilterElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kFilterTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGFilterElement>(To<SVGElement>(node));
  }
};
#define ToSVGFilterElement(x) blink::To<blink::SVGFilterElement>(x)
#define ToSVGFilterElementOrNull(x) blink::DynamicTo<blink::SVGFilterElement>(x)

template <>
struct DowncastTraits<SVGForeignObjectElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kForeignObjectTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGForeignObjectElement>(To<SVGElement>(node));
  }
};
#define ToSVGForeignObjectElement(x) blink::To<blink::SVGForeignObjectElement>(x)
#define ToSVGForeignObjectElementOrNull(x) blink::DynamicTo<blink::SVGForeignObjectElement>(x)

template <>
struct DowncastTraits<SVGGElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kGTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGGElement>(To<SVGElement>(node));
  }
};
#define ToSVGGElement(x) blink::To<blink::SVGGElement>(x)
#define ToSVGGElementOrNull(x) blink::DynamicTo<blink::SVGGElement>(x)

template <>
struct DowncastTraits<SVGImageElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kImageTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGImageElement>(To<SVGElement>(node));
  }
};
#define ToSVGImageElement(x) blink::To<blink::SVGImageElement>(x)
#define ToSVGImageElementOrNull(x) blink::DynamicTo<blink::SVGImageElement>(x)

template <>
struct DowncastTraits<SVGLineElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kLineTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGLineElement>(To<SVGElement>(node));
  }
};
#define ToSVGLineElement(x) blink::To<blink::SVGLineElement>(x)
#define ToSVGLineElementOrNull(x) blink::DynamicTo<blink::SVGLineElement>(x)

template <>
struct DowncastTraits<SVGLinearGradientElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kLinearGradientTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGLinearGradientElement>(To<SVGElement>(node));
  }
};
#define ToSVGLinearGradientElement(x) blink::To<blink::SVGLinearGradientElement>(x)
#define ToSVGLinearGradientElementOrNull(x) blink::DynamicTo<blink::SVGLinearGradientElement>(x)

template <>
struct DowncastTraits<SVGMPathElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kMPathTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGMPathElement>(To<SVGElement>(node));
  }
};
#define ToSVGMPathElement(x) blink::To<blink::SVGMPathElement>(x)
#define ToSVGMPathElementOrNull(x) blink::DynamicTo<blink::SVGMPathElement>(x)

template <>
struct DowncastTraits<SVGMarkerElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kMarkerTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGMarkerElement>(To<SVGElement>(node));
  }
};
#define ToSVGMarkerElement(x) blink::To<blink::SVGMarkerElement>(x)
#define ToSVGMarkerElementOrNull(x) blink::DynamicTo<blink::SVGMarkerElement>(x)

template <>
struct DowncastTraits<SVGMaskElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kMaskTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGMaskElement>(To<SVGElement>(node));
  }
};
#define ToSVGMaskElement(x) blink::To<blink::SVGMaskElement>(x)
#define ToSVGMaskElementOrNull(x) blink::DynamicTo<blink::SVGMaskElement>(x)

template <>
struct DowncastTraits<SVGMetadataElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kMetadataTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGMetadataElement>(To<SVGElement>(node));
  }
};
#define ToSVGMetadataElement(x) blink::To<blink::SVGMetadataElement>(x)
#define ToSVGMetadataElementOrNull(x) blink::DynamicTo<blink::SVGMetadataElement>(x)

template <>
struct DowncastTraits<SVGPathElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kPathTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGPathElement>(To<SVGElement>(node));
  }
};
#define ToSVGPathElement(x) blink::To<blink::SVGPathElement>(x)
#define ToSVGPathElementOrNull(x) blink::DynamicTo<blink::SVGPathElement>(x)

template <>
struct DowncastTraits<SVGPatternElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kPatternTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGPatternElement>(To<SVGElement>(node));
  }
};
#define ToSVGPatternElement(x) blink::To<blink::SVGPatternElement>(x)
#define ToSVGPatternElementOrNull(x) blink::DynamicTo<blink::SVGPatternElement>(x)

template <>
struct DowncastTraits<SVGPolygonElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kPolygonTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGPolygonElement>(To<SVGElement>(node));
  }
};
#define ToSVGPolygonElement(x) blink::To<blink::SVGPolygonElement>(x)
#define ToSVGPolygonElementOrNull(x) blink::DynamicTo<blink::SVGPolygonElement>(x)

template <>
struct DowncastTraits<SVGPolylineElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kPolylineTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGPolylineElement>(To<SVGElement>(node));
  }
};
#define ToSVGPolylineElement(x) blink::To<blink::SVGPolylineElement>(x)
#define ToSVGPolylineElementOrNull(x) blink::DynamicTo<blink::SVGPolylineElement>(x)

template <>
struct DowncastTraits<SVGRadialGradientElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kRadialGradientTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGRadialGradientElement>(To<SVGElement>(node));
  }
};
#define ToSVGRadialGradientElement(x) blink::To<blink::SVGRadialGradientElement>(x)
#define ToSVGRadialGradientElementOrNull(x) blink::DynamicTo<blink::SVGRadialGradientElement>(x)

template <>
struct DowncastTraits<SVGRectElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kRectTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGRectElement>(To<SVGElement>(node));
  }
};
#define ToSVGRectElement(x) blink::To<blink::SVGRectElement>(x)
#define ToSVGRectElementOrNull(x) blink::DynamicTo<blink::SVGRectElement>(x)

template <>
struct DowncastTraits<SVGSVGElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kSVGTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGSVGElement>(To<SVGElement>(node));
  }
};
#define ToSVGSVGElement(x) blink::To<blink::SVGSVGElement>(x)
#define ToSVGSVGElementOrNull(x) blink::DynamicTo<blink::SVGSVGElement>(x)

template <>
struct DowncastTraits<SVGSetElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kSetTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGSetElement>(To<SVGElement>(node));
  }
};
#define ToSVGSetElement(x) blink::To<blink::SVGSetElement>(x)
#define ToSVGSetElementOrNull(x) blink::DynamicTo<blink::SVGSetElement>(x)

template <>
struct DowncastTraits<SVGStopElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kStopTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGStopElement>(To<SVGElement>(node));
  }
};
#define ToSVGStopElement(x) blink::To<blink::SVGStopElement>(x)
#define ToSVGStopElementOrNull(x) blink::DynamicTo<blink::SVGStopElement>(x)

template <>
struct DowncastTraits<SVGSwitchElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kSwitchTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGSwitchElement>(To<SVGElement>(node));
  }
};
#define ToSVGSwitchElement(x) blink::To<blink::SVGSwitchElement>(x)
#define ToSVGSwitchElementOrNull(x) blink::DynamicTo<blink::SVGSwitchElement>(x)

template <>
struct DowncastTraits<SVGSymbolElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kSymbolTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGSymbolElement>(To<SVGElement>(node));
  }
};
#define ToSVGSymbolElement(x) blink::To<blink::SVGSymbolElement>(x)
#define ToSVGSymbolElementOrNull(x) blink::DynamicTo<blink::SVGSymbolElement>(x)

template <>
struct DowncastTraits<SVGTSpanElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kTSpanTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGTSpanElement>(To<SVGElement>(node));
  }
};
#define ToSVGTSpanElement(x) blink::To<blink::SVGTSpanElement>(x)
#define ToSVGTSpanElementOrNull(x) blink::DynamicTo<blink::SVGTSpanElement>(x)

template <>
struct DowncastTraits<SVGTextElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kTextTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGTextElement>(To<SVGElement>(node));
  }
};
#define ToSVGTextElement(x) blink::To<blink::SVGTextElement>(x)
#define ToSVGTextElementOrNull(x) blink::DynamicTo<blink::SVGTextElement>(x)

template <>
struct DowncastTraits<SVGTextPathElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kTextPathTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGTextPathElement>(To<SVGElement>(node));
  }
};
#define ToSVGTextPathElement(x) blink::To<blink::SVGTextPathElement>(x)
#define ToSVGTextPathElementOrNull(x) blink::DynamicTo<blink::SVGTextPathElement>(x)

template <>
struct DowncastTraits<SVGTitleElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kTitleTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGTitleElement>(To<SVGElement>(node));
  }
};
#define ToSVGTitleElement(x) blink::To<blink::SVGTitleElement>(x)
#define ToSVGTitleElementOrNull(x) blink::DynamicTo<blink::SVGTitleElement>(x)

template <>
struct DowncastTraits<SVGUseElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kUseTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGUseElement>(To<SVGElement>(node));
  }
};
#define ToSVGUseElement(x) blink::To<blink::SVGUseElement>(x)
#define ToSVGUseElementOrNull(x) blink::DynamicTo<blink::SVGUseElement>(x)

template <>
struct DowncastTraits<SVGViewElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kViewTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGViewElement>(To<SVGElement>(node));
  }
};
#define ToSVGViewElement(x) blink::To<blink::SVGViewElement>(x)
#define ToSVGViewElementOrNull(x) blink::DynamicTo<blink::SVGViewElement>(x)

template <>
struct DowncastTraits<SVGScriptElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kScriptTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGScriptElement>(To<SVGElement>(node));
  }
};
#define ToSVGScriptElement(x) blink::To<blink::SVGScriptElement>(x)
#define ToSVGScriptElementOrNull(x) blink::DynamicTo<blink::SVGScriptElement>(x)

template <>
struct DowncastTraits<SVGStyleElement> {
  static bool AllowFrom(const Element& element) {
    return element.HasTagName(svg_names::kStyleTag);
  }
  static bool AllowFrom(const Node& node) {
    return node.IsSVGElement() && IsA<SVGStyleElement>(To<SVGElement>(node));
  }
};
#define ToSVGStyleElement(x) blink::To<blink::SVGStyleElement>(x)
#define ToSVGStyleElementOrNull(x) blink::DynamicTo<blink::SVGStyleElement>(x)


}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_SVG_ELEMENT_TYPE_HELPERS_H_
