// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_ACCESSIBILITY_MOJOM_AX_RELATIVE_BOUNDS_MOJOM_H_
#define UI_ACCESSIBILITY_MOJOM_AX_RELATIVE_BOUNDS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/accessibility/mojom/ax_relative_bounds.mojom-shared.h"
#include "ui/accessibility/mojom/ax_relative_bounds.mojom-forward.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/transform.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "ui/accessibility/ax_relative_bounds.h"




namespace ax {
namespace mojom {








class  AXRelativeBounds {
 public:
  using DataView = AXRelativeBoundsDataView;
  using Data_ = internal::AXRelativeBounds_Data;

  template <typename... Args>
  static AXRelativeBoundsPtr New(Args&&... args) {
    return AXRelativeBoundsPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AXRelativeBoundsPtr From(const U& u) {
    return mojo::TypeConverter<AXRelativeBoundsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AXRelativeBounds>::Convert(*this);
  }


  AXRelativeBounds();

  AXRelativeBounds(
      int32_t offset_container_id,
      const gfx::RectF& bounds,
      const gfx::Transform& transform);

  ~AXRelativeBounds();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AXRelativeBoundsPtr>
  AXRelativeBoundsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AXRelativeBounds>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AXRelativeBounds::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AXRelativeBounds::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AXRelativeBounds_UnserializedMessageContext<
            UserType, AXRelativeBounds::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AXRelativeBounds::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AXRelativeBounds::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AXRelativeBounds_UnserializedMessageContext<
            UserType, AXRelativeBounds::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AXRelativeBounds::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int32_t offset_container_id;
  
  gfx::RectF bounds;
  
  gfx::Transform transform;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
AXRelativeBoundsPtr AXRelativeBounds::Clone() const {
  return New(
      mojo::Clone(offset_container_id),
      mojo::Clone(bounds),
      mojo::Clone(transform)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AXRelativeBounds>::value>::type*>
bool AXRelativeBounds::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->offset_container_id, other_struct.offset_container_id))
    return false;
  if (!mojo::Equals(this->bounds, other_struct.bounds))
    return false;
  if (!mojo::Equals(this->transform, other_struct.transform))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace ax

namespace mojo {


template <>
struct  StructTraits<::ax::mojom::AXRelativeBounds::DataView,
                                         ::ax::mojom::AXRelativeBoundsPtr> {
  static bool IsNull(const ::ax::mojom::AXRelativeBoundsPtr& input) { return !input; }
  static void SetToNull(::ax::mojom::AXRelativeBoundsPtr* output) { output->reset(); }

  static decltype(::ax::mojom::AXRelativeBounds::offset_container_id) offset_container_id(
      const ::ax::mojom::AXRelativeBoundsPtr& input) {
    return input->offset_container_id;
  }

  static const decltype(::ax::mojom::AXRelativeBounds::bounds)& bounds(
      const ::ax::mojom::AXRelativeBoundsPtr& input) {
    return input->bounds;
  }

  static const decltype(::ax::mojom::AXRelativeBounds::transform)& transform(
      const ::ax::mojom::AXRelativeBoundsPtr& input) {
    return input->transform;
  }

  static bool Read(::ax::mojom::AXRelativeBounds::DataView input, ::ax::mojom::AXRelativeBoundsPtr* output);
};

}  // namespace mojo

#endif  // UI_ACCESSIBILITY_MOJOM_AX_RELATIVE_BOUNDS_MOJOM_H_